/* Generated automatically. */
static const char configuration_arguments[] = "../gcc-16.2.0/configure --includedir=/usr/include/i686-linux-gnu --host=x86_64-linux-gnu --target=i686-linux-gnu --enable-languages=c,c++ --with-native-system-header-dir=/usr/include/i686-linux-gnu --with-headers=/usr/include/i686-linux-gnu --with-slibdir=/usr/lib/i686-linux-gnu --enable-shared --disable-multilib --enable-clocale=gnu --enable-threads=posix --enable-__cxa_atexit --disable-nls --disable-libstdcxx-pch --disable-bootstrap --with-gxx-include-dir=/usr/include/i686-linux-gnu/c++/16.2.0 --with-linker-hash-style=gnu --enable-default-pie --enable-default-ssp --without-zstd --disable-libsanitizer";
static const char thread_model[] = "posix";

static const struct {
  const char *name, *value;
} configure_default_options[] = { { "cpu", "generic" }, { "arch", "pentiumpro" } };
