/* Autogenerated by regenerate-attr-urls.py.  */

const attr_url_entry all_attrs[] = {
 { "OS_Task", "gcc/H8_002f300-Attributes.html#index-OS_005fTask_002c-H8_002f300", "H8/300", 7},
 { "OS_main", "gcc/AVR-Attributes.html#index-OS_005fmain_002c-AVR", "AVR", 7},
 { "OS_task", "gcc/AVR-Attributes.html#index-OS_005ftask_002c-AVR", "AVR", 7},
 { "abi_tag", "gcc/C_002b_002b-Attributes.html#index-abi_005ftag", "", 7},
 { "absdata", "gcc/AVR-Attributes.html#index-absdata_002c-AVR", "AVR", 7},
 { "access", "gcc/Common-Attributes.html#index-access", "", 6},
 { "address", "gcc/AVR-Attributes.html#index-address_002c-AVR", "AVR", 7},
 { "alias", "gcc/Common-Attributes.html#index-alias", "", 5},
 { "aligned", "gcc/Common-Attributes.html#index-aligned", "", 7},
 { "alloc_align", "gcc/Common-Attributes.html#index-alloc_005falign", "", 11},
 { "alloc_size", "gcc/Common-Attributes.html#index-alloc_005fsize", "", 10},
 { "altivec", "gcc/PowerPC-Attributes.html#index-altivec_002c-PowerPC", "PowerPC", 7},
 { "always_inline", "gcc/Common-Attributes.html#index-always_005finline", "", 13},
 { "amdgpu_hsa_kernel", "gcc/AMD-GCN-Attributes.html#index-amdgpu_005fhsa_005fkernel_002c-AMD-GCN", "AMD GCN", 17},
 { "arch=", "gcc/ARM-Attributes.html#index-arch_003d_002c-ARM", "ARM", 5},
 { "artificial", "gcc/Common-Attributes.html#index-artificial", "", 10},
 { "assume", "gcc/Common-Attributes.html#index-assume", "", 6},
 { "assume_aligned", "gcc/Common-Attributes.html#index-assume_005faligned", "", 14},
 { "aux", "gcc/ARC-Attributes.html#index-aux_002c-ARC", "ARC", 3},
 { "bank_switch", "gcc/M32C-Attributes.html#index-bank_005fswitch_002c-M32C", "M32C", 11},
 { "below100", "gcc/Xstormy16-Attributes.html#index-below100_002c-Xstormy16", "Xstormy16", 8},
 { "break_handler", "gcc/MicroBlaze-Attributes.html#index-break_005fhandler_002c-MicroBlaze", "MicroBlaze", 13},
 { "brk_interrupt", "gcc/RL78-Attributes.html#index-brk_005finterrupt_002c-RL78", "RL78", 13},
 { "btf_decl_tag", "gcc/Common-Attributes.html#index-btf_005fdecl_005ftag", "", 12},
 { "btf_type_tag", "gcc/Common-Attributes.html#index-btf_005ftype_005ftag", "", 12},
 { "callee_pop_aggregate_return", "gcc/x86-Attributes.html#index-callee_005fpop_005faggregate_005freturn_002c-x86", "x86", 27},
 { "cdecl", "gcc/x86-Attributes.html#index-cdecl_002c-x86-32", "x86-32", 5},
 { "cf_check", "gcc/x86-Attributes.html#index-cf_005fcheck_002c-x86", "x86", 8},
 { "cleanup", "gcc/Common-Attributes.html#index-cleanup", "", 7},
 { "code_readable", "gcc/MIPS-Attributes.html#index-code_005freadable_002c-MIPS", "MIPS", 13},
 { "cold", "gcc/C_002b_002b-Attributes.html#index-cold-1", "", 4},
 { "cold", "gcc/Common-Attributes.html#index-cold", "", 4},
 { "common", "gcc/Common-Attributes.html#index-common", "", 6},
 { "const", "gcc/Common-Attributes.html#index-const", "", 5},
 { "constructor", "gcc/Common-Attributes.html#index-constructor", "", 11},
 { "copy", "gcc/Common-Attributes.html#index-copy", "", 4},
 { "counted_by", "gcc/Common-Attributes.html#index-counted_005fby", "", 10},
 { "critical", "gcc/MSP430-Attributes.html#index-critical_002c-MSP430", "MSP430", 8},
 { "deprecated", "gcc/Common-Attributes.html#index-deprecated", "", 10},
 { "designated_init", "gcc/Common-Attributes.html#index-designated_005finit", "", 15},
 { "destructor", "gcc/Common-Attributes.html#index-destructor", "", 10},
 { "disinterrupt", "gcc/Epiphany-Attributes.html#index-disinterrupt_002c-Epiphany", "Epiphany", 12},
 { "dllexport", "gcc/Microsoft-Windows-Attributes.html#index-dllexport_002c-Microsoft-Windows", "Microsoft Windows", 9},
 { "dllimport", "gcc/Microsoft-Windows-Attributes.html#index-dllimport_002c-Microsoft-Windows", "Microsoft Windows", 9},
 { "eightbit_data", "gcc/H8_002f300-Attributes.html#index-eightbit_005fdata_002c-H8_002f300", "H8/300", 13},
 { "either", "gcc/MSP430-Attributes.html#index-either_002c-MSP430", "MSP430", 6},
 { "error", "gcc/Common-Attributes.html#index-error", "", 5},
 { "exception", "gcc/NDS32-Attributes.html#index-exception_002c-NDS32", "NDS32", 9},
 { "exception_handler", "gcc/Blackfin-Attributes.html#index-exception_005fhandler_002c-Blackfin", "Blackfin", 17},
 { "expected_throw", "gcc/Common-Attributes.html#index-expected_005fthrow", "", 14},
 { "externally_visible", "gcc/Common-Attributes.html#index-externally_005fvisible", "", 18},
 { "fallthrough", "gcc/Common-Attributes.html#index-fallthrough", "", 11},
 { "far", "gcc/MIPS-Attributes.html#index-far_002c-MIPS", "MIPS", 3},
 { "fast_interrupt", "gcc/M32C-Attributes.html#index-fast_005finterrupt_002c-M32C", "M32C", 14},
 { "fast_interrupt", "gcc/MicroBlaze-Attributes.html#index-fast_005finterrupt_002c-MicroBlaze", "MicroBlaze", 14},
 { "fast_interrupt", "gcc/RX-Attributes.html#index-fast_005finterrupt_002c-RX", "RX", 14},
 { "fastcall", "gcc/x86-Attributes.html#index-fastcall_002c-x86-32", "x86-32", 8},
 { "fd_arg", "gcc/Common-Attributes.html#index-fd_005farg", "", 6},
 { "fd_arg_read", "gcc/Common-Attributes.html#index-fd_005farg_005fread", "", 11},
 { "fd_arg_write", "gcc/Common-Attributes.html#index-fd_005farg_005fwrite", "", 12},
 { "fentry_name", "gcc/x86-Attributes.html#index-fentry_005fname_002c-x86", "x86", 11},
 { "fentry_section", "gcc/x86-Attributes.html#index-fentry_005fsection_002c-x86", "x86", 14},
 { "flatten", "gcc/Common-Attributes.html#index-flatten", "", 7},
 { "force_align_arg_pointer", "gcc/x86-Attributes.html#index-force_005falign_005farg_005fpointer_002c-x86", "x86", 23},
 { "format", "gcc/Common-Attributes.html#index-format", "", 6},
 { "format_arg", "gcc/Common-Attributes.html#index-format_005farg", "", 10},
 { "forwarder_section", "gcc/Epiphany-Attributes.html#index-forwarder_005fsection_002c-Epiphany", "Epiphany", 17},
 { "function_return", "gcc/x86-Attributes.html#index-function_005freturn_002c-x86", "x86", 15},
 { "function_vector", "gcc/H8_002f300-Attributes.html#index-function_005fvector_002c-H8_002f300", "H8/300", 15},
 { "function_vector", "gcc/M32C-Attributes.html#index-function_005fvector_002c-M16C_002fM32C", "M16C/M32C", 15},
 { "function_vector", "gcc/SH-Attributes.html#index-function_005fvector_002c-SH", "SH", 15},
 { "gcc_struct", "gcc/PowerPC-Attributes.html#index-gcc_005fstruct_002c-PowerPC", "PowerPC", 10},
 { "gcc_struct", "gcc/x86-Attributes.html#index-gcc_005fstruct_002c-x86", "x86", 10},
 { "general-regs-only", "gcc/ARM-Attributes.html#index-general-regs-only_002c-ARM", "ARM", 17},
 { "gnu_inline", "gcc/Common-Attributes.html#index-gnu_005finline", "", 10},
 { "hardbool", "gcc/Common-Attributes.html#index-hardbool", "", 8},
 { "hot", "gcc/C_002b_002b-Attributes.html#index-hot-1", "", 3},
 { "hot", "gcc/Common-Attributes.html#index-hot", "", 3},
 { "hotpatch", "gcc/S_002f390-Attributes.html#index-hotpatch_002c-S_002f390", "S/390", 8},
 { "ifunc", "gcc/Common-Attributes.html#index-ifunc", "", 5},
 { "indirect_branch", "gcc/x86-Attributes.html#index-indirect_005fbranch_002c-x86", "x86", 15},
 { "indirect_return", "gcc/x86-Attributes.html#index-indirect_005freturn_002c-x86", "x86", 15},
 { "init_priority", "gcc/C_002b_002b-Attributes.html#index-init_005fpriority", "", 13},
 { "interrupt", "gcc/ARC-Attributes.html#index-interrupt_002c-ARC", "ARC", 9},
 { "interrupt", "gcc/ARM-Attributes.html#index-interrupt_002c-ARM", "ARM", 9},
 { "interrupt", "gcc/AVR-Attributes.html#index-interrupt_002c-AVR", "AVR", 9},
 { "interrupt", "gcc/C-SKY-Attributes.html#index-interrupt_002c-C-SKY", "C-SKY", 9},
 { "interrupt", "gcc/Common-Attributes.html#index-interrupt", "", 9},
 { "interrupt", "gcc/Epiphany-Attributes.html#index-interrupt_002c-Epiphany", "Epiphany", 9},
 { "interrupt", "gcc/M32C-Attributes.html#index-interrupt_002c-M32C", "M32C", 9},
 { "interrupt", "gcc/M32R_002fD-Attributes.html#index-interrupt_002c-M32R_002fD", "M32R/D", 9},
 { "interrupt", "gcc/MIPS-Attributes.html#index-interrupt_002c-MIPS", "MIPS", 9},
 { "interrupt", "gcc/MSP430-Attributes.html#index-interrupt_002c-MSP430", "MSP430", 9},
 { "interrupt", "gcc/NDS32-Attributes.html#index-interrupt_002c-NDS32", "NDS32", 9},
 { "interrupt", "gcc/RISC-V-Attributes.html#index-interrupt_002c-RISC-V", "RISC-V", 9},
 { "interrupt", "gcc/RL78-Attributes.html#index-interrupt_002c-RL78", "RL78", 9},
 { "interrupt", "gcc/RX-Attributes.html#index-interrupt_002c-RX", "RX", 9},
 { "interrupt", "gcc/V850-Attributes.html#index-interrupt_002c-V850", "V850", 9},
 { "interrupt", "gcc/Visium-Attributes.html#index-interrupt_002c-Visium", "Visium", 9},
 { "interrupt", "gcc/Xstormy16-Attributes.html#index-interrupt_002c-Xstormy16", "Xstormy16", 9},
 { "interrupt", "gcc/m68k-Attributes.html#index-interrupt_002c-m68k", "m68k", 9},
 { "interrupt", "gcc/x86-Attributes.html#index-interrupt_002c-x86", "x86", 9},
 { "interrupt_handler", "gcc/Blackfin-Attributes.html#index-interrupt_005fhandler_002c-Blackfin", "Blackfin", 17},
 { "interrupt_handler", "gcc/Common-Attributes.html#index-interrupt_005fhandler", "", 17},
 { "interrupt_handler", "gcc/H8_002f300-Attributes.html#index-interrupt_005fhandler_002c-H8_002f300", "H8/300", 17},
 { "interrupt_handler", "gcc/MicroBlaze-Attributes.html#index-interrupt_005fhandler_002c-MicroBlaze", "MicroBlaze", 17},
 { "interrupt_handler", "gcc/SH-Attributes.html#index-interrupt_005fhandler_002c-SH", "SH", 17},
 { "interrupt_handler", "gcc/V850-Attributes.html#index-interrupt_005fhandler_002c-V850", "V850", 17},
 { "interrupt_handler", "gcc/m68k-Attributes.html#index-interrupt_005fhandler_002c-m68k", "m68k", 17},
 { "interrupt_thread", "gcc/m68k-Attributes.html#index-interrupt_005fthread_002c-fido", "fido", 16},
 { "io", "gcc/AVR-Attributes.html#index-io_002c-AVR", "AVR", 2},
 { "io_low", "gcc/AVR-Attributes.html#index-io_005flow_002c-AVR", "AVR", 6},
 { "isr", "gcc/ARM-Attributes.html#index-isr_002c-ARM", "ARM", 3},
 { "isr", "gcc/C-SKY-Attributes.html#index-isr_002c-C-SKY", "C-SKY", 3},
 { "jli_always", "gcc/ARC-Attributes.html#index-jli_005falways_002c-ARC", "ARC", 10},
 { "jli_fixed", "gcc/ARC-Attributes.html#index-jli_005ffixed_002c-ARC", "ARC", 9},
 { "keep_interrupts_masked", "gcc/MIPS-Attributes.html#index-keep_005finterrupts_005fmasked_002c-MIPS", "MIPS", 22},
 { "kernel", "gcc/Nvidia-PTX-Attributes.html#index-kernel_002c-Nvidia-PTX", "Nvidia PTX", 6},
 { "kspisusp", "gcc/Blackfin-Attributes.html#index-kspisusp_002c-Blackfin", "Blackfin", 8},
 { "l1_data", "gcc/Blackfin-Attributes.html#index-l1_005fdata_002c-Blackfin", "Blackfin", 7},
 { "l1_data_A", "gcc/Blackfin-Attributes.html#index-l1_005fdata_005fA_002c-Blackfin", "Blackfin", 9},
 { "l1_data_B", "gcc/Blackfin-Attributes.html#index-l1_005fdata_005fB_002c-Blackfin", "Blackfin", 9},
 { "l1_text", "gcc/Blackfin-Attributes.html#index-l1_005ftext_002c-Blackfin", "Blackfin", 7},
 { "l2", "gcc/Blackfin-Attributes.html#index-l2_002c-Blackfin", "Blackfin", 2},
 { "leaf", "gcc/Common-Attributes.html#index-leaf", "", 4},
 { "long_call", "gcc/ARC-Attributes.html#index-long_005fcall_002c-ARC", "ARC", 9},
 { "long_call", "gcc/ARM-Attributes.html#index-long_005fcall_002c-ARM", "ARM", 9},
 { "long_call", "gcc/Epiphany-Attributes.html#index-long_005fcall_002c-Epiphany", "Epiphany", 9},
 { "long_call", "gcc/MIPS-Attributes.html#index-long_005fcall_002c-MIPS", "MIPS", 9},
 { "longcall", "gcc/Blackfin-Attributes.html#index-longcall_002c-Blackfin", "Blackfin", 8},
 { "longcall", "gcc/PowerPC-Attributes.html#index-longcall_002c-PowerPC", "PowerPC", 8},
 { "lower", "gcc/MSP430-Attributes.html#index-lower_002c-MSP430", "MSP430", 5},
 { "malloc", "gcc/Common-Attributes.html#index-malloc", "", 6},
 { "may_alias", "gcc/Common-Attributes.html#index-may_005falias", "", 9},
 { "medium_call", "gcc/ARC-Attributes.html#index-medium_005fcall_002c-ARC", "ARC", 11},
 { "micromips", "gcc/MIPS-Attributes.html#index-micromips_002c-MIPS", "MIPS", 9},
 { "mips16", "gcc/MIPS-Attributes.html#index-mips16_002c-MIPS", "MIPS", 6},
 { "mode", "gcc/Common-Attributes.html#index-mode", "", 4},
 { "model", "gcc/IA-64-Attributes.html#index-model_002c-IA-64", "IA-64", 5},
 { "model", "gcc/LoongArch-Attributes.html#index-model_002c-LoongArch", "LoongArch", 5},
 { "model", "gcc/M32R_002fD-Attributes.html#index-model_002c-M32R_002fD", "M32R/D", 5},
 { "monitor", "gcc/H8_002f300-Attributes.html#index-monitor_002c-H8_002f300", "H8/300", 7},
 { "ms_abi", "gcc/x86-Attributes.html#index-ms_005fabi_002c-x86", "x86", 6},
 { "ms_hook_prologue", "gcc/x86-Attributes.html#index-ms_005fhook_005fprologue_002c-x86", "x86", 16},
 { "ms_struct", "gcc/PowerPC-Attributes.html#index-ms_005fstruct_002c-PowerPC", "PowerPC", 9},
 { "ms_struct", "gcc/x86-Attributes.html#index-ms_005fstruct_002c-x86", "x86", 9},
 { "musttail", "gcc/Common-Attributes.html#index-musttail", "", 8},
 { "naked", "gcc/Common-Attributes.html#index-naked", "", 5},
 { "near", "gcc/MIPS-Attributes.html#index-near_002c-MIPS", "MIPS", 4},
 { "nested", "gcc/NDS32-Attributes.html#index-nested_002c-NDS32", "NDS32", 6},
 { "nested_ready", "gcc/NDS32-Attributes.html#index-nested_005fready_002c-NDS32", "NDS32", 12},
 { "nesting", "gcc/Blackfin-Attributes.html#index-nesting_002c-Blackfin", "Blackfin", 7},
 { "nmi", "gcc/NDS32-Attributes.html#index-nmi_002c-NDS32", "NDS32", 3},
 { "nmi_handler", "gcc/Blackfin-Attributes.html#index-nmi_005fhandler_002c-Blackfin", "Blackfin", 11},
 { "no_callee_saved_registers", "gcc/x86-Attributes.html#index-no_005fcallee_005fsaved_005fregisters_002c-x86", "x86", 25},
 { "no_caller_saved_registers", "gcc/x86-Attributes.html#index-no_005fcaller_005fsaved_005fregisters_002c-x86", "x86", 25},
 { "no_dangling", "gcc/C_002b_002b-Attributes.html#index-no_005fdangling", "", 11},
 { "no_gccisr", "gcc/AVR-Attributes.html#index-no_005fgccisr_002c-AVR", "AVR", 9},
 { "no_icf", "gcc/Common-Attributes.html#index-no_005ficf", "", 6},
 { "no_instrument_function", "gcc/Common-Attributes.html#index-no_005finstrument_005ffunction", "", 22},
 { "no_reorder", "gcc/Common-Attributes.html#index-no_005freorder", "", 10},
 { "no_sanitize_address", "gcc/Common-Attributes.html#index-no_005fsanitize_005faddress", "", 19},
 { "no_sanitize_thread", "gcc/Common-Attributes.html#index-no_005fsanitize_005fthread", "", 18},
 { "no_sanitize_undefined", "gcc/Common-Attributes.html#index-no_005fsanitize_005fundefined", "", 21},
 { "no_split_stack", "gcc/Common-Attributes.html#index-no_005fsplit_005fstack", "", 14},
 { "no_stack_limit", "gcc/Common-Attributes.html#index-no_005fstack_005flimit", "", 14},
 { "no_stack_protector", "gcc/Common-Attributes.html#index-no_005fstack_005fprotector", "", 18},
 { "noblock", "gcc/AVR-Attributes.html#index-noblock_002c-AVR", "AVR", 7},
 { "nocf_check", "gcc/x86-Attributes.html#index-nocf_005fcheck_002c-x86", "x86", 10},
 { "noclone", "gcc/Common-Attributes.html#index-noclone", "", 7},
 { "nocommon", "gcc/Common-Attributes.html#index-nocommon", "", 8},
 { "nocompression", "gcc/MIPS-Attributes.html#index-nocompression_002c-MIPS", "MIPS", 13},
 { "nodirect_extern_access", "gcc/x86-Attributes.html#index-nodirect_005fextern_005faccess_002c-x86", "x86", 22},
 { "noinit", "gcc/Common-Attributes.html#index-noinit", "", 6},
 { "noinline", "gcc/Common-Attributes.html#index-noinline", "", 8},
 { "noipa", "gcc/Common-Attributes.html#index-noipa", "", 5},
 { "nomicromips", "gcc/MIPS-Attributes.html#index-nomicromips_002c-MIPS", "MIPS", 11},
 { "nomips16", "gcc/MIPS-Attributes.html#index-nomips16_002c-MIPS", "MIPS", 8},
 { "nonnull", "gcc/Common-Attributes.html#index-nonnull", "", 7},
 { "nonnull_if_nonzero", "gcc/Common-Attributes.html#index-nonnull_005fif_005fnonzero", "", 18},
 { "nonstring", "gcc/Common-Attributes.html#index-nonstring", "", 9},
 { "noplt", "gcc/Common-Attributes.html#index-noplt", "", 5},
 { "noreturn", "gcc/Common-Attributes.html#index-noreturn", "", 8},
 { "nosave_low_regs", "gcc/SH-Attributes.html#index-nosave_005flow_005fregs_002c-SH", "SH", 15},
 { "not_nested", "gcc/NDS32-Attributes.html#index-not_005fnested_002c-NDS32", "NDS32", 10},
 { "nothrow", "gcc/Common-Attributes.html#index-nothrow", "", 7},
 { "notshared", "gcc/ARM-Attributes.html#index-notshared_002c-ARM", "ARM", 9},
 { "null_terminated_string_arg", "gcc/Common-Attributes.html#index-null_005fterminated_005fstring_005farg", "", 26},
 { "objc_nullability", "gcc/Common-Attributes.html#index-objc_005fnullability", "", 16},
 { "objc_root_class", "gcc/Common-Attributes.html#index-objc_005froot_005fclass", "", 15},
 { "optimize", "gcc/Common-Attributes.html#index-optimize", "", 8},
 { "packed", "gcc/Common-Attributes.html#index-packed", "", 6},
 { "partial_save", "gcc/NDS32-Attributes.html#index-partial_005fsave_002c-NDS32", "NDS32", 12},
 { "patchable_function_entry", "gcc/Common-Attributes.html#index-patchable_005ffunction_005fentry", "", 24},
 { "pcs", "gcc/ARM-Attributes.html#index-pcs_002c-ARM", "ARM", 3},
 { "persistent", "gcc/Common-Attributes.html#index-persistent", "", 10},
 { "prefer-vector-width", "gcc/x86-Attributes.html#index-prefer-vector-width_002c-x86", "x86", 19},
 { "preserve_access_index", "gcc/BPF-Attributes.html#index-preserve_005faccess_005findex_002c-BPF", "BPF", 21},
 { "preserve_none", "gcc/x86-Attributes.html#index-preserve_005fnone_002c-x86", "x86", 13},
 { "progmem", "gcc/AVR-Attributes.html#index-progmem_002c-AVR", "AVR", 7},
 { "pure", "gcc/Common-Attributes.html#index-pure", "", 4},
 { "reentrant", "gcc/MSP430-Attributes.html#index-reentrant_002c-MSP430", "MSP430", 9},
 { "regparm", "gcc/x86-Attributes.html#index-regparm_002c-x86", "x86", 7},
 { "renesas", "gcc/SH-Attributes.html#index-renesas_002c-SH", "SH", 7},
 { "reproducible", "gcc/Common-Attributes.html#index-reproducible", "", 12},
 { "resbank", "gcc/SH-Attributes.html#index-resbank_002c-SH", "SH", 7},
 { "reset", "gcc/NDS32-Attributes.html#index-reset_002c-NDS32", "NDS32", 5},
 { "retain", "gcc/Common-Attributes.html#index-retain", "", 6},
 { "returns_nonnull", "gcc/Common-Attributes.html#index-returns_005fnonnull", "", 15},
 { "returns_twice", "gcc/Common-Attributes.html#index-returns_005ftwice", "", 13},
 { "riscv_vector_cc", "gcc/RISC-V-Attributes.html#index-riscv_005fvector_005fcc_002c-RISC-V", "RISC-V", 15},
 { "saddr", "gcc/RL78-Attributes.html#index-saddr_002c-RL78", "RL78", 5},
 { "save_all", "gcc/NDS32-Attributes.html#index-save_005fall_002c-NDS32", "NDS32", 8},
 { "save_volatiles", "gcc/MicroBlaze-Attributes.html#index-save_005fvolatiles_002c-MicroBlaze", "MicroBlaze", 14},
 { "saveall", "gcc/Blackfin-Attributes.html#index-saveall_002c-Blackfin", "Blackfin", 7},
 { "saveall", "gcc/H8_002f300-Attributes.html#index-saveall_002c-H8_002f300", "H8/300", 7},
 { "scalar_storage_order", "gcc/Common-Attributes.html#index-scalar_005fstorage_005forder", "", 20},
 { "sda", "gcc/V850-Attributes.html#index-sda_002c-V850", "V850", 3},
 { "section", "gcc/Common-Attributes.html#index-section", "", 7},
 { "secure_call", "gcc/ARC-Attributes.html#index-secure_005fcall_002c-ARC", "ARC", 11},
 { "selectany", "gcc/Microsoft-Windows-Attributes.html#index-selectany_002c-Microsoft-Windows", "Microsoft Windows", 9},
 { "sentinel", "gcc/Common-Attributes.html#index-sentinel", "", 8},
 { "shared", "gcc/Microsoft-Windows-Attributes.html#index-shared_002c-Microsoft-Windows", "Microsoft Windows", 6},
 { "shared", "gcc/Nvidia-PTX-Attributes.html#index-shared_002c-Nvidia-PTX", "Nvidia PTX", 6},
 { "short_call", "gcc/ARC-Attributes.html#index-short_005fcall_002c-ARC", "ARC", 10},
 { "short_call", "gcc/ARM-Attributes.html#index-short_005fcall_002c-ARM", "ARM", 10},
 { "short_call", "gcc/Epiphany-Attributes.html#index-short_005fcall_002c-Epiphany", "Epiphany", 10},
 { "short_call", "gcc/MIPS-Attributes.html#index-short_005fcall_002c-MIPS", "MIPS", 10},
 { "shortcall", "gcc/Blackfin-Attributes.html#index-shortcall_002c-Blackfin", "Blackfin", 9},
 { "shortcall", "gcc/PowerPC-Attributes.html#index-shortcall_002c-PowerPC", "PowerPC", 9},
 { "signal", "gcc/AVR-Attributes.html#index-signal_002c-AVR", "AVR", 6},
 { "simd", "gcc/Common-Attributes.html#index-simd", "", 4},
 { "sp_switch", "gcc/SH-Attributes.html#index-sp_005fswitch_002c-SH", "SH", 9},
 { "sseregparm", "gcc/x86-Attributes.html#index-sseregparm_002c-x86", "x86", 10},
 { "stack_protect", "gcc/Common-Attributes.html#index-stack_005fprotect", "", 13},
 { "stdcall", "gcc/x86-Attributes.html#index-stdcall_002c-x86-32", "x86-32", 7},
 { "strict_flex_array", "gcc/Common-Attributes.html#index-strict_005fflex_005farray", "", 17},
 { "strub", "gcc/Common-Attributes.html#index-strub", "", 5},
 { "symver", "gcc/Common-Attributes.html#index-symver", "", 6},
 { "syscall_linkage", "gcc/IA-64-Attributes.html#index-syscall_005flinkage_002c-IA-64", "IA-64", 15},
 { "sysv_abi", "gcc/x86-Attributes.html#index-sysv_005fabi_002c-x86", "x86", 8},
 { "tainted_args", "gcc/Common-Attributes.html#index-tainted_005fargs", "", 12},
 { "target", "gcc/AArch64-Attributes.html#index-target_002c-AArch64", "AArch64", 6},
 { "target", "gcc/ARM-Attributes.html#index-target_002c-ARM", "ARM", 6},
 { "target", "gcc/Common-Attributes.html#index-target", "", 6},
 { "target", "gcc/LoongArch-Attributes.html#index-target_002c-LoongArch", "LoongArch", 6},
 { "target", "gcc/PowerPC-Attributes.html#index-target_002c-PowerPC", "PowerPC", 6},
 { "target", "gcc/RISC-V-Attributes.html#index-target_002c-RISC-V", "RISC-V", 6},
 { "target", "gcc/S_002f390-Attributes.html#index-target_002c-S_002f390", "S/390", 6},
 { "target", "gcc/x86-Attributes.html#index-target_002c-x86", "x86", 6},
 { "target_clones", "gcc/Common-Attributes.html#index-target_005fclones", "", 13},
 { "target_clones", "gcc/LoongArch-Attributes.html#index-target_005fclones_002c-LoongArch", "LoongArch", 13},
 { "target_version", "gcc/Common-Attributes.html#index-target_005fversion", "", 14},
 { "target_version", "gcc/LoongArch-Attributes.html#index-target_005fversion_002c-LoongArch", "LoongArch", 14},
 { "tda", "gcc/V850-Attributes.html#index-tda_002c-V850", "V850", 3},
 { "thiscall", "gcc/x86-Attributes.html#index-thiscall_002c-x86-32", "x86-32", 8},
 { "tiny_data", "gcc/H8_002f300-Attributes.html#index-tiny_005fdata_002c-H8_002f300", "H8/300", 9},
 { "tls_model", "gcc/Common-Attributes.html#index-tls_005fmodel", "", 9},
 { "transparent_union", "gcc/Common-Attributes.html#index-transparent_005funion", "", 17},
 { "trap_exit", "gcc/SH-Attributes.html#index-trap_005fexit_002c-SH", "SH", 9},
 { "trapa_handler", "gcc/SH-Attributes.html#index-trapa_005fhandler_002c-SH", "SH", 13},
 { "unavailable", "gcc/Common-Attributes.html#index-unavailable", "", 11},
 { "uncached", "gcc/ARC-Attributes.html#index-uncached_002c-ARC", "ARC", 8},
 { "uninitialized", "gcc/Common-Attributes.html#index-uninitialized", "", 13},
 { "unsequenced", "gcc/Common-Attributes.html#index-unsequenced", "", 11},
 { "unused", "gcc/Common-Attributes.html#index-unused", "", 6},
 { "upper", "gcc/MSP430-Attributes.html#index-upper_002c-MSP430", "MSP430", 5},
 { "use_debug_exception_return", "gcc/MIPS-Attributes.html#index-use_005fdebug_005fexception_005freturn_002c-MIPS", "MIPS", 26},
 { "use_hazard_barrier_return", "gcc/MIPS-Attributes.html#index-use_005fhazard_005fbarrier_005freturn_002c-MIPS", "MIPS", 25},
 { "use_shadow_register_set", "gcc/MIPS-Attributes.html#index-use_005fshadow_005fregister_005fset_002c-MIPS", "MIPS", 23},
 { "used", "gcc/Common-Attributes.html#index-used", "", 4},
 { "vector", "gcc/RX-Attributes.html#index-vector_002c-RX", "RX", 6},
 { "vector_size", "gcc/Common-Attributes.html#index-vector_005fsize", "", 11},
 { "version_id", "gcc/IA-64-Attributes.html#index-version_005fid_002c-IA-64", "IA-64", 10},
 { "visibility", "gcc/Common-Attributes.html#index-visibility", "", 10},
 { "wakeup", "gcc/MSP430-Attributes.html#index-wakeup_002c-MSP430", "MSP430", 6},
 { "warm", "gcc/NDS32-Attributes.html#index-warm_002c-NDS32", "NDS32", 4},
 { "warn_unused", "gcc/C_002b_002b-Attributes.html#index-warn_005funused", "", 11},
 { "warn_unused_result", "gcc/Common-Attributes.html#index-warn_005funused_005fresult", "", 18},
 { "warning", "gcc/Common-Attributes.html#index-warning", "", 7},
 { "weak", "gcc/Common-Attributes.html#index-weak", "", 4},
 { "weakref", "gcc/Common-Attributes.html#index-weakref", "", 7},
 { "zda", "gcc/V850-Attributes.html#index-zda_002c-V850", "V850", 3},
 { "zero_call_used_regs", "gcc/Common-Attributes.html#index-zero_005fcall_005fused_005fregs", "", 19},
};

static const struct attr_url_table {
  const attr_url_entry *m_table;
  const size_t m_table_sz;
} attr_url_tables[] = {
  { all_attrs, ARRAY_SIZE (all_attrs) },
};
