/* Generated automatically by the program `gencodes'
   from the machine description file `md'.  */

#ifndef GCC_INSN_CODES_H
#define GCC_INSN_CODES_H

enum insn_code {
  CODE_FOR_nothing = 0,
  CODE_FOR_ccmpqi = 1,
  CODE_FOR_ccmphi = 2,
  CODE_FOR_ccmpsi = 3,
   CODE_FOR_ccmpdi = CODE_FOR_nothing,
  CODE_FOR_x86_sahf_1 = 50,
  CODE_FOR_x86_stc = 66,
  CODE_FOR_pushflsi2 = 78,
  CODE_FOR_pushfldi2 = 79,
  CODE_FOR_popflsi1 = 80,
  CODE_FOR_popfldi1 = 81,
  CODE_FOR_swapsi = 102,
   CODE_FOR_swapdi = CODE_FOR_nothing,
  CODE_FOR_insvhi_1 = 119,
  CODE_FOR_insvsi_1 = 120,
   CODE_FOR_insvdi_1 = CODE_FOR_nothing,
  CODE_FOR_push2_di = 137,
  CODE_FOR_pop2_di = 138,
   CODE_FOR_pushp_di = CODE_FOR_nothing,
  CODE_FOR_popp_di = 139,
  CODE_FOR_push2p_di = 140,
  CODE_FOR_pop2p_di = 141,
   CODE_FOR_zero_extendditi2 = CODE_FOR_nothing,
   CODE_FOR_zero_extendqidi2 = CODE_FOR_nothing,
   CODE_FOR_zero_extendhidi2 = CODE_FOR_nothing,
  CODE_FOR_zero_extendqisi2_and = 151,
  CODE_FOR_zero_extendhisi2_and = 152,
  CODE_FOR_zero_extendqihi2_and = 155,
  CODE_FOR_extendsidi2_1 = 157,
   CODE_FOR_extendditi2 = CODE_FOR_nothing,
   CODE_FOR_extendqidi2 = CODE_FOR_nothing,
   CODE_FOR_extendhidi2 = CODE_FOR_nothing,
  CODE_FOR_extendhisi2 = 158,
  CODE_FOR_extendqisi2 = 159,
  CODE_FOR_extendqihi2 = 160,
  CODE_FOR_extendbfsf2_1 = 168,
  CODE_FOR_truncdfsf2 = 171,
  CODE_FOR_truncxfsf2 = 172,
  CODE_FOR_truncxfdf2 = 173,
  CODE_FOR_truncsfbf2 = 176,
  CODE_FOR_fix_trunchfsi2 = 177,
  CODE_FOR_fixuns_trunchfsi2 = 178,
   CODE_FOR_fix_trunchfdi2 = CODE_FOR_nothing,
   CODE_FOR_fixuns_trunchfdi2 = CODE_FOR_nothing,
   CODE_FOR_fixuns_truncsfdi2 = CODE_FOR_nothing,
   CODE_FOR_fixuns_truncdfdi2 = CODE_FOR_nothing,
  CODE_FOR_fixuns_truncsfsi2_avx512f = 179,
  CODE_FOR_fixuns_truncdfsi2_avx512f = 180,
  CODE_FOR_fix_truncsfsi_sse = 183,
   CODE_FOR_fix_truncsfdi_sse = CODE_FOR_nothing,
  CODE_FOR_fix_truncdfsi_sse = 184,
   CODE_FOR_fix_truncdfdi_sse = CODE_FOR_nothing,
  CODE_FOR_fix_trunchi_i387_fisttp = 185,
  CODE_FOR_fix_truncsi_i387_fisttp = 186,
  CODE_FOR_fix_truncdi_i387_fisttp = 187,
  CODE_FOR_fix_truncdi_i387 = 191,
  CODE_FOR_fix_trunchi_i387 = 192,
  CODE_FOR_fix_truncsi_i387 = 193,
  CODE_FOR_x86_fnstcw_1 = 194,
  CODE_FOR_floathisf2 = 195,
  CODE_FOR_floathidf2 = 196,
  CODE_FOR_floathixf2 = 197,
  CODE_FOR_floatsixf2 = 198,
  CODE_FOR_floatdixf2 = 199,
  CODE_FOR_floatsihf2 = 202,
  CODE_FOR_floatunssihf2 = 203,
   CODE_FOR_floatdihf2 = CODE_FOR_nothing,
   CODE_FOR_floatunsdihf2 = CODE_FOR_nothing,
  CODE_FOR_floatdisf2_i387_with_xmm = 206,
  CODE_FOR_floatdidf2_i387_with_xmm = 207,
  CODE_FOR_floatdixf2_i387_with_xmm = 208,
  CODE_FOR_floatunssisf2_i387_with_xmm = 211,
  CODE_FOR_floatunssidf2_i387_with_xmm = 212,
  CODE_FOR_floatunssixf2_i387_with_xmm = 213,
   CODE_FOR_addsi_1_zext = CODE_FOR_nothing,
  CODE_FOR_addvqi4_1 = 281,
  CODE_FOR_addvhi4_1 = 282,
  CODE_FOR_addvsi4_1 = 283,
  CODE_FOR_addvdi4_1 = 284,
  CODE_FOR_subvqi4_1 = 357,
  CODE_FOR_subvhi4_1 = 358,
  CODE_FOR_subvsi4_1 = 359,
  CODE_FOR_subvdi4_1 = 360,
  CODE_FOR_addqi3_carry = 379,
  CODE_FOR_addhi3_carry = 380,
  CODE_FOR_addsi3_carry = 381,
  CODE_FOR_adddi3_carry = 382,
  CODE_FOR_addcarrysi = 418,
  CODE_FOR_addcarrydi = 419,
  CODE_FOR_subqi3_carry = 422,
  CODE_FOR_subhi3_carry = 423,
  CODE_FOR_subsi3_carry = 424,
  CODE_FOR_subdi3_carry = 425,
  CODE_FOR_subsi3_carry_ccc = 461,
   CODE_FOR_subdi3_carry_ccc = CODE_FOR_nothing,
  CODE_FOR_subsi3_carry_ccgz = 463,
   CODE_FOR_subdi3_carry_ccgz = CODE_FOR_nothing,
  CODE_FOR_subborrowsi = 464,
  CODE_FOR_subborrowdi = 465,
  CODE_FOR_addqi3_cc_overflow_1 = 478,
  CODE_FOR_addhi3_cc_overflow_1 = 479,
  CODE_FOR_addsi3_cc_overflow_1 = 480,
  CODE_FOR_adddi3_cc_overflow_1 = 481,
  CODE_FOR_smulsi3_highpart = 553,
  CODE_FOR_umulsi3_highpart = 554,
   CODE_FOR_smuldi3_highpart = CODE_FOR_nothing,
   CODE_FOR_umuldi3_highpart = CODE_FOR_nothing,
  CODE_FOR_divmodsi4_1 = 557,
   CODE_FOR_divmoddi4_1 = CODE_FOR_nothing,
  CODE_FOR_udivmodsi4_1 = 558,
   CODE_FOR_udivmoddi4_1 = CODE_FOR_nothing,
   CODE_FOR_divmodsi4_zext_1 = CODE_FOR_nothing,
   CODE_FOR_udivmodsi4_zext_1 = CODE_FOR_nothing,
   CODE_FOR_divmodsi4_zext_2 = CODE_FOR_nothing,
   CODE_FOR_udivmodsi4_zext_2 = CODE_FOR_nothing,
  CODE_FOR_divmodhiqi3_nf = 574,
  CODE_FOR_divmodhiqi3 = 575,
  CODE_FOR_udivmodhiqi3_nf = 576,
  CODE_FOR_udivmodhiqi3 = 577,
  CODE_FOR_ashldi3_doubleword = 894,
   CODE_FOR_ashlti3_doubleword = CODE_FOR_nothing,
   CODE_FOR_x86_64_shld_nf = CODE_FOR_nothing,
   CODE_FOR_x86_64_shld = CODE_FOR_nothing,
  CODE_FOR_x86_64_shld_ndd_nf = 897,
  CODE_FOR_x86_64_shld_ndd = 898,
   CODE_FOR_x86_64_shld_1_nf = CODE_FOR_nothing,
   CODE_FOR_x86_64_shld_1 = CODE_FOR_nothing,
  CODE_FOR_x86_64_shld_ndd_1_nf = 899,
  CODE_FOR_x86_64_shld_ndd_1 = 900,
  CODE_FOR_x86_shld_nf = 902,
  CODE_FOR_x86_shld = 903,
  CODE_FOR_x86_shld_ndd_nf = 904,
  CODE_FOR_x86_shld_ndd = 905,
  CODE_FOR_x86_shld_1_nf = 906,
  CODE_FOR_x86_shld_1 = 907,
  CODE_FOR_x86_shld_ndd_1_nf = 908,
  CODE_FOR_x86_shld_ndd_1 = 909,
  CODE_FOR_lshrdi3_doubleword = 952,
  CODE_FOR_ashrdi3_doubleword = 953,
   CODE_FOR_lshrti3_doubleword = CODE_FOR_nothing,
   CODE_FOR_ashrti3_doubleword = CODE_FOR_nothing,
  CODE_FOR_lshrdi3_doubleword_lowpart_nf = 954,
  CODE_FOR_ashrdi3_doubleword_lowpart_nf = 955,
   CODE_FOR_lshrti3_doubleword_lowpart_nf = CODE_FOR_nothing,
   CODE_FOR_ashrti3_doubleword_lowpart_nf = CODE_FOR_nothing,
  CODE_FOR_lshrdi3_doubleword_lowpart = 956,
  CODE_FOR_ashrdi3_doubleword_lowpart = 957,
   CODE_FOR_lshrti3_doubleword_lowpart = CODE_FOR_nothing,
   CODE_FOR_ashrti3_doubleword_lowpart = CODE_FOR_nothing,
   CODE_FOR_x86_64_shrd_nf = CODE_FOR_nothing,
   CODE_FOR_x86_64_shrd = CODE_FOR_nothing,
  CODE_FOR_x86_64_shrd_ndd_nf = 958,
  CODE_FOR_x86_64_shrd_ndd = 959,
   CODE_FOR_x86_64_shrd_1_nf = CODE_FOR_nothing,
   CODE_FOR_x86_64_shrd_1 = CODE_FOR_nothing,
  CODE_FOR_x86_64_shrd_ndd_1_nf = 960,
  CODE_FOR_x86_64_shrd_ndd_1 = 961,
  CODE_FOR_x86_shrd_nf = 963,
  CODE_FOR_x86_shrd = 964,
  CODE_FOR_x86_shrd_ndd_nf = 965,
  CODE_FOR_x86_shrd_ndd = 966,
  CODE_FOR_x86_shrd_1_nf = 967,
  CODE_FOR_x86_shrd_1 = 968,
  CODE_FOR_x86_shrd_ndd_1_nf = 969,
  CODE_FOR_x86_shrd_ndd_1 = 970,
  CODE_FOR_ashrsi3_cvt_nf = 974,
  CODE_FOR_ashrsi3_cvt = 975,
  CODE_FOR_ashrdi3_cvt_nf = 976,
  CODE_FOR_ashrdi3_cvt = 977,
  CODE_FOR_ix86_rotldi3_doubleword = 1127,
   CODE_FOR_ix86_rotlti3_doubleword = CODE_FOR_nothing,
  CODE_FOR_ix86_rotrdi3_doubleword = 1128,
   CODE_FOR_ix86_rotrti3_doubleword = CODE_FOR_nothing,
  CODE_FOR_rotl32di2_doubleword = 1129,
  CODE_FOR_rotr32di2_doubleword = 1130,
   CODE_FOR_rotl64ti2_doubleword = CODE_FOR_nothing,
   CODE_FOR_rotr64ti2_doubleword = CODE_FOR_nothing,
  CODE_FOR_rcrsi2 = 1173,
   CODE_FOR_rcrdi2 = CODE_FOR_nothing,
  CODE_FOR_lshrsi3_carry = 1174,
  CODE_FOR_ashrsi3_carry = 1175,
   CODE_FOR_lshrdi3_carry = CODE_FOR_nothing,
   CODE_FOR_ashrdi3_carry = CODE_FOR_nothing,
  CODE_FOR_setcc_sf_sse = 1231,
  CODE_FOR_setcc_df_sse = 1232,
  CODE_FOR_setcc_hf_mask = 1233,
  CODE_FOR_jump = 1235,
  CODE_FOR_blockage = 1262,
  CODE_FOR_prologue_use = 1264,
  CODE_FOR_simple_return_internal = 1265,
  CODE_FOR_interrupt_return = 1266,
  CODE_FOR_simple_return_internal_long = 1267,
  CODE_FOR_simple_return_pop_internal = 1268,
  CODE_FOR_nop = 1271,
  CODE_FOR_nops = 1272,
  CODE_FOR_max_skip_align = 1273,
   CODE_FOR_set_got_rex64 = CODE_FOR_nothing,
   CODE_FOR_set_rip_rex64 = CODE_FOR_nothing,
  CODE_FOR_set_got_offset_rex64 = 1276,
  CODE_FOR_eh_return_internal = 1277,
  CODE_FOR_split_stack_return = 1279,
  CODE_FOR_ffssi2_no_cmove = 1280,
  CODE_FOR_ctzsi2 = 1284,
   CODE_FOR_ctzdi2 = CODE_FOR_nothing,
   CODE_FOR_bsr_rex64 = CODE_FOR_nothing,
   CODE_FOR_bsr_rex64_1 = CODE_FOR_nothing,
   CODE_FOR_bsr_rex64_1_zext = CODE_FOR_nothing,
  CODE_FOR_bsr = 1286,
  CODE_FOR_bsr_1 = 1287,
   CODE_FOR_bsr_zext_1 = CODE_FOR_nothing,
  CODE_FOR_clzsi2_lzcnt_nf = 1288,
   CODE_FOR_clzdi2_lzcnt_nf = CODE_FOR_nothing,
  CODE_FOR_clzsi2_lzcnt = 1289,
   CODE_FOR_clzdi2_lzcnt = CODE_FOR_nothing,
  CODE_FOR_tzcnt_si_nf = 1294,
  CODE_FOR_lzcnt_si_nf = 1295,
   CODE_FOR_tzcnt_di_nf = CODE_FOR_nothing,
   CODE_FOR_lzcnt_di_nf = CODE_FOR_nothing,
  CODE_FOR_tzcnt_si = 1296,
  CODE_FOR_lzcnt_si = 1297,
   CODE_FOR_tzcnt_di = CODE_FOR_nothing,
   CODE_FOR_lzcnt_di = CODE_FOR_nothing,
  CODE_FOR_tzcnt_hi_nf = 1302,
  CODE_FOR_tzcnt_hi = 1303,
  CODE_FOR_lzcnt_hi_nf = 1304,
  CODE_FOR_lzcnt_hi = 1305,
  CODE_FOR_bmi_bextr_si = 1306,
   CODE_FOR_bmi_bextr_di = CODE_FOR_nothing,
  CODE_FOR_bmi2_pdep_si3 = 1325,
   CODE_FOR_bmi2_pdep_di3 = CODE_FOR_nothing,
  CODE_FOR_bmi2_pext_si3 = 1326,
   CODE_FOR_bmi2_pext_di3 = CODE_FOR_nothing,
  CODE_FOR_tbm_bextri_si = 1327,
   CODE_FOR_tbm_bextri_di = CODE_FOR_nothing,
  CODE_FOR_popcountsi2_nf = 1337,
   CODE_FOR_popcountdi2_nf = CODE_FOR_nothing,
  CODE_FOR_popcountsi2 = 1338,
   CODE_FOR_popcountdi2 = CODE_FOR_nothing,
  CODE_FOR_popcounthi2_nf = 1343,
  CODE_FOR_popcounthi2 = 1344,
  CODE_FOR_bswaphisi2_lowpart = 1349,
  CODE_FOR_parityhi2_cmp = 1350,
  CODE_FOR_parityqi2_cmp = 1351,
  CODE_FOR_rcphf2 = 1372,
  CODE_FOR_truncxfsf2_i387_noop_unspec = 1397,
  CODE_FOR_truncxfdf2_i387_noop_unspec = 1398,
  CODE_FOR_sqrtxf2 = 1399,
  CODE_FOR_rsqrthf2 = 1401,
  CODE_FOR_sqrthf2 = 1402,
  CODE_FOR_x86_fnstsw_1 = 1405,
  CODE_FOR_fpremxf4_i387 = 1406,
  CODE_FOR_fprem1xf4_i387 = 1407,
  CODE_FOR_sinxf2 = 1408,
  CODE_FOR_cosxf2 = 1409,
  CODE_FOR_sincosxf3 = 1410,
  CODE_FOR_fptanxf4_i387 = 1411,
  CODE_FOR_atan2xf3 = 1412,
  CODE_FOR_fyl2xxf3_i387 = 1413,
  CODE_FOR_fyl2xp1xf3_i387 = 1414,
  CODE_FOR_fxtractxf3_i387 = 1415,
  CODE_FOR_fscalexf4_i387 = 1417,
  CODE_FOR_avx512f_scalefsf2 = 1418,
  CODE_FOR_avx512f_scalefdf2 = 1419,
  CODE_FOR_sse4_1_roundhf2 = 1420,
  CODE_FOR_sse4_1_roundsf2 = 1421,
  CODE_FOR_sse4_1_rounddf2 = 1422,
  CODE_FOR_rintxf2 = 1423,
  CODE_FOR_lrintxfdi2 = 1424,
  CODE_FOR_lrintxfhi2 = 1425,
  CODE_FOR_lrintxfsi2 = 1426,
  CODE_FOR_frndintxf2_roundeven = 1427,
  CODE_FOR_frndintxf2_floor = 1428,
  CODE_FOR_frndintxf2_ceil = 1429,
  CODE_FOR_frndintxf2_trunc = 1430,
  CODE_FOR_frndintxf2_roundeven_i387 = 1431,
  CODE_FOR_frndintxf2_floor_i387 = 1432,
  CODE_FOR_frndintxf2_ceil_i387 = 1433,
  CODE_FOR_frndintxf2_trunc_i387 = 1434,
  CODE_FOR_fistdi2_floor = 1441,
  CODE_FOR_fistdi2_ceil = 1442,
  CODE_FOR_fisthi2_floor = 1443,
  CODE_FOR_fisthi2_ceil = 1444,
  CODE_FOR_fistsi2_floor = 1445,
  CODE_FOR_fistsi2_ceil = 1446,
  CODE_FOR_fxamsf2_i387 = 1447,
  CODE_FOR_fxamdf2_i387 = 1448,
  CODE_FOR_fxamxf2_i387 = 1449,
  CODE_FOR_movmsk_df = 1450,
  CODE_FOR_cld = 1451,
  CODE_FOR_movhf_mask = 1486,
  CODE_FOR_smaxsf3 = 1492,
  CODE_FOR_sminsf3 = 1493,
  CODE_FOR_smaxdf3 = 1494,
  CODE_FOR_smindf3 = 1495,
  CODE_FOR_smaxhf3 = 1496,
  CODE_FOR_sminhf3 = 1497,
  CODE_FOR_pro_epilogue_adjust_stack_add_si = 1550,
  CODE_FOR_pro_epilogue_adjust_stack_add_di = 1551,
  CODE_FOR_pro_epilogue_adjust_stack_add_noccsi = 1552,
  CODE_FOR_pro_epilogue_adjust_stack_add_noccdi = 1553,
  CODE_FOR_pro_epilogue_adjust_stack_sub_si = 1554,
  CODE_FOR_pro_epilogue_adjust_stack_sub_di = 1555,
  CODE_FOR_allocate_stack_worker_probe_si = 1556,
  CODE_FOR_allocate_stack_worker_probe_di = 1557,
  CODE_FOR_probe_stack_1 = 1558,
  CODE_FOR_adjust_stack_and_probe_si = 1559,
  CODE_FOR_adjust_stack_and_probe_di = 1560,
  CODE_FOR_probe_stack_range_si = 1561,
  CODE_FOR_probe_stack_range_di = 1562,
  CODE_FOR_stack_protect_set_1_si_si = 1563,
  CODE_FOR_stack_protect_set_1_di_si = 1564,
  CODE_FOR_stack_protect_set_1_si_di = 1565,
  CODE_FOR_stack_protect_set_1_di_di = 1566,
  CODE_FOR_stack_protect_test_1_si = 1573,
  CODE_FOR_stack_protect_test_1_di = 1574,
  CODE_FOR_trap = 1575,
  CODE_FOR_ud2 = 1576,
   CODE_FOR_prefetchi = CODE_FOR_nothing,
  CODE_FOR_sse4_2_crc32qi = 1580,
  CODE_FOR_sse4_2_crc32hi = 1581,
  CODE_FOR_sse4_2_crc32si = 1582,
   CODE_FOR_sse4_2_crc32di = CODE_FOR_nothing,
  CODE_FOR_rdpmc = 1583,
   CODE_FOR_rdpmc_rex64 = CODE_FOR_nothing,
  CODE_FOR_rdtsc = 1584,
   CODE_FOR_rdtsc_rex64 = CODE_FOR_nothing,
  CODE_FOR_rdtscp = 1585,
   CODE_FOR_rdtscp_rex64 = CODE_FOR_nothing,
  CODE_FOR_fxsave = 1586,
   CODE_FOR_fxsave64 = CODE_FOR_nothing,
  CODE_FOR_fxrstor = 1587,
   CODE_FOR_fxrstor64 = CODE_FOR_nothing,
  CODE_FOR_xsave = 1588,
  CODE_FOR_xsaveopt = 1589,
  CODE_FOR_xsavec = 1590,
  CODE_FOR_xsaves = 1591,
   CODE_FOR_xsave_rex64 = CODE_FOR_nothing,
   CODE_FOR_xsaveopt_rex64 = CODE_FOR_nothing,
   CODE_FOR_xsavec_rex64 = CODE_FOR_nothing,
   CODE_FOR_xsaves_rex64 = CODE_FOR_nothing,
   CODE_FOR_xsave64 = CODE_FOR_nothing,
   CODE_FOR_xsaveopt64 = CODE_FOR_nothing,
   CODE_FOR_xsavec64 = CODE_FOR_nothing,
   CODE_FOR_xsaves64 = CODE_FOR_nothing,
  CODE_FOR_xrstor = 1592,
  CODE_FOR_xrstors = 1593,
   CODE_FOR_xrstor_rex64 = CODE_FOR_nothing,
   CODE_FOR_xrstors_rex64 = CODE_FOR_nothing,
   CODE_FOR_xrstor64 = CODE_FOR_nothing,
   CODE_FOR_xrstors64 = CODE_FOR_nothing,
  CODE_FOR_xsetbv = 1594,
   CODE_FOR_xsetbv_rex64 = CODE_FOR_nothing,
  CODE_FOR_xgetbv = 1595,
   CODE_FOR_xgetbv_rex64 = CODE_FOR_nothing,
  CODE_FOR_fnstenv = 1596,
  CODE_FOR_fldenv = 1597,
  CODE_FOR_fnstsw = 1598,
  CODE_FOR_fnclex = 1599,
  CODE_FOR_lwp_llwpcbsi = 1600,
  CODE_FOR_lwp_llwpcbdi = 1601,
  CODE_FOR_lwp_slwpcbsi = 1602,
  CODE_FOR_lwp_slwpcbdi = 1603,
  CODE_FOR_lwp_lwpvalsi = 1604,
   CODE_FOR_lwp_lwpvaldi = CODE_FOR_nothing,
  CODE_FOR_lwp_lwpinssi = 1605,
   CODE_FOR_lwp_lwpinsdi = CODE_FOR_nothing,
   CODE_FOR_rdfsbasesi = CODE_FOR_nothing,
   CODE_FOR_rdgsbasesi = CODE_FOR_nothing,
   CODE_FOR_rdfsbasedi = CODE_FOR_nothing,
   CODE_FOR_rdgsbasedi = CODE_FOR_nothing,
   CODE_FOR_wrfsbasesi = CODE_FOR_nothing,
   CODE_FOR_wrgsbasesi = CODE_FOR_nothing,
   CODE_FOR_wrfsbasedi = CODE_FOR_nothing,
   CODE_FOR_wrgsbasedi = CODE_FOR_nothing,
  CODE_FOR_ptwritesi = 1606,
   CODE_FOR_ptwritedi = CODE_FOR_nothing,
  CODE_FOR_rdrandhi = 1607,
  CODE_FOR_rdrandsi = 1608,
   CODE_FOR_rdranddi = CODE_FOR_nothing,
  CODE_FOR_rdseedhi = 1609,
  CODE_FOR_rdseedsi = 1610,
   CODE_FOR_rdseeddi = CODE_FOR_nothing,
  CODE_FOR_rdsspsi = 1612,
   CODE_FOR_rdsspdi = CODE_FOR_nothing,
  CODE_FOR_incsspsi = 1613,
   CODE_FOR_incsspdi = CODE_FOR_nothing,
  CODE_FOR_saveprevssp = 1614,
  CODE_FOR_rstorssp = 1615,
  CODE_FOR_wrsssi = 1616,
   CODE_FOR_wrssdi = CODE_FOR_nothing,
  CODE_FOR_wrusssi = 1617,
   CODE_FOR_wrussdi = CODE_FOR_nothing,
  CODE_FOR_setssbsy = 1618,
  CODE_FOR_clrssbsy = 1619,
  CODE_FOR_nop_endbr = 1620,
  CODE_FOR_xbegin_1 = 1621,
  CODE_FOR_xend = 1622,
  CODE_FOR_xabort = 1623,
  CODE_FOR_xtest_1 = 1624,
  CODE_FOR_clwb = 1625,
  CODE_FOR_clflushopt = 1626,
  CODE_FOR_mwaitx = 1627,
  CODE_FOR_monitorx_si = 1628,
  CODE_FOR_monitorx_di = 1629,
  CODE_FOR_clzero_si = 1630,
  CODE_FOR_clzero_di = 1631,
  CODE_FOR_rdpid = 1634,
   CODE_FOR_rdpid_rex64 = CODE_FOR_nothing,
  CODE_FOR_wbinvd = 1635,
  CODE_FOR_wbnoinvd = 1636,
  CODE_FOR_movdirisi = 1637,
   CODE_FOR_movdiridi = CODE_FOR_nothing,
  CODE_FOR_movdir64b_si = 1638,
  CODE_FOR_movdir64b_di = 1639,
  CODE_FOR_xsusldtrk = 1640,
  CODE_FOR_xresldtrk = 1641,
  CODE_FOR_enqcmd_si = 1642,
  CODE_FOR_enqcmds_si = 1643,
  CODE_FOR_enqcmd_di = 1644,
  CODE_FOR_enqcmds_di = 1645,
   CODE_FOR_clui = CODE_FOR_nothing,
   CODE_FOR_stui = CODE_FOR_nothing,
   CODE_FOR_testui = CODE_FOR_nothing,
   CODE_FOR_senduipi = CODE_FOR_nothing,
  CODE_FOR_umwait = 1646,
   CODE_FOR_umwait_rex64 = CODE_FOR_nothing,
  CODE_FOR_umonitor_si = 1647,
  CODE_FOR_umonitor_di = 1648,
  CODE_FOR_tpause = 1649,
   CODE_FOR_tpause_rex64 = CODE_FOR_nothing,
  CODE_FOR_cldemote = 1650,
  CODE_FOR_speculation_barrier = 1651,
  CODE_FOR_serialize = 1652,
  CODE_FOR_patchable_area = 1653,
  CODE_FOR_hreset = 1654,
   CODE_FOR_urdmsr = CODE_FOR_nothing,
   CODE_FOR_uwrmsr = CODE_FOR_nothing,
  CODE_FOR_ldtilecfg = 1655,
  CODE_FOR_sttilecfg = 1656,
   CODE_FOR_movrsqi = CODE_FOR_nothing,
   CODE_FOR_movrshi = CODE_FOR_nothing,
   CODE_FOR_movrssi = CODE_FOR_nothing,
   CODE_FOR_movrsdi = CODE_FOR_nothing,
  CODE_FOR_sse_movntq = 1677,
  CODE_FOR_mmx_ieee_maxv2sf3 = 1683,
  CODE_FOR_mmx_ieee_minv2sf3 = 1684,
  CODE_FOR_mmx_rcpv2sf2 = 1685,
  CODE_FOR_mmx_rcpit1v2sf3 = 1686,
  CODE_FOR_mmx_rcpit2v2sf3 = 1687,
  CODE_FOR_mmx_rsqrtv2sf2 = 1688,
  CODE_FOR_mmx_rsqit1v2sf3 = 1689,
  CODE_FOR_mmx_hsubv2sf3 = 1691,
  CODE_FOR_mmx_gtv2sf3 = 1694,
  CODE_FOR_mmx_gev2sf3 = 1695,
   CODE_FOR_sse4_1_insertps_v2sf = CODE_FOR_nothing,
   CODE_FOR_sse4_1_insertps_v2si = CODE_FOR_nothing,
   CODE_FOR_mmx_blendvps = CODE_FOR_nothing,
   CODE_FOR_andv2sf3 = CODE_FOR_nothing,
   CODE_FOR_iorv2sf3 = CODE_FOR_nothing,
   CODE_FOR_xorv2sf3 = CODE_FOR_nothing,
  CODE_FOR_mmx_fix_truncv2sfv2si2 = 1696,
  CODE_FOR_mmx_floatv2siv2sf2 = 1697,
  CODE_FOR_mmx_pf2iw = 1698,
  CODE_FOR_mmx_pi2fw = 1699,
  CODE_FOR_mmx_pswapdv2sf2 = 1700,
  CODE_FOR_andv2bf3 = 1713,
  CODE_FOR_iorv2bf3 = 1714,
  CODE_FOR_xorv2bf3 = 1715,
   CODE_FOR_andv4bf3 = CODE_FOR_nothing,
   CODE_FOR_iorv4bf3 = CODE_FOR_nothing,
   CODE_FOR_xorv4bf3 = CODE_FOR_nothing,
  CODE_FOR_andv2hf3 = 1716,
  CODE_FOR_iorv2hf3 = 1717,
  CODE_FOR_xorv2hf3 = 1718,
   CODE_FOR_andv4hf3 = CODE_FOR_nothing,
   CODE_FOR_iorv4hf3 = CODE_FOR_nothing,
   CODE_FOR_xorv4hf3 = CODE_FOR_nothing,
  CODE_FOR_negv2qi2 = 1719,
  CODE_FOR_addv4qi3 = 1728,
  CODE_FOR_subv4qi3 = 1729,
  CODE_FOR_addv2hi3 = 1730,
  CODE_FOR_subv2hi3 = 1731,
  CODE_FOR_addv2qi3 = 1732,
  CODE_FOR_subv2qi3 = 1733,
  CODE_FOR_ssaddv4qi3 = 1742,
  CODE_FOR_usaddv4qi3 = 1743,
  CODE_FOR_sssubv4qi3 = 1744,
  CODE_FOR_ussubv4qi3 = 1745,
  CODE_FOR_ssaddv2qi3 = 1746,
  CODE_FOR_usaddv2qi3 = 1747,
  CODE_FOR_sssubv2qi3 = 1748,
  CODE_FOR_ussubv2qi3 = 1749,
  CODE_FOR_ssaddv2hi3 = 1750,
  CODE_FOR_usaddv2hi3 = 1751,
  CODE_FOR_sssubv2hi3 = 1752,
  CODE_FOR_ussubv2hi3 = 1753,
   CODE_FOR_mulv2si3 = CODE_FOR_nothing,
  CODE_FOR_mulv2hi3 = 1755,
  CODE_FOR_smulv2hi3_highpart = 1758,
  CODE_FOR_umulv2hi3_highpart = 1759,
   CODE_FOR_smaxv8qi3 = CODE_FOR_nothing,
   CODE_FOR_sminv8qi3 = CODE_FOR_nothing,
   CODE_FOR_smaxv2si3 = CODE_FOR_nothing,
   CODE_FOR_sminv2si3 = CODE_FOR_nothing,
  CODE_FOR_smaxv4qi3 = 1765,
  CODE_FOR_sminv4qi3 = 1766,
  CODE_FOR_smaxv2qi3 = 1767,
  CODE_FOR_sminv2qi3 = 1768,
  CODE_FOR_smaxv2hi3 = 1769,
  CODE_FOR_sminv2hi3 = 1770,
   CODE_FOR_umaxv4hi3 = CODE_FOR_nothing,
   CODE_FOR_uminv4hi3 = CODE_FOR_nothing,
   CODE_FOR_umaxv2si3 = CODE_FOR_nothing,
   CODE_FOR_uminv2si3 = CODE_FOR_nothing,
  CODE_FOR_umaxv4qi3 = 1773,
  CODE_FOR_uminv4qi3 = 1774,
  CODE_FOR_umaxv2qi3 = 1775,
  CODE_FOR_uminv2qi3 = 1776,
  CODE_FOR_umaxv2hi3 = 1777,
  CODE_FOR_uminv2hi3 = 1778,
  CODE_FOR_ssse3_absv8qi2 = 1779,
  CODE_FOR_ssse3_absv4hi2 = 1780,
  CODE_FOR_ssse3_absv2si2 = 1781,
  CODE_FOR_absv4qi2 = 1782,
  CODE_FOR_absv2qi2 = 1783,
  CODE_FOR_absv2hi2 = 1784,
  CODE_FOR_mmx_ashrv4hi3 = 1785,
  CODE_FOR_mmx_ashrv2si3 = 1786,
  CODE_FOR_mmx_ashlv4hi3 = 1787,
  CODE_FOR_mmx_lshrv4hi3 = 1788,
  CODE_FOR_mmx_ashlv2si3 = 1789,
  CODE_FOR_mmx_lshrv2si3 = 1790,
  CODE_FOR_mmx_ashlv1di3 = 1791,
  CODE_FOR_mmx_lshrv1di3 = 1792,
  CODE_FOR_mmx_ashlv1si3 = 1793,
  CODE_FOR_mmx_lshrv1si3 = 1794,
  CODE_FOR_ashlv2hi3 = 1795,
  CODE_FOR_lshrv2hi3 = 1796,
  CODE_FOR_ashrv2hi3 = 1797,
  CODE_FOR_ashlv2qi3 = 1799,
  CODE_FOR_lshrv2qi3 = 1800,
  CODE_FOR_ashrv2qi3 = 1801,
  CODE_FOR_mmx_gtv8qi3 = 1808,
  CODE_FOR_mmx_gtv4hi3 = 1809,
  CODE_FOR_mmx_gtv2si3 = 1810,
   CODE_FOR_mmx_pblendvb_v8qi = CODE_FOR_nothing,
  CODE_FOR_mmx_pblendvb_v4qi = 1826,
  CODE_FOR_mmx_pblendvb_v2qi = 1827,
  CODE_FOR_mmx_pblendvb_v2hi = 1828,
   CODE_FOR_mmx_ppermv64 = CODE_FOR_nothing,
  CODE_FOR_mmx_ppermv32 = 1838,
  CODE_FOR_one_cmplv4qi2 = 1839,
  CODE_FOR_one_cmplv2qi2 = 1840,
  CODE_FOR_one_cmplv2hi2 = 1841,
  CODE_FOR_mmx_andnotv8qi3 = 1842,
  CODE_FOR_mmx_andnotv4hi3 = 1843,
  CODE_FOR_mmx_andnotv2si3 = 1844,
  CODE_FOR_mmx_packsswb = 1866,
  CODE_FOR_mmx_packuswb = 1867,
  CODE_FOR_mmx_packssdw = 1868,
   CODE_FOR_mmx_packusdw = CODE_FOR_nothing,
  CODE_FOR_mmx_punpckhbw = 1869,
  CODE_FOR_mmx_punpckhbw_low = 1870,
  CODE_FOR_mmx_punpcklbw = 1871,
  CODE_FOR_mmx_punpcklbw_low = 1872,
  CODE_FOR_mmx_punpckhwd = 1873,
  CODE_FOR_mmx_punpcklwd = 1874,
  CODE_FOR_mmx_punpckhdq = 1875,
  CODE_FOR_mmx_punpckldq = 1876,
   CODE_FOR_sse4_1_sign_extendv4qiv4hi2 = CODE_FOR_nothing,
   CODE_FOR_sse4_1_zero_extendv4qiv4hi2 = CODE_FOR_nothing,
   CODE_FOR_sse4_1_sign_extendv2hiv2si2 = CODE_FOR_nothing,
   CODE_FOR_sse4_1_zero_extendv2hiv2si2 = CODE_FOR_nothing,
   CODE_FOR_sse4_1_sign_extendv2qiv2si2 = CODE_FOR_nothing,
   CODE_FOR_sse4_1_zero_extendv2qiv2si2 = CODE_FOR_nothing,
  CODE_FOR_sse4_1_sign_extendv2qiv2hi2 = 1877,
  CODE_FOR_sse4_1_zero_extendv2qiv2hi2 = 1878,
   CODE_FOR_avx512vl_truncv4hiv4qi2 = CODE_FOR_nothing,
  CODE_FOR_avx512vl_truncv2hiv2qi2 = 1879,
   CODE_FOR_avx512vl_truncv2siv2qi2 = CODE_FOR_nothing,
   CODE_FOR_avx512vl_truncv2siv2hi2 = CODE_FOR_nothing,
   CODE_FOR_mmx_pshufbv8qi3 = CODE_FOR_nothing,
  CODE_FOR_mmx_pshufbv4qi3 = 1887,
  CODE_FOR_mmx_pshufwv4hf_1 = 1888,
  CODE_FOR_mmx_pshufwv4bf_1 = 1889,
  CODE_FOR_mmx_pshufwv4hi_1 = 1890,
  CODE_FOR_mmx_pswapdv2si2 = 1894,
  CODE_FOR_uavgv4qi3_ceil = 1924,
  CODE_FOR_uavgv2qi3_ceil = 1925,
  CODE_FOR_uavgv2hi3_ceil = 1926,
  CODE_FOR_mmx_pmovmskb = 1928,
   CODE_FOR_popcountv8qi2 = CODE_FOR_nothing,
  CODE_FOR_popcountv4qi2 = 1933,
  CODE_FOR_popcountv2qi2 = 1934,
   CODE_FOR_popcountv4hi2 = CODE_FOR_nothing,
  CODE_FOR_popcountv2hi2 = 1935,
   CODE_FOR_popcountv2si2 = CODE_FOR_nothing,
  CODE_FOR_movv64qi_internal = 1936,
  CODE_FOR_movv32qi_internal = 1937,
  CODE_FOR_movv16qi_internal = 1938,
  CODE_FOR_movv32hi_internal = 1939,
  CODE_FOR_movv16hi_internal = 1940,
  CODE_FOR_movv8hi_internal = 1941,
  CODE_FOR_movv16si_internal = 1942,
  CODE_FOR_movv8si_internal = 1943,
  CODE_FOR_movv4si_internal = 1944,
  CODE_FOR_movv8di_internal = 1945,
  CODE_FOR_movv4di_internal = 1946,
  CODE_FOR_movv2di_internal = 1947,
  CODE_FOR_movv4ti_internal = 1948,
  CODE_FOR_movv2ti_internal = 1949,
  CODE_FOR_movv1ti_internal = 1950,
  CODE_FOR_movv32hf_internal = 1951,
  CODE_FOR_movv16hf_internal = 1952,
  CODE_FOR_movv8hf_internal = 1953,
  CODE_FOR_movv32bf_internal = 1954,
  CODE_FOR_movv16bf_internal = 1955,
  CODE_FOR_movv8bf_internal = 1956,
  CODE_FOR_movv16sf_internal = 1957,
  CODE_FOR_movv8sf_internal = 1958,
  CODE_FOR_movv4sf_internal = 1959,
  CODE_FOR_movv8df_internal = 1960,
  CODE_FOR_movv4df_internal = 1961,
  CODE_FOR_movv2df_internal = 1962,
  CODE_FOR_avx512f_movhf_mask = 2044,
  CODE_FOR_avx512f_movsf_mask = 2045,
  CODE_FOR_avx512f_movdf_mask = 2046,
  CODE_FOR_avx512f_storehf_mask = 2050,
  CODE_FOR_avx512f_storesf_mask = 2051,
  CODE_FOR_avx512f_storedf_mask = 2052,
  CODE_FOR_avx512f_blendmv16si = 2053,
  CODE_FOR_avx512vl_blendmv8si = 2054,
  CODE_FOR_avx512vl_blendmv4si = 2055,
  CODE_FOR_avx512f_blendmv8di = 2056,
  CODE_FOR_avx512vl_blendmv4di = 2057,
  CODE_FOR_avx512vl_blendmv2di = 2058,
  CODE_FOR_avx512f_blendmv16sf = 2059,
  CODE_FOR_avx512vl_blendmv8sf = 2060,
  CODE_FOR_avx512vl_blendmv4sf = 2061,
  CODE_FOR_avx512f_blendmv8df = 2062,
  CODE_FOR_avx512vl_blendmv4df = 2063,
  CODE_FOR_avx512vl_blendmv2df = 2064,
  CODE_FOR_avx512bw_blendmv64qi = 2065,
  CODE_FOR_avx512vl_blendmv16qi = 2066,
  CODE_FOR_avx512vl_blendmv32qi = 2067,
  CODE_FOR_avx512bw_blendmv32hi = 2068,
  CODE_FOR_avx512vl_blendmv16hi = 2069,
  CODE_FOR_avx512vl_blendmv8hi = 2070,
  CODE_FOR_avx512bw_blendmv32hf = 2071,
  CODE_FOR_avx512vl_blendmv16hf = 2072,
  CODE_FOR_avx512fp16_blendmv8hf = 2073,
  CODE_FOR_avx512bw_blendmv32bf = 2074,
  CODE_FOR_avx512vl_blendmv16bf = 2075,
  CODE_FOR_avx512vl_blendmv8bf = 2076,
  CODE_FOR_movdi_to_sse = 2105,
  CODE_FOR_avx_lddqu256 = 2106,
  CODE_FOR_sse3_lddqu = 2107,
  CODE_FOR_sse2_movntisi = 2108,
   CODE_FOR_sse2_movntidi = CODE_FOR_nothing,
  CODE_FOR_avx512f_movntv16sf = 2109,
  CODE_FOR_avx_movntv8sf = 2110,
  CODE_FOR_sse_movntv4sf = 2111,
  CODE_FOR_avx512f_movntv8df = 2112,
  CODE_FOR_avx_movntv4df = 2113,
  CODE_FOR_sse2_movntv2df = 2114,
  CODE_FOR_avx512f_movntv8di = 2115,
  CODE_FOR_avx_movntv4di = 2116,
  CODE_FOR_sse2_movntv2di = 2117,
  CODE_FOR_kandqi = 2118,
  CODE_FOR_kiorqi = 2119,
  CODE_FOR_kxorqi = 2120,
  CODE_FOR_kandhi = 2121,
  CODE_FOR_kiorhi = 2122,
  CODE_FOR_kxorhi = 2123,
  CODE_FOR_kandsi = 2124,
  CODE_FOR_kiorsi = 2125,
  CODE_FOR_kxorsi = 2126,
  CODE_FOR_kanddi = 2127,
  CODE_FOR_kiordi = 2128,
  CODE_FOR_kxordi = 2129,
  CODE_FOR_kandnqi = 2130,
  CODE_FOR_kandnhi = 2131,
  CODE_FOR_kandnsi = 2132,
  CODE_FOR_kandndi = 2133,
  CODE_FOR_kxnorqi = 2134,
  CODE_FOR_kxnorhi = 2135,
  CODE_FOR_kxnorsi = 2136,
  CODE_FOR_kxnordi = 2137,
  CODE_FOR_knotqi = 2138,
  CODE_FOR_knothi = 2139,
  CODE_FOR_knotsi = 2140,
  CODE_FOR_knotdi = 2141,
  CODE_FOR_kaddqi = 2143,
  CODE_FOR_kaddhi = 2144,
  CODE_FOR_kaddsi = 2145,
  CODE_FOR_kadddi = 2146,
  CODE_FOR_kashiftqi = 2147,
  CODE_FOR_klshiftrtqi = 2148,
  CODE_FOR_kashifthi = 2149,
  CODE_FOR_klshiftrthi = 2150,
  CODE_FOR_kashiftsi = 2151,
  CODE_FOR_klshiftrtsi = 2152,
  CODE_FOR_kashiftdi = 2153,
  CODE_FOR_klshiftrtdi = 2154,
  CODE_FOR_ktestqi = 2156,
  CODE_FOR_ktesthi = 2157,
  CODE_FOR_ktestsi = 2158,
  CODE_FOR_ktestdi = 2159,
  CODE_FOR_kortestqi_ccc = 2164,
  CODE_FOR_kortesthi_ccc = 2165,
  CODE_FOR_kortestsi_ccc = 2166,
  CODE_FOR_kortestdi_ccc = 2167,
  CODE_FOR_kortestqi_ccz = 2168,
  CODE_FOR_kortesthi_ccz = 2169,
  CODE_FOR_kortestsi_ccz = 2170,
  CODE_FOR_kortestdi_ccz = 2171,
  CODE_FOR_kunpckhi = 2187,
  CODE_FOR_kunpcksi = 2188,
  CODE_FOR_kunpckdi = 2189,
  CODE_FOR_avx512fp16_vmaddv8hf3 = 2298,
  CODE_FOR_avx512fp16_vmaddv8hf3_round = 2299,
  CODE_FOR_avx512fp16_vmaddv8hf3_mask = 2300,
  CODE_FOR_avx512fp16_vmaddv8hf3_mask_round = 2301,
  CODE_FOR_avx512fp16_vmsubv8hf3 = 2302,
  CODE_FOR_avx512fp16_vmsubv8hf3_round = 2303,
  CODE_FOR_avx512fp16_vmsubv8hf3_mask = 2304,
  CODE_FOR_avx512fp16_vmsubv8hf3_mask_round = 2305,
  CODE_FOR_sse_vmaddv4sf3 = 2306,
  CODE_FOR_sse_vmaddv4sf3_round = 2307,
  CODE_FOR_sse_vmaddv4sf3_mask = 2308,
  CODE_FOR_sse_vmaddv4sf3_mask_round = 2309,
  CODE_FOR_sse_vmsubv4sf3 = 2310,
  CODE_FOR_sse_vmsubv4sf3_round = 2311,
  CODE_FOR_sse_vmsubv4sf3_mask = 2312,
  CODE_FOR_sse_vmsubv4sf3_mask_round = 2313,
  CODE_FOR_sse2_vmaddv2df3 = 2314,
  CODE_FOR_sse2_vmaddv2df3_round = 2315,
  CODE_FOR_sse2_vmaddv2df3_mask = 2316,
  CODE_FOR_sse2_vmaddv2df3_mask_round = 2317,
  CODE_FOR_sse2_vmsubv2df3 = 2318,
  CODE_FOR_sse2_vmsubv2df3_round = 2319,
  CODE_FOR_sse2_vmsubv2df3_mask = 2320,
  CODE_FOR_sse2_vmsubv2df3_mask_round = 2321,
  CODE_FOR_avx512fp16_vmmulv8hf3 = 2364,
  CODE_FOR_avx512fp16_vmmulv8hf3_round = 2365,
  CODE_FOR_avx512fp16_vmmulv8hf3_mask = 2366,
  CODE_FOR_avx512fp16_vmmulv8hf3_mask_round = 2367,
  CODE_FOR_avx512fp16_vmdivv8hf3 = 2368,
  CODE_FOR_avx512fp16_vmdivv8hf3_round = 2369,
  CODE_FOR_avx512fp16_vmdivv8hf3_mask = 2370,
  CODE_FOR_avx512fp16_vmdivv8hf3_mask_round = 2371,
  CODE_FOR_sse_vmmulv4sf3 = 2372,
  CODE_FOR_sse_vmmulv4sf3_round = 2373,
  CODE_FOR_sse_vmmulv4sf3_mask = 2374,
  CODE_FOR_sse_vmmulv4sf3_mask_round = 2375,
  CODE_FOR_sse_vmdivv4sf3 = 2376,
  CODE_FOR_sse_vmdivv4sf3_round = 2377,
  CODE_FOR_sse_vmdivv4sf3_mask = 2378,
  CODE_FOR_sse_vmdivv4sf3_mask_round = 2379,
  CODE_FOR_sse2_vmmulv2df3 = 2380,
  CODE_FOR_sse2_vmmulv2df3_round = 2381,
  CODE_FOR_sse2_vmmulv2df3_mask = 2382,
  CODE_FOR_sse2_vmmulv2df3_mask_round = 2383,
  CODE_FOR_sse2_vmdivv2df3 = 2384,
  CODE_FOR_sse2_vmdivv2df3_round = 2385,
  CODE_FOR_sse2_vmdivv2df3_mask = 2386,
  CODE_FOR_sse2_vmdivv2df3_mask_round = 2387,
  CODE_FOR_avx512fp16_divv32hf3 = 2388,
  CODE_FOR_avx512fp16_divv32hf3_round = 2389,
  CODE_FOR_avx512fp16_divv32hf3_mask = 2390,
  CODE_FOR_avx512fp16_divv32hf3_mask_round = 2391,
  CODE_FOR_avx512fp16_divv16hf3 = 2392,
   CODE_FOR_avx512fp16_divv16hf3_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_divv16hf3_mask = 2393,
   CODE_FOR_avx512fp16_divv16hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_divv8hf3 = 2394,
   CODE_FOR_avx512fp16_divv8hf3_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_divv8hf3_mask = 2395,
   CODE_FOR_avx512fp16_divv8hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_divv16sf3 = 2396,
  CODE_FOR_avx512f_divv16sf3_round = 2397,
  CODE_FOR_avx512f_divv16sf3_mask = 2398,
  CODE_FOR_avx512f_divv16sf3_mask_round = 2399,
  CODE_FOR_avx_divv8sf3 = 2400,
   CODE_FOR_avx_divv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_avx_divv8sf3_mask = 2401,
   CODE_FOR_avx_divv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse_divv4sf3 = 2402,
   CODE_FOR_sse_divv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_sse_divv4sf3_mask = 2403,
   CODE_FOR_sse_divv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_divv8df3 = 2404,
  CODE_FOR_avx512f_divv8df3_round = 2405,
  CODE_FOR_avx512f_divv8df3_mask = 2406,
  CODE_FOR_avx512f_divv8df3_mask_round = 2407,
  CODE_FOR_avx_divv4df3 = 2408,
   CODE_FOR_avx_divv4df3_round = CODE_FOR_nothing,
  CODE_FOR_avx_divv4df3_mask = 2409,
   CODE_FOR_avx_divv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse2_divv2df3 = 2410,
   CODE_FOR_sse2_divv2df3_round = CODE_FOR_nothing,
  CODE_FOR_sse2_divv2df3_mask = 2411,
   CODE_FOR_sse2_divv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx_rcpv8sf2 = 2412,
  CODE_FOR_sse_rcpv4sf2 = 2413,
  CODE_FOR_sse_vmrcpv4sf2 = 2414,
  CODE_FOR_avx512fp16_rcpv32hf2 = 2416,
  CODE_FOR_avx512fp16_rcpv32hf2_mask = 2417,
  CODE_FOR_avx512fp16_rcpv16hf2 = 2418,
  CODE_FOR_avx512fp16_rcpv16hf2_mask = 2419,
  CODE_FOR_avx512fp16_rcpv8hf2 = 2420,
  CODE_FOR_avx512fp16_rcpv8hf2_mask = 2421,
  CODE_FOR_avx512fp16_vmrcpv8hf2 = 2422,
  CODE_FOR_avx512fp16_vmrcpv8hf2_mask = 2423,
  CODE_FOR_rcp14v16sf_mask = 2426,
  CODE_FOR_rcp14v8sf_mask = 2428,
  CODE_FOR_rcp14v4sf_mask = 2430,
  CODE_FOR_rcp14v8df_mask = 2432,
  CODE_FOR_rcp14v4df_mask = 2434,
  CODE_FOR_rcp14v2df_mask = 2436,
  CODE_FOR_srcp14v4sf = 2437,
  CODE_FOR_srcp14v2df = 2438,
  CODE_FOR_srcp14v4sf_mask = 2439,
  CODE_FOR_srcp14v2df_mask = 2440,
  CODE_FOR_avx512fp16_sqrtv32hf2 = 2441,
  CODE_FOR_avx512fp16_sqrtv32hf2_round = 2442,
  CODE_FOR_avx512fp16_sqrtv32hf2_mask = 2443,
  CODE_FOR_avx512fp16_sqrtv32hf2_mask_round = 2444,
  CODE_FOR_avx512fp16_sqrtv16hf2 = 2445,
   CODE_FOR_avx512fp16_sqrtv16hf2_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_sqrtv16hf2_mask = 2446,
   CODE_FOR_avx512fp16_sqrtv16hf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_sqrtv8hf2 = 2447,
   CODE_FOR_avx512fp16_sqrtv8hf2_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_sqrtv8hf2_mask = 2448,
   CODE_FOR_avx512fp16_sqrtv8hf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_sqrtv16sf2 = 2449,
  CODE_FOR_avx512f_sqrtv16sf2_round = 2450,
  CODE_FOR_avx512f_sqrtv16sf2_mask = 2451,
  CODE_FOR_avx512f_sqrtv16sf2_mask_round = 2452,
  CODE_FOR_avx_sqrtv8sf2 = 2453,
   CODE_FOR_avx_sqrtv8sf2_round = CODE_FOR_nothing,
  CODE_FOR_avx_sqrtv8sf2_mask = 2454,
   CODE_FOR_avx_sqrtv8sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse_sqrtv4sf2 = 2455,
   CODE_FOR_sse_sqrtv4sf2_round = CODE_FOR_nothing,
  CODE_FOR_sse_sqrtv4sf2_mask = 2456,
   CODE_FOR_sse_sqrtv4sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_sqrtv8df2 = 2457,
  CODE_FOR_avx512f_sqrtv8df2_round = 2458,
  CODE_FOR_avx512f_sqrtv8df2_mask = 2459,
  CODE_FOR_avx512f_sqrtv8df2_mask_round = 2460,
  CODE_FOR_avx_sqrtv4df2 = 2461,
   CODE_FOR_avx_sqrtv4df2_round = CODE_FOR_nothing,
  CODE_FOR_avx_sqrtv4df2_mask = 2462,
   CODE_FOR_avx_sqrtv4df2_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse2_sqrtv2df2 = 2463,
   CODE_FOR_sse2_sqrtv2df2_round = CODE_FOR_nothing,
  CODE_FOR_sse2_sqrtv2df2_mask = 2464,
   CODE_FOR_sse2_sqrtv2df2_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_vmsqrtv8hf2 = 2465,
  CODE_FOR_avx512fp16_vmsqrtv8hf2_round = 2466,
  CODE_FOR_avx512fp16_vmsqrtv8hf2_mask = 2467,
  CODE_FOR_avx512fp16_vmsqrtv8hf2_mask_round = 2468,
  CODE_FOR_sse_vmsqrtv4sf2 = 2469,
  CODE_FOR_sse_vmsqrtv4sf2_round = 2470,
  CODE_FOR_sse_vmsqrtv4sf2_mask = 2471,
  CODE_FOR_sse_vmsqrtv4sf2_mask_round = 2472,
  CODE_FOR_sse2_vmsqrtv2df2 = 2473,
  CODE_FOR_sse2_vmsqrtv2df2_round = 2474,
  CODE_FOR_sse2_vmsqrtv2df2_mask = 2475,
  CODE_FOR_sse2_vmsqrtv2df2_mask_round = 2476,
  CODE_FOR_avx_rsqrtv8sf2 = 2489,
  CODE_FOR_sse_rsqrtv4sf2 = 2490,
  CODE_FOR_avx512fp16_rsqrtv32hf2 = 2491,
  CODE_FOR_avx512fp16_rsqrtv32hf2_mask = 2492,
  CODE_FOR_avx512fp16_rsqrtv16hf2 = 2493,
  CODE_FOR_avx512fp16_rsqrtv16hf2_mask = 2494,
  CODE_FOR_avx512fp16_rsqrtv8hf2 = 2495,
  CODE_FOR_avx512fp16_rsqrtv8hf2_mask = 2496,
  CODE_FOR_rsqrt14v16sf_mask = 2498,
  CODE_FOR_rsqrt14v8sf_mask = 2500,
  CODE_FOR_rsqrt14v4sf_mask = 2502,
  CODE_FOR_rsqrt14v8df_mask = 2504,
  CODE_FOR_rsqrt14v4df_mask = 2506,
  CODE_FOR_rsqrt14v2df_mask = 2508,
  CODE_FOR_rsqrt14v4sf = 2509,
  CODE_FOR_rsqrt14v2df = 2510,
  CODE_FOR_rsqrt14_v4sf_mask = 2511,
  CODE_FOR_rsqrt14_v2df_mask = 2512,
  CODE_FOR_sse_vmrsqrtv4sf2 = 2513,
  CODE_FOR_avx512fp16_vmrsqrtv8hf2 = 2515,
  CODE_FOR_avx512fp16_vmrsqrtv8hf2_mask = 2516,
  CODE_FOR_ieee_maxv32hf3 = 2588,
  CODE_FOR_ieee_maxv32hf3_round = 2589,
  CODE_FOR_ieee_maxv32hf3_mask = 2590,
  CODE_FOR_ieee_maxv32hf3_mask_round = 2591,
  CODE_FOR_ieee_minv32hf3 = 2592,
  CODE_FOR_ieee_minv32hf3_round = 2593,
  CODE_FOR_ieee_minv32hf3_mask = 2594,
  CODE_FOR_ieee_minv32hf3_mask_round = 2595,
  CODE_FOR_ieee_maxv16hf3 = 2596,
   CODE_FOR_ieee_maxv16hf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv16hf3_mask = 2597,
   CODE_FOR_ieee_maxv16hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv16hf3 = 2598,
   CODE_FOR_ieee_minv16hf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv16hf3_mask = 2599,
   CODE_FOR_ieee_minv16hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv8hf3 = 2600,
   CODE_FOR_ieee_maxv8hf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv8hf3_mask = 2601,
   CODE_FOR_ieee_maxv8hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv8hf3 = 2602,
   CODE_FOR_ieee_minv8hf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv8hf3_mask = 2603,
   CODE_FOR_ieee_minv8hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv16sf3 = 2604,
  CODE_FOR_ieee_maxv16sf3_round = 2605,
  CODE_FOR_ieee_maxv16sf3_mask = 2606,
  CODE_FOR_ieee_maxv16sf3_mask_round = 2607,
  CODE_FOR_ieee_minv16sf3 = 2608,
  CODE_FOR_ieee_minv16sf3_round = 2609,
  CODE_FOR_ieee_minv16sf3_mask = 2610,
  CODE_FOR_ieee_minv16sf3_mask_round = 2611,
  CODE_FOR_ieee_maxv8sf3 = 2612,
   CODE_FOR_ieee_maxv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv8sf3_mask = 2613,
   CODE_FOR_ieee_maxv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv8sf3 = 2614,
   CODE_FOR_ieee_minv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv8sf3_mask = 2615,
   CODE_FOR_ieee_minv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv4sf3 = 2616,
   CODE_FOR_ieee_maxv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv4sf3_mask = 2617,
   CODE_FOR_ieee_maxv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv4sf3 = 2618,
   CODE_FOR_ieee_minv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv4sf3_mask = 2619,
   CODE_FOR_ieee_minv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv8df3 = 2620,
  CODE_FOR_ieee_maxv8df3_round = 2621,
  CODE_FOR_ieee_maxv8df3_mask = 2622,
  CODE_FOR_ieee_maxv8df3_mask_round = 2623,
  CODE_FOR_ieee_minv8df3 = 2624,
  CODE_FOR_ieee_minv8df3_round = 2625,
  CODE_FOR_ieee_minv8df3_mask = 2626,
  CODE_FOR_ieee_minv8df3_mask_round = 2627,
  CODE_FOR_ieee_maxv4df3 = 2628,
   CODE_FOR_ieee_maxv4df3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv4df3_mask = 2629,
   CODE_FOR_ieee_maxv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv4df3 = 2630,
   CODE_FOR_ieee_minv4df3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv4df3_mask = 2631,
   CODE_FOR_ieee_minv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv2df3 = 2632,
   CODE_FOR_ieee_maxv2df3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv2df3_mask = 2633,
   CODE_FOR_ieee_maxv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv2df3 = 2634,
   CODE_FOR_ieee_minv2df3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv2df3_mask = 2635,
   CODE_FOR_ieee_minv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_ieee_vmmaxv8hf3 = 2666,
  CODE_FOR_avx512fp16_ieee_vmmaxv8hf3_mask = 2667,
  CODE_FOR_avx512fp16_ieee_vmmaxv8hf3_round = 2668,
  CODE_FOR_avx512fp16_ieee_vmmaxv8hf3_mask_round = 2669,
  CODE_FOR_avx512fp16_ieee_vmminv8hf3 = 2670,
  CODE_FOR_avx512fp16_ieee_vmminv8hf3_mask = 2671,
  CODE_FOR_avx512fp16_ieee_vmminv8hf3_round = 2672,
  CODE_FOR_avx512fp16_ieee_vmminv8hf3_mask_round = 2673,
  CODE_FOR_sse_ieee_vmmaxv4sf3 = 2674,
  CODE_FOR_sse_ieee_vmmaxv4sf3_mask = 2675,
  CODE_FOR_sse_ieee_vmmaxv4sf3_round = 2676,
  CODE_FOR_sse_ieee_vmmaxv4sf3_mask_round = 2677,
  CODE_FOR_sse_ieee_vmminv4sf3 = 2678,
  CODE_FOR_sse_ieee_vmminv4sf3_mask = 2679,
  CODE_FOR_sse_ieee_vmminv4sf3_round = 2680,
  CODE_FOR_sse_ieee_vmminv4sf3_mask_round = 2681,
  CODE_FOR_sse2_ieee_vmmaxv2df3 = 2682,
  CODE_FOR_sse2_ieee_vmmaxv2df3_mask = 2683,
  CODE_FOR_sse2_ieee_vmmaxv2df3_round = 2684,
  CODE_FOR_sse2_ieee_vmmaxv2df3_mask_round = 2685,
  CODE_FOR_sse2_ieee_vmminv2df3 = 2686,
  CODE_FOR_sse2_ieee_vmminv2df3_mask = 2687,
  CODE_FOR_sse2_ieee_vmminv2df3_round = 2688,
  CODE_FOR_sse2_ieee_vmminv2df3_mask_round = 2689,
  CODE_FOR_vec_addsubv8sf3 = 2690,
  CODE_FOR_vec_addsubv4sf3 = 2691,
  CODE_FOR_vec_addsubv4df3 = 2692,
  CODE_FOR_vec_addsubv2df3 = 2693,
  CODE_FOR_avx_haddv4df3 = 2694,
  CODE_FOR_avx_hsubv4df3 = 2695,
  CODE_FOR_sse3_hsubv2df3 = 2697,
  CODE_FOR_avx_haddv8sf3 = 2700,
  CODE_FOR_avx_hsubv8sf3 = 2701,
  CODE_FOR_sse3_haddv4sf3 = 2702,
  CODE_FOR_sse3_hsubv4sf3 = 2703,
  CODE_FOR_reducepv32hf_mask = 2706,
  CODE_FOR_reducepv32hf_mask_round = 2707,
  CODE_FOR_reducepv16hf_mask = 2710,
  CODE_FOR_reducepv16hf_mask_round = 2711,
  CODE_FOR_reducepv8hf_mask = 2714,
  CODE_FOR_reducepv8hf_mask_round = 2715,
  CODE_FOR_reducepv16sf_mask = 2718,
  CODE_FOR_reducepv16sf_mask_round = 2719,
  CODE_FOR_reducepv8sf_mask = 2722,
  CODE_FOR_reducepv8sf_mask_round = 2723,
  CODE_FOR_reducepv4sf_mask = 2726,
  CODE_FOR_reducepv4sf_mask_round = 2727,
  CODE_FOR_reducepv8df_mask = 2730,
  CODE_FOR_reducepv8df_mask_round = 2731,
  CODE_FOR_reducepv4df_mask = 2734,
  CODE_FOR_reducepv4df_mask_round = 2735,
  CODE_FOR_reducepv2df_mask = 2738,
  CODE_FOR_reducepv2df_mask_round = 2739,
  CODE_FOR_reducesv8hf = 2740,
  CODE_FOR_reducesv8hf_mask = 2741,
  CODE_FOR_reducesv8hf_round = 2742,
  CODE_FOR_reducesv8hf_mask_round = 2743,
  CODE_FOR_reducesv4sf = 2744,
  CODE_FOR_reducesv4sf_mask = 2745,
  CODE_FOR_reducesv4sf_round = 2746,
  CODE_FOR_reducesv4sf_mask_round = 2747,
  CODE_FOR_reducesv2df = 2748,
  CODE_FOR_reducesv2df_mask = 2749,
  CODE_FOR_reducesv2df_round = 2750,
  CODE_FOR_reducesv2df_mask_round = 2751,
  CODE_FOR_avx_cmpv8sf3 = 2752,
  CODE_FOR_avx_cmpv4sf3 = 2753,
  CODE_FOR_avx_cmpv4df3 = 2754,
  CODE_FOR_avx_cmpv2df3 = 2755,
  CODE_FOR_avx_vmcmpv4sf3 = 2784,
  CODE_FOR_avx_vmcmpv2df3 = 2785,
  CODE_FOR_avx_maskcmpv8sf3 = 2790,
  CODE_FOR_sse_maskcmpv4sf3 = 2791,
  CODE_FOR_avx_maskcmpv4df3 = 2792,
  CODE_FOR_sse2_maskcmpv2df3 = 2793,
  CODE_FOR_sse_vmmaskcmpv4sf3 = 2794,
  CODE_FOR_sse2_vmmaskcmpv2df3 = 2795,
  CODE_FOR_avx512f_cmpv16si3 = 2796,
  CODE_FOR_avx512f_cmpv16si3_round = 2797,
  CODE_FOR_avx512f_cmpv16si3_mask = 2798,
  CODE_FOR_avx512f_cmpv16si3_mask_round = 2799,
  CODE_FOR_avx512vl_cmpv8si3 = 2800,
   CODE_FOR_avx512vl_cmpv8si3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv8si3_mask = 2801,
   CODE_FOR_avx512vl_cmpv8si3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4si3 = 2802,
   CODE_FOR_avx512vl_cmpv4si3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4si3_mask = 2803,
   CODE_FOR_avx512vl_cmpv4si3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_cmpv8di3 = 2804,
  CODE_FOR_avx512f_cmpv8di3_round = 2805,
  CODE_FOR_avx512f_cmpv8di3_mask = 2806,
  CODE_FOR_avx512f_cmpv8di3_mask_round = 2807,
  CODE_FOR_avx512vl_cmpv4di3 = 2808,
   CODE_FOR_avx512vl_cmpv4di3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4di3_mask = 2809,
   CODE_FOR_avx512vl_cmpv4di3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv2di3 = 2810,
   CODE_FOR_avx512vl_cmpv2di3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv2di3_mask = 2811,
   CODE_FOR_avx512vl_cmpv2di3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512bw_cmpv32hf3 = 2812,
  CODE_FOR_avx512bw_cmpv32hf3_round = 2813,
  CODE_FOR_avx512bw_cmpv32hf3_mask = 2814,
  CODE_FOR_avx512bw_cmpv32hf3_mask_round = 2815,
  CODE_FOR_avx512vl_cmpv16hf3 = 2816,
   CODE_FOR_avx512vl_cmpv16hf3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv16hf3_mask = 2817,
   CODE_FOR_avx512vl_cmpv16hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_cmpv8hf3 = 2818,
   CODE_FOR_avx512fp16_cmpv8hf3_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_cmpv8hf3_mask = 2819,
   CODE_FOR_avx512fp16_cmpv8hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_cmpv16sf3 = 2820,
  CODE_FOR_avx512f_cmpv16sf3_round = 2821,
  CODE_FOR_avx512f_cmpv16sf3_mask = 2822,
  CODE_FOR_avx512f_cmpv16sf3_mask_round = 2823,
  CODE_FOR_avx512vl_cmpv8sf3 = 2824,
   CODE_FOR_avx512vl_cmpv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv8sf3_mask = 2825,
   CODE_FOR_avx512vl_cmpv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4sf3 = 2826,
   CODE_FOR_avx512vl_cmpv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4sf3_mask = 2827,
   CODE_FOR_avx512vl_cmpv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_cmpv8df3 = 2828,
  CODE_FOR_avx512f_cmpv8df3_round = 2829,
  CODE_FOR_avx512f_cmpv8df3_mask = 2830,
  CODE_FOR_avx512f_cmpv8df3_mask_round = 2831,
  CODE_FOR_avx512vl_cmpv4df3 = 2832,
   CODE_FOR_avx512vl_cmpv4df3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4df3_mask = 2833,
   CODE_FOR_avx512vl_cmpv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv2df3 = 2834,
   CODE_FOR_avx512vl_cmpv2df3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv2df3_mask = 2835,
   CODE_FOR_avx512vl_cmpv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512bw_cmpv64qi3 = 2956,
  CODE_FOR_avx512bw_cmpv64qi3_mask = 2957,
  CODE_FOR_avx512vl_cmpv16qi3 = 2958,
  CODE_FOR_avx512vl_cmpv16qi3_mask = 2959,
  CODE_FOR_avx512vl_cmpv32qi3 = 2960,
  CODE_FOR_avx512vl_cmpv32qi3_mask = 2961,
  CODE_FOR_avx512bw_cmpv32hi3 = 2962,
  CODE_FOR_avx512bw_cmpv32hi3_mask = 2963,
  CODE_FOR_avx512vl_cmpv16hi3 = 2964,
  CODE_FOR_avx512vl_cmpv16hi3_mask = 2965,
  CODE_FOR_avx512vl_cmpv8hi3 = 2966,
  CODE_FOR_avx512vl_cmpv8hi3_mask = 2967,
  CODE_FOR_avx512bw_ucmpv64qi3 = 3064,
  CODE_FOR_avx512bw_ucmpv64qi3_mask = 3065,
  CODE_FOR_avx512vl_ucmpv16qi3 = 3066,
  CODE_FOR_avx512vl_ucmpv16qi3_mask = 3067,
  CODE_FOR_avx512vl_ucmpv32qi3 = 3068,
  CODE_FOR_avx512vl_ucmpv32qi3_mask = 3069,
  CODE_FOR_avx512bw_ucmpv32hi3 = 3070,
  CODE_FOR_avx512bw_ucmpv32hi3_mask = 3071,
  CODE_FOR_avx512vl_ucmpv16hi3 = 3072,
  CODE_FOR_avx512vl_ucmpv16hi3_mask = 3073,
  CODE_FOR_avx512vl_ucmpv8hi3 = 3074,
  CODE_FOR_avx512vl_ucmpv8hi3_mask = 3075,
  CODE_FOR_avx512f_ucmpv16si3 = 3136,
  CODE_FOR_avx512f_ucmpv16si3_mask = 3137,
  CODE_FOR_avx512vl_ucmpv8si3 = 3138,
  CODE_FOR_avx512vl_ucmpv8si3_mask = 3139,
  CODE_FOR_avx512vl_ucmpv4si3 = 3140,
  CODE_FOR_avx512vl_ucmpv4si3_mask = 3141,
  CODE_FOR_avx512f_ucmpv8di3 = 3142,
  CODE_FOR_avx512f_ucmpv8di3_mask = 3143,
  CODE_FOR_avx512vl_ucmpv4di3 = 3144,
  CODE_FOR_avx512vl_ucmpv4di3_mask = 3145,
  CODE_FOR_avx512vl_ucmpv2di3 = 3146,
  CODE_FOR_avx512vl_ucmpv2di3_mask = 3147,
  CODE_FOR_avx512f_vmcmpv8hf3 = 3202,
  CODE_FOR_avx512f_vmcmpv8hf3_round = 3203,
  CODE_FOR_avx512f_vmcmpv4sf3 = 3204,
  CODE_FOR_avx512f_vmcmpv4sf3_round = 3205,
  CODE_FOR_avx512f_vmcmpv2df3 = 3206,
  CODE_FOR_avx512f_vmcmpv2df3_round = 3207,
  CODE_FOR_avx512f_vmcmpv8hf3_mask = 3208,
  CODE_FOR_avx512f_vmcmpv8hf3_mask_round = 3209,
  CODE_FOR_avx512f_vmcmpv4sf3_mask = 3210,
  CODE_FOR_avx512f_vmcmpv4sf3_mask_round = 3211,
  CODE_FOR_avx512f_vmcmpv2df3_mask = 3212,
  CODE_FOR_avx512f_vmcmpv2df3_mask_round = 3213,
  CODE_FOR_avx10_2_comxhf = 3214,
  CODE_FOR_avx10_2_comxhf_round = 3215,
  CODE_FOR_avx10_2_ucomxhf = 3216,
  CODE_FOR_avx10_2_ucomxhf_round = 3217,
  CODE_FOR_avx10_2_comxsf = 3218,
  CODE_FOR_avx10_2_comxsf_round = 3219,
  CODE_FOR_avx10_2_ucomxsf = 3220,
  CODE_FOR_avx10_2_ucomxsf_round = 3221,
  CODE_FOR_avx10_2_comxdf = 3222,
  CODE_FOR_avx10_2_comxdf_round = 3223,
  CODE_FOR_avx10_2_ucomxdf = 3224,
  CODE_FOR_avx10_2_ucomxdf_round = 3225,
  CODE_FOR_avx512fp16_comi = 3226,
  CODE_FOR_avx512fp16_comi_round = 3227,
  CODE_FOR_avx512fp16_ucomi = 3228,
  CODE_FOR_avx512fp16_ucomi_round = 3229,
  CODE_FOR_sse_comi = 3230,
  CODE_FOR_sse_comi_round = 3231,
  CODE_FOR_sse_ucomi = 3232,
  CODE_FOR_sse_ucomi_round = 3233,
  CODE_FOR_sse2_comi = 3234,
  CODE_FOR_sse2_comi_round = 3235,
  CODE_FOR_sse2_ucomi = 3236,
  CODE_FOR_sse2_ucomi_round = 3237,
  CODE_FOR_avx10_2_comisbf16_v8bf = 3238,
  CODE_FOR_avx512bf16_andnotv16bf3 = 3239,
   CODE_FOR_avx512bf16_andnotv16bf3_mask = CODE_FOR_nothing,
  CODE_FOR_avx512bf16_andnotv8bf3 = 3240,
   CODE_FOR_avx512bf16_andnotv8bf3_mask = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_andnotv16hf3 = 3241,
   CODE_FOR_avx512fp16_andnotv16hf3_mask = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_andnotv8hf3 = 3242,
   CODE_FOR_avx512fp16_andnotv8hf3_mask = CODE_FOR_nothing,
  CODE_FOR_avx_andnotv8sf3 = 3243,
  CODE_FOR_avx_andnotv8sf3_mask = 3244,
  CODE_FOR_sse_andnotv4sf3 = 3245,
  CODE_FOR_sse_andnotv4sf3_mask = 3246,
  CODE_FOR_avx_andnotv4df3 = 3247,
  CODE_FOR_avx_andnotv4df3_mask = 3248,
  CODE_FOR_sse2_andnotv2df3 = 3249,
  CODE_FOR_sse2_andnotv2df3_mask = 3250,
  CODE_FOR_avx512bf16_andnotv32bf3 = 3251,
   CODE_FOR_avx512bf16_andnotv32bf3_mask = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_andnotv32hf3 = 3252,
   CODE_FOR_avx512fp16_andnotv32hf3_mask = CODE_FOR_nothing,
  CODE_FOR_avx512f_andnotv16sf3 = 3253,
  CODE_FOR_avx512f_andnotv16sf3_mask = 3254,
  CODE_FOR_avx512f_andnotv8df3 = 3255,
  CODE_FOR_avx512f_andnotv8df3_mask = 3256,
  CODE_FOR_andbf3 = 3329,
  CODE_FOR_iorbf3 = 3330,
  CODE_FOR_xorbf3 = 3331,
  CODE_FOR_andhf3 = 3332,
  CODE_FOR_iorhf3 = 3333,
  CODE_FOR_xorhf3 = 3334,
  CODE_FOR_andsf3 = 3335,
  CODE_FOR_iorsf3 = 3336,
  CODE_FOR_xorsf3 = 3337,
  CODE_FOR_anddf3 = 3338,
  CODE_FOR_iordf3 = 3339,
  CODE_FOR_xordf3 = 3340,
  CODE_FOR_fma_fmadd_v32hf_maskz_1 = 3351,
  CODE_FOR_fma_fmadd_v32hf_maskz_1_round = 3353,
  CODE_FOR_fma_fmadd_v16hf_maskz_1 = 3355,
   CODE_FOR_fma_fmadd_v16hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmadd_v8hf_maskz_1 = 3357,
   CODE_FOR_fma_fmadd_v8hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmadd_v16sf_maskz_1 = 3361,
  CODE_FOR_fma_fmadd_v16sf_maskz_1_round = 3363,
  CODE_FOR_fma_fmadd_v8sf_maskz_1 = 3365,
   CODE_FOR_fma_fmadd_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmadd_v4sf_maskz_1 = 3367,
   CODE_FOR_fma_fmadd_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmadd_v8df_maskz_1 = 3370,
  CODE_FOR_fma_fmadd_v8df_maskz_1_round = 3372,
  CODE_FOR_fma_fmadd_v4df_maskz_1 = 3374,
   CODE_FOR_fma_fmadd_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmadd_v2df_maskz_1 = 3376,
   CODE_FOR_fma_fmadd_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512bw_fmadd_v32hf_mask = 3377,
  CODE_FOR_avx512bw_fmadd_v32hf_mask_round = 3378,
  CODE_FOR_avx512vl_fmadd_v16hf_mask = 3379,
   CODE_FOR_avx512vl_fmadd_v16hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_fmadd_v8hf_mask = 3380,
   CODE_FOR_avx512fp16_fmadd_v8hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmadd_v16sf_mask = 3381,
  CODE_FOR_avx512f_fmadd_v16sf_mask_round = 3382,
  CODE_FOR_avx512vl_fmadd_v8sf_mask = 3383,
   CODE_FOR_avx512vl_fmadd_v8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fmadd_v4sf_mask = 3384,
   CODE_FOR_avx512vl_fmadd_v4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmadd_v8df_mask = 3385,
  CODE_FOR_avx512f_fmadd_v8df_mask_round = 3386,
  CODE_FOR_avx512vl_fmadd_v4df_mask = 3387,
   CODE_FOR_avx512vl_fmadd_v4df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fmadd_v2df_mask = 3388,
   CODE_FOR_avx512vl_fmadd_v2df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512bw_fmadd_v32hf_mask3 = 3389,
  CODE_FOR_avx512bw_fmadd_v32hf_mask3_round = 3390,
  CODE_FOR_avx512vl_fmadd_v16hf_mask3 = 3391,
  CODE_FOR_avx512vl_fmadd_v16hf_mask3_round = 3392,
  CODE_FOR_avx512fp16_fmadd_v8hf_mask3 = 3393,
  CODE_FOR_avx512fp16_fmadd_v8hf_mask3_round = 3394,
  CODE_FOR_avx512f_fmadd_v16sf_mask3 = 3395,
  CODE_FOR_avx512f_fmadd_v16sf_mask3_round = 3396,
  CODE_FOR_avx512vl_fmadd_v8sf_mask3 = 3397,
  CODE_FOR_avx512vl_fmadd_v8sf_mask3_round = 3398,
  CODE_FOR_avx512vl_fmadd_v4sf_mask3 = 3399,
  CODE_FOR_avx512vl_fmadd_v4sf_mask3_round = 3400,
  CODE_FOR_avx512f_fmadd_v8df_mask3 = 3401,
  CODE_FOR_avx512f_fmadd_v8df_mask3_round = 3402,
  CODE_FOR_avx512vl_fmadd_v4df_mask3 = 3403,
  CODE_FOR_avx512vl_fmadd_v4df_mask3_round = 3404,
  CODE_FOR_avx512vl_fmadd_v2df_mask3 = 3405,
  CODE_FOR_avx512vl_fmadd_v2df_mask3_round = 3406,
  CODE_FOR_fma_fmsub_v32hf_maskz_1 = 3414,
  CODE_FOR_fma_fmsub_v32hf_maskz_1_round = 3416,
  CODE_FOR_fma_fmsub_v16hf_maskz_1 = 3418,
   CODE_FOR_fma_fmsub_v16hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsub_v8hf_maskz_1 = 3420,
   CODE_FOR_fma_fmsub_v8hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsub_v16sf_maskz_1 = 3424,
  CODE_FOR_fma_fmsub_v16sf_maskz_1_round = 3426,
  CODE_FOR_fma_fmsub_v8sf_maskz_1 = 3428,
   CODE_FOR_fma_fmsub_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsub_v4sf_maskz_1 = 3430,
   CODE_FOR_fma_fmsub_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsub_v8df_maskz_1 = 3433,
  CODE_FOR_fma_fmsub_v8df_maskz_1_round = 3435,
  CODE_FOR_fma_fmsub_v4df_maskz_1 = 3437,
   CODE_FOR_fma_fmsub_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsub_v2df_maskz_1 = 3439,
   CODE_FOR_fma_fmsub_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512bw_fmsub_v32hf_mask = 3440,
  CODE_FOR_avx512bw_fmsub_v32hf_mask_round = 3441,
  CODE_FOR_avx512vl_fmsub_v16hf_mask = 3442,
  CODE_FOR_avx512vl_fmsub_v16hf_mask_round = 3443,
  CODE_FOR_avx512fp16_fmsub_v8hf_mask = 3444,
  CODE_FOR_avx512fp16_fmsub_v8hf_mask_round = 3445,
  CODE_FOR_avx512f_fmsub_v16sf_mask = 3446,
  CODE_FOR_avx512f_fmsub_v16sf_mask_round = 3447,
  CODE_FOR_avx512vl_fmsub_v8sf_mask = 3448,
  CODE_FOR_avx512vl_fmsub_v8sf_mask_round = 3449,
  CODE_FOR_avx512vl_fmsub_v4sf_mask = 3450,
  CODE_FOR_avx512vl_fmsub_v4sf_mask_round = 3451,
  CODE_FOR_avx512f_fmsub_v8df_mask = 3452,
  CODE_FOR_avx512f_fmsub_v8df_mask_round = 3453,
  CODE_FOR_avx512vl_fmsub_v4df_mask = 3454,
  CODE_FOR_avx512vl_fmsub_v4df_mask_round = 3455,
  CODE_FOR_avx512vl_fmsub_v2df_mask = 3456,
  CODE_FOR_avx512vl_fmsub_v2df_mask_round = 3457,
  CODE_FOR_avx512bw_fmsub_v32hf_mask3 = 3458,
  CODE_FOR_avx512bw_fmsub_v32hf_mask3_round = 3459,
  CODE_FOR_avx512vl_fmsub_v16hf_mask3 = 3460,
   CODE_FOR_avx512vl_fmsub_v16hf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_fmsub_v8hf_mask3 = 3461,
   CODE_FOR_avx512fp16_fmsub_v8hf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmsub_v16sf_mask3 = 3462,
  CODE_FOR_avx512f_fmsub_v16sf_mask3_round = 3463,
  CODE_FOR_avx512vl_fmsub_v8sf_mask3 = 3464,
   CODE_FOR_avx512vl_fmsub_v8sf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fmsub_v4sf_mask3 = 3465,
   CODE_FOR_avx512vl_fmsub_v4sf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmsub_v8df_mask3 = 3466,
  CODE_FOR_avx512f_fmsub_v8df_mask3_round = 3467,
  CODE_FOR_avx512vl_fmsub_v4df_mask3 = 3468,
   CODE_FOR_avx512vl_fmsub_v4df_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fmsub_v2df_mask3 = 3469,
   CODE_FOR_avx512vl_fmsub_v2df_mask3_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmadd_v32hf_maskz_1 = 3477,
  CODE_FOR_fma_fnmadd_v32hf_maskz_1_round = 3479,
  CODE_FOR_fma_fnmadd_v16hf_maskz_1 = 3481,
   CODE_FOR_fma_fnmadd_v16hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmadd_v8hf_maskz_1 = 3483,
   CODE_FOR_fma_fnmadd_v8hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmadd_v16sf_maskz_1 = 3487,
  CODE_FOR_fma_fnmadd_v16sf_maskz_1_round = 3489,
  CODE_FOR_fma_fnmadd_v8sf_maskz_1 = 3491,
   CODE_FOR_fma_fnmadd_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmadd_v4sf_maskz_1 = 3493,
   CODE_FOR_fma_fnmadd_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmadd_v8df_maskz_1 = 3496,
  CODE_FOR_fma_fnmadd_v8df_maskz_1_round = 3498,
  CODE_FOR_fma_fnmadd_v4df_maskz_1 = 3500,
   CODE_FOR_fma_fnmadd_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmadd_v2df_maskz_1 = 3502,
   CODE_FOR_fma_fnmadd_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512bw_fnmadd_v32hf_mask = 3503,
  CODE_FOR_avx512bw_fnmadd_v32hf_mask_round = 3504,
  CODE_FOR_avx512vl_fnmadd_v16hf_mask = 3505,
   CODE_FOR_avx512vl_fnmadd_v16hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_fnmadd_v8hf_mask = 3506,
   CODE_FOR_avx512fp16_fnmadd_v8hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmadd_v16sf_mask = 3507,
  CODE_FOR_avx512f_fnmadd_v16sf_mask_round = 3508,
  CODE_FOR_avx512vl_fnmadd_v8sf_mask = 3509,
   CODE_FOR_avx512vl_fnmadd_v8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmadd_v4sf_mask = 3510,
   CODE_FOR_avx512vl_fnmadd_v4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmadd_v8df_mask = 3511,
  CODE_FOR_avx512f_fnmadd_v8df_mask_round = 3512,
  CODE_FOR_avx512vl_fnmadd_v4df_mask = 3513,
   CODE_FOR_avx512vl_fnmadd_v4df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmadd_v2df_mask = 3514,
   CODE_FOR_avx512vl_fnmadd_v2df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512bw_fnmadd_v32hf_mask3 = 3515,
  CODE_FOR_avx512bw_fnmadd_v32hf_mask3_round = 3516,
  CODE_FOR_avx512vl_fnmadd_v16hf_mask3 = 3517,
   CODE_FOR_avx512vl_fnmadd_v16hf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_fnmadd_v8hf_mask3 = 3518,
   CODE_FOR_avx512fp16_fnmadd_v8hf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmadd_v16sf_mask3 = 3519,
  CODE_FOR_avx512f_fnmadd_v16sf_mask3_round = 3520,
  CODE_FOR_avx512vl_fnmadd_v8sf_mask3 = 3521,
   CODE_FOR_avx512vl_fnmadd_v8sf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmadd_v4sf_mask3 = 3522,
   CODE_FOR_avx512vl_fnmadd_v4sf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmadd_v8df_mask3 = 3523,
  CODE_FOR_avx512f_fnmadd_v8df_mask3_round = 3524,
  CODE_FOR_avx512vl_fnmadd_v4df_mask3 = 3525,
   CODE_FOR_avx512vl_fnmadd_v4df_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmadd_v2df_mask3 = 3526,
   CODE_FOR_avx512vl_fnmadd_v2df_mask3_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmsub_v32hf_maskz_1 = 3548,
  CODE_FOR_fma_fnmsub_v32hf_maskz_1_round = 3550,
  CODE_FOR_fma_fnmsub_v16hf_maskz_1 = 3552,
   CODE_FOR_fma_fnmsub_v16hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmsub_v8hf_maskz_1 = 3554,
   CODE_FOR_fma_fnmsub_v8hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmsub_v16sf_maskz_1 = 3558,
  CODE_FOR_fma_fnmsub_v16sf_maskz_1_round = 3560,
  CODE_FOR_fma_fnmsub_v8sf_maskz_1 = 3562,
   CODE_FOR_fma_fnmsub_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmsub_v4sf_maskz_1 = 3564,
   CODE_FOR_fma_fnmsub_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmsub_v8df_maskz_1 = 3567,
  CODE_FOR_fma_fnmsub_v8df_maskz_1_round = 3569,
  CODE_FOR_fma_fnmsub_v4df_maskz_1 = 3571,
   CODE_FOR_fma_fnmsub_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmsub_v2df_maskz_1 = 3573,
   CODE_FOR_fma_fnmsub_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512bw_fnmsub_v32hf_mask = 3574,
  CODE_FOR_avx512bw_fnmsub_v32hf_mask_round = 3575,
  CODE_FOR_avx512vl_fnmsub_v16hf_mask = 3576,
   CODE_FOR_avx512vl_fnmsub_v16hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_fnmsub_v8hf_mask = 3577,
   CODE_FOR_avx512fp16_fnmsub_v8hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmsub_v16sf_mask = 3578,
  CODE_FOR_avx512f_fnmsub_v16sf_mask_round = 3579,
  CODE_FOR_avx512vl_fnmsub_v8sf_mask = 3580,
   CODE_FOR_avx512vl_fnmsub_v8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmsub_v4sf_mask = 3581,
   CODE_FOR_avx512vl_fnmsub_v4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmsub_v8df_mask = 3582,
  CODE_FOR_avx512f_fnmsub_v8df_mask_round = 3583,
  CODE_FOR_avx512vl_fnmsub_v4df_mask = 3584,
   CODE_FOR_avx512vl_fnmsub_v4df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmsub_v2df_mask = 3585,
   CODE_FOR_avx512vl_fnmsub_v2df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512bw_fnmsub_v32hf_mask3 = 3586,
  CODE_FOR_avx512bw_fnmsub_v32hf_mask3_round = 3587,
  CODE_FOR_avx512vl_fnmsub_v16hf_mask3 = 3588,
  CODE_FOR_avx512vl_fnmsub_v16hf_mask3_round = 3589,
  CODE_FOR_avx512fp16_fnmsub_v8hf_mask3 = 3590,
  CODE_FOR_avx512fp16_fnmsub_v8hf_mask3_round = 3591,
  CODE_FOR_avx512f_fnmsub_v16sf_mask3 = 3592,
  CODE_FOR_avx512f_fnmsub_v16sf_mask3_round = 3593,
  CODE_FOR_avx512vl_fnmsub_v8sf_mask3 = 3594,
  CODE_FOR_avx512vl_fnmsub_v8sf_mask3_round = 3595,
  CODE_FOR_avx512vl_fnmsub_v4sf_mask3 = 3596,
  CODE_FOR_avx512vl_fnmsub_v4sf_mask3_round = 3597,
  CODE_FOR_avx512f_fnmsub_v8df_mask3 = 3598,
  CODE_FOR_avx512f_fnmsub_v8df_mask3_round = 3599,
  CODE_FOR_avx512vl_fnmsub_v4df_mask3 = 3600,
  CODE_FOR_avx512vl_fnmsub_v4df_mask3_round = 3601,
  CODE_FOR_avx512vl_fnmsub_v2df_mask3 = 3602,
  CODE_FOR_avx512vl_fnmsub_v2df_mask3_round = 3603,
  CODE_FOR_fma_fmaddsub_v32hf_maskz_1 = 3609,
  CODE_FOR_fma_fmaddsub_v32hf_maskz_1_round = 3611,
  CODE_FOR_fma_fmaddsub_v16hf_maskz_1 = 3613,
   CODE_FOR_fma_fmaddsub_v16hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmaddsub_v8hf_maskz_1 = 3615,
   CODE_FOR_fma_fmaddsub_v8hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmaddsub_v16sf_maskz_1 = 3617,
  CODE_FOR_fma_fmaddsub_v16sf_maskz_1_round = 3619,
  CODE_FOR_fma_fmaddsub_v8sf_maskz_1 = 3621,
   CODE_FOR_fma_fmaddsub_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmaddsub_v4sf_maskz_1 = 3623,
   CODE_FOR_fma_fmaddsub_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmaddsub_v8df_maskz_1 = 3625,
  CODE_FOR_fma_fmaddsub_v8df_maskz_1_round = 3627,
  CODE_FOR_fma_fmaddsub_v4df_maskz_1 = 3629,
   CODE_FOR_fma_fmaddsub_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmaddsub_v2df_maskz_1 = 3631,
   CODE_FOR_fma_fmaddsub_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512bw_fmaddsub_v32hf_mask = 3632,
  CODE_FOR_avx512bw_fmaddsub_v32hf_mask_round = 3633,
  CODE_FOR_avx512vl_fmaddsub_v16hf_mask = 3634,
  CODE_FOR_avx512vl_fmaddsub_v16hf_mask_round = 3635,
  CODE_FOR_avx512fp16_fmaddsub_v8hf_mask = 3636,
  CODE_FOR_avx512fp16_fmaddsub_v8hf_mask_round = 3637,
  CODE_FOR_avx512f_fmaddsub_v16sf_mask = 3638,
  CODE_FOR_avx512f_fmaddsub_v16sf_mask_round = 3639,
  CODE_FOR_avx512vl_fmaddsub_v8sf_mask = 3640,
  CODE_FOR_avx512vl_fmaddsub_v8sf_mask_round = 3641,
  CODE_FOR_avx512vl_fmaddsub_v4sf_mask = 3642,
  CODE_FOR_avx512vl_fmaddsub_v4sf_mask_round = 3643,
  CODE_FOR_avx512f_fmaddsub_v8df_mask = 3644,
  CODE_FOR_avx512f_fmaddsub_v8df_mask_round = 3645,
  CODE_FOR_avx512vl_fmaddsub_v4df_mask = 3646,
  CODE_FOR_avx512vl_fmaddsub_v4df_mask_round = 3647,
  CODE_FOR_avx512vl_fmaddsub_v2df_mask = 3648,
  CODE_FOR_avx512vl_fmaddsub_v2df_mask_round = 3649,
  CODE_FOR_avx512bw_fmaddsub_v32hf_mask3 = 3650,
  CODE_FOR_avx512bw_fmaddsub_v32hf_mask3_round = 3651,
  CODE_FOR_avx512vl_fmaddsub_v16hf_mask3 = 3652,
  CODE_FOR_avx512vl_fmaddsub_v16hf_mask3_round = 3653,
  CODE_FOR_avx512fp16_fmaddsub_v8hf_mask3 = 3654,
  CODE_FOR_avx512fp16_fmaddsub_v8hf_mask3_round = 3655,
  CODE_FOR_avx512f_fmaddsub_v16sf_mask3 = 3656,
  CODE_FOR_avx512f_fmaddsub_v16sf_mask3_round = 3657,
  CODE_FOR_avx512vl_fmaddsub_v8sf_mask3 = 3658,
  CODE_FOR_avx512vl_fmaddsub_v8sf_mask3_round = 3659,
  CODE_FOR_avx512vl_fmaddsub_v4sf_mask3 = 3660,
  CODE_FOR_avx512vl_fmaddsub_v4sf_mask3_round = 3661,
  CODE_FOR_avx512f_fmaddsub_v8df_mask3 = 3662,
  CODE_FOR_avx512f_fmaddsub_v8df_mask3_round = 3663,
  CODE_FOR_avx512vl_fmaddsub_v4df_mask3 = 3664,
  CODE_FOR_avx512vl_fmaddsub_v4df_mask3_round = 3665,
  CODE_FOR_avx512vl_fmaddsub_v2df_mask3 = 3666,
  CODE_FOR_avx512vl_fmaddsub_v2df_mask3_round = 3667,
  CODE_FOR_fma_fmsubadd_v32hf_maskz_1 = 3673,
  CODE_FOR_fma_fmsubadd_v32hf_maskz_1_round = 3675,
  CODE_FOR_fma_fmsubadd_v16hf_maskz_1 = 3677,
   CODE_FOR_fma_fmsubadd_v16hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsubadd_v8hf_maskz_1 = 3679,
   CODE_FOR_fma_fmsubadd_v8hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsubadd_v16sf_maskz_1 = 3681,
  CODE_FOR_fma_fmsubadd_v16sf_maskz_1_round = 3683,
  CODE_FOR_fma_fmsubadd_v8sf_maskz_1 = 3685,
   CODE_FOR_fma_fmsubadd_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsubadd_v4sf_maskz_1 = 3687,
   CODE_FOR_fma_fmsubadd_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsubadd_v8df_maskz_1 = 3689,
  CODE_FOR_fma_fmsubadd_v8df_maskz_1_round = 3691,
  CODE_FOR_fma_fmsubadd_v4df_maskz_1 = 3693,
   CODE_FOR_fma_fmsubadd_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsubadd_v2df_maskz_1 = 3695,
   CODE_FOR_fma_fmsubadd_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512bw_fmsubadd_v32hf_mask = 3696,
  CODE_FOR_avx512bw_fmsubadd_v32hf_mask_round = 3697,
  CODE_FOR_avx512vl_fmsubadd_v16hf_mask = 3698,
  CODE_FOR_avx512vl_fmsubadd_v16hf_mask_round = 3699,
  CODE_FOR_avx512fp16_fmsubadd_v8hf_mask = 3700,
  CODE_FOR_avx512fp16_fmsubadd_v8hf_mask_round = 3701,
  CODE_FOR_avx512f_fmsubadd_v16sf_mask = 3702,
  CODE_FOR_avx512f_fmsubadd_v16sf_mask_round = 3703,
  CODE_FOR_avx512vl_fmsubadd_v8sf_mask = 3704,
  CODE_FOR_avx512vl_fmsubadd_v8sf_mask_round = 3705,
  CODE_FOR_avx512vl_fmsubadd_v4sf_mask = 3706,
  CODE_FOR_avx512vl_fmsubadd_v4sf_mask_round = 3707,
  CODE_FOR_avx512f_fmsubadd_v8df_mask = 3708,
  CODE_FOR_avx512f_fmsubadd_v8df_mask_round = 3709,
  CODE_FOR_avx512vl_fmsubadd_v4df_mask = 3710,
  CODE_FOR_avx512vl_fmsubadd_v4df_mask_round = 3711,
  CODE_FOR_avx512vl_fmsubadd_v2df_mask = 3712,
  CODE_FOR_avx512vl_fmsubadd_v2df_mask_round = 3713,
  CODE_FOR_avx512bw_fmsubadd_v32hf_mask3 = 3714,
  CODE_FOR_avx512bw_fmsubadd_v32hf_mask3_round = 3715,
  CODE_FOR_avx512vl_fmsubadd_v16hf_mask3 = 3716,
  CODE_FOR_avx512vl_fmsubadd_v16hf_mask3_round = 3717,
  CODE_FOR_avx512fp16_fmsubadd_v8hf_mask3 = 3718,
  CODE_FOR_avx512fp16_fmsubadd_v8hf_mask3_round = 3719,
  CODE_FOR_avx512f_fmsubadd_v16sf_mask3 = 3720,
  CODE_FOR_avx512f_fmsubadd_v16sf_mask3_round = 3721,
  CODE_FOR_avx512vl_fmsubadd_v8sf_mask3 = 3722,
  CODE_FOR_avx512vl_fmsubadd_v8sf_mask3_round = 3723,
  CODE_FOR_avx512vl_fmsubadd_v4sf_mask3 = 3724,
  CODE_FOR_avx512vl_fmsubadd_v4sf_mask3_round = 3725,
  CODE_FOR_avx512f_fmsubadd_v8df_mask3 = 3726,
  CODE_FOR_avx512f_fmsubadd_v8df_mask3_round = 3727,
  CODE_FOR_avx512vl_fmsubadd_v4df_mask3 = 3728,
  CODE_FOR_avx512vl_fmsubadd_v4df_mask3_round = 3729,
  CODE_FOR_avx512vl_fmsubadd_v2df_mask3 = 3730,
  CODE_FOR_avx512vl_fmsubadd_v2df_mask3_round = 3731,
  CODE_FOR_avx512f_vmfmadd_v8hf_mask = 3756,
  CODE_FOR_avx512f_vmfmadd_v8hf_mask_round = 3757,
  CODE_FOR_avx512f_vmfmadd_v4sf_mask = 3758,
  CODE_FOR_avx512f_vmfmadd_v4sf_mask_round = 3759,
  CODE_FOR_avx512f_vmfmadd_v2df_mask = 3760,
  CODE_FOR_avx512f_vmfmadd_v2df_mask_round = 3761,
  CODE_FOR_avx512f_vmfmadd_v8hf_mask3 = 3762,
  CODE_FOR_avx512f_vmfmadd_v8hf_mask3_round = 3763,
  CODE_FOR_avx512f_vmfmadd_v4sf_mask3 = 3764,
  CODE_FOR_avx512f_vmfmadd_v4sf_mask3_round = 3765,
  CODE_FOR_avx512f_vmfmadd_v2df_mask3 = 3766,
  CODE_FOR_avx512f_vmfmadd_v2df_mask3_round = 3767,
  CODE_FOR_avx512f_vmfmadd_v8hf_maskz_1 = 3768,
  CODE_FOR_avx512f_vmfmadd_v8hf_maskz_1_round = 3769,
  CODE_FOR_avx512f_vmfmadd_v4sf_maskz_1 = 3770,
  CODE_FOR_avx512f_vmfmadd_v4sf_maskz_1_round = 3771,
  CODE_FOR_avx512f_vmfmadd_v2df_maskz_1 = 3772,
  CODE_FOR_avx512f_vmfmadd_v2df_maskz_1_round = 3773,
  CODE_FOR_avx512f_vmfmsub_v8hf_mask3 = 3780,
  CODE_FOR_avx512f_vmfmsub_v8hf_mask3_round = 3781,
  CODE_FOR_avx512f_vmfmsub_v4sf_mask3 = 3782,
  CODE_FOR_avx512f_vmfmsub_v4sf_mask3_round = 3783,
  CODE_FOR_avx512f_vmfmsub_v2df_mask3 = 3784,
  CODE_FOR_avx512f_vmfmsub_v2df_mask3_round = 3785,
  CODE_FOR_avx512f_vmfnmadd_v8hf_mask = 3792,
  CODE_FOR_avx512f_vmfnmadd_v8hf_mask_round = 3793,
  CODE_FOR_avx512f_vmfnmadd_v4sf_mask = 3794,
  CODE_FOR_avx512f_vmfnmadd_v4sf_mask_round = 3795,
  CODE_FOR_avx512f_vmfnmadd_v2df_mask = 3796,
  CODE_FOR_avx512f_vmfnmadd_v2df_mask_round = 3797,
  CODE_FOR_avx512f_vmfnmadd_v8hf_mask3 = 3798,
  CODE_FOR_avx512f_vmfnmadd_v8hf_mask3_round = 3799,
  CODE_FOR_avx512f_vmfnmadd_v4sf_mask3 = 3800,
  CODE_FOR_avx512f_vmfnmadd_v4sf_mask3_round = 3801,
  CODE_FOR_avx512f_vmfnmadd_v2df_mask3 = 3802,
  CODE_FOR_avx512f_vmfnmadd_v2df_mask3_round = 3803,
  CODE_FOR_avx512f_vmfnmadd_v8hf_maskz_1 = 3804,
  CODE_FOR_avx512f_vmfnmadd_v8hf_maskz_1_round = 3805,
  CODE_FOR_avx512f_vmfnmadd_v4sf_maskz_1 = 3806,
  CODE_FOR_avx512f_vmfnmadd_v4sf_maskz_1_round = 3807,
  CODE_FOR_avx512f_vmfnmadd_v2df_maskz_1 = 3808,
  CODE_FOR_avx512f_vmfnmadd_v2df_maskz_1_round = 3809,
  CODE_FOR_fma_fmaddc_v32hf = 3836,
  CODE_FOR_fma_fmaddc_v32hf_round = 3837,
  CODE_FOR_fma_fmaddc_v32hf_maskz_1 = 3838,
  CODE_FOR_fma_fmaddc_v32hf_maskz_1_round = 3839,
  CODE_FOR_fma_fcmaddc_v32hf = 3840,
  CODE_FOR_fma_fcmaddc_v32hf_round = 3841,
  CODE_FOR_fma_fcmaddc_v32hf_maskz_1 = 3842,
  CODE_FOR_fma_fcmaddc_v32hf_maskz_1_round = 3843,
  CODE_FOR_fma_fmaddc_v16hf = 3844,
   CODE_FOR_fma_fmaddc_v16hf_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmaddc_v16hf_maskz_1 = 3845,
   CODE_FOR_fma_fmaddc_v16hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fcmaddc_v16hf = 3846,
   CODE_FOR_fma_fcmaddc_v16hf_round = CODE_FOR_nothing,
  CODE_FOR_fma_fcmaddc_v16hf_maskz_1 = 3847,
   CODE_FOR_fma_fcmaddc_v16hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmaddc_v8hf = 3848,
   CODE_FOR_fma_fmaddc_v8hf_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmaddc_v8hf_maskz_1 = 3849,
   CODE_FOR_fma_fmaddc_v8hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fcmaddc_v8hf = 3850,
   CODE_FOR_fma_fcmaddc_v8hf_round = CODE_FOR_nothing,
  CODE_FOR_fma_fcmaddc_v8hf_maskz_1 = 3851,
   CODE_FOR_fma_fcmaddc_v8hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_v32hf_fadd_fmul = 3852,
  CODE_FOR_fma_v16hf_fadd_fmul = 3853,
  CODE_FOR_fma_v8hf_fadd_fmul = 3854,
  CODE_FOR_fma_v32hf_fadd_fcmul = 3855,
  CODE_FOR_fma_v16hf_fadd_fcmul = 3856,
  CODE_FOR_fma_v8hf_fadd_fcmul = 3857,
  CODE_FOR_fma_fmaddc_v32hf_fma_zero = 3858,
  CODE_FOR_fma_fcmaddc_v32hf_fma_zero = 3859,
  CODE_FOR_fma_fmaddc_v16hf_fma_zero = 3860,
  CODE_FOR_fma_fcmaddc_v16hf_fma_zero = 3861,
  CODE_FOR_fma_fmaddc_v8hf_fma_zero = 3862,
  CODE_FOR_fma_fcmaddc_v8hf_fma_zero = 3863,
  CODE_FOR_fma_fmaddc_v16sf_pair = 3864,
  CODE_FOR_fma_fcmaddc_v16sf_pair = 3865,
  CODE_FOR_fma_fmaddc_v8sf_pair = 3866,
  CODE_FOR_fma_fcmaddc_v8sf_pair = 3867,
  CODE_FOR_fma_fmaddc_v4sf_pair = 3868,
  CODE_FOR_fma_fcmaddc_v4sf_pair = 3869,
  CODE_FOR_fma_v32hf_fmaddc_bcst = 3870,
  CODE_FOR_fma_v16hf_fmaddc_bcst = 3871,
  CODE_FOR_fma_v8hf_fmaddc_bcst = 3872,
  CODE_FOR_fma_v32hf_fcmaddc_bcst = 3873,
  CODE_FOR_fma_v16hf_fcmaddc_bcst = 3874,
  CODE_FOR_fma_v8hf_fcmaddc_bcst = 3875,
  CODE_FOR_avx512bw_fmaddc_v32hf_mask = 3876,
  CODE_FOR_avx512bw_fmaddc_v32hf_mask_round = 3877,
  CODE_FOR_avx512bw_fcmaddc_v32hf_mask = 3878,
  CODE_FOR_avx512bw_fcmaddc_v32hf_mask_round = 3879,
  CODE_FOR_avx512vl_fmaddc_v16hf_mask = 3880,
   CODE_FOR_avx512vl_fmaddc_v16hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fcmaddc_v16hf_mask = 3881,
   CODE_FOR_avx512vl_fcmaddc_v16hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_fmaddc_v8hf_mask = 3882,
   CODE_FOR_avx512fp16_fmaddc_v8hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_fcmaddc_v8hf_mask = 3883,
   CODE_FOR_avx512fp16_fcmaddc_v8hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512bw_fmulc_v32hf = 3884,
  CODE_FOR_avx512bw_fmulc_v32hf_mask = 3885,
  CODE_FOR_avx512bw_fmulc_v32hf_round = 3886,
  CODE_FOR_avx512bw_fmulc_v32hf_mask_round = 3887,
  CODE_FOR_avx512bw_fcmulc_v32hf = 3888,
  CODE_FOR_avx512bw_fcmulc_v32hf_mask = 3889,
  CODE_FOR_avx512bw_fcmulc_v32hf_round = 3890,
  CODE_FOR_avx512bw_fcmulc_v32hf_mask_round = 3891,
  CODE_FOR_avx512vl_fmulc_v16hf = 3892,
  CODE_FOR_avx512vl_fmulc_v16hf_mask = 3893,
   CODE_FOR_avx512vl_fmulc_v16hf_round = CODE_FOR_nothing,
   CODE_FOR_avx512vl_fmulc_v16hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fcmulc_v16hf = 3894,
  CODE_FOR_avx512vl_fcmulc_v16hf_mask = 3895,
   CODE_FOR_avx512vl_fcmulc_v16hf_round = CODE_FOR_nothing,
   CODE_FOR_avx512vl_fcmulc_v16hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_fmulc_v8hf = 3896,
  CODE_FOR_avx512fp16_fmulc_v8hf_mask = 3897,
   CODE_FOR_avx512fp16_fmulc_v8hf_round = CODE_FOR_nothing,
   CODE_FOR_avx512fp16_fmulc_v8hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_fcmulc_v8hf = 3898,
  CODE_FOR_avx512fp16_fcmulc_v8hf_mask = 3899,
   CODE_FOR_avx512fp16_fcmulc_v8hf_round = CODE_FOR_nothing,
   CODE_FOR_avx512fp16_fcmulc_v8hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_fma_fmaddcsh_v8hf = 3900,
  CODE_FOR_avx512fp16_fma_fmaddcsh_v8hf_maskz = 3901,
  CODE_FOR_avx512fp16_fma_fmaddcsh_v8hf_round = 3902,
  CODE_FOR_avx512fp16_fma_fmaddcsh_v8hf_maskz_round = 3903,
  CODE_FOR_avx512fp16_fma_fcmaddcsh_v8hf = 3904,
  CODE_FOR_avx512fp16_fma_fcmaddcsh_v8hf_maskz = 3905,
  CODE_FOR_avx512fp16_fma_fcmaddcsh_v8hf_round = 3906,
  CODE_FOR_avx512fp16_fma_fcmaddcsh_v8hf_maskz_round = 3907,
  CODE_FOR_avx512fp16_fmaddcsh_v8hf_mask = 3908,
  CODE_FOR_avx512fp16_fmaddcsh_v8hf_mask_round = 3909,
  CODE_FOR_avx512fp16_fcmaddcsh_v8hf_mask = 3910,
  CODE_FOR_avx512fp16_fcmaddcsh_v8hf_mask_round = 3911,
  CODE_FOR_avx512fp16_fmulcsh_v8hf = 3912,
  CODE_FOR_avx512fp16_fmulcsh_v8hf_round = 3913,
  CODE_FOR_avx512fp16_fmulcsh_v8hf_mask = 3914,
  CODE_FOR_avx512fp16_fmulcsh_v8hf_mask_round = 3915,
  CODE_FOR_avx512fp16_fcmulcsh_v8hf = 3916,
  CODE_FOR_avx512fp16_fcmulcsh_v8hf_round = 3917,
  CODE_FOR_avx512fp16_fcmulcsh_v8hf_mask = 3918,
  CODE_FOR_avx512fp16_fcmulcsh_v8hf_mask_round = 3919,
  CODE_FOR_avx512fp16_vcvtph2uw_v32hi = 3920,
  CODE_FOR_avx512fp16_vcvtph2uw_v32hi_round = 3921,
  CODE_FOR_avx512fp16_vcvtph2uw_v32hi_mask = 3922,
  CODE_FOR_avx512fp16_vcvtph2uw_v32hi_mask_round = 3923,
  CODE_FOR_avx512fp16_vcvtph2w_v32hi = 3924,
  CODE_FOR_avx512fp16_vcvtph2w_v32hi_round = 3925,
  CODE_FOR_avx512fp16_vcvtph2w_v32hi_mask = 3926,
  CODE_FOR_avx512fp16_vcvtph2w_v32hi_mask_round = 3927,
  CODE_FOR_avx512fp16_vcvtph2udq_v16si = 3928,
  CODE_FOR_avx512fp16_vcvtph2udq_v16si_round = 3929,
  CODE_FOR_avx512fp16_vcvtph2udq_v16si_mask = 3930,
  CODE_FOR_avx512fp16_vcvtph2udq_v16si_mask_round = 3931,
  CODE_FOR_avx512fp16_vcvtph2dq_v16si = 3932,
  CODE_FOR_avx512fp16_vcvtph2dq_v16si_round = 3933,
  CODE_FOR_avx512fp16_vcvtph2dq_v16si_mask = 3934,
  CODE_FOR_avx512fp16_vcvtph2dq_v16si_mask_round = 3935,
  CODE_FOR_avx512fp16_vcvtph2uqq_v8di = 3936,
  CODE_FOR_avx512fp16_vcvtph2uqq_v8di_round = 3937,
  CODE_FOR_avx512fp16_vcvtph2uqq_v8di_mask = 3938,
  CODE_FOR_avx512fp16_vcvtph2uqq_v8di_mask_round = 3939,
  CODE_FOR_avx512fp16_vcvtph2qq_v8di = 3940,
  CODE_FOR_avx512fp16_vcvtph2qq_v8di_round = 3941,
  CODE_FOR_avx512fp16_vcvtph2qq_v8di_mask = 3942,
  CODE_FOR_avx512fp16_vcvtph2qq_v8di_mask_round = 3943,
  CODE_FOR_avx512fp16_vcvtph2uw_v16hi = 3944,
  CODE_FOR_avx512fp16_vcvtph2uw_v16hi_round = 3945,
  CODE_FOR_avx512fp16_vcvtph2uw_v16hi_mask = 3946,
  CODE_FOR_avx512fp16_vcvtph2uw_v16hi_mask_round = 3947,
  CODE_FOR_avx512fp16_vcvtph2w_v16hi = 3948,
  CODE_FOR_avx512fp16_vcvtph2w_v16hi_round = 3949,
  CODE_FOR_avx512fp16_vcvtph2w_v16hi_mask = 3950,
  CODE_FOR_avx512fp16_vcvtph2w_v16hi_mask_round = 3951,
  CODE_FOR_avx512fp16_vcvtph2udq_v8si = 3952,
  CODE_FOR_avx512fp16_vcvtph2udq_v8si_round = 3953,
  CODE_FOR_avx512fp16_vcvtph2udq_v8si_mask = 3954,
  CODE_FOR_avx512fp16_vcvtph2udq_v8si_mask_round = 3955,
  CODE_FOR_avx512fp16_vcvtph2dq_v8si = 3956,
  CODE_FOR_avx512fp16_vcvtph2dq_v8si_round = 3957,
  CODE_FOR_avx512fp16_vcvtph2dq_v8si_mask = 3958,
  CODE_FOR_avx512fp16_vcvtph2dq_v8si_mask_round = 3959,
  CODE_FOR_avx512fp16_vcvtph2uqq_v4di = 3960,
  CODE_FOR_avx512fp16_vcvtph2uqq_v4di_round = 3961,
  CODE_FOR_avx512fp16_vcvtph2uqq_v4di_mask = 3962,
  CODE_FOR_avx512fp16_vcvtph2uqq_v4di_mask_round = 3963,
  CODE_FOR_avx512fp16_vcvtph2qq_v4di = 3964,
  CODE_FOR_avx512fp16_vcvtph2qq_v4di_round = 3965,
  CODE_FOR_avx512fp16_vcvtph2qq_v4di_mask = 3966,
  CODE_FOR_avx512fp16_vcvtph2qq_v4di_mask_round = 3967,
  CODE_FOR_avx512fp16_vcvtph2uw_v8hi = 3968,
  CODE_FOR_avx512fp16_vcvtph2uw_v8hi_round = 3969,
  CODE_FOR_avx512fp16_vcvtph2uw_v8hi_mask = 3970,
  CODE_FOR_avx512fp16_vcvtph2uw_v8hi_mask_round = 3971,
  CODE_FOR_avx512fp16_vcvtph2w_v8hi = 3972,
  CODE_FOR_avx512fp16_vcvtph2w_v8hi_round = 3973,
  CODE_FOR_avx512fp16_vcvtph2w_v8hi_mask = 3974,
  CODE_FOR_avx512fp16_vcvtph2w_v8hi_mask_round = 3975,
  CODE_FOR_avx512fp16_vcvtph2udq_v4si = 3976,
  CODE_FOR_avx512fp16_vcvtph2udq_v4si_round = 3977,
  CODE_FOR_avx512fp16_vcvtph2udq_v4si_mask = 3978,
  CODE_FOR_avx512fp16_vcvtph2udq_v4si_mask_round = 3979,
  CODE_FOR_avx512fp16_vcvtph2dq_v4si = 3980,
  CODE_FOR_avx512fp16_vcvtph2dq_v4si_round = 3981,
  CODE_FOR_avx512fp16_vcvtph2dq_v4si_mask = 3982,
  CODE_FOR_avx512fp16_vcvtph2dq_v4si_mask_round = 3983,
  CODE_FOR_avx512fp16_vcvtph2uqq_v2di = 3984,
  CODE_FOR_avx512fp16_vcvtph2uqq_v2di_round = 3985,
  CODE_FOR_avx512fp16_vcvtph2uqq_v2di_mask = 3986,
  CODE_FOR_avx512fp16_vcvtph2uqq_v2di_mask_round = 3987,
  CODE_FOR_avx512fp16_vcvtph2qq_v2di = 3988,
  CODE_FOR_avx512fp16_vcvtph2qq_v2di_round = 3989,
  CODE_FOR_avx512fp16_vcvtph2qq_v2di_mask = 3990,
  CODE_FOR_avx512fp16_vcvtph2qq_v2di_mask_round = 3991,
  CODE_FOR_avx512fp16_vcvtw2ph_v8hi = 3992,
  CODE_FOR_avx512fp16_vcvtw2ph_v8hi_round = 3993,
  CODE_FOR_avx512fp16_vcvtw2ph_v8hi_mask = 3994,
  CODE_FOR_avx512fp16_vcvtw2ph_v8hi_mask_round = 3995,
  CODE_FOR_avx512fp16_vcvtuw2ph_v8hi = 3996,
  CODE_FOR_avx512fp16_vcvtuw2ph_v8hi_round = 3997,
  CODE_FOR_avx512fp16_vcvtuw2ph_v8hi_mask = 3998,
  CODE_FOR_avx512fp16_vcvtuw2ph_v8hi_mask_round = 3999,
  CODE_FOR_avx512fp16_vcvtw2ph_v16hi = 4000,
  CODE_FOR_avx512fp16_vcvtw2ph_v16hi_round = 4001,
  CODE_FOR_avx512fp16_vcvtw2ph_v16hi_mask = 4002,
  CODE_FOR_avx512fp16_vcvtw2ph_v16hi_mask_round = 4003,
  CODE_FOR_avx512fp16_vcvtuw2ph_v16hi = 4004,
  CODE_FOR_avx512fp16_vcvtuw2ph_v16hi_round = 4005,
  CODE_FOR_avx512fp16_vcvtuw2ph_v16hi_mask = 4006,
  CODE_FOR_avx512fp16_vcvtuw2ph_v16hi_mask_round = 4007,
  CODE_FOR_avx512fp16_vcvtw2ph_v32hi = 4008,
  CODE_FOR_avx512fp16_vcvtw2ph_v32hi_round = 4009,
  CODE_FOR_avx512fp16_vcvtw2ph_v32hi_mask = 4010,
  CODE_FOR_avx512fp16_vcvtw2ph_v32hi_mask_round = 4011,
  CODE_FOR_avx512fp16_vcvtuw2ph_v32hi = 4012,
  CODE_FOR_avx512fp16_vcvtuw2ph_v32hi_round = 4013,
  CODE_FOR_avx512fp16_vcvtuw2ph_v32hi_mask = 4014,
  CODE_FOR_avx512fp16_vcvtuw2ph_v32hi_mask_round = 4015,
  CODE_FOR_avx512fp16_vcvtdq2ph_v8si = 4016,
  CODE_FOR_avx512fp16_vcvtdq2ph_v8si_round = 4017,
  CODE_FOR_avx512fp16_vcvtdq2ph_v8si_mask = 4018,
  CODE_FOR_avx512fp16_vcvtdq2ph_v8si_mask_round = 4019,
  CODE_FOR_avx512fp16_vcvtudq2ph_v8si = 4020,
  CODE_FOR_avx512fp16_vcvtudq2ph_v8si_round = 4021,
  CODE_FOR_avx512fp16_vcvtudq2ph_v8si_mask = 4022,
  CODE_FOR_avx512fp16_vcvtudq2ph_v8si_mask_round = 4023,
  CODE_FOR_avx512fp16_vcvtdq2ph_v16si = 4024,
  CODE_FOR_avx512fp16_vcvtdq2ph_v16si_round = 4025,
  CODE_FOR_avx512fp16_vcvtdq2ph_v16si_mask = 4026,
  CODE_FOR_avx512fp16_vcvtdq2ph_v16si_mask_round = 4027,
  CODE_FOR_avx512fp16_vcvtudq2ph_v16si = 4028,
  CODE_FOR_avx512fp16_vcvtudq2ph_v16si_round = 4029,
  CODE_FOR_avx512fp16_vcvtudq2ph_v16si_mask = 4030,
  CODE_FOR_avx512fp16_vcvtudq2ph_v16si_mask_round = 4031,
  CODE_FOR_avx512fp16_vcvtqq2ph_v8di = 4032,
  CODE_FOR_avx512fp16_vcvtqq2ph_v8di_round = 4033,
  CODE_FOR_avx512fp16_vcvtqq2ph_v8di_mask = 4034,
  CODE_FOR_avx512fp16_vcvtqq2ph_v8di_mask_round = 4035,
  CODE_FOR_avx512fp16_vcvtuqq2ph_v8di = 4036,
  CODE_FOR_avx512fp16_vcvtuqq2ph_v8di_round = 4037,
  CODE_FOR_avx512fp16_vcvtuqq2ph_v8di_mask = 4038,
  CODE_FOR_avx512fp16_vcvtuqq2ph_v8di_mask_round = 4039,
  CODE_FOR_avx512fp16_vcvtsh2usi = 4058,
  CODE_FOR_avx512fp16_vcvtsh2usi_round = 4059,
  CODE_FOR_avx512fp16_vcvtsh2si = 4060,
  CODE_FOR_avx512fp16_vcvtsh2si_round = 4061,
   CODE_FOR_avx512fp16_vcvtsh2usiq = CODE_FOR_nothing,
   CODE_FOR_avx512fp16_vcvtsh2usiq_round = CODE_FOR_nothing,
   CODE_FOR_avx512fp16_vcvtsh2siq = CODE_FOR_nothing,
   CODE_FOR_avx512fp16_vcvtsh2siq_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_vcvtsh2usi_2 = 4062,
  CODE_FOR_avx512fp16_vcvtsh2si_2 = 4063,
   CODE_FOR_avx512fp16_vcvtsh2usiq_2 = CODE_FOR_nothing,
   CODE_FOR_avx512fp16_vcvtsh2siq_2 = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_vcvtsi2sh = 4064,
  CODE_FOR_avx512fp16_vcvtsi2sh_round = 4065,
  CODE_FOR_avx512fp16_vcvtusi2sh = 4066,
  CODE_FOR_avx512fp16_vcvtusi2sh_round = 4067,
   CODE_FOR_avx512fp16_vcvtsi2shq = CODE_FOR_nothing,
   CODE_FOR_avx512fp16_vcvtsi2shq_round = CODE_FOR_nothing,
   CODE_FOR_avx512fp16_vcvtusi2shq = CODE_FOR_nothing,
   CODE_FOR_avx512fp16_vcvtusi2shq_round = CODE_FOR_nothing,
  CODE_FOR_unspec_avx512fp16_fix_truncv8hi2 = 4068,
  CODE_FOR_unspec_avx512fp16_fix_truncv8hi2_round = 4069,
  CODE_FOR_unspec_avx512fp16_fix_truncv8hi2_mask = 4070,
  CODE_FOR_unspec_avx512fp16_fix_truncv8hi2_mask_round = 4071,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv8hi2 = 4072,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv8hi2_round = 4073,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv8hi2_mask = 4074,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv8hi2_mask_round = 4075,
  CODE_FOR_unspec_avx512fp16_fix_truncv16hi2 = 4076,
  CODE_FOR_unspec_avx512fp16_fix_truncv16hi2_round = 4077,
  CODE_FOR_unspec_avx512fp16_fix_truncv16hi2_mask = 4078,
  CODE_FOR_unspec_avx512fp16_fix_truncv16hi2_mask_round = 4079,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv16hi2 = 4080,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv16hi2_round = 4081,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv16hi2_mask = 4082,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv16hi2_mask_round = 4083,
  CODE_FOR_unspec_avx512fp16_fix_truncv32hi2 = 4084,
  CODE_FOR_unspec_avx512fp16_fix_truncv32hi2_round = 4085,
  CODE_FOR_unspec_avx512fp16_fix_truncv32hi2_mask = 4086,
  CODE_FOR_unspec_avx512fp16_fix_truncv32hi2_mask_round = 4087,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv32hi2 = 4088,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv32hi2_round = 4089,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv32hi2_mask = 4090,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv32hi2_mask_round = 4091,
  CODE_FOR_unspec_avx512fp16_fix_truncv8si2 = 4092,
  CODE_FOR_unspec_avx512fp16_fix_truncv8si2_round = 4093,
  CODE_FOR_unspec_avx512fp16_fix_truncv8si2_mask = 4094,
  CODE_FOR_unspec_avx512fp16_fix_truncv8si2_mask_round = 4095,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv8si2 = 4096,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv8si2_round = 4097,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv8si2_mask = 4098,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv8si2_mask_round = 4099,
  CODE_FOR_unspec_avx512fp16_fix_truncv16si2 = 4100,
  CODE_FOR_unspec_avx512fp16_fix_truncv16si2_round = 4101,
  CODE_FOR_unspec_avx512fp16_fix_truncv16si2_mask = 4102,
  CODE_FOR_unspec_avx512fp16_fix_truncv16si2_mask_round = 4103,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv16si2 = 4104,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv16si2_round = 4105,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv16si2_mask = 4106,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv16si2_mask_round = 4107,
  CODE_FOR_unspec_avx512fp16_fix_truncv8di2 = 4108,
  CODE_FOR_unspec_avx512fp16_fix_truncv8di2_round = 4109,
  CODE_FOR_unspec_avx512fp16_fix_truncv8di2_mask = 4110,
  CODE_FOR_unspec_avx512fp16_fix_truncv8di2_mask_round = 4111,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv8di2 = 4112,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv8di2_round = 4113,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv8di2_mask = 4114,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv8di2_mask_round = 4115,
  CODE_FOR_avx512fp16_fix_truncv8hi2 = 4116,
  CODE_FOR_avx512fp16_fix_truncv8hi2_round = 4117,
  CODE_FOR_avx512fp16_fix_truncv8hi2_mask = 4118,
  CODE_FOR_avx512fp16_fix_truncv8hi2_mask_round = 4119,
  CODE_FOR_avx512fp16_fixuns_truncv8hi2 = 4120,
  CODE_FOR_avx512fp16_fixuns_truncv8hi2_round = 4121,
  CODE_FOR_avx512fp16_fixuns_truncv8hi2_mask = 4122,
  CODE_FOR_avx512fp16_fixuns_truncv8hi2_mask_round = 4123,
  CODE_FOR_avx512fp16_fix_truncv16hi2 = 4124,
  CODE_FOR_avx512fp16_fix_truncv16hi2_round = 4125,
  CODE_FOR_avx512fp16_fix_truncv16hi2_mask = 4126,
  CODE_FOR_avx512fp16_fix_truncv16hi2_mask_round = 4127,
  CODE_FOR_avx512fp16_fixuns_truncv16hi2 = 4128,
  CODE_FOR_avx512fp16_fixuns_truncv16hi2_round = 4129,
  CODE_FOR_avx512fp16_fixuns_truncv16hi2_mask = 4130,
  CODE_FOR_avx512fp16_fixuns_truncv16hi2_mask_round = 4131,
  CODE_FOR_avx512fp16_fix_truncv32hi2 = 4132,
  CODE_FOR_avx512fp16_fix_truncv32hi2_round = 4133,
  CODE_FOR_avx512fp16_fix_truncv32hi2_mask = 4134,
  CODE_FOR_avx512fp16_fix_truncv32hi2_mask_round = 4135,
  CODE_FOR_avx512fp16_fixuns_truncv32hi2 = 4136,
  CODE_FOR_avx512fp16_fixuns_truncv32hi2_round = 4137,
  CODE_FOR_avx512fp16_fixuns_truncv32hi2_mask = 4138,
  CODE_FOR_avx512fp16_fixuns_truncv32hi2_mask_round = 4139,
  CODE_FOR_avx512fp16_fix_truncv8si2 = 4140,
  CODE_FOR_avx512fp16_fix_truncv8si2_round = 4141,
  CODE_FOR_avx512fp16_fix_truncv8si2_mask = 4142,
  CODE_FOR_avx512fp16_fix_truncv8si2_mask_round = 4143,
  CODE_FOR_avx512fp16_fixuns_truncv8si2 = 4144,
  CODE_FOR_avx512fp16_fixuns_truncv8si2_round = 4145,
  CODE_FOR_avx512fp16_fixuns_truncv8si2_mask = 4146,
  CODE_FOR_avx512fp16_fixuns_truncv8si2_mask_round = 4147,
  CODE_FOR_avx512fp16_fix_truncv16si2 = 4148,
  CODE_FOR_avx512fp16_fix_truncv16si2_round = 4149,
  CODE_FOR_avx512fp16_fix_truncv16si2_mask = 4150,
  CODE_FOR_avx512fp16_fix_truncv16si2_mask_round = 4151,
  CODE_FOR_avx512fp16_fixuns_truncv16si2 = 4152,
  CODE_FOR_avx512fp16_fixuns_truncv16si2_round = 4153,
  CODE_FOR_avx512fp16_fixuns_truncv16si2_mask = 4154,
  CODE_FOR_avx512fp16_fixuns_truncv16si2_mask_round = 4155,
  CODE_FOR_avx512fp16_fix_truncv8di2 = 4156,
  CODE_FOR_avx512fp16_fix_truncv8di2_round = 4157,
  CODE_FOR_avx512fp16_fix_truncv8di2_mask = 4158,
  CODE_FOR_avx512fp16_fix_truncv8di2_mask_round = 4159,
  CODE_FOR_avx512fp16_fixuns_truncv8di2 = 4160,
  CODE_FOR_avx512fp16_fixuns_truncv8di2_round = 4161,
  CODE_FOR_avx512fp16_fixuns_truncv8di2_mask = 4162,
  CODE_FOR_avx512fp16_fixuns_truncv8di2_mask_round = 4163,
  CODE_FOR_unspec_avx512fp16_fix_truncv4si2 = 4164,
  CODE_FOR_unspec_avx512fp16_fix_truncv4si2_mask = 4165,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv4si2 = 4166,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv4si2_mask = 4167,
  CODE_FOR_unspec_avx512fp16_fix_truncv4di2 = 4168,
  CODE_FOR_unspec_avx512fp16_fix_truncv4di2_mask = 4169,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv4di2 = 4170,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv4di2_mask = 4171,
  CODE_FOR_avx512fp16_fix_truncv4si2 = 4172,
  CODE_FOR_avx512fp16_fix_truncv4si2_mask = 4173,
  CODE_FOR_avx512fp16_fixuns_truncv4si2 = 4174,
  CODE_FOR_avx512fp16_fixuns_truncv4si2_mask = 4175,
  CODE_FOR_avx512fp16_fix_truncv4di2 = 4176,
  CODE_FOR_avx512fp16_fix_truncv4di2_mask = 4177,
  CODE_FOR_avx512fp16_fixuns_truncv4di2 = 4178,
  CODE_FOR_avx512fp16_fixuns_truncv4di2_mask = 4179,
  CODE_FOR_unspec_avx512fp16_fix_truncv2di2 = 4188,
  CODE_FOR_unspec_avx512fp16_fix_truncv2di2_mask = 4189,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv2di2 = 4190,
  CODE_FOR_unspec_avx512fp16_fixuns_truncv2di2_mask = 4191,
  CODE_FOR_avx512fp16_fix_truncv2di2 = 4192,
  CODE_FOR_avx512fp16_fix_truncv2di2_mask = 4193,
  CODE_FOR_avx512fp16_fixuns_truncv2di2 = 4194,
  CODE_FOR_avx512fp16_fixuns_truncv2di2_mask = 4195,
  CODE_FOR_unspec_avx512fp16_fix_truncsi2 = 4200,
  CODE_FOR_unspec_avx512fp16_fix_truncsi2_round = 4201,
  CODE_FOR_unspec_avx512fp16_fixuns_truncsi2 = 4202,
  CODE_FOR_unspec_avx512fp16_fixuns_truncsi2_round = 4203,
   CODE_FOR_unspec_avx512fp16_fix_truncdi2 = CODE_FOR_nothing,
   CODE_FOR_unspec_avx512fp16_fix_truncdi2_round = CODE_FOR_nothing,
   CODE_FOR_unspec_avx512fp16_fixuns_truncdi2 = CODE_FOR_nothing,
   CODE_FOR_unspec_avx512fp16_fixuns_truncdi2_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_fix_truncsi2 = 4204,
  CODE_FOR_avx512fp16_fix_truncsi2_round = 4205,
  CODE_FOR_avx512fp16_fixuns_truncsi2 = 4206,
  CODE_FOR_avx512fp16_fixuns_truncsi2_round = 4207,
   CODE_FOR_avx512fp16_fix_truncdi2 = CODE_FOR_nothing,
   CODE_FOR_avx512fp16_fix_truncdi2_round = CODE_FOR_nothing,
   CODE_FOR_avx512fp16_fixuns_truncdi2 = CODE_FOR_nothing,
   CODE_FOR_avx512fp16_fixuns_truncdi2_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_fix_truncsi2_mem = 4208,
  CODE_FOR_avx512fp16_fixuns_truncsi2_mem = 4209,
   CODE_FOR_avx512fp16_fix_truncdi2_mem = CODE_FOR_nothing,
   CODE_FOR_avx512fp16_fixuns_truncdi2_mem = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_float_extend_phv8df2 = 4210,
  CODE_FOR_avx512fp16_float_extend_phv8df2_round = 4211,
  CODE_FOR_avx512fp16_float_extend_phv8df2_mask = 4212,
  CODE_FOR_avx512fp16_float_extend_phv8df2_mask_round = 4213,
  CODE_FOR_avx512fp16_float_extend_phv16sf2 = 4214,
  CODE_FOR_avx512fp16_float_extend_phv16sf2_round = 4215,
  CODE_FOR_avx512fp16_float_extend_phv16sf2_mask = 4216,
  CODE_FOR_avx512fp16_float_extend_phv16sf2_mask_round = 4217,
  CODE_FOR_avx512fp16_float_extend_phv8sf2 = 4218,
  CODE_FOR_avx512fp16_float_extend_phv8sf2_round = 4219,
  CODE_FOR_avx512fp16_float_extend_phv8sf2_mask = 4220,
  CODE_FOR_avx512fp16_float_extend_phv8sf2_mask_round = 4221,
  CODE_FOR_avx512fp16_float_extend_phv4df2 = 4222,
  CODE_FOR_avx512fp16_float_extend_phv4df2_mask = 4223,
  CODE_FOR_avx512fp16_float_extend_phv4sf2 = 4224,
  CODE_FOR_avx512fp16_float_extend_phv4sf2_mask = 4225,
  CODE_FOR_avx512fp16_float_extend_phv2df2 = 4230,
  CODE_FOR_avx512fp16_float_extend_phv2df2_mask = 4231,
  CODE_FOR_avx512fp16_vcvtpd2ph_v8df = 4234,
  CODE_FOR_avx512fp16_vcvtpd2ph_v8df_round = 4235,
  CODE_FOR_avx512fp16_vcvtpd2ph_v8df_mask = 4236,
  CODE_FOR_avx512fp16_vcvtpd2ph_v8df_mask_round = 4237,
  CODE_FOR_avx512fp16_vcvtps2ph_v16sf = 4238,
  CODE_FOR_avx512fp16_vcvtps2ph_v16sf_round = 4239,
  CODE_FOR_avx512fp16_vcvtps2ph_v16sf_mask = 4240,
  CODE_FOR_avx512fp16_vcvtps2ph_v16sf_mask_round = 4241,
  CODE_FOR_avx512fp16_vcvtps2ph_v8sf = 4242,
  CODE_FOR_avx512fp16_vcvtps2ph_v8sf_round = 4243,
  CODE_FOR_avx512fp16_vcvtps2ph_v8sf_mask = 4244,
  CODE_FOR_avx512fp16_vcvtps2ph_v8sf_mask_round = 4245,
  CODE_FOR_avx512fp16_vcvtsh2sd = 4255,
  CODE_FOR_avx512fp16_vcvtsh2sd_mask = 4256,
  CODE_FOR_avx512fp16_vcvtsh2sd_round = 4257,
  CODE_FOR_avx512fp16_vcvtsh2sd_mask_round = 4258,
  CODE_FOR_avx512fp16_vcvtsh2ss = 4259,
  CODE_FOR_avx512fp16_vcvtsh2ss_mask = 4260,
  CODE_FOR_avx512fp16_vcvtsh2ss_round = 4261,
  CODE_FOR_avx512fp16_vcvtsh2ss_mask_round = 4262,
  CODE_FOR_avx512fp16_vcvtsh2sd_mem = 4263,
  CODE_FOR_avx512fp16_vcvtsh2sd_mask_mem = 4264,
  CODE_FOR_avx512fp16_vcvtsh2ss_mem = 4265,
  CODE_FOR_avx512fp16_vcvtsh2ss_mask_mem = 4266,
  CODE_FOR_avx512fp16_vcvtsd2sh = 4267,
  CODE_FOR_avx512fp16_vcvtsd2sh_round = 4268,
  CODE_FOR_avx512fp16_vcvtsd2sh_mask = 4269,
  CODE_FOR_avx512fp16_vcvtsd2sh_mask_round = 4270,
  CODE_FOR_avx512fp16_vcvtss2sh = 4271,
  CODE_FOR_avx512fp16_vcvtss2sh_round = 4272,
  CODE_FOR_avx512fp16_vcvtss2sh_mask = 4273,
  CODE_FOR_avx512fp16_vcvtss2sh_mask_round = 4274,
  CODE_FOR_avx512fp16_vcvtss2sh_mem = 4275,
  CODE_FOR_avx512fp16_vcvtss2sh_mask_mem = 4276,
  CODE_FOR_avx512fp16_vcvtsd2sh_mem = 4277,
  CODE_FOR_avx512fp16_vcvtsd2sh_mask_mem = 4278,
  CODE_FOR_sse_cvtpi2ps = 4279,
  CODE_FOR_sse_cvtps2pi = 4280,
  CODE_FOR_unspec_sse_cvttps2pi = 4281,
  CODE_FOR_sse_cvttps2pi = 4282,
  CODE_FOR_sse_cvtsi2ss = 4283,
  CODE_FOR_sse_cvtsi2ss_round = 4284,
   CODE_FOR_sse_cvtsi2ssq = CODE_FOR_nothing,
   CODE_FOR_sse_cvtsi2ssq_round = CODE_FOR_nothing,
  CODE_FOR_sse_cvtss2si = 4285,
  CODE_FOR_sse_cvtss2si_round = 4286,
   CODE_FOR_sse_cvtss2siq = CODE_FOR_nothing,
   CODE_FOR_sse_cvtss2siq_round = CODE_FOR_nothing,
  CODE_FOR_sse_cvtss2si_2 = 4287,
   CODE_FOR_sse_cvtss2siq_2 = CODE_FOR_nothing,
  CODE_FOR_unspec_sse_cvttss2si = 4288,
  CODE_FOR_unspec_sse_cvttss2si_round = 4289,
   CODE_FOR_unspec_sse_cvttss2siq = CODE_FOR_nothing,
   CODE_FOR_unspec_sse_cvttss2siq_round = CODE_FOR_nothing,
  CODE_FOR_sse_cvttss2si = 4290,
  CODE_FOR_sse_cvttss2si_round = 4291,
   CODE_FOR_sse_cvttss2siq = CODE_FOR_nothing,
   CODE_FOR_sse_cvttss2siq_round = CODE_FOR_nothing,
  CODE_FOR_cvtusi2ss32 = 4292,
  CODE_FOR_cvtusi2ss32_round = 4293,
  CODE_FOR_cvtusi2sd32 = 4294,
   CODE_FOR_cvtusi2sd32_round = CODE_FOR_nothing,
   CODE_FOR_cvtusi2ss64 = CODE_FOR_nothing,
   CODE_FOR_cvtusi2ss64_round = CODE_FOR_nothing,
   CODE_FOR_cvtusi2sd64 = CODE_FOR_nothing,
   CODE_FOR_cvtusi2sd64_round = CODE_FOR_nothing,
  CODE_FOR_floatv16siv16sf2 = 4295,
  CODE_FOR_floatv16siv16sf2_round = 4296,
  CODE_FOR_floatv16siv16sf2_mask = 4297,
  CODE_FOR_floatv16siv16sf2_mask_round = 4298,
  CODE_FOR_floatv8siv8sf2 = 4299,
   CODE_FOR_floatv8siv8sf2_round = CODE_FOR_nothing,
  CODE_FOR_floatv8siv8sf2_mask = 4300,
   CODE_FOR_floatv8siv8sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_floatv4siv4sf2 = 4301,
   CODE_FOR_floatv4siv4sf2_round = CODE_FOR_nothing,
  CODE_FOR_floatv4siv4sf2_mask = 4302,
   CODE_FOR_floatv4siv4sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_floatunsv16siv16sf2_mask = 4305,
  CODE_FOR_floatunsv16siv16sf2_mask_round = 4306,
  CODE_FOR_floatunsv8siv8sf2_mask = 4309,
  CODE_FOR_floatunsv8siv8sf2_mask_round = 4310,
  CODE_FOR_floatunsv4siv4sf2_mask = 4313,
  CODE_FOR_floatunsv4siv4sf2_mask_round = 4314,
  CODE_FOR_avx_fix_notruncv8sfv8si = 4315,
  CODE_FOR_avx_fix_notruncv8sfv8si_mask = 4316,
  CODE_FOR_sse2_fix_notruncv4sfv4si = 4317,
  CODE_FOR_sse2_fix_notruncv4sfv4si_mask = 4318,
  CODE_FOR_avx512f_fix_notruncv16sfv16si = 4319,
  CODE_FOR_avx512f_fix_notruncv16sfv16si_round = 4320,
  CODE_FOR_avx512f_fix_notruncv16sfv16si_mask = 4321,
  CODE_FOR_avx512f_fix_notruncv16sfv16si_mask_round = 4322,
  CODE_FOR_avx512f_fixuns_notruncv16sfv16si_mask = 4325,
  CODE_FOR_avx512f_fixuns_notruncv16sfv16si_mask_round = 4326,
  CODE_FOR_avx512vl_fixuns_notruncv8sfv8si_mask = 4329,
  CODE_FOR_avx512vl_fixuns_notruncv8sfv8si_mask_round = 4330,
  CODE_FOR_avx512vl_fixuns_notruncv4sfv4si_mask = 4333,
  CODE_FOR_avx512vl_fixuns_notruncv4sfv4si_mask_round = 4334,
  CODE_FOR_avx512dq_cvtps2qqv8di_mask = 4337,
  CODE_FOR_avx512dq_cvtps2qqv8di_mask_round = 4338,
  CODE_FOR_avx512dq_cvtps2qqv4di_mask = 4340,
   CODE_FOR_avx512dq_cvtps2qqv4di_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_cvtps2qqv2di_mask = 4342,
  CODE_FOR_avx512dq_cvtps2uqqv8di_mask = 4345,
  CODE_FOR_avx512dq_cvtps2uqqv8di_mask_round = 4346,
  CODE_FOR_avx512dq_cvtps2uqqv4di_mask = 4348,
   CODE_FOR_avx512dq_cvtps2uqqv4di_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_cvtps2uqqv2di_mask = 4350,
  CODE_FOR_unspec_fix_truncv16sfv16si2 = 4351,
  CODE_FOR_unspec_fix_truncv16sfv16si2_round = 4352,
  CODE_FOR_unspec_fix_truncv16sfv16si2_mask = 4353,
  CODE_FOR_unspec_fix_truncv16sfv16si2_mask_round = 4354,
  CODE_FOR_unspec_fixuns_truncv16sfv16si2 = 4355,
  CODE_FOR_unspec_fixuns_truncv16sfv16si2_round = 4356,
  CODE_FOR_unspec_fixuns_truncv16sfv16si2_mask = 4357,
  CODE_FOR_unspec_fixuns_truncv16sfv16si2_mask_round = 4358,
  CODE_FOR_fix_truncv16sfv16si2 = 4359,
  CODE_FOR_fix_truncv16sfv16si2_round = 4360,
  CODE_FOR_fix_truncv16sfv16si2_mask = 4361,
  CODE_FOR_fix_truncv16sfv16si2_mask_round = 4362,
  CODE_FOR_fixuns_truncv16sfv16si2 = 4363,
  CODE_FOR_fixuns_truncv16sfv16si2_round = 4364,
  CODE_FOR_fixuns_truncv16sfv16si2_mask = 4365,
  CODE_FOR_fixuns_truncv16sfv16si2_mask_round = 4366,
  CODE_FOR_unspec_fix_truncv8sfv8si2 = 4367,
  CODE_FOR_unspec_fix_truncv8sfv8si2_mask = 4368,
  CODE_FOR_fix_truncv8sfv8si2 = 4369,
  CODE_FOR_fix_truncv8sfv8si2_mask = 4370,
  CODE_FOR_unspec_fix_truncv4sfv4si2 = 4371,
  CODE_FOR_unspec_fix_truncv4sfv4si2_mask = 4372,
  CODE_FOR_fix_truncv4sfv4si2 = 4373,
  CODE_FOR_fix_truncv4sfv4si2_mask = 4374,
  CODE_FOR_sse2_cvtpi2pd = 4375,
   CODE_FOR_floatunsv2siv2df2 = CODE_FOR_nothing,
  CODE_FOR_sse2_cvtpd2pi = 4376,
  CODE_FOR_unspec_sse2_cvttpd2pi = 4377,
  CODE_FOR_sse2_cvttpd2pi = 4378,
   CODE_FOR_unspec_fixuns_truncv2dfv2si2 = CODE_FOR_nothing,
   CODE_FOR_fixuns_truncv2dfv2si2 = CODE_FOR_nothing,
  CODE_FOR_sse2_cvtsi2sd = 4379,
   CODE_FOR_sse2_cvtsi2sdq = CODE_FOR_nothing,
   CODE_FOR_sse2_cvtsi2sdq_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_vcvtss2usi = 4380,
  CODE_FOR_avx512f_vcvtss2usi_round = 4381,
   CODE_FOR_avx512f_vcvtss2usiq = CODE_FOR_nothing,
   CODE_FOR_avx512f_vcvtss2usiq_round = CODE_FOR_nothing,
  CODE_FOR_unspec_avx512f_vcvttss2usi = 4382,
  CODE_FOR_unspec_avx512f_vcvttss2usi_round = 4383,
   CODE_FOR_unspec_avx512f_vcvttss2usiq = CODE_FOR_nothing,
   CODE_FOR_unspec_avx512f_vcvttss2usiq_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_vcvttss2usi = 4384,
  CODE_FOR_avx512f_vcvttss2usi_round = 4385,
   CODE_FOR_avx512f_vcvttss2usiq = CODE_FOR_nothing,
   CODE_FOR_avx512f_vcvttss2usiq_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_vcvtsd2usi = 4386,
  CODE_FOR_avx512f_vcvtsd2usi_round = 4387,
   CODE_FOR_avx512f_vcvtsd2usiq = CODE_FOR_nothing,
   CODE_FOR_avx512f_vcvtsd2usiq_round = CODE_FOR_nothing,
  CODE_FOR_unspec_avx512f_vcvttsd2usi = 4388,
  CODE_FOR_unspec_avx512f_vcvttsd2usi_round = 4389,
   CODE_FOR_unspec_avx512f_vcvttsd2usiq = CODE_FOR_nothing,
   CODE_FOR_unspec_avx512f_vcvttsd2usiq_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_vcvttsd2usi = 4390,
  CODE_FOR_avx512f_vcvttsd2usi_round = 4391,
   CODE_FOR_avx512f_vcvttsd2usiq = CODE_FOR_nothing,
   CODE_FOR_avx512f_vcvttsd2usiq_round = CODE_FOR_nothing,
  CODE_FOR_sse2_cvtsd2si = 4392,
  CODE_FOR_sse2_cvtsd2si_round = 4393,
   CODE_FOR_sse2_cvtsd2siq = CODE_FOR_nothing,
   CODE_FOR_sse2_cvtsd2siq_round = CODE_FOR_nothing,
  CODE_FOR_sse2_cvtsd2si_2 = 4394,
   CODE_FOR_sse2_cvtsd2siq_2 = CODE_FOR_nothing,
  CODE_FOR_unspec_sse2_cvttsd2si = 4395,
  CODE_FOR_unspec_sse2_cvttsd2si_round = 4396,
   CODE_FOR_unspec_sse2_cvttsd2siq = CODE_FOR_nothing,
   CODE_FOR_unspec_sse2_cvttsd2siq_round = CODE_FOR_nothing,
  CODE_FOR_sse2_cvttsd2si = 4397,
  CODE_FOR_sse2_cvttsd2si_round = 4398,
   CODE_FOR_sse2_cvttsd2siq = CODE_FOR_nothing,
   CODE_FOR_sse2_cvttsd2siq_round = CODE_FOR_nothing,
  CODE_FOR_floatv8siv8df2 = 4399,
  CODE_FOR_floatv8siv8df2_mask = 4400,
  CODE_FOR_floatv4siv4df2 = 4401,
  CODE_FOR_floatv4siv4df2_mask = 4402,
  CODE_FOR_floatv8div8df2 = 4403,
  CODE_FOR_floatv8div8df2_round = 4404,
  CODE_FOR_floatv8div8df2_mask = 4405,
  CODE_FOR_floatv8div8df2_mask_round = 4406,
  CODE_FOR_floatunsv8div8df2 = 4407,
  CODE_FOR_floatunsv8div8df2_round = 4408,
  CODE_FOR_floatunsv8div8df2_mask = 4409,
  CODE_FOR_floatunsv8div8df2_mask_round = 4410,
  CODE_FOR_floatv4div4df2 = 4411,
  CODE_FOR_floatv4div4df2_round = 4412,
  CODE_FOR_floatv4div4df2_mask = 4413,
  CODE_FOR_floatv4div4df2_mask_round = 4414,
  CODE_FOR_floatunsv4div4df2 = 4415,
  CODE_FOR_floatunsv4div4df2_round = 4416,
  CODE_FOR_floatunsv4div4df2_mask = 4417,
  CODE_FOR_floatunsv4div4df2_mask_round = 4418,
  CODE_FOR_floatv2div2df2 = 4419,
  CODE_FOR_floatv2div2df2_round = 4420,
  CODE_FOR_floatv2div2df2_mask = 4421,
  CODE_FOR_floatv2div2df2_mask_round = 4422,
  CODE_FOR_floatunsv2div2df2 = 4423,
  CODE_FOR_floatunsv2div2df2_round = 4424,
  CODE_FOR_floatunsv2div2df2_mask = 4425,
  CODE_FOR_floatunsv2div2df2_mask_round = 4426,
  CODE_FOR_floatv8div8sf2 = 4427,
  CODE_FOR_floatv8div8sf2_round = 4428,
  CODE_FOR_floatv8div8sf2_mask = 4429,
  CODE_FOR_floatv8div8sf2_mask_round = 4430,
  CODE_FOR_floatunsv8div8sf2 = 4431,
  CODE_FOR_floatunsv8div8sf2_round = 4432,
  CODE_FOR_floatunsv8div8sf2_mask = 4433,
  CODE_FOR_floatunsv8div8sf2_mask_round = 4434,
  CODE_FOR_floatv4div4sf2 = 4435,
   CODE_FOR_floatv4div4sf2_round = CODE_FOR_nothing,
  CODE_FOR_floatv4div4sf2_mask = 4436,
   CODE_FOR_floatv4div4sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_floatunsv4div4sf2 = 4437,
   CODE_FOR_floatunsv4div4sf2_round = CODE_FOR_nothing,
  CODE_FOR_floatunsv4div4sf2_mask = 4438,
   CODE_FOR_floatunsv4div4sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_floatunsv8siv8df2 = 4445,
  CODE_FOR_floatunsv8siv8df2_mask = 4446,
  CODE_FOR_floatunsv4siv4df2 = 4447,
  CODE_FOR_floatunsv4siv4df2_mask = 4448,
  CODE_FOR_floatunsv2siv2df2_mask = 4450,
  CODE_FOR_avx512f_cvtdq2pd512_2 = 4451,
  CODE_FOR_avx_cvtdq2pd256_2 = 4452,
  CODE_FOR_sse2_cvtdq2pd = 4453,
  CODE_FOR_sse2_cvtdq2pd_mask = 4454,
  CODE_FOR_avx512f_cvtpd2dq512 = 4455,
  CODE_FOR_avx512f_cvtpd2dq512_round = 4456,
  CODE_FOR_avx512f_cvtpd2dq512_mask = 4457,
  CODE_FOR_avx512f_cvtpd2dq512_mask_round = 4458,
  CODE_FOR_avx_cvtpd2dq256 = 4459,
  CODE_FOR_avx_cvtpd2dq256_mask = 4460,
  CODE_FOR_sse2_cvtpd2dq = 4462,
  CODE_FOR_sse2_cvtpd2dq_mask = 4463,
  CODE_FOR_fixuns_notruncv8dfv8si2 = 4465,
  CODE_FOR_fixuns_notruncv8dfv8si2_round = 4466,
  CODE_FOR_fixuns_notruncv8dfv8si2_mask = 4467,
  CODE_FOR_fixuns_notruncv8dfv8si2_mask_round = 4468,
  CODE_FOR_fixuns_notruncv4dfv4si2 = 4469,
  CODE_FOR_fixuns_notruncv4dfv4si2_round = 4470,
  CODE_FOR_fixuns_notruncv4dfv4si2_mask = 4471,
  CODE_FOR_fixuns_notruncv4dfv4si2_mask_round = 4472,
  CODE_FOR_fixuns_notruncv2dfv2si2 = 4473,
  CODE_FOR_fixuns_notruncv2dfv2si2_mask = 4474,
  CODE_FOR_unspec_fix_truncv8dfv8si2 = 4476,
  CODE_FOR_unspec_fix_truncv8dfv8si2_round = 4477,
  CODE_FOR_unspec_fix_truncv8dfv8si2_mask = 4478,
  CODE_FOR_unspec_fix_truncv8dfv8si2_mask_round = 4479,
  CODE_FOR_unspec_fixuns_truncv8dfv8si2 = 4480,
  CODE_FOR_unspec_fixuns_truncv8dfv8si2_round = 4481,
  CODE_FOR_unspec_fixuns_truncv8dfv8si2_mask = 4482,
  CODE_FOR_unspec_fixuns_truncv8dfv8si2_mask_round = 4483,
  CODE_FOR_fix_truncv8dfv8si2 = 4484,
  CODE_FOR_fix_truncv8dfv8si2_round = 4485,
  CODE_FOR_fix_truncv8dfv8si2_mask = 4486,
  CODE_FOR_fix_truncv8dfv8si2_mask_round = 4487,
  CODE_FOR_fixuns_truncv8dfv8si2 = 4488,
  CODE_FOR_fixuns_truncv8dfv8si2_round = 4489,
  CODE_FOR_fixuns_truncv8dfv8si2_mask = 4490,
  CODE_FOR_fixuns_truncv8dfv8si2_mask_round = 4491,
  CODE_FOR_unspec_fixuns_truncv2dfv2si2_mask = 4494,
  CODE_FOR_fixuns_truncv2dfv2si2_mask = 4495,
  CODE_FOR_unspec_fix_truncv4dfv4si2 = 4497,
  CODE_FOR_unspec_fix_truncv4dfv4si2_mask = 4498,
  CODE_FOR_fix_truncv4dfv4si2 = 4499,
  CODE_FOR_fix_truncv4dfv4si2_mask = 4500,
  CODE_FOR_unspec_fixuns_truncv4dfv4si2 = 4501,
  CODE_FOR_unspec_fixuns_truncv4dfv4si2_mask = 4502,
  CODE_FOR_fixuns_truncv4dfv4si2 = 4503,
  CODE_FOR_fixuns_truncv4dfv4si2_mask = 4504,
  CODE_FOR_unspec_fix_truncv8dfv8di2 = 4505,
  CODE_FOR_unspec_fix_truncv8dfv8di2_round = 4506,
  CODE_FOR_unspec_fix_truncv8dfv8di2_mask = 4507,
  CODE_FOR_unspec_fix_truncv8dfv8di2_mask_round = 4508,
  CODE_FOR_unspec_fixuns_truncv8dfv8di2 = 4509,
  CODE_FOR_unspec_fixuns_truncv8dfv8di2_round = 4510,
  CODE_FOR_unspec_fixuns_truncv8dfv8di2_mask = 4511,
  CODE_FOR_unspec_fixuns_truncv8dfv8di2_mask_round = 4512,
  CODE_FOR_unspec_fix_truncv4dfv4di2 = 4513,
   CODE_FOR_unspec_fix_truncv4dfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_unspec_fix_truncv4dfv4di2_mask = 4514,
   CODE_FOR_unspec_fix_truncv4dfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_unspec_fixuns_truncv4dfv4di2 = 4515,
   CODE_FOR_unspec_fixuns_truncv4dfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_unspec_fixuns_truncv4dfv4di2_mask = 4516,
   CODE_FOR_unspec_fixuns_truncv4dfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_unspec_fix_truncv2dfv2di2 = 4517,
   CODE_FOR_unspec_fix_truncv2dfv2di2_round = CODE_FOR_nothing,
  CODE_FOR_unspec_fix_truncv2dfv2di2_mask = 4518,
   CODE_FOR_unspec_fix_truncv2dfv2di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_unspec_fixuns_truncv2dfv2di2 = 4519,
   CODE_FOR_unspec_fixuns_truncv2dfv2di2_round = CODE_FOR_nothing,
  CODE_FOR_unspec_fixuns_truncv2dfv2di2_mask = 4520,
   CODE_FOR_unspec_fixuns_truncv2dfv2di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv8dfv8di2 = 4521,
  CODE_FOR_fix_truncv8dfv8di2_round = 4522,
  CODE_FOR_fix_truncv8dfv8di2_mask = 4523,
  CODE_FOR_fix_truncv8dfv8di2_mask_round = 4524,
  CODE_FOR_fixuns_truncv8dfv8di2 = 4525,
  CODE_FOR_fixuns_truncv8dfv8di2_round = 4526,
  CODE_FOR_fixuns_truncv8dfv8di2_mask = 4527,
  CODE_FOR_fixuns_truncv8dfv8di2_mask_round = 4528,
  CODE_FOR_fix_truncv4dfv4di2 = 4529,
   CODE_FOR_fix_truncv4dfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv4dfv4di2_mask = 4530,
   CODE_FOR_fix_truncv4dfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_truncv4dfv4di2 = 4531,
   CODE_FOR_fixuns_truncv4dfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_truncv4dfv4di2_mask = 4532,
   CODE_FOR_fixuns_truncv4dfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv2dfv2di2 = 4533,
   CODE_FOR_fix_truncv2dfv2di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv2dfv2di2_mask = 4534,
   CODE_FOR_fix_truncv2dfv2di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_truncv2dfv2di2 = 4535,
   CODE_FOR_fixuns_truncv2dfv2di2_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_truncv2dfv2di2_mask = 4536,
   CODE_FOR_fixuns_truncv2dfv2di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fix_notruncv8dfv8di2 = 4537,
  CODE_FOR_fix_notruncv8dfv8di2_round = 4538,
  CODE_FOR_fix_notruncv8dfv8di2_mask = 4539,
  CODE_FOR_fix_notruncv8dfv8di2_mask_round = 4540,
  CODE_FOR_fix_notruncv4dfv4di2 = 4541,
   CODE_FOR_fix_notruncv4dfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_notruncv4dfv4di2_mask = 4542,
   CODE_FOR_fix_notruncv4dfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fix_notruncv2dfv2di2 = 4543,
   CODE_FOR_fix_notruncv2dfv2di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_notruncv2dfv2di2_mask = 4544,
   CODE_FOR_fix_notruncv2dfv2di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_notruncv8dfv8di2 = 4545,
  CODE_FOR_fixuns_notruncv8dfv8di2_round = 4546,
  CODE_FOR_fixuns_notruncv8dfv8di2_mask = 4547,
  CODE_FOR_fixuns_notruncv8dfv8di2_mask_round = 4548,
  CODE_FOR_fixuns_notruncv4dfv4di2 = 4549,
   CODE_FOR_fixuns_notruncv4dfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_notruncv4dfv4di2_mask = 4550,
   CODE_FOR_fixuns_notruncv4dfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_notruncv2dfv2di2 = 4551,
   CODE_FOR_fixuns_notruncv2dfv2di2_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_notruncv2dfv2di2_mask = 4552,
   CODE_FOR_fixuns_notruncv2dfv2di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_unspec_fix_truncv8sfv8di2 = 4553,
  CODE_FOR_unspec_fix_truncv8sfv8di2_round = 4554,
  CODE_FOR_unspec_fix_truncv8sfv8di2_mask = 4555,
  CODE_FOR_unspec_fix_truncv8sfv8di2_mask_round = 4556,
  CODE_FOR_unspec_fixuns_truncv8sfv8di2 = 4557,
  CODE_FOR_unspec_fixuns_truncv8sfv8di2_round = 4558,
  CODE_FOR_unspec_fixuns_truncv8sfv8di2_mask = 4559,
  CODE_FOR_unspec_fixuns_truncv8sfv8di2_mask_round = 4560,
  CODE_FOR_unspec_fix_truncv4sfv4di2 = 4561,
   CODE_FOR_unspec_fix_truncv4sfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_unspec_fix_truncv4sfv4di2_mask = 4562,
   CODE_FOR_unspec_fix_truncv4sfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_unspec_fixuns_truncv4sfv4di2 = 4563,
   CODE_FOR_unspec_fixuns_truncv4sfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_unspec_fixuns_truncv4sfv4di2_mask = 4564,
   CODE_FOR_unspec_fixuns_truncv4sfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv8sfv8di2 = 4565,
  CODE_FOR_fix_truncv8sfv8di2_round = 4566,
  CODE_FOR_fix_truncv8sfv8di2_mask = 4567,
  CODE_FOR_fix_truncv8sfv8di2_mask_round = 4568,
  CODE_FOR_fixuns_truncv8sfv8di2 = 4569,
  CODE_FOR_fixuns_truncv8sfv8di2_round = 4570,
  CODE_FOR_fixuns_truncv8sfv8di2_mask = 4571,
  CODE_FOR_fixuns_truncv8sfv8di2_mask_round = 4572,
  CODE_FOR_fix_truncv4sfv4di2 = 4573,
   CODE_FOR_fix_truncv4sfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv4sfv4di2_mask = 4574,
   CODE_FOR_fix_truncv4sfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_truncv4sfv4di2 = 4575,
   CODE_FOR_fixuns_truncv4sfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_truncv4sfv4di2_mask = 4576,
   CODE_FOR_fixuns_truncv4sfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_unspec_avx512dq_fix_truncv2sfv2di2 = 4577,
  CODE_FOR_unspec_avx512dq_fix_truncv2sfv2di2_mask = 4578,
  CODE_FOR_unspec_avx512dq_fixuns_truncv2sfv2di2 = 4579,
  CODE_FOR_unspec_avx512dq_fixuns_truncv2sfv2di2_mask = 4580,
  CODE_FOR_avx512dq_fix_truncv2sfv2di2 = 4581,
  CODE_FOR_avx512dq_fix_truncv2sfv2di2_mask = 4582,
  CODE_FOR_avx512dq_fixuns_truncv2sfv2di2 = 4583,
  CODE_FOR_avx512dq_fixuns_truncv2sfv2di2_mask = 4584,
  CODE_FOR_unspec_fixuns_truncv8sfv8si2_mask = 4586,
  CODE_FOR_unspec_fixuns_truncv4sfv4si2_mask = 4588,
  CODE_FOR_fixuns_truncv8sfv8si2_mask = 4590,
  CODE_FOR_fixuns_truncv4sfv4si2_mask = 4592,
  CODE_FOR_unspec_sse2_cvttpd2dq = 4593,
  CODE_FOR_unspec_sse2_cvttpd2dq_mask = 4594,
  CODE_FOR_sse2_cvttpd2dq = 4595,
  CODE_FOR_sse2_cvttpd2dq_mask = 4596,
  CODE_FOR_sse2_cvtsd2ss = 4598,
  CODE_FOR_sse2_cvtsd2ss_round = 4599,
  CODE_FOR_sse2_cvtsd2ss_mask = 4600,
  CODE_FOR_sse2_cvtsd2ss_mask_round = 4601,
  CODE_FOR_sse2_cvtss2sd = 4603,
  CODE_FOR_sse2_cvtss2sd_round = 4604,
  CODE_FOR_sse2_cvtss2sd_mask = 4605,
  CODE_FOR_sse2_cvtss2sd_mask_round = 4606,
  CODE_FOR_avx512f_cvtpd2ps512_mask = 4610,
  CODE_FOR_avx512f_cvtpd2ps512_mask_round = 4611,
  CODE_FOR_avx_cvtpd2ps256 = 4612,
  CODE_FOR_avx_cvtpd2ps256_mask = 4613,
   CODE_FOR_truncv2dfv2sf2 = CODE_FOR_nothing,
  CODE_FOR_avx512f_cvtps2pd512 = 4617,
  CODE_FOR_avx512f_cvtps2pd512_round = 4618,
  CODE_FOR_avx512f_cvtps2pd512_mask = 4619,
  CODE_FOR_avx512f_cvtps2pd512_mask_round = 4620,
  CODE_FOR_avx_cvtps2pd256 = 4621,
   CODE_FOR_avx_cvtps2pd256_round = CODE_FOR_nothing,
  CODE_FOR_avx_cvtps2pd256_mask = 4622,
   CODE_FOR_avx_cvtps2pd256_mask_round = CODE_FOR_nothing,
  CODE_FOR_vec_unpacks_lo_v16sf = 4624,
  CODE_FOR_avx512bw_cvtb2maskv64qi = 4625,
  CODE_FOR_avx512vl_cvtb2maskv16qi = 4626,
  CODE_FOR_avx512vl_cvtb2maskv32qi = 4627,
  CODE_FOR_avx512bw_cvtw2maskv32hi = 4628,
  CODE_FOR_avx512vl_cvtw2maskv16hi = 4629,
  CODE_FOR_avx512vl_cvtw2maskv8hi = 4630,
  CODE_FOR_avx512f_cvtd2maskv16si = 4631,
  CODE_FOR_avx512vl_cvtd2maskv8si = 4632,
  CODE_FOR_avx512vl_cvtd2maskv4si = 4633,
  CODE_FOR_avx512f_cvtq2maskv8di = 4634,
  CODE_FOR_avx512vl_cvtq2maskv4di = 4635,
  CODE_FOR_avx512vl_cvtq2maskv2di = 4636,
  CODE_FOR_sse2_cvtps2pd = 4667,
  CODE_FOR_sse2_cvtps2pd_mask = 4668,
  CODE_FOR_sse2_cvtps2pd_1 = 4669,
  CODE_FOR_sse2_cvtps2pd_mask_1 = 4670,
  CODE_FOR_sse_movhlps = 4671,
  CODE_FOR_sse_movlhps = 4672,
  CODE_FOR_sse_movlhps_v8hi = 4673,
  CODE_FOR_sse_movlhps_v8hf = 4674,
  CODE_FOR_sse_movlhps_v8bf = 4675,
  CODE_FOR_avx512f_unpckhps512_mask = 4677,
  CODE_FOR_avx_unpckhps256 = 4678,
  CODE_FOR_avx_unpckhps256_mask = 4679,
  CODE_FOR_vec_interleave_highv4sf = 4680,
  CODE_FOR_vec_interleave_highv4sf_mask = 4681,
  CODE_FOR_avx512f_unpcklps512_mask = 4683,
  CODE_FOR_avx_unpcklps256 = 4684,
  CODE_FOR_avx_unpcklps256_mask = 4685,
  CODE_FOR_unpcklps128_mask = 4686,
  CODE_FOR_vec_interleave_lowv4sf = 4687,
  CODE_FOR_avx_movshdup256 = 4688,
  CODE_FOR_avx_movshdup256_mask = 4689,
  CODE_FOR_sse3_movshdup = 4690,
  CODE_FOR_sse3_movshdup_mask = 4691,
  CODE_FOR_avx512f_movshdup512_mask = 4693,
  CODE_FOR_avx_movsldup256 = 4694,
  CODE_FOR_avx_movsldup256_mask = 4695,
  CODE_FOR_sse3_movsldup = 4696,
  CODE_FOR_sse3_movsldup_mask = 4697,
  CODE_FOR_avx512f_movsldup512_mask = 4699,
  CODE_FOR_avx_shufps256_1 = 4700,
  CODE_FOR_avx_shufps256_1_mask = 4701,
  CODE_FOR_sse_shufps_v4sf_mask = 4702,
  CODE_FOR_sse_shufps_v4si = 4703,
  CODE_FOR_sse_shufps_v4sf = 4704,
  CODE_FOR_sse_storehps = 4705,
  CODE_FOR_sse_loadhps = 4706,
  CODE_FOR_sse_storelps = 4707,
  CODE_FOR_sse_loadlps = 4708,
  CODE_FOR_sse_movss_v4si = 4709,
  CODE_FOR_sse_movss_v4sf = 4710,
  CODE_FOR_avx2_vec_dupv8sf = 4711,
  CODE_FOR_avx2_vec_dupv4sf = 4712,
  CODE_FOR_avx2_vec_dupv8sf_1 = 4713,
  CODE_FOR_avx512f_vec_dupv16sf_1 = 4714,
  CODE_FOR_avx512f_vec_dupv8df_1 = 4715,
  CODE_FOR_vec_setv4si_0 = 4727,
  CODE_FOR_vec_setv4sf_0 = 4728,
  CODE_FOR_vec_setv8hi_0 = 4729,
  CODE_FOR_vec_setv8hf_0 = 4730,
  CODE_FOR_vec_setv8bf_0 = 4731,
  CODE_FOR_vec_setv16hi_0 = 4735,
  CODE_FOR_vec_setv32hi_0 = 4736,
  CODE_FOR_vec_setv16hf_0 = 4737,
  CODE_FOR_vec_setv32hf_0 = 4738,
  CODE_FOR_vec_setv16bf_0 = 4739,
  CODE_FOR_vec_setv32bf_0 = 4740,
  CODE_FOR_vec_setv8si_0 = 4752,
  CODE_FOR_vec_setv8sf_0 = 4753,
  CODE_FOR_vec_setv16si_0 = 4754,
  CODE_FOR_vec_setv16sf_0 = 4755,
  CODE_FOR_sse4_1_insertps_v4si = 4760,
  CODE_FOR_sse4_1_insertps_v4sf = 4761,
  CODE_FOR_vec_setv2df_0 = 4762,
  CODE_FOR_avx512dq_vextractf64x2_1_mask = 4766,
  CODE_FOR_avx512dq_vextracti64x2_1_mask = 4767,
  CODE_FOR_avx512f_vextractf32x4_1_mask = 4770,
  CODE_FOR_avx512f_vextracti32x4_1_mask = 4771,
  CODE_FOR_vec_extract_lo_v8df_mask = 4774,
  CODE_FOR_vec_extract_lo_v8di_mask = 4775,
  CODE_FOR_vec_extract_lo_v8df = 4776,
  CODE_FOR_vec_extract_lo_v8di = 4777,
  CODE_FOR_vec_extract_hi_v8df_mask = 4778,
  CODE_FOR_vec_extract_hi_v8di_mask = 4779,
  CODE_FOR_vec_extract_hi_v8df = 4780,
  CODE_FOR_vec_extract_hi_v8di = 4781,
  CODE_FOR_vec_extract_hi_v16sf_mask = 4782,
  CODE_FOR_vec_extract_hi_v16si_mask = 4783,
  CODE_FOR_vec_extract_hi_v16sf = 4784,
  CODE_FOR_vec_extract_hi_v16si = 4785,
  CODE_FOR_vec_extract_lo_v16sf_mask = 4786,
  CODE_FOR_vec_extract_lo_v16si_mask = 4787,
  CODE_FOR_vec_extract_lo_v16sf = 4788,
  CODE_FOR_vec_extract_lo_v16si = 4789,
  CODE_FOR_vec_extract_lo_v4di_mask = 4790,
  CODE_FOR_vec_extract_lo_v4df_mask = 4791,
  CODE_FOR_vec_extract_lo_v4di = 4792,
  CODE_FOR_vec_extract_lo_v4df = 4793,
  CODE_FOR_vec_extract_hi_v4di_mask = 4794,
  CODE_FOR_vec_extract_hi_v4df_mask = 4795,
  CODE_FOR_vec_extract_hi_v4di = 4796,
  CODE_FOR_vec_extract_hi_v4df = 4797,
  CODE_FOR_vec_extract_lo_v8si_mask = 4798,
  CODE_FOR_vec_extract_lo_v8sf_mask = 4799,
  CODE_FOR_vec_extract_lo_v8si = 4800,
  CODE_FOR_vec_extract_lo_v8sf = 4801,
  CODE_FOR_vec_extract_hi_v8si_mask = 4802,
  CODE_FOR_vec_extract_hi_v8sf_mask = 4803,
  CODE_FOR_vec_extract_hi_v8si = 4804,
  CODE_FOR_vec_extract_hi_v8sf = 4805,
  CODE_FOR_vec_extract_lo_v32hi = 4806,
  CODE_FOR_vec_extract_lo_v32hf = 4807,
  CODE_FOR_vec_extract_lo_v32bf = 4808,
  CODE_FOR_vec_extract_hi_v32hi = 4809,
  CODE_FOR_vec_extract_hi_v32hf = 4810,
  CODE_FOR_vec_extract_hi_v32bf = 4811,
  CODE_FOR_vec_extract_lo_v16hi = 4812,
  CODE_FOR_vec_extract_lo_v16hf = 4813,
  CODE_FOR_vec_extract_lo_v16bf = 4814,
  CODE_FOR_vec_extract_hi_v16hi = 4815,
  CODE_FOR_vec_extract_hi_v16hf = 4816,
  CODE_FOR_vec_extract_hi_v16bf = 4817,
  CODE_FOR_vec_extract_lo_v64qi = 4818,
  CODE_FOR_vec_extract_hi_v64qi = 4819,
  CODE_FOR_vec_extract_lo_v32qi = 4820,
  CODE_FOR_vec_extract_hi_v32qi = 4821,
  CODE_FOR_avx512f_unpckhpd512_mask = 4831,
  CODE_FOR_avx_unpckhpd256 = 4832,
  CODE_FOR_avx_unpckhpd256_mask = 4833,
  CODE_FOR_avx512vl_unpckhpd128_mask = 4834,
  CODE_FOR_avx512f_movddup512 = 4836,
  CODE_FOR_avx512f_movddup512_mask = 4837,
  CODE_FOR_avx512f_unpcklpd512 = 4838,
  CODE_FOR_avx512f_unpcklpd512_mask = 4839,
  CODE_FOR_avx_movddup256 = 4840,
  CODE_FOR_avx_movddup256_mask = 4841,
  CODE_FOR_avx_unpcklpd256 = 4842,
  CODE_FOR_avx_unpcklpd256_mask = 4843,
  CODE_FOR_avx512vl_unpcklpd128_mask = 4844,
  CODE_FOR_avx512f_vmscalefv8hf = 4846,
  CODE_FOR_avx512f_vmscalefv8hf_round = 4847,
  CODE_FOR_avx512f_vmscalefv8hf_mask = 4848,
  CODE_FOR_avx512f_vmscalefv8hf_mask_round = 4849,
  CODE_FOR_avx512f_vmscalefv4sf = 4850,
  CODE_FOR_avx512f_vmscalefv4sf_round = 4851,
  CODE_FOR_avx512f_vmscalefv4sf_mask = 4852,
  CODE_FOR_avx512f_vmscalefv4sf_mask_round = 4853,
  CODE_FOR_avx512f_vmscalefv2df = 4854,
  CODE_FOR_avx512f_vmscalefv2df_round = 4855,
  CODE_FOR_avx512f_vmscalefv2df_mask = 4856,
  CODE_FOR_avx512f_vmscalefv2df_mask_round = 4857,
  CODE_FOR_avx512bw_scalefv32hf = 4858,
  CODE_FOR_avx512bw_scalefv32hf_round = 4859,
  CODE_FOR_avx512bw_scalefv32hf_mask = 4860,
  CODE_FOR_avx512bw_scalefv32hf_mask_round = 4861,
  CODE_FOR_avx512vl_scalefv16hf = 4862,
  CODE_FOR_avx512vl_scalefv16hf_round = 4863,
  CODE_FOR_avx512vl_scalefv16hf_mask = 4864,
  CODE_FOR_avx512vl_scalefv16hf_mask_round = 4865,
  CODE_FOR_avx512fp16_scalefv8hf = 4866,
  CODE_FOR_avx512fp16_scalefv8hf_round = 4867,
  CODE_FOR_avx512fp16_scalefv8hf_mask = 4868,
  CODE_FOR_avx512fp16_scalefv8hf_mask_round = 4869,
  CODE_FOR_avx512f_scalefv16sf = 4870,
  CODE_FOR_avx512f_scalefv16sf_round = 4871,
  CODE_FOR_avx512f_scalefv16sf_mask = 4872,
  CODE_FOR_avx512f_scalefv16sf_mask_round = 4873,
  CODE_FOR_avx512vl_scalefv8sf = 4874,
  CODE_FOR_avx512vl_scalefv8sf_round = 4875,
  CODE_FOR_avx512vl_scalefv8sf_mask = 4876,
  CODE_FOR_avx512vl_scalefv8sf_mask_round = 4877,
  CODE_FOR_avx512vl_scalefv4sf = 4878,
  CODE_FOR_avx512vl_scalefv4sf_round = 4879,
  CODE_FOR_avx512vl_scalefv4sf_mask = 4880,
  CODE_FOR_avx512vl_scalefv4sf_mask_round = 4881,
  CODE_FOR_avx512f_scalefv8df = 4882,
  CODE_FOR_avx512f_scalefv8df_round = 4883,
  CODE_FOR_avx512f_scalefv8df_mask = 4884,
  CODE_FOR_avx512f_scalefv8df_mask_round = 4885,
  CODE_FOR_avx512vl_scalefv4df = 4886,
  CODE_FOR_avx512vl_scalefv4df_round = 4887,
  CODE_FOR_avx512vl_scalefv4df_mask = 4888,
  CODE_FOR_avx512vl_scalefv4df_mask_round = 4889,
  CODE_FOR_avx512vl_scalefv2df = 4890,
  CODE_FOR_avx512vl_scalefv2df_round = 4891,
  CODE_FOR_avx512vl_scalefv2df_mask = 4892,
  CODE_FOR_avx512vl_scalefv2df_mask_round = 4893,
  CODE_FOR_avx512f_vternlogv16si = 4894,
  CODE_FOR_avx512f_vternlogv16si_maskz_1 = 4895,
  CODE_FOR_avx512vl_vternlogv8si = 4896,
  CODE_FOR_avx512vl_vternlogv8si_maskz_1 = 4897,
  CODE_FOR_avx512vl_vternlogv4si = 4898,
  CODE_FOR_avx512vl_vternlogv4si_maskz_1 = 4899,
  CODE_FOR_avx512f_vternlogv8di = 4900,
  CODE_FOR_avx512f_vternlogv8di_maskz_1 = 4901,
  CODE_FOR_avx512vl_vternlogv4di = 4902,
  CODE_FOR_avx512vl_vternlogv4di_maskz_1 = 4903,
  CODE_FOR_avx512vl_vternlogv2di = 4904,
  CODE_FOR_avx512vl_vternlogv2di_maskz_1 = 4905,
  CODE_FOR_avx512bw_getexpv32hf = 6472,
  CODE_FOR_avx512bw_getexpv32hf_round = 6473,
  CODE_FOR_avx512bw_getexpv32hf_mask = 6474,
  CODE_FOR_avx512bw_getexpv32hf_mask_round = 6475,
  CODE_FOR_avx512vl_getexpv16hf = 6476,
  CODE_FOR_avx512vl_getexpv16hf_round = 6477,
  CODE_FOR_avx512vl_getexpv16hf_mask = 6478,
  CODE_FOR_avx512vl_getexpv16hf_mask_round = 6479,
  CODE_FOR_avx512fp16_getexpv8hf = 6480,
  CODE_FOR_avx512fp16_getexpv8hf_round = 6481,
  CODE_FOR_avx512fp16_getexpv8hf_mask = 6482,
  CODE_FOR_avx512fp16_getexpv8hf_mask_round = 6483,
  CODE_FOR_avx512f_getexpv16sf = 6484,
  CODE_FOR_avx512f_getexpv16sf_round = 6485,
  CODE_FOR_avx512f_getexpv16sf_mask = 6486,
  CODE_FOR_avx512f_getexpv16sf_mask_round = 6487,
  CODE_FOR_avx512vl_getexpv8sf = 6488,
  CODE_FOR_avx512vl_getexpv8sf_round = 6489,
  CODE_FOR_avx512vl_getexpv8sf_mask = 6490,
  CODE_FOR_avx512vl_getexpv8sf_mask_round = 6491,
  CODE_FOR_avx512vl_getexpv4sf = 6492,
  CODE_FOR_avx512vl_getexpv4sf_round = 6493,
  CODE_FOR_avx512vl_getexpv4sf_mask = 6494,
  CODE_FOR_avx512vl_getexpv4sf_mask_round = 6495,
  CODE_FOR_avx512f_getexpv8df = 6496,
  CODE_FOR_avx512f_getexpv8df_round = 6497,
  CODE_FOR_avx512f_getexpv8df_mask = 6498,
  CODE_FOR_avx512f_getexpv8df_mask_round = 6499,
  CODE_FOR_avx512vl_getexpv4df = 6500,
  CODE_FOR_avx512vl_getexpv4df_round = 6501,
  CODE_FOR_avx512vl_getexpv4df_mask = 6502,
  CODE_FOR_avx512vl_getexpv4df_mask_round = 6503,
  CODE_FOR_avx512vl_getexpv2df = 6504,
  CODE_FOR_avx512vl_getexpv2df_round = 6505,
  CODE_FOR_avx512vl_getexpv2df_mask = 6506,
  CODE_FOR_avx512vl_getexpv2df_mask_round = 6507,
  CODE_FOR_avx512f_sgetexpv8hf = 6508,
  CODE_FOR_avx512f_sgetexpv8hf_mask = 6509,
  CODE_FOR_avx512f_sgetexpv8hf_round = 6510,
  CODE_FOR_avx512f_sgetexpv8hf_mask_round = 6511,
  CODE_FOR_avx512f_sgetexpv4sf = 6512,
  CODE_FOR_avx512f_sgetexpv4sf_mask = 6513,
  CODE_FOR_avx512f_sgetexpv4sf_round = 6514,
  CODE_FOR_avx512f_sgetexpv4sf_mask_round = 6515,
  CODE_FOR_avx512f_sgetexpv2df = 6516,
  CODE_FOR_avx512f_sgetexpv2df_mask = 6517,
  CODE_FOR_avx512f_sgetexpv2df_round = 6518,
  CODE_FOR_avx512f_sgetexpv2df_mask_round = 6519,
  CODE_FOR_avx512f_alignv16si_mask = 6521,
  CODE_FOR_avx512vl_alignv8si_mask = 6523,
  CODE_FOR_avx512vl_alignv4si_mask = 6525,
  CODE_FOR_avx512f_alignv8di_mask = 6527,
  CODE_FOR_avx512vl_alignv4di_mask = 6529,
  CODE_FOR_avx512vl_alignv2di_mask = 6531,
  CODE_FOR_avx512f_fixupimmv16sf = 6540,
  CODE_FOR_avx512f_fixupimmv16sf_round = 6541,
  CODE_FOR_avx512f_fixupimmv16sf_maskz_1 = 6542,
  CODE_FOR_avx512f_fixupimmv16sf_maskz_1_round = 6543,
  CODE_FOR_avx512vl_fixupimmv8sf = 6544,
  CODE_FOR_avx512vl_fixupimmv8sf_round = 6545,
  CODE_FOR_avx512vl_fixupimmv8sf_maskz_1 = 6546,
  CODE_FOR_avx512vl_fixupimmv8sf_maskz_1_round = 6547,
  CODE_FOR_avx512vl_fixupimmv4sf = 6548,
  CODE_FOR_avx512vl_fixupimmv4sf_round = 6549,
  CODE_FOR_avx512vl_fixupimmv4sf_maskz_1 = 6550,
  CODE_FOR_avx512vl_fixupimmv4sf_maskz_1_round = 6551,
  CODE_FOR_avx512f_fixupimmv8df = 6552,
  CODE_FOR_avx512f_fixupimmv8df_round = 6553,
  CODE_FOR_avx512f_fixupimmv8df_maskz_1 = 6554,
  CODE_FOR_avx512f_fixupimmv8df_maskz_1_round = 6555,
  CODE_FOR_avx512vl_fixupimmv4df = 6556,
  CODE_FOR_avx512vl_fixupimmv4df_round = 6557,
  CODE_FOR_avx512vl_fixupimmv4df_maskz_1 = 6558,
  CODE_FOR_avx512vl_fixupimmv4df_maskz_1_round = 6559,
  CODE_FOR_avx512vl_fixupimmv2df = 6560,
  CODE_FOR_avx512vl_fixupimmv2df_round = 6561,
  CODE_FOR_avx512vl_fixupimmv2df_maskz_1 = 6562,
  CODE_FOR_avx512vl_fixupimmv2df_maskz_1_round = 6563,
  CODE_FOR_avx512f_fixupimmv16sf_mask = 6564,
  CODE_FOR_avx512f_fixupimmv16sf_mask_round = 6565,
  CODE_FOR_avx512vl_fixupimmv8sf_mask = 6566,
  CODE_FOR_avx512vl_fixupimmv8sf_mask_round = 6567,
  CODE_FOR_avx512vl_fixupimmv4sf_mask = 6568,
  CODE_FOR_avx512vl_fixupimmv4sf_mask_round = 6569,
  CODE_FOR_avx512f_fixupimmv8df_mask = 6570,
  CODE_FOR_avx512f_fixupimmv8df_mask_round = 6571,
  CODE_FOR_avx512vl_fixupimmv4df_mask = 6572,
  CODE_FOR_avx512vl_fixupimmv4df_mask_round = 6573,
  CODE_FOR_avx512vl_fixupimmv2df_mask = 6574,
  CODE_FOR_avx512vl_fixupimmv2df_mask_round = 6575,
  CODE_FOR_avx512f_sfixupimmv4sf = 6576,
  CODE_FOR_avx512f_sfixupimmv4sf_round = 6577,
  CODE_FOR_avx512f_sfixupimmv4sf_maskz_1 = 6578,
  CODE_FOR_avx512f_sfixupimmv4sf_maskz_1_round = 6579,
  CODE_FOR_avx512f_sfixupimmv2df = 6580,
  CODE_FOR_avx512f_sfixupimmv2df_round = 6581,
  CODE_FOR_avx512f_sfixupimmv2df_maskz_1 = 6582,
  CODE_FOR_avx512f_sfixupimmv2df_maskz_1_round = 6583,
  CODE_FOR_avx512f_sfixupimmv4sf_mask = 6584,
  CODE_FOR_avx512f_sfixupimmv4sf_mask_round = 6585,
  CODE_FOR_avx512f_sfixupimmv2df_mask = 6586,
  CODE_FOR_avx512f_sfixupimmv2df_mask_round = 6587,
  CODE_FOR_avx512bw_rndscalev32hf = 6588,
  CODE_FOR_avx512bw_rndscalev32hf_round = 6589,
  CODE_FOR_avx512bw_rndscalev32hf_mask = 6590,
  CODE_FOR_avx512bw_rndscalev32hf_mask_round = 6591,
  CODE_FOR_avx512vl_rndscalev16hf = 6592,
  CODE_FOR_avx512vl_rndscalev16hf_round = 6593,
  CODE_FOR_avx512vl_rndscalev16hf_mask = 6594,
  CODE_FOR_avx512vl_rndscalev16hf_mask_round = 6595,
  CODE_FOR_avx512fp16_rndscalev8hf = 6596,
  CODE_FOR_avx512fp16_rndscalev8hf_round = 6597,
  CODE_FOR_avx512fp16_rndscalev8hf_mask = 6598,
  CODE_FOR_avx512fp16_rndscalev8hf_mask_round = 6599,
  CODE_FOR_avx512f_rndscalev16sf = 6600,
  CODE_FOR_avx512f_rndscalev16sf_round = 6601,
  CODE_FOR_avx512f_rndscalev16sf_mask = 6602,
  CODE_FOR_avx512f_rndscalev16sf_mask_round = 6603,
  CODE_FOR_avx512vl_rndscalev8sf = 6604,
  CODE_FOR_avx512vl_rndscalev8sf_round = 6605,
  CODE_FOR_avx512vl_rndscalev8sf_mask = 6606,
  CODE_FOR_avx512vl_rndscalev8sf_mask_round = 6607,
  CODE_FOR_avx512vl_rndscalev4sf = 6608,
  CODE_FOR_avx512vl_rndscalev4sf_round = 6609,
  CODE_FOR_avx512vl_rndscalev4sf_mask = 6610,
  CODE_FOR_avx512vl_rndscalev4sf_mask_round = 6611,
  CODE_FOR_avx512f_rndscalev8df = 6612,
  CODE_FOR_avx512f_rndscalev8df_round = 6613,
  CODE_FOR_avx512f_rndscalev8df_mask = 6614,
  CODE_FOR_avx512f_rndscalev8df_mask_round = 6615,
  CODE_FOR_avx512vl_rndscalev4df = 6616,
  CODE_FOR_avx512vl_rndscalev4df_round = 6617,
  CODE_FOR_avx512vl_rndscalev4df_mask = 6618,
  CODE_FOR_avx512vl_rndscalev4df_mask_round = 6619,
  CODE_FOR_avx512vl_rndscalev2df = 6620,
  CODE_FOR_avx512vl_rndscalev2df_round = 6621,
  CODE_FOR_avx512vl_rndscalev2df_mask = 6622,
  CODE_FOR_avx512vl_rndscalev2df_mask_round = 6623,
  CODE_FOR_avx512f_rndscalev8hf = 6624,
  CODE_FOR_avx512f_rndscalev8hf_mask = 6625,
  CODE_FOR_avx512f_rndscalev8hf_round = 6626,
  CODE_FOR_avx512f_rndscalev8hf_mask_round = 6627,
  CODE_FOR_avx512f_rndscalev4sf = 6628,
  CODE_FOR_avx512f_rndscalev4sf_mask = 6629,
  CODE_FOR_avx512f_rndscalev4sf_round = 6630,
  CODE_FOR_avx512f_rndscalev4sf_mask_round = 6631,
  CODE_FOR_avx512f_rndscalev2df = 6632,
  CODE_FOR_avx512f_rndscalev2df_mask = 6633,
  CODE_FOR_avx512f_rndscalev2df_round = 6634,
  CODE_FOR_avx512f_rndscalev2df_mask_round = 6635,
  CODE_FOR_avx512f_shufps512_1 = 6642,
  CODE_FOR_avx512f_shufps512_1_mask = 6643,
  CODE_FOR_avx512f_shufpd512_1 = 6644,
  CODE_FOR_avx512f_shufpd512_1_mask = 6645,
  CODE_FOR_avx_shufpd256_1 = 6646,
  CODE_FOR_avx_shufpd256_1_mask = 6647,
  CODE_FOR_sse2_shufpd_v2df_mask = 6648,
  CODE_FOR_avx2_interleave_highv4di = 6649,
  CODE_FOR_avx2_interleave_highv4di_mask = 6650,
  CODE_FOR_avx512f_interleave_highv8di_mask = 6652,
  CODE_FOR_vec_interleave_highv2di = 6653,
  CODE_FOR_vec_interleave_highv2di_mask = 6654,
  CODE_FOR_avx2_interleave_lowv4di = 6655,
  CODE_FOR_avx2_interleave_lowv4di_mask = 6656,
  CODE_FOR_avx512f_interleave_lowv8di_mask = 6658,
  CODE_FOR_vec_interleave_lowv2di = 6659,
  CODE_FOR_vec_interleave_lowv2di_mask = 6660,
  CODE_FOR_sse2_shufpd_v2di = 6661,
  CODE_FOR_sse2_shufpd_v2df = 6662,
  CODE_FOR_sse2_storehpd = 6663,
  CODE_FOR_sse2_storelpd = 6665,
  CODE_FOR_sse2_loadhpd = 6667,
  CODE_FOR_sse2_loadlpd = 6668,
  CODE_FOR_sse2_movsd_v2di = 6669,
  CODE_FOR_sse2_movsd_v2df = 6670,
  CODE_FOR_vec_dupv2df = 6671,
  CODE_FOR_vec_dupv2df_mask = 6672,
  CODE_FOR_vec_concatv2df = 6673,
  CODE_FOR_vec_setv8df_0 = 6674,
  CODE_FOR_vec_setv4df_0 = 6675,
  CODE_FOR_avx512f_ss_truncatev16siv16qi2_mask = 6693,
  CODE_FOR_avx512f_truncatev16siv16qi2_mask = 6694,
  CODE_FOR_avx512f_us_truncatev16siv16qi2_mask = 6695,
  CODE_FOR_avx512f_ss_truncatev16siv16hi2_mask = 6696,
  CODE_FOR_avx512f_truncatev16siv16hi2_mask = 6697,
  CODE_FOR_avx512f_us_truncatev16siv16hi2_mask = 6698,
  CODE_FOR_avx512f_ss_truncatev8div8si2_mask = 6699,
  CODE_FOR_avx512f_truncatev8div8si2_mask = 6700,
  CODE_FOR_avx512f_us_truncatev8div8si2_mask = 6701,
  CODE_FOR_avx512f_ss_truncatev8div8hi2_mask = 6702,
  CODE_FOR_avx512f_truncatev8div8hi2_mask = 6703,
  CODE_FOR_avx512f_us_truncatev8div8hi2_mask = 6704,
  CODE_FOR_avx512bw_ss_truncatev32hiv32qi2 = 6705,
  CODE_FOR_avx512bw_truncatev32hiv32qi2 = 6706,
  CODE_FOR_avx512bw_us_truncatev32hiv32qi2 = 6707,
  CODE_FOR_avx512bw_ss_truncatev32hiv32qi2_mask = 6709,
  CODE_FOR_avx512bw_truncatev32hiv32qi2_mask = 6710,
  CODE_FOR_avx512bw_us_truncatev32hiv32qi2_mask = 6711,
  CODE_FOR_avx512vl_ss_truncatev4div4si2_mask = 6723,
  CODE_FOR_avx512vl_truncatev4div4si2_mask = 6724,
  CODE_FOR_avx512vl_us_truncatev4div4si2_mask = 6725,
  CODE_FOR_avx512vl_ss_truncatev8siv8hi2_mask = 6726,
  CODE_FOR_avx512vl_truncatev8siv8hi2_mask = 6727,
  CODE_FOR_avx512vl_us_truncatev8siv8hi2_mask = 6728,
  CODE_FOR_avx512vl_ss_truncatev16hiv16qi2_mask = 6729,
  CODE_FOR_avx512vl_truncatev16hiv16qi2_mask = 6730,
  CODE_FOR_avx512vl_us_truncatev16hiv16qi2_mask = 6731,
  CODE_FOR_avx512vl_ss_truncatev4div4qi2 = 6732,
  CODE_FOR_avx512vl_truncatev4div4qi2 = 6733,
  CODE_FOR_avx512vl_us_truncatev4div4qi2 = 6734,
  CODE_FOR_avx512vl_ss_truncatev2div2qi2 = 6735,
  CODE_FOR_avx512vl_truncatev2div2qi2 = 6736,
  CODE_FOR_avx512vl_us_truncatev2div2qi2 = 6737,
  CODE_FOR_avx512vl_ss_truncatev8siv8qi2 = 6738,
  CODE_FOR_avx512vl_truncatev8siv8qi2 = 6739,
  CODE_FOR_avx512vl_us_truncatev8siv8qi2 = 6740,
  CODE_FOR_avx512vl_ss_truncatev4siv4qi2 = 6741,
  CODE_FOR_avx512vl_truncatev4siv4qi2 = 6742,
  CODE_FOR_avx512vl_us_truncatev4siv4qi2 = 6743,
  CODE_FOR_avx512vl_ss_truncatev8hiv8qi2 = 6744,
  CODE_FOR_avx512vl_truncatev8hiv8qi2 = 6745,
  CODE_FOR_avx512vl_us_truncatev8hiv8qi2 = 6746,
  CODE_FOR_avx512vl_ss_truncatev2div2qi2_mask = 6754,
  CODE_FOR_avx512vl_truncatev2div2qi2_mask = 6755,
  CODE_FOR_avx512vl_us_truncatev2div2qi2_mask = 6756,
  CODE_FOR_avx512vl_ss_truncatev2div2qi2_mask_store_1 = 6760,
  CODE_FOR_avx512vl_truncatev2div2qi2_mask_store_1 = 6761,
  CODE_FOR_avx512vl_us_truncatev2div2qi2_mask_store_1 = 6762,
  CODE_FOR_avx512vl_ss_truncatev4siv4qi2_mask = 6775,
  CODE_FOR_avx512vl_truncatev4siv4qi2_mask = 6776,
  CODE_FOR_avx512vl_us_truncatev4siv4qi2_mask = 6777,
  CODE_FOR_avx512vl_ss_truncatev4div4qi2_mask = 6778,
  CODE_FOR_avx512vl_truncatev4div4qi2_mask = 6779,
  CODE_FOR_avx512vl_us_truncatev4div4qi2_mask = 6780,
  CODE_FOR_avx512vl_ss_truncatev4siv4qi2_mask_store_1 = 6787,
  CODE_FOR_avx512vl_truncatev4siv4qi2_mask_store_1 = 6788,
  CODE_FOR_avx512vl_us_truncatev4siv4qi2_mask_store_1 = 6789,
  CODE_FOR_avx512vl_ss_truncatev4div4qi2_mask_store_1 = 6790,
  CODE_FOR_avx512vl_truncatev4div4qi2_mask_store_1 = 6791,
  CODE_FOR_avx512vl_us_truncatev4div4qi2_mask_store_1 = 6792,
  CODE_FOR_avx512vl_ss_truncatev8hiv8qi2_mask = 6805,
  CODE_FOR_avx512vl_truncatev8hiv8qi2_mask = 6806,
  CODE_FOR_avx512vl_us_truncatev8hiv8qi2_mask = 6807,
  CODE_FOR_avx512vl_ss_truncatev8siv8qi2_mask = 6808,
  CODE_FOR_avx512vl_truncatev8siv8qi2_mask = 6809,
  CODE_FOR_avx512vl_us_truncatev8siv8qi2_mask = 6810,
  CODE_FOR_avx512vl_ss_truncatev8hiv8qi2_mask_store_1 = 6817,
  CODE_FOR_avx512vl_truncatev8hiv8qi2_mask_store_1 = 6818,
  CODE_FOR_avx512vl_us_truncatev8hiv8qi2_mask_store_1 = 6819,
  CODE_FOR_avx512vl_ss_truncatev8siv8qi2_mask_store_1 = 6820,
  CODE_FOR_avx512vl_truncatev8siv8qi2_mask_store_1 = 6821,
  CODE_FOR_avx512vl_us_truncatev8siv8qi2_mask_store_1 = 6822,
  CODE_FOR_avx512vl_ss_truncatev4div4hi2 = 6823,
  CODE_FOR_avx512vl_truncatev4div4hi2 = 6824,
  CODE_FOR_avx512vl_us_truncatev4div4hi2 = 6825,
  CODE_FOR_avx512vl_ss_truncatev2div2hi2 = 6826,
  CODE_FOR_avx512vl_truncatev2div2hi2 = 6827,
  CODE_FOR_avx512vl_us_truncatev2div2hi2 = 6828,
  CODE_FOR_avx512vl_ss_truncatev4siv4hi2 = 6829,
  CODE_FOR_avx512vl_truncatev4siv4hi2 = 6830,
  CODE_FOR_avx512vl_us_truncatev4siv4hi2 = 6831,
  CODE_FOR_avx512vl_ss_truncatev4siv4hi2_mask = 6845,
  CODE_FOR_avx512vl_truncatev4siv4hi2_mask = 6846,
  CODE_FOR_avx512vl_us_truncatev4siv4hi2_mask = 6847,
  CODE_FOR_avx512vl_ss_truncatev4div4hi2_mask = 6848,
  CODE_FOR_avx512vl_truncatev4div4hi2_mask = 6849,
  CODE_FOR_avx512vl_us_truncatev4div4hi2_mask = 6850,
  CODE_FOR_avx512vl_ss_truncatev4siv4hi2_mask_store_1 = 6857,
  CODE_FOR_avx512vl_truncatev4siv4hi2_mask_store_1 = 6858,
  CODE_FOR_avx512vl_us_truncatev4siv4hi2_mask_store_1 = 6859,
  CODE_FOR_avx512vl_ss_truncatev4div4hi2_mask_store_1 = 6860,
  CODE_FOR_avx512vl_truncatev4div4hi2_mask_store_1 = 6861,
  CODE_FOR_avx512vl_us_truncatev4div4hi2_mask_store_1 = 6862,
  CODE_FOR_avx512vl_ss_truncatev2div2hi2_mask = 6869,
  CODE_FOR_avx512vl_truncatev2div2hi2_mask = 6870,
  CODE_FOR_avx512vl_us_truncatev2div2hi2_mask = 6871,
  CODE_FOR_avx512vl_ss_truncatev2div2hi2_mask_store_1 = 6875,
  CODE_FOR_avx512vl_truncatev2div2hi2_mask_store_1 = 6876,
  CODE_FOR_avx512vl_us_truncatev2div2hi2_mask_store_1 = 6877,
  CODE_FOR_avx512vl_ss_truncatev2div2si2 = 6878,
  CODE_FOR_avx512vl_truncatev2div2si2 = 6879,
  CODE_FOR_avx512vl_us_truncatev2div2si2 = 6880,
  CODE_FOR_avx512vl_ss_truncatev2div2si2_mask = 6888,
  CODE_FOR_avx512vl_truncatev2div2si2_mask = 6889,
  CODE_FOR_avx512vl_us_truncatev2div2si2_mask = 6890,
  CODE_FOR_avx512vl_ss_truncatev2div2si2_mask_store_1 = 6894,
  CODE_FOR_avx512vl_truncatev2div2si2_mask_store_1 = 6895,
  CODE_FOR_avx512vl_us_truncatev2div2si2_mask_store_1 = 6896,
  CODE_FOR_avx512f_ss_truncatev8div16qi2 = 6897,
  CODE_FOR_avx512f_truncatev8div16qi2 = 6898,
  CODE_FOR_avx512f_us_truncatev8div16qi2 = 6899,
  CODE_FOR_avx512f_ss_truncatev8div16qi2_mask = 6906,
  CODE_FOR_avx512f_truncatev8div16qi2_mask = 6907,
  CODE_FOR_avx512f_us_truncatev8div16qi2_mask = 6908,
  CODE_FOR_avx512f_ss_truncatev8div16qi2_mask_store_1 = 6912,
  CODE_FOR_avx512f_truncatev8div16qi2_mask_store_1 = 6913,
  CODE_FOR_avx512f_us_truncatev8div16qi2_mask_store_1 = 6914,
  CODE_FOR_avx512bw_pmaddwd512v32hi = 7041,
  CODE_FOR_avx512bw_pmaddwd512v32hi_mask = 7042,
  CODE_FOR_avx512bw_pmaddwd512v16hi = 7043,
  CODE_FOR_avx512bw_pmaddwd512v16hi_mask = 7044,
  CODE_FOR_avx512bw_pmaddwd512v8hi = 7045,
  CODE_FOR_avx512bw_pmaddwd512v8hi_mask = 7046,
  CODE_FOR_ashrv16hi3_mask = 7062,
  CODE_FOR_ashrv8hi3_mask = 7064,
  CODE_FOR_ashrv8si3_mask = 7066,
  CODE_FOR_ashrv4si3_mask = 7068,
  CODE_FOR_ashrv2di3_mask = 7070,
  CODE_FOR_ashrv16hi3 = 7071,
  CODE_FOR_ashrv8hi3 = 7072,
  CODE_FOR_ashrv8si3 = 7073,
  CODE_FOR_ashrv4si3 = 7074,
  CODE_FOR_ashrv32hi3_mask = 7080,
  CODE_FOR_ashrv4di3_mask = 7082,
  CODE_FOR_ashrv16si3_mask = 7084,
  CODE_FOR_ashrv8di3_mask = 7086,
  CODE_FOR_ashlv16hi3_mask = 7097,
  CODE_FOR_lshrv16hi3_mask = 7099,
  CODE_FOR_ashlv8hi3_mask = 7101,
  CODE_FOR_lshrv8hi3_mask = 7103,
  CODE_FOR_ashlv8si3_mask = 7105,
  CODE_FOR_lshrv8si3_mask = 7107,
  CODE_FOR_ashlv4si3_mask = 7109,
  CODE_FOR_lshrv4si3_mask = 7111,
  CODE_FOR_ashlv4di3_mask = 7113,
  CODE_FOR_lshrv4di3_mask = 7115,
  CODE_FOR_ashlv2di3_mask = 7117,
  CODE_FOR_lshrv2di3_mask = 7119,
  CODE_FOR_ashlv16hi3 = 7120,
  CODE_FOR_lshrv16hi3 = 7121,
  CODE_FOR_ashlv8hi3 = 7122,
  CODE_FOR_lshrv8hi3 = 7123,
  CODE_FOR_ashlv8si3 = 7124,
  CODE_FOR_lshrv8si3 = 7125,
  CODE_FOR_ashlv4si3 = 7126,
  CODE_FOR_lshrv4si3 = 7127,
  CODE_FOR_ashlv4di3 = 7128,
  CODE_FOR_lshrv4di3 = 7129,
  CODE_FOR_ashlv2di3 = 7130,
  CODE_FOR_lshrv2di3 = 7131,
  CODE_FOR_ashlv32hi3 = 7136,
  CODE_FOR_ashlv32hi3_mask = 7137,
  CODE_FOR_lshrv32hi3 = 7138,
  CODE_FOR_lshrv32hi3_mask = 7139,
  CODE_FOR_ashlv16si3 = 7140,
  CODE_FOR_ashlv16si3_mask = 7141,
  CODE_FOR_lshrv16si3 = 7142,
  CODE_FOR_lshrv16si3_mask = 7143,
  CODE_FOR_ashlv8di3 = 7144,
  CODE_FOR_ashlv8di3_mask = 7145,
  CODE_FOR_lshrv8di3 = 7146,
  CODE_FOR_lshrv8di3_mask = 7147,
  CODE_FOR_avx512bw_ashlv4ti3 = 7148,
  CODE_FOR_avx512bw_lshrv4ti3 = 7149,
  CODE_FOR_avx512bw_ashlv2ti3 = 7150,
  CODE_FOR_avx512bw_lshrv2ti3 = 7151,
  CODE_FOR_avx512bw_ashlv1ti3 = 7152,
  CODE_FOR_avx512bw_lshrv1ti3 = 7153,
  CODE_FOR_avx2_ashlv2ti3 = 7154,
  CODE_FOR_avx2_lshrv2ti3 = 7155,
  CODE_FOR_sse2_ashlv1ti3 = 7156,
  CODE_FOR_sse2_lshrv1ti3 = 7157,
  CODE_FOR_avx512f_rolvv16si = 7158,
  CODE_FOR_avx512f_rolvv16si_mask = 7159,
  CODE_FOR_avx512f_rorvv16si = 7160,
  CODE_FOR_avx512f_rorvv16si_mask = 7161,
  CODE_FOR_avx512vl_rolvv8si = 7162,
  CODE_FOR_avx512vl_rolvv8si_mask = 7163,
  CODE_FOR_avx512vl_rorvv8si = 7164,
  CODE_FOR_avx512vl_rorvv8si_mask = 7165,
  CODE_FOR_avx512vl_rolvv4si = 7166,
  CODE_FOR_avx512vl_rolvv4si_mask = 7167,
  CODE_FOR_avx512vl_rorvv4si = 7168,
  CODE_FOR_avx512vl_rorvv4si_mask = 7169,
  CODE_FOR_avx512f_rolvv8di = 7170,
  CODE_FOR_avx512f_rolvv8di_mask = 7171,
  CODE_FOR_avx512f_rorvv8di = 7172,
  CODE_FOR_avx512f_rorvv8di_mask = 7173,
  CODE_FOR_avx512vl_rolvv4di = 7174,
  CODE_FOR_avx512vl_rolvv4di_mask = 7175,
  CODE_FOR_avx512vl_rorvv4di = 7176,
  CODE_FOR_avx512vl_rorvv4di_mask = 7177,
  CODE_FOR_avx512vl_rolvv2di = 7178,
  CODE_FOR_avx512vl_rolvv2di_mask = 7179,
  CODE_FOR_avx512vl_rorvv2di = 7180,
  CODE_FOR_avx512vl_rorvv2di_mask = 7181,
  CODE_FOR_avx512f_rolv16si = 7182,
  CODE_FOR_avx512f_rolv16si_mask = 7183,
  CODE_FOR_avx512f_rorv16si = 7184,
  CODE_FOR_avx512f_rorv16si_mask = 7185,
  CODE_FOR_avx512vl_rolv8si = 7186,
  CODE_FOR_avx512vl_rolv8si_mask = 7187,
  CODE_FOR_avx512vl_rorv8si = 7188,
  CODE_FOR_avx512vl_rorv8si_mask = 7189,
  CODE_FOR_avx512vl_rolv4si = 7190,
  CODE_FOR_avx512vl_rolv4si_mask = 7191,
  CODE_FOR_avx512vl_rorv4si = 7192,
  CODE_FOR_avx512vl_rorv4si_mask = 7193,
  CODE_FOR_avx512f_rolv8di = 7194,
  CODE_FOR_avx512f_rolv8di_mask = 7195,
  CODE_FOR_avx512f_rorv8di = 7196,
  CODE_FOR_avx512f_rorv8di_mask = 7197,
  CODE_FOR_avx512vl_rolv4di = 7198,
  CODE_FOR_avx512vl_rolv4di_mask = 7199,
  CODE_FOR_avx512vl_rorv4di = 7200,
  CODE_FOR_avx512vl_rorv4di_mask = 7201,
  CODE_FOR_avx512vl_rolv2di = 7202,
  CODE_FOR_avx512vl_rolv2di_mask = 7203,
  CODE_FOR_avx512vl_rorv2di = 7204,
  CODE_FOR_avx512vl_rorv2di_mask = 7205,
  CODE_FOR_sse4_2_gtv2di3 = 7404,
  CODE_FOR_avx2_gtv32qi3 = 7405,
  CODE_FOR_avx2_gtv16hi3 = 7406,
  CODE_FOR_avx2_gtv8si3 = 7407,
  CODE_FOR_avx2_gtv4di3 = 7408,
  CODE_FOR_one_cmplv16si2_mask = 7413,
  CODE_FOR_one_cmplv8di2_mask = 7415,
   CODE_FOR_one_cmplv64qi2_mask = CODE_FOR_nothing,
   CODE_FOR_one_cmplv32qi2_mask = CODE_FOR_nothing,
   CODE_FOR_one_cmplv16qi2_mask = CODE_FOR_nothing,
   CODE_FOR_one_cmplv32hi2_mask = CODE_FOR_nothing,
   CODE_FOR_one_cmplv16hi2_mask = CODE_FOR_nothing,
   CODE_FOR_one_cmplv8hi2_mask = CODE_FOR_nothing,
  CODE_FOR_one_cmplv8si2_mask = 7423,
  CODE_FOR_one_cmplv4si2_mask = 7425,
  CODE_FOR_one_cmplv4di2_mask = 7427,
  CODE_FOR_one_cmplv2di2_mask = 7429,
  CODE_FOR_andv1ti3 = 7514,
  CODE_FOR_iorv1ti3 = 7515,
  CODE_FOR_xorv1ti3 = 7516,
  CODE_FOR_avx512bw_testmv64qi3 = 7565,
  CODE_FOR_avx512bw_testmv64qi3_mask = 7566,
  CODE_FOR_avx512vl_testmv32qi3 = 7567,
  CODE_FOR_avx512vl_testmv32qi3_mask = 7568,
  CODE_FOR_avx512vl_testmv16qi3 = 7569,
  CODE_FOR_avx512vl_testmv16qi3_mask = 7570,
  CODE_FOR_avx512bw_testmv32hi3 = 7571,
  CODE_FOR_avx512bw_testmv32hi3_mask = 7572,
  CODE_FOR_avx512vl_testmv16hi3 = 7573,
  CODE_FOR_avx512vl_testmv16hi3_mask = 7574,
  CODE_FOR_avx512vl_testmv8hi3 = 7575,
  CODE_FOR_avx512vl_testmv8hi3_mask = 7576,
  CODE_FOR_avx512f_testmv16si3 = 7577,
  CODE_FOR_avx512f_testmv16si3_mask = 7578,
  CODE_FOR_avx512vl_testmv8si3 = 7579,
  CODE_FOR_avx512vl_testmv8si3_mask = 7580,
  CODE_FOR_avx512vl_testmv4si3 = 7581,
  CODE_FOR_avx512vl_testmv4si3_mask = 7582,
  CODE_FOR_avx512f_testmv8di3 = 7583,
  CODE_FOR_avx512f_testmv8di3_mask = 7584,
  CODE_FOR_avx512vl_testmv4di3 = 7585,
  CODE_FOR_avx512vl_testmv4di3_mask = 7586,
  CODE_FOR_avx512vl_testmv2di3 = 7587,
  CODE_FOR_avx512vl_testmv2di3_mask = 7588,
  CODE_FOR_avx512bw_testnmv64qi3 = 7589,
  CODE_FOR_avx512bw_testnmv64qi3_mask = 7590,
  CODE_FOR_avx512vl_testnmv32qi3 = 7591,
  CODE_FOR_avx512vl_testnmv32qi3_mask = 7592,
  CODE_FOR_avx512vl_testnmv16qi3 = 7593,
  CODE_FOR_avx512vl_testnmv16qi3_mask = 7594,
  CODE_FOR_avx512bw_testnmv32hi3 = 7595,
  CODE_FOR_avx512bw_testnmv32hi3_mask = 7596,
  CODE_FOR_avx512vl_testnmv16hi3 = 7597,
  CODE_FOR_avx512vl_testnmv16hi3_mask = 7598,
  CODE_FOR_avx512vl_testnmv8hi3 = 7599,
  CODE_FOR_avx512vl_testnmv8hi3_mask = 7600,
  CODE_FOR_avx512f_testnmv16si3 = 7601,
  CODE_FOR_avx512f_testnmv16si3_mask = 7602,
  CODE_FOR_avx512vl_testnmv8si3 = 7603,
  CODE_FOR_avx512vl_testnmv8si3_mask = 7604,
  CODE_FOR_avx512vl_testnmv4si3 = 7605,
  CODE_FOR_avx512vl_testnmv4si3_mask = 7606,
  CODE_FOR_avx512f_testnmv8di3 = 7607,
  CODE_FOR_avx512f_testnmv8di3_mask = 7608,
  CODE_FOR_avx512vl_testnmv4di3 = 7609,
  CODE_FOR_avx512vl_testnmv4di3_mask = 7610,
  CODE_FOR_avx512vl_testnmv2di3 = 7611,
  CODE_FOR_avx512vl_testnmv2di3_mask = 7612,
  CODE_FOR_sse2_packsswb = 7757,
  CODE_FOR_sse2_packsswb_mask = 7758,
  CODE_FOR_avx2_packsswb = 7759,
  CODE_FOR_avx2_packsswb_mask = 7760,
  CODE_FOR_avx512bw_packsswb = 7761,
  CODE_FOR_avx512bw_packsswb_mask = 7762,
  CODE_FOR_sse2_packssdw = 7763,
  CODE_FOR_sse2_packssdw_mask = 7764,
  CODE_FOR_avx2_packssdw = 7765,
  CODE_FOR_avx2_packssdw_mask = 7766,
  CODE_FOR_avx512bw_packssdw = 7767,
  CODE_FOR_avx512bw_packssdw_mask = 7768,
  CODE_FOR_avx512bw_packuswb = 7769,
  CODE_FOR_avx512bw_packuswb_mask = 7770,
  CODE_FOR_avx2_packuswb = 7771,
  CODE_FOR_avx2_packuswb_mask = 7772,
  CODE_FOR_sse2_packuswb = 7773,
  CODE_FOR_sse2_packuswb_mask = 7774,
  CODE_FOR_avx512bw_interleave_highv64qi = 7775,
  CODE_FOR_avx512bw_interleave_highv64qi_mask = 7776,
  CODE_FOR_avx2_interleave_highv32qi = 7777,
  CODE_FOR_avx2_interleave_highv32qi_mask = 7778,
  CODE_FOR_vec_interleave_highv16qi = 7779,
  CODE_FOR_vec_interleave_highv16qi_mask = 7780,
  CODE_FOR_avx512bw_interleave_lowv64qi = 7781,
  CODE_FOR_avx512bw_interleave_lowv64qi_mask = 7782,
  CODE_FOR_avx2_interleave_lowv32qi = 7783,
  CODE_FOR_avx2_interleave_lowv32qi_mask = 7784,
  CODE_FOR_vec_interleave_lowv16qi = 7785,
  CODE_FOR_vec_interleave_lowv16qi_mask = 7786,
  CODE_FOR_avx512bw_interleave_highv32hi = 7787,
  CODE_FOR_avx512bw_interleave_highv32hi_mask = 7788,
  CODE_FOR_avx512bw_interleave_highv32hf = 7789,
  CODE_FOR_avx512bw_interleave_highv32hf_mask = 7790,
  CODE_FOR_avx512bw_interleave_highv32bf = 7791,
  CODE_FOR_avx512bw_interleave_highv32bf_mask = 7792,
  CODE_FOR_avx2_interleave_highv16hi = 7793,
  CODE_FOR_avx2_interleave_highv16hi_mask = 7794,
  CODE_FOR_avx2_interleave_highv16hf = 7795,
  CODE_FOR_avx2_interleave_highv16hf_mask = 7796,
  CODE_FOR_avx2_interleave_highv16bf = 7797,
  CODE_FOR_avx2_interleave_highv16bf_mask = 7798,
  CODE_FOR_vec_interleave_highv8hi = 7799,
  CODE_FOR_vec_interleave_highv8hi_mask = 7800,
  CODE_FOR_vec_interleave_highv8hf = 7801,
  CODE_FOR_vec_interleave_highv8hf_mask = 7802,
  CODE_FOR_vec_interleave_highv8bf = 7803,
  CODE_FOR_vec_interleave_highv8bf_mask = 7804,
  CODE_FOR_avx512bw_interleave_lowv32hi_mask = 7806,
  CODE_FOR_avx512bw_interleave_lowv32hf_mask = 7808,
  CODE_FOR_avx512bw_interleave_lowv32bf_mask = 7810,
  CODE_FOR_avx2_interleave_lowv16hi = 7811,
  CODE_FOR_avx2_interleave_lowv16hi_mask = 7812,
  CODE_FOR_avx2_interleave_lowv16hf = 7813,
  CODE_FOR_avx2_interleave_lowv16hf_mask = 7814,
  CODE_FOR_avx2_interleave_lowv16bf = 7815,
  CODE_FOR_avx2_interleave_lowv16bf_mask = 7816,
  CODE_FOR_vec_interleave_lowv8hi = 7817,
  CODE_FOR_vec_interleave_lowv8hi_mask = 7818,
  CODE_FOR_vec_interleave_lowv8hf = 7819,
  CODE_FOR_vec_interleave_lowv8hf_mask = 7820,
  CODE_FOR_vec_interleave_lowv8bf = 7821,
  CODE_FOR_vec_interleave_lowv8bf_mask = 7822,
  CODE_FOR_avx2_interleave_highv8si = 7823,
  CODE_FOR_avx2_interleave_highv8si_mask = 7824,
  CODE_FOR_avx512f_interleave_highv16si_mask = 7826,
  CODE_FOR_vec_interleave_highv4si = 7827,
  CODE_FOR_vec_interleave_highv4si_mask = 7828,
  CODE_FOR_avx2_interleave_lowv8si = 7829,
  CODE_FOR_avx2_interleave_lowv8si_mask = 7830,
  CODE_FOR_avx512f_interleave_lowv16si_mask = 7832,
  CODE_FOR_vec_interleave_lowv4si = 7833,
  CODE_FOR_vec_interleave_lowv4si_mask = 7834,
  CODE_FOR_sse4_1_pinsrb = 7835,
  CODE_FOR_sse2_pinsrw = 7836,
  CODE_FOR_sse2_pinsrph = 7837,
  CODE_FOR_sse2_pinsrbf = 7838,
  CODE_FOR_sse4_1_pinsrd = 7839,
  CODE_FOR_sse4_1_pinsrq = 7840,
  CODE_FOR_avx512dq_vinsertf64x2_1_mask = 7846,
  CODE_FOR_avx512dq_vinserti64x2_1_mask = 7848,
  CODE_FOR_avx512f_vinsertf32x4_1_mask = 7850,
  CODE_FOR_avx512f_vinserti32x4_1_mask = 7852,
  CODE_FOR_vec_set_lo_v16sf = 7853,
  CODE_FOR_vec_set_lo_v16sf_mask = 7854,
  CODE_FOR_vec_set_lo_v16si = 7855,
  CODE_FOR_vec_set_lo_v16si_mask = 7856,
  CODE_FOR_vec_set_hi_v16sf = 7857,
  CODE_FOR_vec_set_hi_v16sf_mask = 7858,
  CODE_FOR_vec_set_hi_v16si = 7859,
  CODE_FOR_vec_set_hi_v16si_mask = 7860,
  CODE_FOR_vec_set_lo_v8df = 7861,
  CODE_FOR_vec_set_lo_v8df_mask = 7862,
  CODE_FOR_vec_set_lo_v8di = 7863,
  CODE_FOR_vec_set_lo_v8di_mask = 7864,
  CODE_FOR_vec_set_hi_v8df = 7865,
  CODE_FOR_vec_set_hi_v8df_mask = 7866,
  CODE_FOR_vec_set_hi_v8di = 7867,
  CODE_FOR_vec_set_hi_v8di_mask = 7868,
  CODE_FOR_avx512dq_shuf_i64x2_1_mask = 7870,
  CODE_FOR_avx512dq_shuf_f64x2_1_mask = 7872,
  CODE_FOR_avx512f_shuf_f64x2_1 = 7873,
  CODE_FOR_avx512f_shuf_f64x2_1_mask = 7874,
  CODE_FOR_avx512f_shuf_i64x2_1 = 7875,
  CODE_FOR_avx512f_shuf_i64x2_1_mask = 7876,
  CODE_FOR_avx512vl_shuf_i32x4_1 = 7881,
  CODE_FOR_avx512vl_shuf_i32x4_1_mask = 7882,
  CODE_FOR_avx512vl_shuf_f32x4_1 = 7883,
  CODE_FOR_avx512vl_shuf_f32x4_1_mask = 7884,
  CODE_FOR_avx512f_shuf_f32x4_1 = 7885,
  CODE_FOR_avx512f_shuf_f32x4_1_mask = 7886,
  CODE_FOR_avx512f_shuf_i32x4_1 = 7887,
  CODE_FOR_avx512f_shuf_i32x4_1_mask = 7888,
  CODE_FOR_avx512f_pshufd_1 = 7893,
  CODE_FOR_avx512f_pshufd_1_mask = 7894,
  CODE_FOR_avx2_pshufd_1 = 7895,
  CODE_FOR_avx2_pshufd_1_mask = 7896,
  CODE_FOR_sse2_pshufd_1 = 7897,
  CODE_FOR_sse2_pshufd_1_mask = 7898,
  CODE_FOR_avx512bw_pshuflwv32hi_mask = 7900,
  CODE_FOR_avx2_pshuflw_1 = 7901,
  CODE_FOR_avx2_pshuflw_1_mask = 7902,
  CODE_FOR_sse2_pshuflw_1 = 7903,
  CODE_FOR_sse2_pshuflw_1_mask = 7904,
  CODE_FOR_avx512bw_pshufhwv32hi_mask = 7906,
  CODE_FOR_avx2_pshufhw_1 = 7907,
  CODE_FOR_avx2_pshufhw_1_mask = 7908,
  CODE_FOR_sse2_pshufhw_1 = 7909,
  CODE_FOR_sse2_pshufhw_1_mask = 7910,
  CODE_FOR_sse2_loadld = 7911,
  CODE_FOR_vec_concatv2di = 7938,
  CODE_FOR_vec_setv8di_0 = 7940,
  CODE_FOR_vec_setv4di_0 = 7941,
  CODE_FOR_avx_movmskps256 = 7957,
  CODE_FOR_sse_movmskps = 7958,
  CODE_FOR_avx_movmskpd256 = 7959,
  CODE_FOR_sse2_movmskpd = 7960,
  CODE_FOR_avx2_pmovmskb = 7973,
  CODE_FOR_sse2_pmovmskb = 7974,
  CODE_FOR_sse_ldmxcsr = 7988,
  CODE_FOR_sse_stmxcsr = 7989,
  CODE_FOR_sse2_clflush = 7990,
  CODE_FOR_sse3_mwait = 7991,
  CODE_FOR_sse3_monitor_si = 7992,
  CODE_FOR_sse3_monitor_di = 7993,
  CODE_FOR_avx2_phaddwv16hi3 = 7994,
  CODE_FOR_avx2_phaddswv16hi3 = 7995,
  CODE_FOR_avx2_phsubwv16hi3 = 7996,
  CODE_FOR_avx2_phsubswv16hi3 = 7997,
  CODE_FOR_ssse3_phaddwv8hi3 = 7998,
  CODE_FOR_ssse3_phaddswv8hi3 = 7999,
  CODE_FOR_ssse3_phsubwv8hi3 = 8000,
  CODE_FOR_ssse3_phsubswv8hi3 = 8001,
  CODE_FOR_ssse3_phaddwv4hi3 = 8002,
  CODE_FOR_ssse3_phaddswv4hi3 = 8003,
  CODE_FOR_ssse3_phsubwv4hi3 = 8004,
  CODE_FOR_ssse3_phsubswv4hi3 = 8005,
  CODE_FOR_avx2_phadddv8si3 = 8006,
  CODE_FOR_avx2_phsubdv8si3 = 8007,
  CODE_FOR_ssse3_phadddv4si3 = 8008,
  CODE_FOR_ssse3_phsubdv4si3 = 8009,
  CODE_FOR_ssse3_phadddv2si3 = 8010,
  CODE_FOR_ssse3_phsubdv2si3 = 8011,
  CODE_FOR_avx2_pmaddubsw256 = 8012,
  CODE_FOR_avx512bw_pmaddubsw512v8hi = 8013,
  CODE_FOR_avx512bw_pmaddubsw512v8hi_mask = 8014,
  CODE_FOR_avx512bw_pmaddubsw512v16hi = 8015,
  CODE_FOR_avx512bw_pmaddubsw512v16hi_mask = 8016,
  CODE_FOR_avx512bw_pmaddubsw512v32hi = 8017,
  CODE_FOR_avx512bw_pmaddubsw512v32hi_mask = 8018,
  CODE_FOR_avx512bw_umulhrswv32hi3 = 8019,
  CODE_FOR_avx512bw_umulhrswv32hi3_mask = 8020,
  CODE_FOR_ssse3_pmaddubsw128 = 8021,
  CODE_FOR_ssse3_pmaddubsw = 8022,
  CODE_FOR_avx512bw_pshufbv64qi3 = 8031,
  CODE_FOR_avx512bw_pshufbv64qi3_mask = 8032,
  CODE_FOR_avx2_pshufbv32qi3 = 8033,
  CODE_FOR_avx2_pshufbv32qi3_mask = 8034,
  CODE_FOR_ssse3_pshufbv16qi3 = 8035,
  CODE_FOR_ssse3_pshufbv16qi3_mask = 8036,
  CODE_FOR_avx2_psignv32qi3 = 8038,
  CODE_FOR_ssse3_psignv16qi3 = 8039,
  CODE_FOR_avx2_psignv16hi3 = 8040,
  CODE_FOR_ssse3_psignv8hi3 = 8041,
  CODE_FOR_avx2_psignv8si3 = 8042,
  CODE_FOR_ssse3_psignv4si3 = 8043,
  CODE_FOR_ssse3_psignv8qi3 = 8044,
  CODE_FOR_ssse3_psignv4hi3 = 8045,
  CODE_FOR_ssse3_psignv2si3 = 8046,
  CODE_FOR_avx512bw_palignrv64qi_mask = 8047,
  CODE_FOR_avx2_palignrv32qi_mask = 8048,
  CODE_FOR_ssse3_palignrv16qi_mask = 8049,
  CODE_FOR_avx512bw_palignrv4ti = 8050,
  CODE_FOR_avx2_palignrv2ti = 8051,
  CODE_FOR_ssse3_palignrv1ti = 8052,
  CODE_FOR_ssse3_palignrdi = 8053,
  CODE_FOR_absv16si2_mask = 8066,
  CODE_FOR_absv8si2_mask = 8067,
  CODE_FOR_absv4si2_mask = 8068,
  CODE_FOR_absv8di2_mask = 8069,
  CODE_FOR_absv4di2_mask = 8070,
  CODE_FOR_absv2di2_mask = 8071,
  CODE_FOR_absv64qi2_mask = 8072,
  CODE_FOR_absv16qi2_mask = 8073,
  CODE_FOR_absv32qi2_mask = 8074,
  CODE_FOR_absv32hi2_mask = 8075,
  CODE_FOR_absv16hi2_mask = 8076,
  CODE_FOR_absv8hi2_mask = 8077,
  CODE_FOR_sse4a_movntsf = 8078,
  CODE_FOR_sse4a_movntdf = 8079,
  CODE_FOR_sse4a_vmmovntv4sf = 8080,
  CODE_FOR_sse4a_vmmovntv2df = 8081,
  CODE_FOR_sse4a_extrqi = 8082,
  CODE_FOR_sse4a_extrq = 8083,
  CODE_FOR_sse4a_insertqi = 8084,
  CODE_FOR_sse4a_insertq = 8085,
  CODE_FOR_avx_blendps256 = 8086,
  CODE_FOR_sse4_1_blendps = 8087,
  CODE_FOR_avx_blendpd256 = 8088,
  CODE_FOR_sse4_1_blendpd = 8089,
  CODE_FOR_avx_blendvps256 = 8090,
  CODE_FOR_sse4_1_blendvps = 8091,
  CODE_FOR_avx_blendvpd256 = 8092,
  CODE_FOR_sse4_1_blendvpd = 8093,
  CODE_FOR_sse4_1_blendvss = 8094,
  CODE_FOR_sse4_1_blendvsd = 8095,
  CODE_FOR_avx_dpps256 = 8120,
  CODE_FOR_sse4_1_dpps = 8121,
  CODE_FOR_avx_dppd256 = 8122,
  CODE_FOR_sse4_1_dppd = 8123,
  CODE_FOR_avx512f_movntdqa = 8124,
  CODE_FOR_avx2_movntdqa = 8125,
  CODE_FOR_sse4_1_movntdqa = 8126,
  CODE_FOR_avx2_mpsadbw = 8127,
  CODE_FOR_sse4_1_mpsadbw = 8128,
  CODE_FOR_avx10_2_mpsadbw = 8129,
  CODE_FOR_avx10_2_mpsadbw_mask = 8130,
  CODE_FOR_avx2_mpsadbw_mask = 8132,
  CODE_FOR_sse4_1_mpsadbw_mask = 8134,
  CODE_FOR_avx512bw_packusdw = 8135,
  CODE_FOR_avx512bw_packusdw_mask = 8136,
  CODE_FOR_avx2_packusdw = 8137,
  CODE_FOR_avx2_packusdw_mask = 8138,
  CODE_FOR_sse4_1_packusdw = 8139,
  CODE_FOR_sse4_1_packusdw_mask = 8140,
  CODE_FOR_avx2_pblendvb = 8141,
  CODE_FOR_sse4_1_pblendvb = 8142,
  CODE_FOR_sse4_1_pblendw = 8151,
  CODE_FOR_sse4_1_pblendph = 8152,
  CODE_FOR_sse4_1_pblendbf = 8153,
  CODE_FOR_avx2_pblenddv8si = 8157,
  CODE_FOR_avx2_pblenddv4si = 8158,
  CODE_FOR_sse4_1_phminposuw = 8159,
  CODE_FOR_avx2_sign_extendv16qiv16hi2 = 8160,
  CODE_FOR_avx2_sign_extendv16qiv16hi2_mask = 8161,
  CODE_FOR_avx2_zero_extendv16qiv16hi2 = 8162,
  CODE_FOR_avx2_zero_extendv16qiv16hi2_mask = 8163,
  CODE_FOR_avx512bw_sign_extendv32qiv32hi2 = 8168,
  CODE_FOR_avx512bw_sign_extendv32qiv32hi2_mask = 8169,
  CODE_FOR_avx512bw_zero_extendv32qiv32hi2 = 8170,
  CODE_FOR_avx512bw_zero_extendv32qiv32hi2_mask = 8171,
  CODE_FOR_sse4_1_sign_extendv8qiv8hi2 = 8176,
  CODE_FOR_sse4_1_sign_extendv8qiv8hi2_mask = 8177,
  CODE_FOR_sse4_1_zero_extendv8qiv8hi2 = 8178,
  CODE_FOR_sse4_1_zero_extendv8qiv8hi2_mask = 8179,
  CODE_FOR_avx512f_sign_extendv16qiv16si2_mask = 8193,
  CODE_FOR_avx512f_zero_extendv16qiv16si2_mask = 8195,
  CODE_FOR_avx2_sign_extendv8qiv8si2 = 8196,
  CODE_FOR_avx2_sign_extendv8qiv8si2_mask = 8197,
  CODE_FOR_avx2_zero_extendv8qiv8si2 = 8198,
  CODE_FOR_avx2_zero_extendv8qiv8si2_mask = 8199,
  CODE_FOR_sse4_1_sign_extendv4qiv4si2 = 8208,
  CODE_FOR_sse4_1_sign_extendv4qiv4si2_mask = 8209,
  CODE_FOR_sse4_1_zero_extendv4qiv4si2 = 8210,
  CODE_FOR_sse4_1_zero_extendv4qiv4si2_mask = 8211,
  CODE_FOR_avx512f_sign_extendv16hiv16si2 = 8222,
  CODE_FOR_avx512f_sign_extendv16hiv16si2_mask = 8223,
  CODE_FOR_avx512f_zero_extendv16hiv16si2 = 8224,
  CODE_FOR_avx512f_zero_extendv16hiv16si2_mask = 8225,
  CODE_FOR_avx512f_zero_extendv16hiv16si2_1 = 8226,
  CODE_FOR_avx2_sign_extendv8hiv8si2 = 8230,
  CODE_FOR_avx2_sign_extendv8hiv8si2_mask = 8231,
  CODE_FOR_avx2_zero_extendv8hiv8si2 = 8232,
  CODE_FOR_avx2_zero_extendv8hiv8si2_mask = 8233,
  CODE_FOR_avx2_zero_extendv8hiv8si2_1 = 8234,
  CODE_FOR_sse4_1_sign_extendv4hiv4si2 = 8238,
  CODE_FOR_sse4_1_sign_extendv4hiv4si2_mask = 8239,
  CODE_FOR_sse4_1_zero_extendv4hiv4si2 = 8240,
  CODE_FOR_sse4_1_zero_extendv4hiv4si2_mask = 8241,
  CODE_FOR_avx512f_sign_extendv8qiv8di2 = 8254,
  CODE_FOR_avx512f_sign_extendv8qiv8di2_mask = 8255,
  CODE_FOR_avx512f_zero_extendv8qiv8di2 = 8256,
  CODE_FOR_avx512f_zero_extendv8qiv8di2_mask = 8257,
  CODE_FOR_avx2_sign_extendv4qiv4di2 = 8266,
  CODE_FOR_avx2_sign_extendv4qiv4di2_mask = 8267,
  CODE_FOR_avx2_zero_extendv4qiv4di2 = 8268,
  CODE_FOR_avx2_zero_extendv4qiv4di2_mask = 8269,
  CODE_FOR_sse4_1_sign_extendv2qiv2di2 = 8278,
  CODE_FOR_sse4_1_sign_extendv2qiv2di2_mask = 8279,
  CODE_FOR_sse4_1_zero_extendv2qiv2di2 = 8280,
  CODE_FOR_sse4_1_zero_extendv2qiv2di2_mask = 8281,
  CODE_FOR_avx512f_sign_extendv8hiv8di2 = 8289,
  CODE_FOR_avx512f_sign_extendv8hiv8di2_mask = 8290,
  CODE_FOR_avx512f_zero_extendv8hiv8di2 = 8291,
  CODE_FOR_avx512f_zero_extendv8hiv8di2_mask = 8292,
  CODE_FOR_avx2_sign_extendv4hiv4di2 = 8293,
  CODE_FOR_avx2_sign_extendv4hiv4di2_mask = 8294,
  CODE_FOR_avx2_zero_extendv4hiv4di2 = 8295,
  CODE_FOR_avx2_zero_extendv4hiv4di2_mask = 8296,
  CODE_FOR_sse4_1_sign_extendv2hiv2di2 = 8305,
  CODE_FOR_sse4_1_sign_extendv2hiv2di2_mask = 8306,
  CODE_FOR_sse4_1_zero_extendv2hiv2di2 = 8307,
  CODE_FOR_sse4_1_zero_extendv2hiv2di2_mask = 8308,
  CODE_FOR_avx512f_sign_extendv8siv8di2 = 8317,
  CODE_FOR_avx512f_sign_extendv8siv8di2_mask = 8318,
  CODE_FOR_avx512f_zero_extendv8siv8di2 = 8319,
  CODE_FOR_avx512f_zero_extendv8siv8di2_mask = 8320,
  CODE_FOR_avx2_sign_extendv4siv4di2 = 8323,
  CODE_FOR_avx2_sign_extendv4siv4di2_mask = 8324,
  CODE_FOR_avx2_zero_extendv4siv4di2 = 8325,
  CODE_FOR_avx2_zero_extendv4siv4di2_mask = 8326,
  CODE_FOR_sse4_1_sign_extendv2siv2di2 = 8329,
  CODE_FOR_sse4_1_sign_extendv2siv2di2_mask = 8330,
  CODE_FOR_sse4_1_zero_extendv2siv2di2 = 8331,
  CODE_FOR_sse4_1_zero_extendv2siv2di2_mask = 8332,
  CODE_FOR_avx_vtestps256 = 8343,
  CODE_FOR_avx_vtestps = 8344,
  CODE_FOR_avx_vtestpd256 = 8345,
  CODE_FOR_avx_vtestpd = 8346,
  CODE_FOR_ptesttf2 = 8361,
  CODE_FOR_avx_roundps256 = 8384,
  CODE_FOR_sse4_1_roundps = 8385,
  CODE_FOR_avx_roundpd256 = 8386,
  CODE_FOR_sse4_1_roundpd = 8387,
  CODE_FOR_sse4_1_roundsh = 8388,
  CODE_FOR_sse4_1_roundss = 8389,
  CODE_FOR_sse4_1_roundsd = 8390,
  CODE_FOR_sse4_2_pcmpestr = 8394,
  CODE_FOR_sse4_2_pcmpestri = 8395,
  CODE_FOR_sse4_2_pcmpestrm = 8396,
  CODE_FOR_sse4_2_pcmpestr_cconly = 8397,
  CODE_FOR_sse4_2_pcmpistr = 8398,
  CODE_FOR_sse4_2_pcmpistri = 8399,
  CODE_FOR_sse4_2_pcmpistrm = 8400,
  CODE_FOR_sse4_2_pcmpistr_cconly = 8401,
  CODE_FOR_xop_pmacsww = 8402,
  CODE_FOR_xop_pmacssww = 8403,
  CODE_FOR_xop_pmacsdd = 8404,
  CODE_FOR_xop_pmacssdd = 8405,
  CODE_FOR_xop_pmacsdql = 8406,
  CODE_FOR_xop_pmacssdql = 8407,
  CODE_FOR_xop_pmacsdqh = 8408,
  CODE_FOR_xop_pmacssdqh = 8409,
  CODE_FOR_xop_pmacswd = 8410,
  CODE_FOR_xop_pmacsswd = 8411,
  CODE_FOR_xop_pmadcswd = 8412,
  CODE_FOR_xop_pmadcsswd = 8413,
  CODE_FOR_xop_pcmov_v32qi256 = 8414,
  CODE_FOR_xop_pcmov_v16qi = 8415,
  CODE_FOR_xop_pcmov_v16hi256 = 8416,
  CODE_FOR_xop_pcmov_v8hi = 8417,
  CODE_FOR_xop_pcmov_v8si256 = 8418,
  CODE_FOR_xop_pcmov_v4si = 8419,
  CODE_FOR_xop_pcmov_v4di256 = 8420,
  CODE_FOR_xop_pcmov_v2di = 8421,
  CODE_FOR_xop_pcmov_v2ti256 = 8422,
  CODE_FOR_xop_pcmov_v1ti = 8423,
  CODE_FOR_xop_pcmov_v16hf256 = 8424,
  CODE_FOR_xop_pcmov_v8hf = 8425,
  CODE_FOR_xop_pcmov_v8sf256 = 8426,
  CODE_FOR_xop_pcmov_v4sf = 8427,
  CODE_FOR_xop_pcmov_v4df256 = 8428,
  CODE_FOR_xop_pcmov_v2df = 8429,
  CODE_FOR_xop_phaddbw = 8430,
  CODE_FOR_xop_phaddubw = 8431,
  CODE_FOR_xop_phaddbd = 8432,
  CODE_FOR_xop_phaddubd = 8433,
  CODE_FOR_xop_phaddbq = 8434,
  CODE_FOR_xop_phaddubq = 8435,
  CODE_FOR_xop_phaddwd = 8436,
  CODE_FOR_xop_phadduwd = 8437,
  CODE_FOR_xop_phaddwq = 8438,
  CODE_FOR_xop_phadduwq = 8439,
  CODE_FOR_xop_phadddq = 8440,
  CODE_FOR_xop_phaddudq = 8441,
  CODE_FOR_xop_phsubbw = 8442,
  CODE_FOR_xop_phsubwd = 8443,
  CODE_FOR_xop_phsubdq = 8444,
  CODE_FOR_xop_pperm = 8445,
  CODE_FOR_xop_pperm_pack_v2di_v4si = 8446,
  CODE_FOR_xop_pperm_pack_v4si_v8hi = 8447,
  CODE_FOR_xop_pperm_pack_v8hi_v16qi = 8448,
  CODE_FOR_xop_rotlv16qi3 = 8449,
  CODE_FOR_xop_rotlv8hi3 = 8450,
  CODE_FOR_xop_rotlv4si3 = 8451,
  CODE_FOR_xop_rotlv2di3 = 8452,
  CODE_FOR_xop_rotrv16qi3 = 8453,
  CODE_FOR_xop_rotrv8hi3 = 8454,
  CODE_FOR_xop_rotrv4si3 = 8455,
  CODE_FOR_xop_rotrv2di3 = 8456,
  CODE_FOR_xop_vrotlv16qi3 = 8457,
  CODE_FOR_xop_vrotlv8hi3 = 8458,
  CODE_FOR_xop_vrotlv4si3 = 8459,
  CODE_FOR_xop_vrotlv2di3 = 8460,
  CODE_FOR_xop_shav16qi3 = 8461,
  CODE_FOR_xop_shav8hi3 = 8462,
  CODE_FOR_xop_shav4si3 = 8463,
  CODE_FOR_xop_shav2di3 = 8464,
  CODE_FOR_xop_shlv16qi3 = 8465,
  CODE_FOR_xop_shlv8hi3 = 8466,
  CODE_FOR_xop_shlv4si3 = 8467,
  CODE_FOR_xop_shlv2di3 = 8468,
  CODE_FOR_xop_frczsf2 = 8470,
  CODE_FOR_xop_frczdf2 = 8471,
  CODE_FOR_xop_frczv4sf2 = 8472,
  CODE_FOR_xop_frczv2df2 = 8473,
  CODE_FOR_xop_frczv8sf2 = 8474,
  CODE_FOR_xop_frczv4df2 = 8475,
  CODE_FOR_xop_maskcmpv16qi3 = 8478,
  CODE_FOR_xop_maskcmpv8hi3 = 8479,
  CODE_FOR_xop_maskcmpv4si3 = 8480,
  CODE_FOR_xop_maskcmpv2di3 = 8481,
  CODE_FOR_xop_maskcmp_unsv16qi3 = 8482,
  CODE_FOR_xop_maskcmp_unsv8hi3 = 8483,
  CODE_FOR_xop_maskcmp_unsv4si3 = 8484,
  CODE_FOR_xop_maskcmp_unsv2di3 = 8485,
  CODE_FOR_xop_maskcmp_uns2v16qi3 = 8486,
  CODE_FOR_xop_maskcmp_uns2v8hi3 = 8487,
  CODE_FOR_xop_maskcmp_uns2v4si3 = 8488,
  CODE_FOR_xop_maskcmp_uns2v2di3 = 8489,
  CODE_FOR_xop_pcom_tfv16qi3 = 8490,
  CODE_FOR_xop_pcom_tfv8hi3 = 8491,
  CODE_FOR_xop_pcom_tfv4si3 = 8492,
  CODE_FOR_xop_pcom_tfv2di3 = 8493,
  CODE_FOR_xop_vpermil2v8sf3 = 8494,
  CODE_FOR_xop_vpermil2v4sf3 = 8495,
  CODE_FOR_xop_vpermil2v4df3 = 8496,
  CODE_FOR_xop_vpermil2v2df3 = 8497,
  CODE_FOR_aesenc = 8498,
  CODE_FOR_aesenclast = 8499,
  CODE_FOR_aesdec = 8500,
  CODE_FOR_aesdeclast = 8501,
  CODE_FOR_aesimc = 8502,
  CODE_FOR_aeskeygenassist = 8503,
  CODE_FOR_pclmulqdq = 8504,
  CODE_FOR_avx_vzeroupper_callee_abi = 8506,
  CODE_FOR_avx2_pbroadcastv16si = 8507,
  CODE_FOR_avx2_pbroadcastv8di = 8508,
  CODE_FOR_avx2_pbroadcastv64qi = 8509,
  CODE_FOR_avx2_pbroadcastv32qi = 8510,
  CODE_FOR_avx2_pbroadcastv16qi = 8511,
  CODE_FOR_avx2_pbroadcastv32hi = 8512,
  CODE_FOR_avx2_pbroadcastv16hi = 8513,
  CODE_FOR_avx2_pbroadcastv8hi = 8514,
  CODE_FOR_avx2_pbroadcastv8si = 8515,
  CODE_FOR_avx2_pbroadcastv4si = 8516,
  CODE_FOR_avx2_pbroadcastv4di = 8517,
  CODE_FOR_avx2_pbroadcastv2di = 8518,
  CODE_FOR_avx2_pbroadcastv32hf = 8519,
  CODE_FOR_avx2_pbroadcastv16hf = 8520,
  CODE_FOR_avx2_pbroadcastv8hf = 8521,
  CODE_FOR_avx2_pbroadcastv32bf = 8522,
  CODE_FOR_avx2_pbroadcastv16bf = 8523,
  CODE_FOR_avx2_pbroadcastv8bf = 8524,
  CODE_FOR_avx2_pbroadcastv32qi_1 = 8525,
  CODE_FOR_avx2_pbroadcastv16hi_1 = 8526,
  CODE_FOR_avx2_pbroadcastv8si_1 = 8527,
  CODE_FOR_avx2_pbroadcastv4di_1 = 8528,
  CODE_FOR_avx2_pbroadcastv16hf_1 = 8529,
  CODE_FOR_avx2_pbroadcastv16bf_1 = 8530,
  CODE_FOR_avx2_permvarv8si = 8531,
  CODE_FOR_avx2_permvarv8si_mask = 8532,
  CODE_FOR_avx2_permvarv8sf = 8533,
  CODE_FOR_avx2_permvarv8sf_mask = 8534,
  CODE_FOR_avx512f_permvarv16si = 8535,
  CODE_FOR_avx512f_permvarv16si_mask = 8536,
  CODE_FOR_avx512f_permvarv16sf = 8537,
  CODE_FOR_avx512f_permvarv16sf_mask = 8538,
  CODE_FOR_avx512f_permvarv8di = 8539,
  CODE_FOR_avx512f_permvarv8di_mask = 8540,
  CODE_FOR_avx512f_permvarv8df = 8541,
  CODE_FOR_avx512f_permvarv8df_mask = 8542,
  CODE_FOR_avx2_permvarv4di = 8543,
  CODE_FOR_avx2_permvarv4di_mask = 8544,
  CODE_FOR_avx2_permvarv4df = 8545,
  CODE_FOR_avx2_permvarv4df_mask = 8546,
  CODE_FOR_avx512bw_permvarv64qi = 8547,
  CODE_FOR_avx512bw_permvarv64qi_mask = 8548,
  CODE_FOR_avx512vl_permvarv16qi = 8549,
  CODE_FOR_avx512vl_permvarv16qi_mask = 8550,
  CODE_FOR_avx512vl_permvarv32qi = 8551,
  CODE_FOR_avx512vl_permvarv32qi_mask = 8552,
  CODE_FOR_avx512vl_permvarv8hi = 8553,
  CODE_FOR_avx512vl_permvarv8hi_mask = 8554,
  CODE_FOR_avx512vl_permvarv16hi = 8555,
  CODE_FOR_avx512vl_permvarv16hi_mask = 8556,
  CODE_FOR_avx512bw_permvarv32hi = 8557,
  CODE_FOR_avx512bw_permvarv32hi_mask = 8558,
  CODE_FOR_avx512fp16_permvarv8hf = 8559,
  CODE_FOR_avx512fp16_permvarv8hf_mask = 8560,
  CODE_FOR_avx512vl_permvarv16hf = 8561,
  CODE_FOR_avx512vl_permvarv16hf_mask = 8562,
  CODE_FOR_avx512bw_permvarv32hf = 8563,
  CODE_FOR_avx512bw_permvarv32hf_mask = 8564,
  CODE_FOR_avx512vl_permvarv8bf = 8565,
  CODE_FOR_avx512vl_permvarv8bf_mask = 8566,
  CODE_FOR_avx512vl_permvarv16bf = 8567,
  CODE_FOR_avx512vl_permvarv16bf_mask = 8568,
  CODE_FOR_avx512bw_permvarv32bf = 8569,
  CODE_FOR_avx512bw_permvarv32bf_mask = 8570,
  CODE_FOR_avx2_permv4di_1 = 8598,
  CODE_FOR_avx2_permv4di_1_mask = 8599,
  CODE_FOR_avx2_permv4df_1 = 8600,
  CODE_FOR_avx2_permv4df_1_mask = 8601,
  CODE_FOR_avx512f_permv8df_1 = 8602,
  CODE_FOR_avx512f_permv8df_1_mask = 8603,
  CODE_FOR_avx512f_permv8di_1 = 8604,
  CODE_FOR_avx512f_permv8di_1_mask = 8605,
  CODE_FOR_avx2_permv2ti = 8606,
  CODE_FOR_avx2_vec_dupv4df = 8607,
  CODE_FOR_avx512f_vec_dupv16si_1 = 8608,
  CODE_FOR_avx512f_vec_dupv8di_1 = 8609,
  CODE_FOR_avx512bw_vec_dupv32hi_1 = 8610,
  CODE_FOR_avx512bw_vec_dupv64qi_1 = 8611,
  CODE_FOR_avx512bw_vec_dupv32hf_1 = 8612,
  CODE_FOR_avx512bw_vec_dupv32bf_1 = 8613,
  CODE_FOR_avx512f_vec_dupv16si = 8614,
  CODE_FOR_avx512f_vec_dupv16si_mask = 8615,
  CODE_FOR_avx512vl_vec_dupv8si = 8616,
  CODE_FOR_avx512vl_vec_dupv8si_mask = 8617,
  CODE_FOR_avx512vl_vec_dupv4si = 8618,
  CODE_FOR_avx512vl_vec_dupv4si_mask = 8619,
  CODE_FOR_avx512f_vec_dupv8di = 8620,
  CODE_FOR_avx512f_vec_dupv8di_mask = 8621,
  CODE_FOR_avx512vl_vec_dupv4di = 8622,
  CODE_FOR_avx512vl_vec_dupv4di_mask = 8623,
  CODE_FOR_avx512vl_vec_dupv2di = 8624,
  CODE_FOR_avx512vl_vec_dupv2di_mask = 8625,
  CODE_FOR_avx512f_vec_dupv16sf = 8626,
  CODE_FOR_avx512f_vec_dupv16sf_mask = 8627,
  CODE_FOR_avx512vl_vec_dupv8sf = 8628,
  CODE_FOR_avx512vl_vec_dupv8sf_mask = 8629,
  CODE_FOR_avx512vl_vec_dupv4sf = 8630,
  CODE_FOR_avx512vl_vec_dupv4sf_mask = 8631,
  CODE_FOR_avx512f_vec_dupv8df = 8632,
  CODE_FOR_avx512f_vec_dupv8df_mask = 8633,
  CODE_FOR_avx512vl_vec_dupv4df = 8634,
  CODE_FOR_avx512vl_vec_dupv4df_mask = 8635,
  CODE_FOR_avx512vl_vec_dupv2df = 8636,
  CODE_FOR_avx512vl_vec_dupv2df_mask = 8637,
  CODE_FOR_avx512bw_vec_dupv64qi = 8638,
  CODE_FOR_avx512bw_vec_dupv64qi_mask = 8639,
  CODE_FOR_avx512vl_vec_dupv16qi = 8640,
  CODE_FOR_avx512vl_vec_dupv16qi_mask = 8641,
  CODE_FOR_avx512vl_vec_dupv32qi = 8642,
  CODE_FOR_avx512vl_vec_dupv32qi_mask = 8643,
  CODE_FOR_avx512bw_vec_dupv32hi = 8644,
  CODE_FOR_avx512bw_vec_dupv32hi_mask = 8645,
  CODE_FOR_avx512vl_vec_dupv16hi = 8646,
  CODE_FOR_avx512vl_vec_dupv16hi_mask = 8647,
  CODE_FOR_avx512vl_vec_dupv8hi = 8648,
  CODE_FOR_avx512vl_vec_dupv8hi_mask = 8649,
  CODE_FOR_avx512bw_vec_dupv32hf = 8650,
  CODE_FOR_avx512bw_vec_dupv32hf_mask = 8651,
  CODE_FOR_avx512vl_vec_dupv16hf = 8652,
  CODE_FOR_avx512vl_vec_dupv16hf_mask = 8653,
  CODE_FOR_avx512fp16_vec_dupv8hf = 8654,
  CODE_FOR_avx512fp16_vec_dupv8hf_mask = 8655,
  CODE_FOR_avx512bw_vec_dupv32bf = 8656,
  CODE_FOR_avx512bw_vec_dupv32bf_mask = 8657,
  CODE_FOR_avx512vl_vec_dupv16bf = 8658,
  CODE_FOR_avx512vl_vec_dupv16bf_mask = 8659,
  CODE_FOR_avx512vl_vec_dupv8bf = 8660,
  CODE_FOR_avx512vl_vec_dupv8bf_mask = 8661,
  CODE_FOR_avx512f_broadcastv16sf_mask = 8663,
  CODE_FOR_avx512f_broadcastv16si_mask = 8665,
  CODE_FOR_avx512f_broadcastv8df_mask = 8667,
  CODE_FOR_avx512f_broadcastv8di_mask = 8669,
  CODE_FOR_avx512bw_vec_dup_gprv64qi_mask = 8671,
  CODE_FOR_avx512vl_vec_dup_gprv16qi_mask = 8673,
  CODE_FOR_avx512vl_vec_dup_gprv32qi_mask = 8675,
  CODE_FOR_avx512bw_vec_dup_gprv32hi_mask = 8677,
  CODE_FOR_avx512vl_vec_dup_gprv16hi_mask = 8679,
  CODE_FOR_avx512vl_vec_dup_gprv8hi_mask = 8681,
  CODE_FOR_avx512bw_vec_dup_gprv32hf_mask = 8683,
  CODE_FOR_avx512vl_vec_dup_gprv16hf_mask = 8685,
  CODE_FOR_avx512fp16_vec_dup_gprv8hf_mask = 8687,
  CODE_FOR_avx512bw_vec_dup_gprv32bf_mask = 8689,
  CODE_FOR_avx512vl_vec_dup_gprv16bf_mask = 8691,
  CODE_FOR_avx512vl_vec_dup_gprv8bf_mask = 8693,
  CODE_FOR_avx512f_vec_dup_gprv16si_mask = 8695,
  CODE_FOR_avx512vl_vec_dup_gprv8si_mask = 8697,
  CODE_FOR_avx512vl_vec_dup_gprv4si_mask = 8699,
  CODE_FOR_avx512f_vec_dup_gprv8di_mask = 8701,
  CODE_FOR_avx512vl_vec_dup_gprv4di_mask = 8703,
  CODE_FOR_avx512vl_vec_dup_gprv2di_mask = 8705,
  CODE_FOR_avx512f_vec_dup_gprv16sf_mask = 8707,
  CODE_FOR_avx512vl_vec_dup_gprv8sf_mask = 8709,
  CODE_FOR_avx512vl_vec_dup_gprv4sf_mask = 8711,
  CODE_FOR_avx512f_vec_dup_gprv8df_mask = 8713,
  CODE_FOR_avx512vl_vec_dup_gprv4df_mask = 8715,
  CODE_FOR_avx512vl_vec_dup_gprv2df_mask = 8717,
  CODE_FOR_vec_dupv4sf = 8718,
  CODE_FOR_avx2_vbroadcasti128_v32qi = 8721,
  CODE_FOR_avx2_vbroadcasti128_v16hi = 8722,
  CODE_FOR_avx2_vbroadcasti128_v8si = 8723,
  CODE_FOR_avx2_vbroadcasti128_v4di = 8724,
  CODE_FOR_avx2_lddqu_inserti_to_bcasti = 8725,
  CODE_FOR_vec_dupv8si = 8736,
  CODE_FOR_vec_dupv8sf = 8737,
  CODE_FOR_vec_dupv4di = 8738,
  CODE_FOR_vec_dupv4df = 8739,
  CODE_FOR_avx_vbroadcastf128_v32qi = 8740,
  CODE_FOR_avx_vbroadcastf128_v16hi = 8741,
  CODE_FOR_avx_vbroadcastf128_v8si = 8742,
  CODE_FOR_avx_vbroadcastf128_v4di = 8743,
  CODE_FOR_avx_vbroadcastf128_v8sf = 8744,
  CODE_FOR_avx_vbroadcastf128_v4df = 8745,
  CODE_FOR_avx_vbroadcastf128_v16hf = 8746,
  CODE_FOR_avx_vbroadcastf128_v16bf = 8747,
  CODE_FOR_avx512dq_broadcastv16si_mask = 8757,
  CODE_FOR_avx512dq_broadcastv8si_mask = 8759,
  CODE_FOR_avx512dq_broadcastv4si_mask = 8761,
  CODE_FOR_avx512dq_broadcastv16sf_mask = 8763,
  CODE_FOR_avx512dq_broadcastv8sf_mask = 8765,
  CODE_FOR_avx512vl_broadcastv8si_mask_1 = 8767,
  CODE_FOR_avx512vl_broadcastv8sf_mask_1 = 8769,
  CODE_FOR_avx512dq_broadcastv16sf_mask_1 = 8771,
  CODE_FOR_avx512dq_broadcastv16si_mask_1 = 8773,
  CODE_FOR_avx512dq_broadcastv8di_mask_1 = 8775,
  CODE_FOR_avx512dq_broadcastv8df_mask_1 = 8777,
  CODE_FOR_avx512dq_broadcastv4di_mask_1 = 8779,
  CODE_FOR_avx512dq_broadcastv4df_mask_1 = 8781,
  CODE_FOR_avx512cd_maskb_vec_dupv8di = 8782,
  CODE_FOR_avx512cd_maskb_vec_dupv4di = 8783,
  CODE_FOR_avx512cd_maskb_vec_dupv2di = 8784,
  CODE_FOR_avx512cd_maskw_vec_dupv16si = 8785,
  CODE_FOR_avx512cd_maskw_vec_dupv8si = 8786,
  CODE_FOR_avx512cd_maskw_vec_dupv4si = 8787,
  CODE_FOR_avx512f_vpermilvarv16sf3 = 8788,
  CODE_FOR_avx512f_vpermilvarv16sf3_mask = 8789,
  CODE_FOR_avx_vpermilvarv8sf3 = 8790,
  CODE_FOR_avx_vpermilvarv8sf3_mask = 8791,
  CODE_FOR_avx_vpermilvarv4sf3 = 8792,
  CODE_FOR_avx_vpermilvarv4sf3_mask = 8793,
  CODE_FOR_avx512f_vpermilvarv8df3 = 8794,
  CODE_FOR_avx512f_vpermilvarv8df3_mask = 8795,
  CODE_FOR_avx_vpermilvarv4df3 = 8796,
  CODE_FOR_avx_vpermilvarv4df3_mask = 8797,
  CODE_FOR_avx_vpermilvarv2df3 = 8798,
  CODE_FOR_avx_vpermilvarv2df3_mask = 8799,
  CODE_FOR_avx512f_vpermt2varv16si3 = 8818,
  CODE_FOR_avx512f_vpermt2varv16si3_maskz_1 = 8819,
  CODE_FOR_avx512f_vpermt2varv16sf3 = 8820,
  CODE_FOR_avx512f_vpermt2varv16sf3_maskz_1 = 8821,
  CODE_FOR_avx512f_vpermt2varv8di3 = 8822,
  CODE_FOR_avx512f_vpermt2varv8di3_maskz_1 = 8823,
  CODE_FOR_avx512f_vpermt2varv8df3 = 8824,
  CODE_FOR_avx512f_vpermt2varv8df3_maskz_1 = 8825,
  CODE_FOR_avx512vl_vpermt2varv8si3 = 8826,
  CODE_FOR_avx512vl_vpermt2varv8si3_maskz_1 = 8827,
  CODE_FOR_avx512vl_vpermt2varv8sf3 = 8828,
  CODE_FOR_avx512vl_vpermt2varv8sf3_maskz_1 = 8829,
  CODE_FOR_avx512vl_vpermt2varv4di3 = 8830,
  CODE_FOR_avx512vl_vpermt2varv4di3_maskz_1 = 8831,
  CODE_FOR_avx512vl_vpermt2varv4df3 = 8832,
  CODE_FOR_avx512vl_vpermt2varv4df3_maskz_1 = 8833,
  CODE_FOR_avx512vl_vpermt2varv4si3 = 8834,
  CODE_FOR_avx512vl_vpermt2varv4si3_maskz_1 = 8835,
  CODE_FOR_avx512vl_vpermt2varv4sf3 = 8836,
  CODE_FOR_avx512vl_vpermt2varv4sf3_maskz_1 = 8837,
  CODE_FOR_avx512vl_vpermt2varv2di3 = 8838,
  CODE_FOR_avx512vl_vpermt2varv2di3_maskz_1 = 8839,
  CODE_FOR_avx512vl_vpermt2varv2df3 = 8840,
  CODE_FOR_avx512vl_vpermt2varv2df3_maskz_1 = 8841,
  CODE_FOR_avx512bw_vpermt2varv32hi3 = 8842,
  CODE_FOR_avx512bw_vpermt2varv32hi3_maskz_1 = 8843,
  CODE_FOR_avx512vl_vpermt2varv16hi3 = 8844,
  CODE_FOR_avx512vl_vpermt2varv16hi3_maskz_1 = 8845,
  CODE_FOR_avx512vl_vpermt2varv8hi3 = 8846,
  CODE_FOR_avx512vl_vpermt2varv8hi3_maskz_1 = 8847,
  CODE_FOR_avx512bw_vpermt2varv64qi3 = 8848,
  CODE_FOR_avx512bw_vpermt2varv64qi3_maskz_1 = 8849,
  CODE_FOR_avx512vl_vpermt2varv32qi3 = 8850,
  CODE_FOR_avx512vl_vpermt2varv32qi3_maskz_1 = 8851,
  CODE_FOR_avx512vl_vpermt2varv16qi3 = 8852,
  CODE_FOR_avx512vl_vpermt2varv16qi3_maskz_1 = 8853,
  CODE_FOR_avx512bw_vpermt2varv32hf3 = 8854,
  CODE_FOR_avx512bw_vpermt2varv32hf3_maskz_1 = 8855,
  CODE_FOR_avx512vl_vpermt2varv16hf3 = 8856,
  CODE_FOR_avx512vl_vpermt2varv16hf3_maskz_1 = 8857,
  CODE_FOR_avx512fp16_vpermt2varv8hf3 = 8858,
  CODE_FOR_avx512fp16_vpermt2varv8hf3_maskz_1 = 8859,
  CODE_FOR_avx512bw_vpermt2varv32bf3 = 8860,
  CODE_FOR_avx512bw_vpermt2varv32bf3_maskz_1 = 8861,
  CODE_FOR_avx512vl_vpermt2varv16bf3 = 8862,
  CODE_FOR_avx512vl_vpermt2varv16bf3_maskz_1 = 8863,
  CODE_FOR_avx512vl_vpermt2varv8bf3 = 8864,
  CODE_FOR_avx512vl_vpermt2varv8bf3_maskz_1 = 8865,
  CODE_FOR_avx512f_vpermt2varv16si3_mask = 8866,
  CODE_FOR_avx512f_vpermt2varv16sf3_mask = 8867,
  CODE_FOR_avx512f_vpermt2varv8di3_mask = 8868,
  CODE_FOR_avx512f_vpermt2varv8df3_mask = 8869,
  CODE_FOR_avx512vl_vpermt2varv8si3_mask = 8870,
  CODE_FOR_avx512vl_vpermt2varv8sf3_mask = 8871,
  CODE_FOR_avx512vl_vpermt2varv4di3_mask = 8872,
  CODE_FOR_avx512vl_vpermt2varv4df3_mask = 8873,
  CODE_FOR_avx512vl_vpermt2varv4si3_mask = 8874,
  CODE_FOR_avx512vl_vpermt2varv4sf3_mask = 8875,
  CODE_FOR_avx512vl_vpermt2varv2di3_mask = 8876,
  CODE_FOR_avx512vl_vpermt2varv2df3_mask = 8877,
  CODE_FOR_avx512bw_vpermt2varv32hi3_mask = 8878,
  CODE_FOR_avx512vl_vpermt2varv16hi3_mask = 8879,
  CODE_FOR_avx512vl_vpermt2varv8hi3_mask = 8880,
  CODE_FOR_avx512bw_vpermt2varv64qi3_mask = 8881,
  CODE_FOR_avx512vl_vpermt2varv32qi3_mask = 8882,
  CODE_FOR_avx512vl_vpermt2varv16qi3_mask = 8883,
  CODE_FOR_vec_set_lo_v4di = 8898,
  CODE_FOR_vec_set_lo_v4di_mask = 8899,
  CODE_FOR_vec_set_lo_v4df = 8900,
  CODE_FOR_vec_set_lo_v4df_mask = 8901,
  CODE_FOR_vec_set_hi_v4di = 8902,
  CODE_FOR_vec_set_hi_v4di_mask = 8903,
  CODE_FOR_vec_set_hi_v4df = 8904,
  CODE_FOR_vec_set_hi_v4df_mask = 8905,
  CODE_FOR_vec_set_lo_v8si = 8906,
  CODE_FOR_vec_set_lo_v8si_mask = 8907,
  CODE_FOR_vec_set_lo_v8sf = 8908,
  CODE_FOR_vec_set_lo_v8sf_mask = 8909,
  CODE_FOR_vec_set_hi_v8si = 8910,
  CODE_FOR_vec_set_hi_v8si_mask = 8911,
  CODE_FOR_vec_set_hi_v8sf = 8912,
  CODE_FOR_vec_set_hi_v8sf_mask = 8913,
  CODE_FOR_vec_set_lo_v16hi = 8914,
  CODE_FOR_vec_set_lo_v16hf = 8915,
  CODE_FOR_vec_set_lo_v16bf = 8916,
  CODE_FOR_vec_set_hi_v16hi = 8917,
  CODE_FOR_vec_set_hi_v16hf = 8918,
  CODE_FOR_vec_set_hi_v16bf = 8919,
  CODE_FOR_vec_set_lo_v32qi = 8920,
  CODE_FOR_vec_set_hi_v32qi = 8921,
  CODE_FOR_avx_maskloadps = 8922,
  CODE_FOR_avx_maskloadpd = 8923,
  CODE_FOR_avx2_maskloadq256 = 8924,
  CODE_FOR_avx2_maskloadq = 8925,
  CODE_FOR_avx_maskloadps256 = 8926,
  CODE_FOR_avx_maskloadpd256 = 8927,
  CODE_FOR_avx2_maskloadd256 = 8928,
  CODE_FOR_avx2_maskloadd = 8929,
  CODE_FOR_avx_maskstoreps = 8930,
  CODE_FOR_avx_maskstorepd = 8931,
  CODE_FOR_avx2_maskstoreq256 = 8932,
  CODE_FOR_avx2_maskstoreq = 8933,
  CODE_FOR_avx_maskstoreps256 = 8934,
  CODE_FOR_avx_maskstorepd256 = 8935,
  CODE_FOR_avx2_maskstored256 = 8936,
  CODE_FOR_avx2_maskstored = 8937,
  CODE_FOR_avx512f_storev16si_mask = 8938,
  CODE_FOR_avx512vl_storev8si_mask = 8939,
  CODE_FOR_avx512vl_storev4si_mask = 8940,
  CODE_FOR_avx512f_storev8di_mask = 8941,
  CODE_FOR_avx512vl_storev4di_mask = 8942,
  CODE_FOR_avx512vl_storev2di_mask = 8943,
  CODE_FOR_avx512f_storev16sf_mask = 8944,
  CODE_FOR_avx512vl_storev8sf_mask = 8945,
  CODE_FOR_avx512vl_storev4sf_mask = 8946,
  CODE_FOR_avx512f_storev8df_mask = 8947,
  CODE_FOR_avx512vl_storev4df_mask = 8948,
  CODE_FOR_avx512vl_storev2df_mask = 8949,
  CODE_FOR_avx512bw_storev64qi_mask = 8950,
  CODE_FOR_avx512vl_storev16qi_mask = 8951,
  CODE_FOR_avx512vl_storev32qi_mask = 8952,
  CODE_FOR_avx512bw_storev32hi_mask = 8953,
  CODE_FOR_avx512vl_storev16hi_mask = 8954,
  CODE_FOR_avx512vl_storev8hi_mask = 8955,
  CODE_FOR_avx512bw_storev32hf_mask = 8956,
  CODE_FOR_avx512vl_storev16hf_mask = 8957,
  CODE_FOR_avx512fp16_storev8hf_mask = 8958,
  CODE_FOR_avx512bw_storev32bf_mask = 8959,
  CODE_FOR_avx512vl_storev16bf_mask = 8960,
  CODE_FOR_avx512vl_storev8bf_mask = 8961,
  CODE_FOR_avx_si256_si = 8986,
  CODE_FOR_avx_ps256_ps = 8987,
  CODE_FOR_avx_pd256_pd = 8988,
  CODE_FOR_avx2_ashrvv4si = 8989,
  CODE_FOR_avx2_ashrvv4si_mask = 8990,
  CODE_FOR_avx2_ashrvv8si = 8991,
  CODE_FOR_avx2_ashrvv8si_mask = 8992,
  CODE_FOR_avx512f_ashrvv16si = 8993,
  CODE_FOR_avx512f_ashrvv16si_mask = 8994,
  CODE_FOR_avx2_ashrvv2di = 8995,
  CODE_FOR_avx2_ashrvv2di_mask = 8996,
  CODE_FOR_avx2_ashrvv4di = 8997,
  CODE_FOR_avx2_ashrvv4di_mask = 8998,
  CODE_FOR_avx512f_ashrvv8di = 8999,
  CODE_FOR_avx512f_ashrvv8di_mask = 9000,
  CODE_FOR_avx512vl_ashrvv8hi = 9001,
  CODE_FOR_avx512vl_ashrvv8hi_mask = 9002,
  CODE_FOR_avx512vl_ashrvv16hi = 9003,
  CODE_FOR_avx512vl_ashrvv16hi_mask = 9004,
  CODE_FOR_avx512bw_ashrvv32hi = 9005,
  CODE_FOR_avx512bw_ashrvv32hi_mask = 9006,
  CODE_FOR_avx512f_ashlvv16si = 9007,
  CODE_FOR_avx512f_ashlvv16si_mask = 9008,
  CODE_FOR_avx512f_lshrvv16si = 9009,
  CODE_FOR_avx512f_lshrvv16si_mask = 9010,
  CODE_FOR_avx2_ashlvv8si = 9011,
  CODE_FOR_avx2_ashlvv8si_mask = 9012,
  CODE_FOR_avx2_lshrvv8si = 9013,
  CODE_FOR_avx2_lshrvv8si_mask = 9014,
  CODE_FOR_avx2_ashlvv4si = 9015,
  CODE_FOR_avx2_ashlvv4si_mask = 9016,
  CODE_FOR_avx2_lshrvv4si = 9017,
  CODE_FOR_avx2_lshrvv4si_mask = 9018,
  CODE_FOR_avx512f_ashlvv8di = 9019,
  CODE_FOR_avx512f_ashlvv8di_mask = 9020,
  CODE_FOR_avx512f_lshrvv8di = 9021,
  CODE_FOR_avx512f_lshrvv8di_mask = 9022,
  CODE_FOR_avx2_ashlvv4di = 9023,
  CODE_FOR_avx2_ashlvv4di_mask = 9024,
  CODE_FOR_avx2_lshrvv4di = 9025,
  CODE_FOR_avx2_lshrvv4di_mask = 9026,
  CODE_FOR_avx2_ashlvv2di = 9027,
  CODE_FOR_avx2_ashlvv2di_mask = 9028,
  CODE_FOR_avx2_lshrvv2di = 9029,
  CODE_FOR_avx2_lshrvv2di_mask = 9030,
  CODE_FOR_avx512vl_ashlvv8hi = 9031,
  CODE_FOR_avx512vl_ashlvv8hi_mask = 9032,
  CODE_FOR_avx512vl_lshrvv8hi = 9033,
  CODE_FOR_avx512vl_lshrvv8hi_mask = 9034,
  CODE_FOR_avx512vl_ashlvv16hi = 9035,
  CODE_FOR_avx512vl_ashlvv16hi_mask = 9036,
  CODE_FOR_avx512vl_lshrvv16hi = 9037,
  CODE_FOR_avx512vl_lshrvv16hi_mask = 9038,
  CODE_FOR_avx512bw_ashlvv32hi = 9039,
  CODE_FOR_avx512bw_ashlvv32hi_mask = 9040,
  CODE_FOR_avx512bw_lshrvv32hi = 9041,
  CODE_FOR_avx512bw_lshrvv32hi_mask = 9042,
  CODE_FOR_avx_vec_concatv32qi = 9043,
  CODE_FOR_avx_vec_concatv16hi = 9044,
  CODE_FOR_avx_vec_concatv16hf = 9045,
  CODE_FOR_avx_vec_concatv16bf = 9046,
  CODE_FOR_avx_vec_concatv8si = 9047,
  CODE_FOR_avx_vec_concatv4di = 9048,
  CODE_FOR_avx_vec_concatv8sf = 9049,
  CODE_FOR_avx_vec_concatv4df = 9050,
  CODE_FOR_avx_vec_concatv64qi = 9051,
  CODE_FOR_avx_vec_concatv32hi = 9052,
  CODE_FOR_avx_vec_concatv32hf = 9053,
  CODE_FOR_avx_vec_concatv32bf = 9054,
  CODE_FOR_avx_vec_concatv16si = 9055,
  CODE_FOR_avx_vec_concatv8di = 9056,
  CODE_FOR_avx_vec_concatv16sf = 9057,
  CODE_FOR_avx_vec_concatv8df = 9058,
  CODE_FOR_vcvtph2ps = 9083,
  CODE_FOR_vcvtph2ps_mask = 9084,
  CODE_FOR_vcvtph2ps256 = 9087,
  CODE_FOR_vcvtph2ps256_mask = 9088,
  CODE_FOR_avx512f_vcvtph2ps512_mask = 9091,
  CODE_FOR_avx512f_vcvtph2ps512_mask_round = 9092,
  CODE_FOR_vcvtps2ph256 = 9096,
  CODE_FOR_vcvtps2ph256_mask = 9097,
  CODE_FOR_avx512f_vcvtps2ph512_mask = 9102,
  CODE_FOR_avx512f_vcvtps2ph512_mask_round = 9103,
  CODE_FOR_avx512f_compressv16si_mask = 9322,
  CODE_FOR_avx512f_compressv16sf_mask = 9323,
  CODE_FOR_avx512f_compressv8di_mask = 9324,
  CODE_FOR_avx512f_compressv8df_mask = 9325,
  CODE_FOR_avx512vl_compressv8si_mask = 9326,
  CODE_FOR_avx512vl_compressv8sf_mask = 9327,
  CODE_FOR_avx512vl_compressv4di_mask = 9328,
  CODE_FOR_avx512vl_compressv4df_mask = 9329,
  CODE_FOR_avx512vl_compressv4si_mask = 9330,
  CODE_FOR_avx512vl_compressv4sf_mask = 9331,
  CODE_FOR_avx512vl_compressv2di_mask = 9332,
  CODE_FOR_avx512vl_compressv2df_mask = 9333,
  CODE_FOR_compressv64qi_mask = 9334,
  CODE_FOR_compressv16qi_mask = 9335,
  CODE_FOR_compressv32qi_mask = 9336,
  CODE_FOR_compressv32hi_mask = 9337,
  CODE_FOR_compressv16hi_mask = 9338,
  CODE_FOR_compressv8hi_mask = 9339,
  CODE_FOR_avx512f_compressstorev16si_mask = 9340,
  CODE_FOR_avx512f_compressstorev16sf_mask = 9341,
  CODE_FOR_avx512f_compressstorev8di_mask = 9342,
  CODE_FOR_avx512f_compressstorev8df_mask = 9343,
  CODE_FOR_avx512vl_compressstorev8si_mask = 9344,
  CODE_FOR_avx512vl_compressstorev8sf_mask = 9345,
  CODE_FOR_avx512vl_compressstorev4di_mask = 9346,
  CODE_FOR_avx512vl_compressstorev4df_mask = 9347,
  CODE_FOR_avx512vl_compressstorev4si_mask = 9348,
  CODE_FOR_avx512vl_compressstorev4sf_mask = 9349,
  CODE_FOR_avx512vl_compressstorev2di_mask = 9350,
  CODE_FOR_avx512vl_compressstorev2df_mask = 9351,
  CODE_FOR_compressstorev64qi_mask = 9352,
  CODE_FOR_compressstorev16qi_mask = 9353,
  CODE_FOR_compressstorev32qi_mask = 9354,
  CODE_FOR_compressstorev32hi_mask = 9355,
  CODE_FOR_compressstorev16hi_mask = 9356,
  CODE_FOR_compressstorev8hi_mask = 9357,
  CODE_FOR_expandv16si_mask = 9358,
  CODE_FOR_expandv16sf_mask = 9359,
  CODE_FOR_expandv8di_mask = 9360,
  CODE_FOR_expandv8df_mask = 9361,
  CODE_FOR_expandv8si_mask = 9362,
  CODE_FOR_expandv8sf_mask = 9363,
  CODE_FOR_expandv4di_mask = 9364,
  CODE_FOR_expandv4df_mask = 9365,
  CODE_FOR_expandv4si_mask = 9366,
  CODE_FOR_expandv4sf_mask = 9367,
  CODE_FOR_expandv2di_mask = 9368,
  CODE_FOR_expandv2df_mask = 9369,
  CODE_FOR_expandv64qi_mask = 9370,
  CODE_FOR_expandv16qi_mask = 9371,
  CODE_FOR_expandv32qi_mask = 9372,
  CODE_FOR_expandv32hi_mask = 9373,
  CODE_FOR_expandv16hi_mask = 9374,
  CODE_FOR_expandv8hi_mask = 9375,
  CODE_FOR_avx512dq_rangepv16sf = 9394,
  CODE_FOR_avx512dq_rangepv16sf_round = 9395,
  CODE_FOR_avx512dq_rangepv16sf_mask = 9396,
  CODE_FOR_avx512dq_rangepv16sf_mask_round = 9397,
  CODE_FOR_avx512dq_rangepv8sf = 9398,
   CODE_FOR_avx512dq_rangepv8sf_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv8sf_mask = 9399,
   CODE_FOR_avx512dq_rangepv8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv4sf = 9400,
   CODE_FOR_avx512dq_rangepv4sf_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv4sf_mask = 9401,
   CODE_FOR_avx512dq_rangepv4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv8df = 9402,
  CODE_FOR_avx512dq_rangepv8df_round = 9403,
  CODE_FOR_avx512dq_rangepv8df_mask = 9404,
  CODE_FOR_avx512dq_rangepv8df_mask_round = 9405,
  CODE_FOR_avx512dq_rangepv4df = 9406,
   CODE_FOR_avx512dq_rangepv4df_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv4df_mask = 9407,
   CODE_FOR_avx512dq_rangepv4df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv2df = 9408,
   CODE_FOR_avx512dq_rangepv2df_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv2df_mask = 9409,
   CODE_FOR_avx512dq_rangepv2df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangesv4sf = 9410,
  CODE_FOR_avx512dq_rangesv4sf_mask = 9411,
  CODE_FOR_avx512dq_rangesv4sf_round = 9412,
  CODE_FOR_avx512dq_rangesv4sf_mask_round = 9413,
  CODE_FOR_avx512dq_rangesv2df = 9414,
  CODE_FOR_avx512dq_rangesv2df_mask = 9415,
  CODE_FOR_avx512dq_rangesv2df_round = 9416,
  CODE_FOR_avx512dq_rangesv2df_mask_round = 9417,
  CODE_FOR_avx512dq_fpclassv32hf = 9418,
  CODE_FOR_avx512dq_fpclassv32hf_mask = 9419,
  CODE_FOR_avx512dq_fpclassv16hf = 9420,
  CODE_FOR_avx512dq_fpclassv16hf_mask = 9421,
  CODE_FOR_avx512dq_fpclassv8hf = 9422,
  CODE_FOR_avx512dq_fpclassv8hf_mask = 9423,
  CODE_FOR_avx512dq_fpclassv16sf = 9424,
  CODE_FOR_avx512dq_fpclassv16sf_mask = 9425,
  CODE_FOR_avx512dq_fpclassv8sf = 9426,
  CODE_FOR_avx512dq_fpclassv8sf_mask = 9427,
  CODE_FOR_avx512dq_fpclassv4sf = 9428,
  CODE_FOR_avx512dq_fpclassv4sf_mask = 9429,
  CODE_FOR_avx512dq_fpclassv8df = 9430,
  CODE_FOR_avx512dq_fpclassv8df_mask = 9431,
  CODE_FOR_avx512dq_fpclassv4df = 9432,
  CODE_FOR_avx512dq_fpclassv4df_mask = 9433,
  CODE_FOR_avx512dq_fpclassv2df = 9434,
  CODE_FOR_avx512dq_fpclassv2df_mask = 9435,
  CODE_FOR_avx512dq_vmfpclassv8hf = 9436,
  CODE_FOR_avx512dq_vmfpclassv8hf_mask = 9437,
  CODE_FOR_avx512dq_vmfpclassv4sf = 9438,
  CODE_FOR_avx512dq_vmfpclassv4sf_mask = 9439,
  CODE_FOR_avx512dq_vmfpclassv2df = 9440,
  CODE_FOR_avx512dq_vmfpclassv2df_mask = 9441,
  CODE_FOR_avx512bw_getmantv32hf = 9442,
  CODE_FOR_avx512bw_getmantv32hf_round = 9443,
  CODE_FOR_avx512bw_getmantv32hf_mask = 9444,
  CODE_FOR_avx512bw_getmantv32hf_mask_round = 9445,
  CODE_FOR_avx512vl_getmantv16hf = 9446,
  CODE_FOR_avx512vl_getmantv16hf_round = 9447,
  CODE_FOR_avx512vl_getmantv16hf_mask = 9448,
  CODE_FOR_avx512vl_getmantv16hf_mask_round = 9449,
  CODE_FOR_avx512fp16_getmantv8hf = 9450,
  CODE_FOR_avx512fp16_getmantv8hf_round = 9451,
  CODE_FOR_avx512fp16_getmantv8hf_mask = 9452,
  CODE_FOR_avx512fp16_getmantv8hf_mask_round = 9453,
  CODE_FOR_avx512f_getmantv16sf = 9454,
  CODE_FOR_avx512f_getmantv16sf_round = 9455,
  CODE_FOR_avx512f_getmantv16sf_mask = 9456,
  CODE_FOR_avx512f_getmantv16sf_mask_round = 9457,
  CODE_FOR_avx512vl_getmantv8sf = 9458,
  CODE_FOR_avx512vl_getmantv8sf_round = 9459,
  CODE_FOR_avx512vl_getmantv8sf_mask = 9460,
  CODE_FOR_avx512vl_getmantv8sf_mask_round = 9461,
  CODE_FOR_avx512vl_getmantv4sf = 9462,
  CODE_FOR_avx512vl_getmantv4sf_round = 9463,
  CODE_FOR_avx512vl_getmantv4sf_mask = 9464,
  CODE_FOR_avx512vl_getmantv4sf_mask_round = 9465,
  CODE_FOR_avx512f_getmantv8df = 9466,
  CODE_FOR_avx512f_getmantv8df_round = 9467,
  CODE_FOR_avx512f_getmantv8df_mask = 9468,
  CODE_FOR_avx512f_getmantv8df_mask_round = 9469,
  CODE_FOR_avx512vl_getmantv4df = 9470,
  CODE_FOR_avx512vl_getmantv4df_round = 9471,
  CODE_FOR_avx512vl_getmantv4df_mask = 9472,
  CODE_FOR_avx512vl_getmantv4df_mask_round = 9473,
  CODE_FOR_avx512vl_getmantv2df = 9474,
  CODE_FOR_avx512vl_getmantv2df_round = 9475,
  CODE_FOR_avx512vl_getmantv2df_mask = 9476,
  CODE_FOR_avx512vl_getmantv2df_mask_round = 9477,
  CODE_FOR_avx512f_vgetmantv8hf = 9478,
  CODE_FOR_avx512f_vgetmantv8hf_mask = 9479,
  CODE_FOR_avx512f_vgetmantv8hf_round = 9480,
  CODE_FOR_avx512f_vgetmantv8hf_mask_round = 9481,
  CODE_FOR_avx512f_vgetmantv4sf = 9482,
  CODE_FOR_avx512f_vgetmantv4sf_mask = 9483,
  CODE_FOR_avx512f_vgetmantv4sf_round = 9484,
  CODE_FOR_avx512f_vgetmantv4sf_mask_round = 9485,
  CODE_FOR_avx512f_vgetmantv2df = 9486,
  CODE_FOR_avx512f_vgetmantv2df_mask = 9487,
  CODE_FOR_avx512f_vgetmantv2df_round = 9488,
  CODE_FOR_avx512f_vgetmantv2df_mask_round = 9489,
  CODE_FOR_avx512bw_dbpsadbwv8hi_mask = 9491,
  CODE_FOR_avx512bw_dbpsadbwv16hi_mask = 9493,
  CODE_FOR_avx512bw_dbpsadbwv32hi_mask = 9495,
  CODE_FOR_clzv16si2 = 9496,
  CODE_FOR_clzv16si2_mask = 9497,
  CODE_FOR_clzv8si2 = 9498,
  CODE_FOR_clzv8si2_mask = 9499,
  CODE_FOR_clzv4si2 = 9500,
  CODE_FOR_clzv4si2_mask = 9501,
  CODE_FOR_clzv8di2 = 9502,
  CODE_FOR_clzv8di2_mask = 9503,
  CODE_FOR_clzv4di2 = 9504,
  CODE_FOR_clzv4di2_mask = 9505,
  CODE_FOR_clzv2di2 = 9506,
  CODE_FOR_clzv2di2_mask = 9507,
  CODE_FOR_conflictv16si_mask = 9509,
  CODE_FOR_conflictv8si_mask = 9511,
  CODE_FOR_conflictv4si_mask = 9513,
  CODE_FOR_conflictv8di_mask = 9515,
  CODE_FOR_conflictv4di_mask = 9517,
  CODE_FOR_conflictv2di_mask = 9519,
  CODE_FOR_sha1msg1 = 9520,
  CODE_FOR_sha1msg2 = 9521,
  CODE_FOR_sha1nexte = 9522,
  CODE_FOR_sha1rnds4 = 9523,
  CODE_FOR_sha256msg1 = 9524,
  CODE_FOR_sha256msg2 = 9525,
  CODE_FOR_sha256rnds2 = 9526,
  CODE_FOR_vsm3msg1 = 9527,
  CODE_FOR_vsm3msg2 = 9528,
  CODE_FOR_vsm3rnds2 = 9529,
  CODE_FOR_vsha512msg1 = 9530,
  CODE_FOR_vsha512msg2 = 9531,
  CODE_FOR_vsha512rnds2 = 9532,
  CODE_FOR_vsm4key4_v16si = 9533,
  CODE_FOR_vsm4key4_v8si = 9534,
  CODE_FOR_vsm4key4_v4si = 9535,
  CODE_FOR_vsm4rnds4_v16si = 9536,
  CODE_FOR_vsm4rnds4_v8si = 9537,
  CODE_FOR_vsm4rnds4_v4si = 9538,
  CODE_FOR_avx512f_si512_si = 9539,
  CODE_FOR_avx512f_ps512_ps = 9540,
  CODE_FOR_avx512f_pd512_pd = 9541,
  CODE_FOR_avx512f_si512_256si = 9542,
  CODE_FOR_avx512f_ps512_256ps = 9543,
  CODE_FOR_avx512f_pd512_256pd = 9544,
  CODE_FOR_vpmadd52luqv8di = 9545,
  CODE_FOR_vpmadd52huqv8di = 9546,
  CODE_FOR_vpmadd52luqv4di = 9547,
  CODE_FOR_vpmadd52huqv4di = 9548,
  CODE_FOR_vpmadd52luqv2di = 9549,
  CODE_FOR_vpmadd52huqv2di = 9550,
  CODE_FOR_vpmadd52luqv8di_maskz_1 = 9551,
  CODE_FOR_vpmadd52huqv8di_maskz_1 = 9552,
  CODE_FOR_vpmadd52luqv4di_maskz_1 = 9553,
  CODE_FOR_vpmadd52huqv4di_maskz_1 = 9554,
  CODE_FOR_vpmadd52luqv2di_maskz_1 = 9555,
  CODE_FOR_vpmadd52huqv2di_maskz_1 = 9556,
  CODE_FOR_vpmadd52luqv8di_mask = 9557,
  CODE_FOR_vpmadd52huqv8di_mask = 9558,
  CODE_FOR_vpmadd52luqv4di_mask = 9559,
  CODE_FOR_vpmadd52huqv4di_mask = 9560,
  CODE_FOR_vpmadd52luqv2di_mask = 9561,
  CODE_FOR_vpmadd52huqv2di_mask = 9562,
  CODE_FOR_vpmultishiftqbv64qi = 9563,
  CODE_FOR_vpmultishiftqbv64qi_mask = 9564,
  CODE_FOR_vpmultishiftqbv16qi = 9565,
  CODE_FOR_vpmultishiftqbv16qi_mask = 9566,
  CODE_FOR_vpmultishiftqbv32qi = 9567,
  CODE_FOR_vpmultishiftqbv32qi_mask = 9568,
  CODE_FOR_vpopcountv16si = 9569,
  CODE_FOR_vpopcountv16si_mask = 9570,
  CODE_FOR_vpopcountv8si = 9571,
  CODE_FOR_vpopcountv8si_mask = 9572,
  CODE_FOR_vpopcountv4si = 9573,
  CODE_FOR_vpopcountv4si_mask = 9574,
  CODE_FOR_vpopcountv8di = 9575,
  CODE_FOR_vpopcountv8di_mask = 9576,
  CODE_FOR_vpopcountv4di = 9577,
  CODE_FOR_vpopcountv4di_mask = 9578,
  CODE_FOR_vpopcountv2di = 9579,
  CODE_FOR_vpopcountv2di_mask = 9580,
  CODE_FOR_vpopcountv64qi = 9581,
  CODE_FOR_vpopcountv64qi_mask = 9582,
  CODE_FOR_vpopcountv16qi = 9583,
  CODE_FOR_vpopcountv16qi_mask = 9584,
  CODE_FOR_vpopcountv32qi = 9585,
  CODE_FOR_vpopcountv32qi_mask = 9586,
  CODE_FOR_vpopcountv32hi = 9587,
  CODE_FOR_vpopcountv32hi_mask = 9588,
  CODE_FOR_vpopcountv16hi = 9589,
  CODE_FOR_vpopcountv16hi_mask = 9590,
  CODE_FOR_vpopcountv8hi = 9591,
  CODE_FOR_vpopcountv8hi_mask = 9592,
  CODE_FOR_vgf2p8affineinvqb_v64qi = 9593,
  CODE_FOR_vgf2p8affineinvqb_v64qi_mask = 9594,
  CODE_FOR_vgf2p8affineinvqb_v32qi = 9595,
  CODE_FOR_vgf2p8affineinvqb_v32qi_mask = 9596,
  CODE_FOR_vgf2p8affineinvqb_v16qi = 9597,
  CODE_FOR_vgf2p8affineinvqb_v16qi_mask = 9598,
  CODE_FOR_vgf2p8affineqb_v64qi = 9599,
  CODE_FOR_vgf2p8affineqb_v64qi_mask = 9600,
  CODE_FOR_vgf2p8affineqb_v32qi = 9601,
  CODE_FOR_vgf2p8affineqb_v32qi_mask = 9602,
  CODE_FOR_vgf2p8affineqb_v16qi = 9603,
  CODE_FOR_vgf2p8affineqb_v16qi_mask = 9604,
  CODE_FOR_vgf2p8mulb_v64qi = 9605,
  CODE_FOR_vgf2p8mulb_v64qi_mask = 9606,
  CODE_FOR_vgf2p8mulb_v32qi = 9607,
  CODE_FOR_vgf2p8mulb_v32qi_mask = 9608,
  CODE_FOR_vgf2p8mulb_v16qi = 9609,
  CODE_FOR_vgf2p8mulb_v16qi_mask = 9610,
  CODE_FOR_vpshrd_v32hi = 9611,
  CODE_FOR_vpshrd_v32hi_mask = 9612,
  CODE_FOR_vpshrd_v16si = 9613,
  CODE_FOR_vpshrd_v16si_mask = 9614,
  CODE_FOR_vpshrd_v8di = 9615,
  CODE_FOR_vpshrd_v8di_mask = 9616,
  CODE_FOR_vpshrd_v16hi = 9617,
  CODE_FOR_vpshrd_v16hi_mask = 9618,
  CODE_FOR_vpshrd_v8si = 9619,
  CODE_FOR_vpshrd_v8si_mask = 9620,
  CODE_FOR_vpshrd_v4di = 9621,
  CODE_FOR_vpshrd_v4di_mask = 9622,
  CODE_FOR_vpshrd_v8hi = 9623,
  CODE_FOR_vpshrd_v8hi_mask = 9624,
  CODE_FOR_vpshrd_v4si = 9625,
  CODE_FOR_vpshrd_v4si_mask = 9626,
  CODE_FOR_vpshrd_v2di = 9627,
  CODE_FOR_vpshrd_v2di_mask = 9628,
  CODE_FOR_vpshld_v32hi = 9629,
  CODE_FOR_vpshld_v32hi_mask = 9630,
  CODE_FOR_vpshld_v16si = 9631,
  CODE_FOR_vpshld_v16si_mask = 9632,
  CODE_FOR_vpshld_v8di = 9633,
  CODE_FOR_vpshld_v8di_mask = 9634,
  CODE_FOR_vpshld_v16hi = 9635,
  CODE_FOR_vpshld_v16hi_mask = 9636,
  CODE_FOR_vpshld_v8si = 9637,
  CODE_FOR_vpshld_v8si_mask = 9638,
  CODE_FOR_vpshld_v4di = 9639,
  CODE_FOR_vpshld_v4di_mask = 9640,
  CODE_FOR_vpshld_v8hi = 9641,
  CODE_FOR_vpshld_v8hi_mask = 9642,
  CODE_FOR_vpshld_v4si = 9643,
  CODE_FOR_vpshld_v4si_mask = 9644,
  CODE_FOR_vpshld_v2di = 9645,
  CODE_FOR_vpshld_v2di_mask = 9646,
  CODE_FOR_vpshrdv_v32hi = 9647,
  CODE_FOR_vpshrdv_v16si = 9648,
  CODE_FOR_vpshrdv_v8di = 9649,
  CODE_FOR_vpshrdv_v16hi = 9650,
  CODE_FOR_vpshrdv_v8si = 9651,
  CODE_FOR_vpshrdv_v4di = 9652,
  CODE_FOR_vpshrdv_v8hi = 9653,
  CODE_FOR_vpshrdv_v4si = 9654,
  CODE_FOR_vpshrdv_v2di = 9655,
  CODE_FOR_vpshrdv_v32hi_mask = 9656,
  CODE_FOR_vpshrdv_v16si_mask = 9657,
  CODE_FOR_vpshrdv_v8di_mask = 9658,
  CODE_FOR_vpshrdv_v16hi_mask = 9659,
  CODE_FOR_vpshrdv_v8si_mask = 9660,
  CODE_FOR_vpshrdv_v4di_mask = 9661,
  CODE_FOR_vpshrdv_v8hi_mask = 9662,
  CODE_FOR_vpshrdv_v4si_mask = 9663,
  CODE_FOR_vpshrdv_v2di_mask = 9664,
  CODE_FOR_vpshrdv_v32hi_maskz_1 = 9665,
  CODE_FOR_vpshrdv_v16si_maskz_1 = 9666,
  CODE_FOR_vpshrdv_v8di_maskz_1 = 9667,
  CODE_FOR_vpshrdv_v16hi_maskz_1 = 9668,
  CODE_FOR_vpshrdv_v8si_maskz_1 = 9669,
  CODE_FOR_vpshrdv_v4di_maskz_1 = 9670,
  CODE_FOR_vpshrdv_v8hi_maskz_1 = 9671,
  CODE_FOR_vpshrdv_v4si_maskz_1 = 9672,
  CODE_FOR_vpshrdv_v2di_maskz_1 = 9673,
  CODE_FOR_vpshldv_v32hi = 9674,
  CODE_FOR_vpshldv_v16si = 9675,
  CODE_FOR_vpshldv_v8di = 9676,
  CODE_FOR_vpshldv_v16hi = 9677,
  CODE_FOR_vpshldv_v8si = 9678,
  CODE_FOR_vpshldv_v4di = 9679,
  CODE_FOR_vpshldv_v8hi = 9680,
  CODE_FOR_vpshldv_v4si = 9681,
  CODE_FOR_vpshldv_v2di = 9682,
  CODE_FOR_vpshldv_v32hi_mask = 9683,
  CODE_FOR_vpshldv_v16si_mask = 9684,
  CODE_FOR_vpshldv_v8di_mask = 9685,
  CODE_FOR_vpshldv_v16hi_mask = 9686,
  CODE_FOR_vpshldv_v8si_mask = 9687,
  CODE_FOR_vpshldv_v4di_mask = 9688,
  CODE_FOR_vpshldv_v8hi_mask = 9689,
  CODE_FOR_vpshldv_v4si_mask = 9690,
  CODE_FOR_vpshldv_v2di_mask = 9691,
  CODE_FOR_vpshldv_v32hi_maskz_1 = 9692,
  CODE_FOR_vpshldv_v16si_maskz_1 = 9693,
  CODE_FOR_vpshldv_v8di_maskz_1 = 9694,
  CODE_FOR_vpshldv_v16hi_maskz_1 = 9695,
  CODE_FOR_vpshldv_v8si_maskz_1 = 9696,
  CODE_FOR_vpshldv_v4di_maskz_1 = 9697,
  CODE_FOR_vpshldv_v8hi_maskz_1 = 9698,
  CODE_FOR_vpshldv_v4si_maskz_1 = 9699,
  CODE_FOR_vpshldv_v2di_maskz_1 = 9700,
  CODE_FOR_vpdpbusd_v16si = 9701,
  CODE_FOR_vpdpbusd_v8si = 9702,
  CODE_FOR_vpdpbusd_v4si = 9703,
  CODE_FOR_vpdpbusd_v16si_mask = 9704,
  CODE_FOR_vpdpbusd_v8si_mask = 9705,
  CODE_FOR_vpdpbusd_v4si_mask = 9706,
  CODE_FOR_vpdpbusd_v16si_maskz_1 = 9707,
  CODE_FOR_vpdpbusd_v8si_maskz_1 = 9708,
  CODE_FOR_vpdpbusd_v4si_maskz_1 = 9709,
  CODE_FOR_vpdpbusds_v16si = 9710,
  CODE_FOR_vpdpbusds_v8si = 9711,
  CODE_FOR_vpdpbusds_v4si = 9712,
  CODE_FOR_vpdpbusds_v16si_mask = 9713,
  CODE_FOR_vpdpbusds_v8si_mask = 9714,
  CODE_FOR_vpdpbusds_v4si_mask = 9715,
  CODE_FOR_vpdpbusds_v16si_maskz_1 = 9716,
  CODE_FOR_vpdpbusds_v8si_maskz_1 = 9717,
  CODE_FOR_vpdpbusds_v4si_maskz_1 = 9718,
  CODE_FOR_vpdpwssd_v16si = 9719,
  CODE_FOR_vpdpwssd_v8si = 9720,
  CODE_FOR_vpdpwssd_v4si = 9721,
  CODE_FOR_vpdpwssd_v16si_mask = 9722,
  CODE_FOR_vpdpwssd_v8si_mask = 9723,
  CODE_FOR_vpdpwssd_v4si_mask = 9724,
  CODE_FOR_vpdpwssd_v16si_maskz_1 = 9725,
  CODE_FOR_vpdpwssd_v8si_maskz_1 = 9726,
  CODE_FOR_vpdpwssd_v4si_maskz_1 = 9727,
  CODE_FOR_vpdpwssds_v16si = 9728,
  CODE_FOR_vpdpwssds_v8si = 9729,
  CODE_FOR_vpdpwssds_v4si = 9730,
  CODE_FOR_vpdpwssds_v16si_mask = 9731,
  CODE_FOR_vpdpwssds_v8si_mask = 9732,
  CODE_FOR_vpdpwssds_v4si_mask = 9733,
  CODE_FOR_vpdpwssds_v16si_maskz_1 = 9734,
  CODE_FOR_vpdpwssds_v8si_maskz_1 = 9735,
  CODE_FOR_vpdpwssds_v4si_maskz_1 = 9736,
  CODE_FOR_vaesdec_v32qi = 9737,
  CODE_FOR_vaesdec_v16qi = 9738,
  CODE_FOR_vaesdec_v64qi = 9739,
  CODE_FOR_vaesdeclast_v32qi = 9740,
  CODE_FOR_vaesdeclast_v16qi = 9741,
  CODE_FOR_vaesdeclast_v64qi = 9742,
  CODE_FOR_vaesenc_v32qi = 9743,
  CODE_FOR_vaesenc_v16qi = 9744,
  CODE_FOR_vaesenc_v64qi = 9745,
  CODE_FOR_vaesenclast_v32qi = 9746,
  CODE_FOR_vaesenclast_v16qi = 9747,
  CODE_FOR_vaesenclast_v64qi = 9748,
  CODE_FOR_vpclmulqdq_v8di = 9749,
  CODE_FOR_vpclmulqdq_v4di = 9750,
  CODE_FOR_vpclmulqdq_v2di = 9751,
  CODE_FOR_avx512vl_vpshufbitqmbv64qi = 9752,
  CODE_FOR_avx512vl_vpshufbitqmbv64qi_mask = 9753,
  CODE_FOR_avx512vl_vpshufbitqmbv16qi = 9754,
  CODE_FOR_avx512vl_vpshufbitqmbv16qi_mask = 9755,
  CODE_FOR_avx512vl_vpshufbitqmbv32qi = 9756,
  CODE_FOR_avx512vl_vpshufbitqmbv32qi_mask = 9757,
  CODE_FOR_avx512vp2intersect_2intersectv8di = 9760,
  CODE_FOR_avx512vp2intersect_2intersectv4di = 9761,
  CODE_FOR_avx512vp2intersect_2intersectv2di = 9762,
  CODE_FOR_avx512vp2intersect_2intersectv8si = 9763,
  CODE_FOR_avx512vp2intersect_2intersectv4si = 9764,
  CODE_FOR_avx512vp2intersect_2intersectv16si = 9765,
  CODE_FOR_avx512f_cvtne2ps2bf16_v32bf = 9766,
  CODE_FOR_avx512f_cvtne2ps2bf16_v32bf_mask = 9767,
  CODE_FOR_avx512f_cvtne2ps2bf16_v16bf = 9768,
  CODE_FOR_avx512f_cvtne2ps2bf16_v16bf_mask = 9769,
  CODE_FOR_avx512f_cvtne2ps2bf16_v8bf = 9770,
  CODE_FOR_avx512f_cvtne2ps2bf16_v8bf_mask = 9771,
  CODE_FOR_avx512f_cvtneps2bf16_v4sf_mask_1 = 9773,
  CODE_FOR_vcvtneps2bf16_v8sf = 9774,
  CODE_FOR_avx512f_cvtneps2bf16_v16sf = 9775,
  CODE_FOR_avx512f_cvtneps2bf16_v16sf_mask = 9776,
  CODE_FOR_avx512f_cvtneps2bf16_v8sf = 9777,
  CODE_FOR_avx512f_cvtneps2bf16_v8sf_mask = 9778,
  CODE_FOR_avx512f_dpbf16ps_v16sf = 9779,
  CODE_FOR_avx512f_dpbf16ps_v16sf_maskz_1 = 9780,
  CODE_FOR_avx512f_dpbf16ps_v8sf = 9781,
  CODE_FOR_avx512f_dpbf16ps_v8sf_maskz_1 = 9782,
  CODE_FOR_avx512f_dpbf16ps_v4sf = 9783,
  CODE_FOR_avx512f_dpbf16ps_v4sf_maskz_1 = 9784,
  CODE_FOR_avx512f_dpbf16ps_v16sf_mask = 9785,
  CODE_FOR_avx512f_dpbf16ps_v8sf_mask = 9786,
  CODE_FOR_avx512f_dpbf16ps_v4sf_mask = 9787,
  CODE_FOR_loadiwkey = 9788,
  CODE_FOR_aesdec128klu8 = 9791,
  CODE_FOR_aesdec256klu8 = 9792,
  CODE_FOR_aesenc128klu8 = 9793,
  CODE_FOR_aesenc256klu8 = 9794,
  CODE_FOR_vpdpbssd_v8si = 9799,
  CODE_FOR_vpdpbssds_v8si = 9800,
  CODE_FOR_vpdpbsud_v8si = 9801,
  CODE_FOR_vpdpbsuds_v8si = 9802,
  CODE_FOR_vpdpbuud_v8si = 9803,
  CODE_FOR_vpdpbuuds_v8si = 9804,
  CODE_FOR_vpdpbssd_v4si = 9805,
  CODE_FOR_vpdpbssds_v4si = 9806,
  CODE_FOR_vpdpbsud_v4si = 9807,
  CODE_FOR_vpdpbsuds_v4si = 9808,
  CODE_FOR_vpdpbuud_v4si = 9809,
  CODE_FOR_vpdpbuuds_v4si = 9810,
  CODE_FOR_vpdpbssd_v16si = 9811,
  CODE_FOR_vpdpbssds_v16si = 9812,
  CODE_FOR_vpdpbsud_v16si = 9813,
  CODE_FOR_vpdpbsuds_v16si = 9814,
  CODE_FOR_vpdpbuud_v16si = 9815,
  CODE_FOR_vpdpbuuds_v16si = 9816,
  CODE_FOR_vpdpbssd_v16si_mask = 9817,
  CODE_FOR_vpdpbssds_v16si_mask = 9818,
  CODE_FOR_vpdpbsud_v16si_mask = 9819,
  CODE_FOR_vpdpbsuds_v16si_mask = 9820,
  CODE_FOR_vpdpbuud_v16si_mask = 9821,
  CODE_FOR_vpdpbuuds_v16si_mask = 9822,
  CODE_FOR_vpdpbssd_v8si_mask = 9823,
  CODE_FOR_vpdpbssds_v8si_mask = 9824,
  CODE_FOR_vpdpbsud_v8si_mask = 9825,
  CODE_FOR_vpdpbsuds_v8si_mask = 9826,
  CODE_FOR_vpdpbuud_v8si_mask = 9827,
  CODE_FOR_vpdpbuuds_v8si_mask = 9828,
  CODE_FOR_vpdpbssd_v4si_mask = 9829,
  CODE_FOR_vpdpbssds_v4si_mask = 9830,
  CODE_FOR_vpdpbsud_v4si_mask = 9831,
  CODE_FOR_vpdpbsuds_v4si_mask = 9832,
  CODE_FOR_vpdpbuud_v4si_mask = 9833,
  CODE_FOR_vpdpbuuds_v4si_mask = 9834,
  CODE_FOR_vbcstnebf162ps_v8sf = 9853,
  CODE_FOR_vbcstnebf162ps_v4sf = 9854,
  CODE_FOR_vbcstnesh2ps_v8sf = 9855,
  CODE_FOR_vbcstnesh2ps_v4sf = 9856,
  CODE_FOR_vcvtneeph2ps_v8hf = 9857,
  CODE_FOR_vcvtneebf162ps_v8bf = 9858,
  CODE_FOR_vcvtneeph2ps_v16hf = 9859,
  CODE_FOR_vcvtneebf162ps_v16bf = 9860,
  CODE_FOR_vcvtneoph2ps_v8hf = 9861,
  CODE_FOR_vcvtneobf162ps_v8bf = 9862,
  CODE_FOR_vcvtneoph2ps_v16hf = 9863,
  CODE_FOR_vcvtneobf162ps_v16bf = 9864,
  CODE_FOR_avx10_2_cvt2ps2phx_v32hf = 9865,
  CODE_FOR_avx10_2_cvt2ps2phx_v32hf_round = 9866,
  CODE_FOR_avx10_2_cvt2ps2phx_v32hf_mask = 9867,
  CODE_FOR_avx10_2_cvt2ps2phx_v32hf_mask_round = 9868,
  CODE_FOR_avx10_2_cvt2ps2phx_v16hf = 9869,
   CODE_FOR_avx10_2_cvt2ps2phx_v16hf_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvt2ps2phx_v16hf_mask = 9870,
   CODE_FOR_avx10_2_cvt2ps2phx_v16hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvt2ps2phx_v8hf = 9871,
   CODE_FOR_avx10_2_cvt2ps2phx_v8hf_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvt2ps2phx_v8hf_mask = 9872,
   CODE_FOR_avx10_2_cvt2ps2phx_v8hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_vcvt2ph2bf8v32hf = 9873,
  CODE_FOR_vcvt2ph2bf8v32hf_mask = 9874,
  CODE_FOR_vcvt2ph2bf8sv32hf = 9875,
  CODE_FOR_vcvt2ph2bf8sv32hf_mask = 9876,
  CODE_FOR_vcvt2ph2hf8v32hf = 9877,
  CODE_FOR_vcvt2ph2hf8v32hf_mask = 9878,
  CODE_FOR_vcvt2ph2hf8sv32hf = 9879,
  CODE_FOR_vcvt2ph2hf8sv32hf_mask = 9880,
  CODE_FOR_vcvt2ph2bf8v16hf = 9881,
  CODE_FOR_vcvt2ph2bf8v16hf_mask = 9882,
  CODE_FOR_vcvt2ph2bf8sv16hf = 9883,
  CODE_FOR_vcvt2ph2bf8sv16hf_mask = 9884,
  CODE_FOR_vcvt2ph2hf8v16hf = 9885,
  CODE_FOR_vcvt2ph2hf8v16hf_mask = 9886,
  CODE_FOR_vcvt2ph2hf8sv16hf = 9887,
  CODE_FOR_vcvt2ph2hf8sv16hf_mask = 9888,
  CODE_FOR_vcvt2ph2bf8v8hf = 9889,
  CODE_FOR_vcvt2ph2bf8v8hf_mask = 9890,
  CODE_FOR_vcvt2ph2bf8sv8hf = 9891,
  CODE_FOR_vcvt2ph2bf8sv8hf_mask = 9892,
  CODE_FOR_vcvt2ph2hf8v8hf = 9893,
  CODE_FOR_vcvt2ph2hf8v8hf_mask = 9894,
  CODE_FOR_vcvt2ph2hf8sv8hf = 9895,
  CODE_FOR_vcvt2ph2hf8sv8hf_mask = 9896,
  CODE_FOR_vcvtbiasph2bf8v32hf = 9905,
  CODE_FOR_vcvtbiasph2bf8v32hf_mask = 9906,
  CODE_FOR_vcvtbiasph2bf8sv32hf = 9907,
  CODE_FOR_vcvtbiasph2bf8sv32hf_mask = 9908,
  CODE_FOR_vcvtbiasph2hf8v32hf = 9909,
  CODE_FOR_vcvtbiasph2hf8v32hf_mask = 9910,
  CODE_FOR_vcvtbiasph2hf8sv32hf = 9911,
  CODE_FOR_vcvtbiasph2hf8sv32hf_mask = 9912,
  CODE_FOR_vcvtbiasph2bf8v16hf = 9913,
  CODE_FOR_vcvtbiasph2bf8v16hf_mask = 9914,
  CODE_FOR_vcvtbiasph2bf8sv16hf = 9915,
  CODE_FOR_vcvtbiasph2bf8sv16hf_mask = 9916,
  CODE_FOR_vcvtbiasph2hf8v16hf = 9917,
  CODE_FOR_vcvtbiasph2hf8v16hf_mask = 9918,
  CODE_FOR_vcvtbiasph2hf8sv16hf = 9919,
  CODE_FOR_vcvtbiasph2hf8sv16hf_mask = 9920,
  CODE_FOR_vcvtph2bf8v16hf = 9929,
  CODE_FOR_vcvtph2bf8v16hf_mask = 9930,
  CODE_FOR_vcvtph2bf8sv16hf = 9931,
  CODE_FOR_vcvtph2bf8sv16hf_mask = 9932,
  CODE_FOR_vcvtph2hf8v16hf = 9933,
  CODE_FOR_vcvtph2hf8v16hf_mask = 9934,
  CODE_FOR_vcvtph2hf8sv16hf = 9935,
  CODE_FOR_vcvtph2hf8sv16hf_mask = 9936,
  CODE_FOR_vcvtph2bf8v32hf = 9937,
  CODE_FOR_vcvtph2bf8v32hf_mask = 9938,
  CODE_FOR_vcvtph2bf8sv32hf = 9939,
  CODE_FOR_vcvtph2bf8sv32hf_mask = 9940,
  CODE_FOR_vcvtph2hf8v32hf = 9941,
  CODE_FOR_vcvtph2hf8v32hf_mask = 9942,
  CODE_FOR_vcvtph2hf8sv32hf = 9943,
  CODE_FOR_vcvtph2hf8sv32hf_mask = 9944,
  CODE_FOR_vcvthf82phv32hf = 9945,
  CODE_FOR_vcvthf82phv32hf_mask = 9946,
  CODE_FOR_vcvthf82phv16hf = 9947,
  CODE_FOR_vcvthf82phv16hf_mask = 9948,
  CODE_FOR_vcvthf82phv8hf = 9949,
  CODE_FOR_vcvthf82phv8hf_mask = 9950,
  CODE_FOR_vpdpwusd_v8si = 9951,
  CODE_FOR_vpdpwusds_v8si = 9952,
  CODE_FOR_vpdpwsud_v8si = 9953,
  CODE_FOR_vpdpwsuds_v8si = 9954,
  CODE_FOR_vpdpwuud_v8si = 9955,
  CODE_FOR_vpdpwuuds_v8si = 9956,
  CODE_FOR_vpdpwusd_v4si = 9957,
  CODE_FOR_vpdpwusds_v4si = 9958,
  CODE_FOR_vpdpwsud_v4si = 9959,
  CODE_FOR_vpdpwsuds_v4si = 9960,
  CODE_FOR_vpdpwuud_v4si = 9961,
  CODE_FOR_vpdpwuuds_v4si = 9962,
  CODE_FOR_vpdpwusd_v16si = 9963,
  CODE_FOR_vpdpwusds_v16si = 9964,
  CODE_FOR_vpdpwsud_v16si = 9965,
  CODE_FOR_vpdpwsuds_v16si = 9966,
  CODE_FOR_vpdpwuud_v16si = 9967,
  CODE_FOR_vpdpwuuds_v16si = 9968,
  CODE_FOR_vpdpwusd_v16si_mask = 9969,
  CODE_FOR_vpdpwusds_v16si_mask = 9970,
  CODE_FOR_vpdpwsud_v16si_mask = 9971,
  CODE_FOR_vpdpwsuds_v16si_mask = 9972,
  CODE_FOR_vpdpwuud_v16si_mask = 9973,
  CODE_FOR_vpdpwuuds_v16si_mask = 9974,
  CODE_FOR_vpdpwusd_v8si_mask = 9975,
  CODE_FOR_vpdpwusds_v8si_mask = 9976,
  CODE_FOR_vpdpwsud_v8si_mask = 9977,
  CODE_FOR_vpdpwsuds_v8si_mask = 9978,
  CODE_FOR_vpdpwuud_v8si_mask = 9979,
  CODE_FOR_vpdpwuuds_v8si_mask = 9980,
  CODE_FOR_vpdpwusd_v4si_mask = 9981,
  CODE_FOR_vpdpwusds_v4si_mask = 9982,
  CODE_FOR_vpdpwsud_v4si_mask = 9983,
  CODE_FOR_vpdpwsuds_v4si_mask = 9984,
  CODE_FOR_vpdpwuud_v4si_mask = 9985,
  CODE_FOR_vpdpwuuds_v4si_mask = 9986,
  CODE_FOR_vdpphps_v16sf = 10005,
  CODE_FOR_vdpphps_v8sf = 10006,
  CODE_FOR_vdpphps_v4sf = 10007,
  CODE_FOR_vdpphps_v16sf_mask = 10008,
  CODE_FOR_vdpphps_v8sf_mask = 10009,
  CODE_FOR_vdpphps_v4sf_mask = 10010,
  CODE_FOR_vdpphps_v16sf_maskz_1 = 10011,
  CODE_FOR_vdpphps_v8sf_maskz_1 = 10012,
  CODE_FOR_vdpphps_v4sf_maskz_1 = 10013,
  CODE_FOR_avx10_2_scalefbf16_v32bf = 10014,
  CODE_FOR_avx10_2_scalefbf16_v32bf_mask = 10015,
  CODE_FOR_avx10_2_scalefbf16_v16bf = 10016,
  CODE_FOR_avx10_2_scalefbf16_v16bf_mask = 10017,
  CODE_FOR_avx10_2_scalefbf16_v8bf = 10018,
  CODE_FOR_avx10_2_scalefbf16_v8bf_mask = 10019,
  CODE_FOR_avx10_2_smaxbf16_v32bf = 10020,
  CODE_FOR_avx10_2_smaxbf16_v32bf_mask = 10021,
  CODE_FOR_avx10_2_sminbf16_v32bf = 10022,
  CODE_FOR_avx10_2_sminbf16_v32bf_mask = 10023,
  CODE_FOR_avx10_2_smaxbf16_v16bf = 10024,
  CODE_FOR_avx10_2_smaxbf16_v16bf_mask = 10025,
  CODE_FOR_avx10_2_sminbf16_v16bf = 10026,
  CODE_FOR_avx10_2_sminbf16_v16bf_mask = 10027,
  CODE_FOR_avx10_2_smaxbf16_v8bf = 10028,
  CODE_FOR_avx10_2_smaxbf16_v8bf_mask = 10029,
  CODE_FOR_avx10_2_sminbf16_v8bf = 10030,
  CODE_FOR_avx10_2_sminbf16_v8bf_mask = 10031,
  CODE_FOR_avx10_2_addbf16_v32bf = 10032,
  CODE_FOR_avx10_2_addbf16_v32bf_mask = 10033,
  CODE_FOR_avx10_2_subbf16_v32bf = 10034,
  CODE_FOR_avx10_2_subbf16_v32bf_mask = 10035,
  CODE_FOR_avx10_2_mulbf16_v32bf = 10036,
  CODE_FOR_avx10_2_mulbf16_v32bf_mask = 10037,
  CODE_FOR_avx10_2_divbf16_v32bf = 10038,
  CODE_FOR_avx10_2_divbf16_v32bf_mask = 10039,
  CODE_FOR_avx10_2_addbf16_v16bf = 10040,
  CODE_FOR_avx10_2_addbf16_v16bf_mask = 10041,
  CODE_FOR_avx10_2_subbf16_v16bf = 10042,
  CODE_FOR_avx10_2_subbf16_v16bf_mask = 10043,
  CODE_FOR_avx10_2_mulbf16_v16bf = 10044,
  CODE_FOR_avx10_2_mulbf16_v16bf_mask = 10045,
  CODE_FOR_avx10_2_divbf16_v16bf = 10046,
  CODE_FOR_avx10_2_divbf16_v16bf_mask = 10047,
  CODE_FOR_avx10_2_addbf16_v8bf = 10048,
  CODE_FOR_avx10_2_addbf16_v8bf_mask = 10049,
  CODE_FOR_avx10_2_subbf16_v8bf = 10050,
  CODE_FOR_avx10_2_subbf16_v8bf_mask = 10051,
  CODE_FOR_avx10_2_mulbf16_v8bf = 10052,
  CODE_FOR_avx10_2_mulbf16_v8bf_mask = 10053,
  CODE_FOR_avx10_2_divbf16_v8bf = 10054,
  CODE_FOR_avx10_2_divbf16_v8bf_mask = 10055,
  CODE_FOR_avx10_2_fmaddbf16_v32bf = 10056,
  CODE_FOR_avx10_2_fmaddbf16_v32bf_maskz_1 = 10057,
  CODE_FOR_avx10_2_fmaddbf16_v16bf = 10058,
  CODE_FOR_avx10_2_fmaddbf16_v16bf_maskz_1 = 10059,
  CODE_FOR_avx10_2_fmaddbf16_v8bf = 10060,
  CODE_FOR_avx10_2_fmaddbf16_v8bf_maskz_1 = 10061,
  CODE_FOR_avx10_2_fmaddbf16_v32bf_mask = 10062,
  CODE_FOR_avx10_2_fmaddbf16_v16bf_mask = 10063,
  CODE_FOR_avx10_2_fmaddbf16_v8bf_mask = 10064,
  CODE_FOR_avx10_2_fmaddbf16_v32bf_mask3 = 10065,
  CODE_FOR_avx10_2_fmaddbf16_v16bf_mask3 = 10066,
  CODE_FOR_avx10_2_fmaddbf16_v8bf_mask3 = 10067,
  CODE_FOR_avx10_2_fnmaddbf16_v32bf = 10068,
  CODE_FOR_avx10_2_fnmaddbf16_v32bf_maskz_1 = 10069,
  CODE_FOR_avx10_2_fnmaddbf16_v16bf = 10070,
  CODE_FOR_avx10_2_fnmaddbf16_v16bf_maskz_1 = 10071,
  CODE_FOR_avx10_2_fnmaddbf16_v8bf = 10072,
  CODE_FOR_avx10_2_fnmaddbf16_v8bf_maskz_1 = 10073,
  CODE_FOR_avx10_2_fnmaddbf16_v32bf_mask = 10074,
  CODE_FOR_avx10_2_fnmaddbf16_v16bf_mask = 10075,
  CODE_FOR_avx10_2_fnmaddbf16_v8bf_mask = 10076,
  CODE_FOR_avx10_2_fnmaddbf16_v32bf_mask3 = 10077,
  CODE_FOR_avx10_2_fnmaddbf16_v16bf_mask3 = 10078,
  CODE_FOR_avx10_2_fnmaddbf16_v8bf_mask3 = 10079,
  CODE_FOR_avx10_2_fmsubbf16_v32bf = 10080,
  CODE_FOR_avx10_2_fmsubbf16_v32bf_maskz_1 = 10081,
  CODE_FOR_avx10_2_fmsubbf16_v16bf = 10082,
  CODE_FOR_avx10_2_fmsubbf16_v16bf_maskz_1 = 10083,
  CODE_FOR_avx10_2_fmsubbf16_v8bf = 10084,
  CODE_FOR_avx10_2_fmsubbf16_v8bf_maskz_1 = 10085,
  CODE_FOR_avx10_2_fmsubbf16_v32bf_mask = 10086,
  CODE_FOR_avx10_2_fmsubbf16_v16bf_mask = 10087,
  CODE_FOR_avx10_2_fmsubbf16_v8bf_mask = 10088,
  CODE_FOR_avx10_2_fmsubbf16_v32bf_mask3 = 10089,
  CODE_FOR_avx10_2_fmsubbf16_v16bf_mask3 = 10090,
  CODE_FOR_avx10_2_fmsubbf16_v8bf_mask3 = 10091,
  CODE_FOR_avx10_2_fnmsubbf16_v32bf = 10092,
  CODE_FOR_avx10_2_fnmsubbf16_v32bf_maskz_1 = 10093,
  CODE_FOR_avx10_2_fnmsubbf16_v16bf = 10094,
  CODE_FOR_avx10_2_fnmsubbf16_v16bf_maskz_1 = 10095,
  CODE_FOR_avx10_2_fnmsubbf16_v8bf = 10096,
  CODE_FOR_avx10_2_fnmsubbf16_v8bf_maskz_1 = 10097,
  CODE_FOR_avx10_2_fnmsubbf16_v32bf_mask = 10098,
  CODE_FOR_avx10_2_fnmsubbf16_v16bf_mask = 10099,
  CODE_FOR_avx10_2_fnmsubbf16_v8bf_mask = 10100,
  CODE_FOR_avx10_2_fnmsubbf16_v32bf_mask3 = 10101,
  CODE_FOR_avx10_2_fnmsubbf16_v16bf_mask3 = 10102,
  CODE_FOR_avx10_2_fnmsubbf16_v8bf_mask3 = 10103,
  CODE_FOR_avx10_2_rsqrtbf16_v32bf = 10104,
  CODE_FOR_avx10_2_rsqrtbf16_v32bf_mask = 10105,
  CODE_FOR_avx10_2_rsqrtbf16_v16bf = 10106,
  CODE_FOR_avx10_2_rsqrtbf16_v16bf_mask = 10107,
  CODE_FOR_avx10_2_rsqrtbf16_v8bf = 10108,
  CODE_FOR_avx10_2_rsqrtbf16_v8bf_mask = 10109,
  CODE_FOR_avx10_2_sqrtbf16_v32bf = 10110,
  CODE_FOR_avx10_2_sqrtbf16_v32bf_mask = 10111,
  CODE_FOR_avx10_2_sqrtbf16_v16bf = 10112,
  CODE_FOR_avx10_2_sqrtbf16_v16bf_mask = 10113,
  CODE_FOR_avx10_2_sqrtbf16_v8bf = 10114,
  CODE_FOR_avx10_2_sqrtbf16_v8bf_mask = 10115,
  CODE_FOR_avx10_2_rcpbf16_v32bf = 10116,
  CODE_FOR_avx10_2_rcpbf16_v32bf_mask = 10117,
  CODE_FOR_avx10_2_rcpbf16_v16bf = 10118,
  CODE_FOR_avx10_2_rcpbf16_v16bf_mask = 10119,
  CODE_FOR_avx10_2_rcpbf16_v8bf = 10120,
  CODE_FOR_avx10_2_rcpbf16_v8bf_mask = 10121,
  CODE_FOR_avx10_2_getexpbf16_v32bf = 10122,
  CODE_FOR_avx10_2_getexpbf16_v32bf_mask = 10123,
  CODE_FOR_avx10_2_getexpbf16_v16bf = 10124,
  CODE_FOR_avx10_2_getexpbf16_v16bf_mask = 10125,
  CODE_FOR_avx10_2_getexpbf16_v8bf = 10126,
  CODE_FOR_avx10_2_getexpbf16_v8bf_mask = 10127,
  CODE_FOR_avx10_2_rndscalebf16_v32bf = 10128,
  CODE_FOR_avx10_2_rndscalebf16_v32bf_mask = 10129,
  CODE_FOR_avx10_2_reducebf16_v32bf = 10130,
  CODE_FOR_avx10_2_reducebf16_v32bf_mask = 10131,
  CODE_FOR_avx10_2_getmantbf16_v32bf = 10132,
  CODE_FOR_avx10_2_getmantbf16_v32bf_mask = 10133,
  CODE_FOR_avx10_2_rndscalebf16_v16bf = 10134,
  CODE_FOR_avx10_2_rndscalebf16_v16bf_mask = 10135,
  CODE_FOR_avx10_2_reducebf16_v16bf = 10136,
  CODE_FOR_avx10_2_reducebf16_v16bf_mask = 10137,
  CODE_FOR_avx10_2_getmantbf16_v16bf = 10138,
  CODE_FOR_avx10_2_getmantbf16_v16bf_mask = 10139,
  CODE_FOR_avx10_2_rndscalebf16_v8bf = 10140,
  CODE_FOR_avx10_2_rndscalebf16_v8bf_mask = 10141,
  CODE_FOR_avx10_2_reducebf16_v8bf = 10142,
  CODE_FOR_avx10_2_reducebf16_v8bf_mask = 10143,
  CODE_FOR_avx10_2_getmantbf16_v8bf = 10144,
  CODE_FOR_avx10_2_getmantbf16_v8bf_mask = 10145,
  CODE_FOR_avx10_2_fpclassbf16_v32bf = 10146,
  CODE_FOR_avx10_2_fpclassbf16_v32bf_mask = 10147,
  CODE_FOR_avx10_2_fpclassbf16_v16bf = 10148,
  CODE_FOR_avx10_2_fpclassbf16_v16bf_mask = 10149,
  CODE_FOR_avx10_2_fpclassbf16_v8bf = 10150,
  CODE_FOR_avx10_2_fpclassbf16_v8bf_mask = 10151,
  CODE_FOR_avx10_2_cmpbf16_v32bf = 10152,
  CODE_FOR_avx10_2_cmpbf16_v32bf_mask = 10153,
  CODE_FOR_avx10_2_cmpbf16_v16bf = 10154,
  CODE_FOR_avx10_2_cmpbf16_v16bf_mask = 10155,
  CODE_FOR_avx10_2_cmpbf16_v8bf = 10156,
  CODE_FOR_avx10_2_cmpbf16_v8bf_mask = 10157,
  CODE_FOR_avx10_2_cvtbf162ibsv32bf = 10158,
  CODE_FOR_avx10_2_cvtbf162ibsv32bf_mask = 10159,
  CODE_FOR_avx10_2_cvtbf162iubsv32bf = 10160,
  CODE_FOR_avx10_2_cvtbf162iubsv32bf_mask = 10161,
  CODE_FOR_avx10_2_cvttbf162ibsv32bf = 10162,
  CODE_FOR_avx10_2_cvttbf162ibsv32bf_mask = 10163,
  CODE_FOR_avx10_2_cvttbf162iubsv32bf = 10164,
  CODE_FOR_avx10_2_cvttbf162iubsv32bf_mask = 10165,
  CODE_FOR_avx10_2_cvtbf162ibsv16bf = 10166,
  CODE_FOR_avx10_2_cvtbf162ibsv16bf_mask = 10167,
  CODE_FOR_avx10_2_cvtbf162iubsv16bf = 10168,
  CODE_FOR_avx10_2_cvtbf162iubsv16bf_mask = 10169,
  CODE_FOR_avx10_2_cvttbf162ibsv16bf = 10170,
  CODE_FOR_avx10_2_cvttbf162ibsv16bf_mask = 10171,
  CODE_FOR_avx10_2_cvttbf162iubsv16bf = 10172,
  CODE_FOR_avx10_2_cvttbf162iubsv16bf_mask = 10173,
  CODE_FOR_avx10_2_cvtbf162ibsv8bf = 10174,
  CODE_FOR_avx10_2_cvtbf162ibsv8bf_mask = 10175,
  CODE_FOR_avx10_2_cvtbf162iubsv8bf = 10176,
  CODE_FOR_avx10_2_cvtbf162iubsv8bf_mask = 10177,
  CODE_FOR_avx10_2_cvttbf162ibsv8bf = 10178,
  CODE_FOR_avx10_2_cvttbf162ibsv8bf_mask = 10179,
  CODE_FOR_avx10_2_cvttbf162iubsv8bf = 10180,
  CODE_FOR_avx10_2_cvttbf162iubsv8bf_mask = 10181,
  CODE_FOR_avx10_2_cvtph2ibsv32hf = 10182,
  CODE_FOR_avx10_2_cvtph2ibsv32hf_round = 10183,
  CODE_FOR_avx10_2_cvtph2ibsv32hf_mask = 10184,
  CODE_FOR_avx10_2_cvtph2ibsv32hf_mask_round = 10185,
  CODE_FOR_avx10_2_cvtph2iubsv32hf = 10186,
  CODE_FOR_avx10_2_cvtph2iubsv32hf_round = 10187,
  CODE_FOR_avx10_2_cvtph2iubsv32hf_mask = 10188,
  CODE_FOR_avx10_2_cvtph2iubsv32hf_mask_round = 10189,
  CODE_FOR_avx10_2_cvtph2ibsv16hf = 10190,
   CODE_FOR_avx10_2_cvtph2ibsv16hf_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvtph2ibsv16hf_mask = 10191,
   CODE_FOR_avx10_2_cvtph2ibsv16hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvtph2iubsv16hf = 10192,
   CODE_FOR_avx10_2_cvtph2iubsv16hf_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvtph2iubsv16hf_mask = 10193,
   CODE_FOR_avx10_2_cvtph2iubsv16hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvtph2ibsv8hf = 10194,
   CODE_FOR_avx10_2_cvtph2ibsv8hf_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvtph2ibsv8hf_mask = 10195,
   CODE_FOR_avx10_2_cvtph2ibsv8hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvtph2iubsv8hf = 10196,
   CODE_FOR_avx10_2_cvtph2iubsv8hf_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvtph2iubsv8hf_mask = 10197,
   CODE_FOR_avx10_2_cvtph2iubsv8hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvttph2ibsv32hf = 10198,
  CODE_FOR_avx10_2_cvttph2ibsv32hf_round = 10199,
  CODE_FOR_avx10_2_cvttph2ibsv32hf_mask = 10200,
  CODE_FOR_avx10_2_cvttph2ibsv32hf_mask_round = 10201,
  CODE_FOR_avx10_2_cvttph2iubsv32hf = 10202,
  CODE_FOR_avx10_2_cvttph2iubsv32hf_round = 10203,
  CODE_FOR_avx10_2_cvttph2iubsv32hf_mask = 10204,
  CODE_FOR_avx10_2_cvttph2iubsv32hf_mask_round = 10205,
  CODE_FOR_avx10_2_cvttph2ibsv16hf = 10206,
   CODE_FOR_avx10_2_cvttph2ibsv16hf_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvttph2ibsv16hf_mask = 10207,
   CODE_FOR_avx10_2_cvttph2ibsv16hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvttph2iubsv16hf = 10208,
   CODE_FOR_avx10_2_cvttph2iubsv16hf_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvttph2iubsv16hf_mask = 10209,
   CODE_FOR_avx10_2_cvttph2iubsv16hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvttph2ibsv8hf = 10210,
   CODE_FOR_avx10_2_cvttph2ibsv8hf_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvttph2ibsv8hf_mask = 10211,
   CODE_FOR_avx10_2_cvttph2ibsv8hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvttph2iubsv8hf = 10212,
   CODE_FOR_avx10_2_cvttph2iubsv8hf_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvttph2iubsv8hf_mask = 10213,
   CODE_FOR_avx10_2_cvttph2iubsv8hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvtps2ibsv16sf = 10214,
  CODE_FOR_avx10_2_cvtps2ibsv16sf_round = 10215,
  CODE_FOR_avx10_2_cvtps2ibsv16sf_mask = 10216,
  CODE_FOR_avx10_2_cvtps2ibsv16sf_mask_round = 10217,
  CODE_FOR_avx10_2_cvtps2iubsv16sf = 10218,
  CODE_FOR_avx10_2_cvtps2iubsv16sf_round = 10219,
  CODE_FOR_avx10_2_cvtps2iubsv16sf_mask = 10220,
  CODE_FOR_avx10_2_cvtps2iubsv16sf_mask_round = 10221,
  CODE_FOR_avx10_2_cvtps2ibsv8sf = 10222,
   CODE_FOR_avx10_2_cvtps2ibsv8sf_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvtps2ibsv8sf_mask = 10223,
   CODE_FOR_avx10_2_cvtps2ibsv8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvtps2iubsv8sf = 10224,
   CODE_FOR_avx10_2_cvtps2iubsv8sf_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvtps2iubsv8sf_mask = 10225,
   CODE_FOR_avx10_2_cvtps2iubsv8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvtps2ibsv4sf = 10226,
   CODE_FOR_avx10_2_cvtps2ibsv4sf_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvtps2ibsv4sf_mask = 10227,
   CODE_FOR_avx10_2_cvtps2ibsv4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvtps2iubsv4sf = 10228,
   CODE_FOR_avx10_2_cvtps2iubsv4sf_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvtps2iubsv4sf_mask = 10229,
   CODE_FOR_avx10_2_cvtps2iubsv4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvttps2ibsv16sf = 10230,
  CODE_FOR_avx10_2_cvttps2ibsv16sf_round = 10231,
  CODE_FOR_avx10_2_cvttps2ibsv16sf_mask = 10232,
  CODE_FOR_avx10_2_cvttps2ibsv16sf_mask_round = 10233,
  CODE_FOR_avx10_2_cvttps2iubsv16sf = 10234,
  CODE_FOR_avx10_2_cvttps2iubsv16sf_round = 10235,
  CODE_FOR_avx10_2_cvttps2iubsv16sf_mask = 10236,
  CODE_FOR_avx10_2_cvttps2iubsv16sf_mask_round = 10237,
  CODE_FOR_avx10_2_cvttps2ibsv8sf = 10238,
   CODE_FOR_avx10_2_cvttps2ibsv8sf_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvttps2ibsv8sf_mask = 10239,
   CODE_FOR_avx10_2_cvttps2ibsv8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvttps2iubsv8sf = 10240,
   CODE_FOR_avx10_2_cvttps2iubsv8sf_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvttps2iubsv8sf_mask = 10241,
   CODE_FOR_avx10_2_cvttps2iubsv8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvttps2ibsv4sf = 10242,
   CODE_FOR_avx10_2_cvttps2ibsv4sf_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvttps2ibsv4sf_mask = 10243,
   CODE_FOR_avx10_2_cvttps2ibsv4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvttps2iubsv4sf = 10244,
   CODE_FOR_avx10_2_cvttps2iubsv4sf_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_cvttps2iubsv4sf_mask = 10245,
   CODE_FOR_avx10_2_cvttps2iubsv4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttps2dqsv16sf = 10246,
  CODE_FOR_avx10_2_vcvttps2dqsv16sf_round = 10247,
  CODE_FOR_avx10_2_vcvttps2dqsv16sf_mask = 10248,
  CODE_FOR_avx10_2_vcvttps2dqsv16sf_mask_round = 10249,
  CODE_FOR_avx10_2_vcvttps2udqsv16sf = 10250,
  CODE_FOR_avx10_2_vcvttps2udqsv16sf_round = 10251,
  CODE_FOR_avx10_2_vcvttps2udqsv16sf_mask = 10252,
  CODE_FOR_avx10_2_vcvttps2udqsv16sf_mask_round = 10253,
  CODE_FOR_avx10_2_vcvttps2dqsv8sf = 10254,
   CODE_FOR_avx10_2_vcvttps2dqsv8sf_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttps2dqsv8sf_mask = 10255,
   CODE_FOR_avx10_2_vcvttps2dqsv8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttps2udqsv8sf = 10256,
   CODE_FOR_avx10_2_vcvttps2udqsv8sf_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttps2udqsv8sf_mask = 10257,
   CODE_FOR_avx10_2_vcvttps2udqsv8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttps2dqsv4sf = 10258,
   CODE_FOR_avx10_2_vcvttps2dqsv4sf_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttps2dqsv4sf_mask = 10259,
   CODE_FOR_avx10_2_vcvttps2dqsv4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttps2udqsv4sf = 10260,
   CODE_FOR_avx10_2_vcvttps2udqsv4sf_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttps2udqsv4sf_mask = 10261,
   CODE_FOR_avx10_2_vcvttps2udqsv4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttpd2dqsv8df = 10262,
  CODE_FOR_avx10_2_vcvttpd2dqsv8df_round = 10263,
  CODE_FOR_avx10_2_vcvttpd2dqsv8df_mask = 10264,
  CODE_FOR_avx10_2_vcvttpd2dqsv8df_mask_round = 10265,
  CODE_FOR_avx10_2_vcvttpd2udqsv8df = 10266,
  CODE_FOR_avx10_2_vcvttpd2udqsv8df_round = 10267,
  CODE_FOR_avx10_2_vcvttpd2udqsv8df_mask = 10268,
  CODE_FOR_avx10_2_vcvttpd2udqsv8df_mask_round = 10269,
  CODE_FOR_avx10_2_vcvttpd2dqsv4df = 10270,
   CODE_FOR_avx10_2_vcvttpd2dqsv4df_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttpd2dqsv4df_mask = 10271,
   CODE_FOR_avx10_2_vcvttpd2dqsv4df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttpd2udqsv4df = 10272,
   CODE_FOR_avx10_2_vcvttpd2udqsv4df_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttpd2udqsv4df_mask = 10273,
   CODE_FOR_avx10_2_vcvttpd2udqsv4df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttpd2dqsv2df = 10274,
   CODE_FOR_avx10_2_vcvttpd2dqsv2df_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttpd2dqsv2df_mask = 10275,
   CODE_FOR_avx10_2_vcvttpd2dqsv2df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttpd2udqsv2df = 10276,
   CODE_FOR_avx10_2_vcvttpd2udqsv2df_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttpd2udqsv2df_mask = 10277,
   CODE_FOR_avx10_2_vcvttpd2udqsv2df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttpd2qqsv8df = 10278,
  CODE_FOR_avx10_2_vcvttpd2qqsv8df_round = 10279,
  CODE_FOR_avx10_2_vcvttpd2qqsv8df_mask = 10280,
  CODE_FOR_avx10_2_vcvttpd2qqsv8df_mask_round = 10281,
  CODE_FOR_avx10_2_vcvttpd2uqqsv8df = 10282,
  CODE_FOR_avx10_2_vcvttpd2uqqsv8df_round = 10283,
  CODE_FOR_avx10_2_vcvttpd2uqqsv8df_mask = 10284,
  CODE_FOR_avx10_2_vcvttpd2uqqsv8df_mask_round = 10285,
  CODE_FOR_avx10_2_vcvttpd2qqsv4df = 10286,
   CODE_FOR_avx10_2_vcvttpd2qqsv4df_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttpd2qqsv4df_mask = 10287,
   CODE_FOR_avx10_2_vcvttpd2qqsv4df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttpd2uqqsv4df = 10288,
   CODE_FOR_avx10_2_vcvttpd2uqqsv4df_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttpd2uqqsv4df_mask = 10289,
   CODE_FOR_avx10_2_vcvttpd2uqqsv4df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttpd2qqsv2df = 10290,
   CODE_FOR_avx10_2_vcvttpd2qqsv2df_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttpd2qqsv2df_mask = 10291,
   CODE_FOR_avx10_2_vcvttpd2qqsv2df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttpd2uqqsv2df = 10292,
   CODE_FOR_avx10_2_vcvttpd2uqqsv2df_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttpd2uqqsv2df_mask = 10293,
   CODE_FOR_avx10_2_vcvttpd2uqqsv2df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttps2qqsv8di = 10294,
  CODE_FOR_avx10_2_vcvttps2qqsv8di_round = 10295,
  CODE_FOR_avx10_2_vcvttps2qqsv8di_mask = 10296,
  CODE_FOR_avx10_2_vcvttps2qqsv8di_mask_round = 10297,
  CODE_FOR_avx10_2_vcvttps2uqqsv8di = 10298,
  CODE_FOR_avx10_2_vcvttps2uqqsv8di_round = 10299,
  CODE_FOR_avx10_2_vcvttps2uqqsv8di_mask = 10300,
  CODE_FOR_avx10_2_vcvttps2uqqsv8di_mask_round = 10301,
  CODE_FOR_avx10_2_vcvttps2qqsv4di = 10302,
   CODE_FOR_avx10_2_vcvttps2qqsv4di_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttps2qqsv4di_mask = 10303,
   CODE_FOR_avx10_2_vcvttps2qqsv4di_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttps2uqqsv4di = 10304,
   CODE_FOR_avx10_2_vcvttps2uqqsv4di_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttps2uqqsv4di_mask = 10305,
   CODE_FOR_avx10_2_vcvttps2uqqsv4di_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttps2qqsv2di = 10306,
   CODE_FOR_avx10_2_vcvttps2qqsv2di_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttps2qqsv2di_mask = 10307,
   CODE_FOR_avx10_2_vcvttps2qqsv2di_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttps2uqqsv2di = 10308,
   CODE_FOR_avx10_2_vcvttps2uqqsv2di_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttps2uqqsv2di_mask = 10309,
   CODE_FOR_avx10_2_vcvttps2uqqsv2di_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttsd2sissi = 10310,
  CODE_FOR_avx10_2_vcvttsd2sissi_round = 10311,
  CODE_FOR_avx10_2_vcvttsd2usissi = 10312,
  CODE_FOR_avx10_2_vcvttsd2usissi_round = 10313,
   CODE_FOR_avx10_2_vcvttsd2sisdi = CODE_FOR_nothing,
   CODE_FOR_avx10_2_vcvttsd2sisdi_round = CODE_FOR_nothing,
   CODE_FOR_avx10_2_vcvttsd2usisdi = CODE_FOR_nothing,
   CODE_FOR_avx10_2_vcvttsd2usisdi_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_vcvttss2sissi = 10314,
  CODE_FOR_avx10_2_vcvttss2sissi_round = 10315,
  CODE_FOR_avx10_2_vcvttss2usissi = 10316,
  CODE_FOR_avx10_2_vcvttss2usissi_round = 10317,
   CODE_FOR_avx10_2_vcvttss2sisdi = CODE_FOR_nothing,
   CODE_FOR_avx10_2_vcvttss2sisdi_round = CODE_FOR_nothing,
   CODE_FOR_avx10_2_vcvttss2usisdi = CODE_FOR_nothing,
   CODE_FOR_avx10_2_vcvttss2usisdi_round = CODE_FOR_nothing,
  CODE_FOR_avx10_2_minmaxbf16_v32bf = 10318,
  CODE_FOR_avx10_2_minmaxbf16_v32bf_mask = 10319,
  CODE_FOR_avx10_2_minmaxbf16_v16bf = 10320,
  CODE_FOR_avx10_2_minmaxbf16_v16bf_mask = 10321,
  CODE_FOR_avx10_2_minmaxbf16_v8bf = 10322,
  CODE_FOR_avx10_2_minmaxbf16_v8bf_mask = 10323,
  CODE_FOR_avx10_2_minmaxpv32hf = 10324,
  CODE_FOR_avx10_2_minmaxpv32hf_round = 10325,
  CODE_FOR_avx10_2_minmaxpv32hf_mask = 10326,
  CODE_FOR_avx10_2_minmaxpv32hf_mask_round = 10327,
  CODE_FOR_avx10_2_minmaxpv16hf = 10328,
  CODE_FOR_avx10_2_minmaxpv16hf_round = 10329,
  CODE_FOR_avx10_2_minmaxpv16hf_mask = 10330,
  CODE_FOR_avx10_2_minmaxpv16hf_mask_round = 10331,
  CODE_FOR_avx10_2_minmaxpv8hf = 10332,
  CODE_FOR_avx10_2_minmaxpv8hf_round = 10333,
  CODE_FOR_avx10_2_minmaxpv8hf_mask = 10334,
  CODE_FOR_avx10_2_minmaxpv8hf_mask_round = 10335,
  CODE_FOR_avx10_2_minmaxpv16sf = 10336,
  CODE_FOR_avx10_2_minmaxpv16sf_round = 10337,
  CODE_FOR_avx10_2_minmaxpv16sf_mask = 10338,
  CODE_FOR_avx10_2_minmaxpv16sf_mask_round = 10339,
  CODE_FOR_avx10_2_minmaxpv8sf = 10340,
  CODE_FOR_avx10_2_minmaxpv8sf_round = 10341,
  CODE_FOR_avx10_2_minmaxpv8sf_mask = 10342,
  CODE_FOR_avx10_2_minmaxpv8sf_mask_round = 10343,
  CODE_FOR_avx10_2_minmaxpv4sf = 10344,
  CODE_FOR_avx10_2_minmaxpv4sf_round = 10345,
  CODE_FOR_avx10_2_minmaxpv4sf_mask = 10346,
  CODE_FOR_avx10_2_minmaxpv4sf_mask_round = 10347,
  CODE_FOR_avx10_2_minmaxpv8df = 10348,
  CODE_FOR_avx10_2_minmaxpv8df_round = 10349,
  CODE_FOR_avx10_2_minmaxpv8df_mask = 10350,
  CODE_FOR_avx10_2_minmaxpv8df_mask_round = 10351,
  CODE_FOR_avx10_2_minmaxpv4df = 10352,
  CODE_FOR_avx10_2_minmaxpv4df_round = 10353,
  CODE_FOR_avx10_2_minmaxpv4df_mask = 10354,
  CODE_FOR_avx10_2_minmaxpv4df_mask_round = 10355,
  CODE_FOR_avx10_2_minmaxpv2df = 10356,
  CODE_FOR_avx10_2_minmaxpv2df_round = 10357,
  CODE_FOR_avx10_2_minmaxpv2df_mask = 10358,
  CODE_FOR_avx10_2_minmaxpv2df_mask_round = 10359,
  CODE_FOR_avx10_2_minmaxsv8hf = 10360,
  CODE_FOR_avx10_2_minmaxsv8hf_mask = 10361,
  CODE_FOR_avx10_2_minmaxsv8hf_round = 10362,
  CODE_FOR_avx10_2_minmaxsv8hf_mask_round = 10363,
  CODE_FOR_avx10_2_minmaxsv4sf = 10364,
  CODE_FOR_avx10_2_minmaxsv4sf_mask = 10365,
  CODE_FOR_avx10_2_minmaxsv4sf_round = 10366,
  CODE_FOR_avx10_2_minmaxsv4sf_mask_round = 10367,
  CODE_FOR_avx10_2_minmaxsv2df = 10368,
  CODE_FOR_avx10_2_minmaxsv2df_mask = 10369,
  CODE_FOR_avx10_2_minmaxsv2df_round = 10370,
  CODE_FOR_avx10_2_minmaxsv2df_mask_round = 10371,
  CODE_FOR_avx10_2_vmovrsbv64qi = 10372,
  CODE_FOR_avx10_2_vmovrsbv64qi_mask = 10373,
  CODE_FOR_avx10_2_vmovrsbv32qi = 10374,
  CODE_FOR_avx10_2_vmovrsbv32qi_mask = 10375,
  CODE_FOR_avx10_2_vmovrsbv16qi = 10376,
  CODE_FOR_avx10_2_vmovrsbv16qi_mask = 10377,
  CODE_FOR_avx10_2_vmovrswv32hi = 10378,
  CODE_FOR_avx10_2_vmovrswv32hi_mask = 10379,
  CODE_FOR_avx10_2_vmovrswv16hi = 10380,
  CODE_FOR_avx10_2_vmovrswv16hi_mask = 10381,
  CODE_FOR_avx10_2_vmovrswv8hi = 10382,
  CODE_FOR_avx10_2_vmovrswv8hi_mask = 10383,
  CODE_FOR_avx10_2_vmovrsdv16si = 10384,
  CODE_FOR_avx10_2_vmovrsdv16si_mask = 10385,
  CODE_FOR_avx10_2_vmovrsdv8si = 10386,
  CODE_FOR_avx10_2_vmovrsdv8si_mask = 10387,
  CODE_FOR_avx10_2_vmovrsdv4si = 10388,
  CODE_FOR_avx10_2_vmovrsdv4si_mask = 10389,
  CODE_FOR_avx10_2_vmovrsqv8di = 10390,
  CODE_FOR_avx10_2_vmovrsqv8di_mask = 10391,
  CODE_FOR_avx10_2_vmovrsqv4di = 10392,
  CODE_FOR_avx10_2_vmovrsqv4di_mask = 10393,
  CODE_FOR_avx10_2_vmovrsqv2di = 10394,
  CODE_FOR_avx10_2_vmovrsqv2di_mask = 10395,
  CODE_FOR_avx512bmm_vbmacor16x16x16_v32hi = 10396,
  CODE_FOR_avx512bmm_vbmacor16x16x16_v16hi = 10397,
  CODE_FOR_avx512bmm_vbmacxor16x16x16_v32hi = 10398,
  CODE_FOR_avx512bmm_vbmacxor16x16x16_v16hi = 10399,
  CODE_FOR_avx512bmm_vbitrevb_v64qi_mask = 10400,
  CODE_FOR_avx512bmm_vbitrevb_v16qi_mask = 10401,
  CODE_FOR_avx512bmm_vbitrevb_v32qi_mask = 10402,
  CODE_FOR_avx512bmm_vbitrevb_v64qi = 10403,
  CODE_FOR_avx512bmm_vbitrevb_v16qi = 10404,
  CODE_FOR_avx512bmm_vbitrevb_v32qi = 10405,
  CODE_FOR_mfence_sse2 = 10408,
  CODE_FOR_mfence_nosse = 10409,
  CODE_FOR_atomic_loaddi_fpu = 10410,
  CODE_FOR_atomic_storeqi_1 = 10411,
  CODE_FOR_atomic_storehi_1 = 10412,
  CODE_FOR_atomic_storesi_1 = 10413,
   CODE_FOR_atomic_storedi_1 = CODE_FOR_nothing,
  CODE_FOR_atomic_storedi_fpu = 10414,
  CODE_FOR_loaddi_via_fpu = 10415,
  CODE_FOR_storedi_via_fpu = 10416,
  CODE_FOR_loaddi_via_sse = 10417,
  CODE_FOR_storedi_via_sse = 10418,
  CODE_FOR_atomic_compare_and_swapdi_doubleword = 10419,
   CODE_FOR_atomic_compare_and_swapti_doubleword = CODE_FOR_nothing,
  CODE_FOR_atomic_compare_and_swapqi_1 = 10420,
  CODE_FOR_atomic_compare_and_swaphi_1 = 10421,
  CODE_FOR_atomic_compare_and_swapsi_1 = 10422,
   CODE_FOR_atomic_compare_and_swapdi_1 = CODE_FOR_nothing,
  CODE_FOR_atomic_fetch_addqi = 10423,
  CODE_FOR_atomic_fetch_addhi = 10424,
  CODE_FOR_atomic_fetch_addsi = 10425,
   CODE_FOR_atomic_fetch_adddi = CODE_FOR_nothing,
  CODE_FOR_atomic_exchangeqi = 10429,
  CODE_FOR_atomic_exchangehi = 10430,
  CODE_FOR_atomic_exchangesi = 10431,
   CODE_FOR_atomic_exchangedi = CODE_FOR_nothing,
  CODE_FOR_rao_aandsi = 10432,
  CODE_FOR_rao_aorsi = 10433,
  CODE_FOR_rao_axorsi = 10434,
  CODE_FOR_rao_aaddsi = 10435,
   CODE_FOR_rao_aanddi = CODE_FOR_nothing,
   CODE_FOR_rao_aordi = CODE_FOR_nothing,
   CODE_FOR_rao_axordi = CODE_FOR_nothing,
   CODE_FOR_rao_aadddi = CODE_FOR_nothing,
  CODE_FOR_atomic_addqi = 10436,
  CODE_FOR_atomic_addhi = 10437,
  CODE_FOR_atomic_addsi = 10438,
   CODE_FOR_atomic_adddi = CODE_FOR_nothing,
  CODE_FOR_atomic_subqi = 10439,
  CODE_FOR_atomic_subhi = 10440,
  CODE_FOR_atomic_subsi = 10441,
   CODE_FOR_atomic_subdi = CODE_FOR_nothing,
  CODE_FOR_atomic_andqi = 10442,
  CODE_FOR_atomic_orqi = 10443,
  CODE_FOR_atomic_xorqi = 10444,
  CODE_FOR_atomic_andhi = 10445,
  CODE_FOR_atomic_orhi = 10446,
  CODE_FOR_atomic_xorhi = 10447,
  CODE_FOR_atomic_andsi = 10448,
  CODE_FOR_atomic_orsi = 10449,
  CODE_FOR_atomic_xorsi = 10450,
   CODE_FOR_atomic_anddi = CODE_FOR_nothing,
   CODE_FOR_atomic_ordi = CODE_FOR_nothing,
   CODE_FOR_atomic_xordi = CODE_FOR_nothing,
  CODE_FOR_atomic_bit_test_and_sethi_1 = 10451,
  CODE_FOR_atomic_bit_test_and_setsi_1 = 10452,
   CODE_FOR_atomic_bit_test_and_setdi_1 = CODE_FOR_nothing,
  CODE_FOR_atomic_bit_test_and_complementhi_1 = 10453,
  CODE_FOR_atomic_bit_test_and_complementsi_1 = 10454,
   CODE_FOR_atomic_bit_test_and_complementdi_1 = CODE_FOR_nothing,
  CODE_FOR_atomic_bit_test_and_resethi_1 = 10455,
  CODE_FOR_atomic_bit_test_and_resetsi_1 = 10456,
   CODE_FOR_atomic_bit_test_and_resetdi_1 = CODE_FOR_nothing,
  CODE_FOR_atomic_add_fetch_cmp_0qi_1 = 10457,
  CODE_FOR_atomic_add_fetch_cmp_0hi_1 = 10458,
  CODE_FOR_atomic_add_fetch_cmp_0si_1 = 10459,
   CODE_FOR_atomic_add_fetch_cmp_0di_1 = CODE_FOR_nothing,
  CODE_FOR_atomic_sub_fetch_cmp_0qi_1 = 10460,
  CODE_FOR_atomic_sub_fetch_cmp_0hi_1 = 10461,
  CODE_FOR_atomic_sub_fetch_cmp_0si_1 = 10462,
   CODE_FOR_atomic_sub_fetch_cmp_0di_1 = CODE_FOR_nothing,
  CODE_FOR_atomic_and_fetch_cmp_0qi_1 = 10463,
  CODE_FOR_atomic_or_fetch_cmp_0qi_1 = 10464,
  CODE_FOR_atomic_xor_fetch_cmp_0qi_1 = 10465,
  CODE_FOR_atomic_and_fetch_cmp_0hi_1 = 10466,
  CODE_FOR_atomic_or_fetch_cmp_0hi_1 = 10467,
  CODE_FOR_atomic_xor_fetch_cmp_0hi_1 = 10468,
  CODE_FOR_atomic_and_fetch_cmp_0si_1 = 10469,
  CODE_FOR_atomic_or_fetch_cmp_0si_1 = 10470,
  CODE_FOR_atomic_xor_fetch_cmp_0si_1 = 10471,
   CODE_FOR_atomic_and_fetch_cmp_0di_1 = CODE_FOR_nothing,
   CODE_FOR_atomic_or_fetch_cmp_0di_1 = CODE_FOR_nothing,
   CODE_FOR_atomic_xor_fetch_cmp_0di_1 = CODE_FOR_nothing,
   CODE_FOR_cmpccxadd_si = CODE_FOR_nothing,
   CODE_FOR_cmpccxadd_di = CODE_FOR_nothing,
  CODE_FOR_cbranchqi4 = 10472,
  CODE_FOR_cbranchhi4 = 10473,
  CODE_FOR_cbranchsi4 = 10474,
  CODE_FOR_cbranchdi4 = 10475,
  CODE_FOR_cbranchti4 = 10476,
  CODE_FOR_cbranchoi4 = 10477,
  CODE_FOR_cbranchxi4 = 10478,
  CODE_FOR_cstoreqi4 = 10479,
  CODE_FOR_cstorehi4 = 10480,
  CODE_FOR_cstoresi4 = 10481,
  CODE_FOR_cstoredi4 = 10482,
   CODE_FOR_cstoreti4 = CODE_FOR_nothing,
  CODE_FOR_cmpqi_1 = 10483,
  CODE_FOR_cmphi_1 = 10484,
  CODE_FOR_cmpsi_1 = 10485,
   CODE_FOR_cmpdi_1 = CODE_FOR_nothing,
  CODE_FOR_cmpqi_ext_3 = 10486,
  CODE_FOR_cbranchxf4 = 10487,
  CODE_FOR_cstorexf4 = 10488,
  CODE_FOR_cbranchhf4 = 10489,
  CODE_FOR_cbranchsf4 = 10490,
  CODE_FOR_cbranchdf4 = 10491,
  CODE_FOR_cbranchbf4 = 10492,
  CODE_FOR_cstorehf4 = 10493,
  CODE_FOR_cstorebf4 = 10494,
  CODE_FOR_cstoresf4 = 10495,
  CODE_FOR_cstoredf4 = 10496,
  CODE_FOR_cbranchcc4 = 10497,
  CODE_FOR_cstoreccgc4 = 10498,
  CODE_FOR_cstoreccgoc4 = 10499,
  CODE_FOR_cstoreccno4 = 10500,
  CODE_FOR_cstoreccgz4 = 10501,
  CODE_FOR_cstorecca4 = 10502,
  CODE_FOR_cstoreccc4 = 10503,
  CODE_FOR_cstorecco4 = 10504,
  CODE_FOR_cstoreccp4 = 10505,
  CODE_FOR_cstoreccs4 = 10506,
  CODE_FOR_cstoreccz4 = 10507,
  CODE_FOR_cstorecc4 = 10508,
   CODE_FOR_reload_noff_store = CODE_FOR_nothing,
   CODE_FOR_reload_noff_load = CODE_FOR_nothing,
  CODE_FOR_movxi = 10509,
  CODE_FOR_movoi = 10510,
  CODE_FOR_movti = 10511,
  CODE_FOR_movcdi = 10512,
  CODE_FOR_movqi = 10513,
  CODE_FOR_movhi = 10514,
  CODE_FOR_movsi = 10515,
  CODE_FOR_movdi = 10516,
  CODE_FOR_movstrictqi = 10517,
  CODE_FOR_movstricthi = 10518,
  CODE_FOR_extvhi = 10519,
  CODE_FOR_extvsi = 10520,
  CODE_FOR_extzvhi = 10521,
  CODE_FOR_extzvsi = 10522,
   CODE_FOR_extzvdi = CODE_FOR_nothing,
  CODE_FOR_insvhi = 10523,
  CODE_FOR_insvsi = 10524,
   CODE_FOR_insvdi = CODE_FOR_nothing,
  CODE_FOR_movtf = 10525,
  CODE_FOR_movhf = 10526,
  CODE_FOR_movsf = 10527,
  CODE_FOR_movdf = 10528,
  CODE_FOR_movxf = 10529,
  CODE_FOR_movbf = 10530,
  CODE_FOR_zero_extendsidi2 = 10531,
  CODE_FOR_zero_extendqisi2 = 10532,
  CODE_FOR_zero_extendhisi2 = 10533,
  CODE_FOR_zero_extendqihi2 = 10534,
  CODE_FOR_extendsidi2 = 10535,
  CODE_FOR_extendsfdf2 = 10536,
  CODE_FOR_extendhfsf2 = 10537,
  CODE_FOR_extendhfdf2 = 10538,
  CODE_FOR_extendbfsf2 = 10539,
  CODE_FOR_extendsfxf2 = 10540,
  CODE_FOR_extenddfxf2 = 10541,
  CODE_FOR_truncsfhf2 = 10542,
  CODE_FOR_truncdfhf2 = 10543,
  CODE_FOR_fix_truncxfdi2 = 10544,
  CODE_FOR_fix_truncsfdi2 = 10545,
  CODE_FOR_fix_truncdfdi2 = 10546,
  CODE_FOR_fix_truncxfsi2 = 10547,
  CODE_FOR_fix_truncsfsi2 = 10548,
  CODE_FOR_fix_truncdfsi2 = 10549,
  CODE_FOR_fix_truncsfhi2 = 10550,
  CODE_FOR_fix_truncdfhi2 = 10551,
  CODE_FOR_fix_truncxfhi2 = 10552,
  CODE_FOR_fixuns_truncsfsi2 = 10553,
  CODE_FOR_fixuns_truncdfsi2 = 10554,
  CODE_FOR_fixuns_trunchfhi2 = 10555,
  CODE_FOR_fixuns_truncsfhi2 = 10556,
  CODE_FOR_fixuns_truncdfhi2 = 10557,
  CODE_FOR_floatsisf2 = 10558,
  CODE_FOR_floatdisf2 = 10559,
  CODE_FOR_floatsidf2 = 10560,
  CODE_FOR_floatdidf2 = 10561,
  CODE_FOR_floatunsqisf2 = 10562,
  CODE_FOR_floatunshisf2 = 10563,
  CODE_FOR_floatunsqidf2 = 10564,
  CODE_FOR_floatunshidf2 = 10565,
  CODE_FOR_floatunssisf2 = 10566,
  CODE_FOR_floatunssidf2 = 10567,
  CODE_FOR_floatunssixf2 = 10568,
   CODE_FOR_floatunsdisf2 = CODE_FOR_nothing,
   CODE_FOR_floatunsdidf2 = CODE_FOR_nothing,
  CODE_FOR_addqi3 = 10569,
  CODE_FOR_addhi3 = 10570,
  CODE_FOR_addsi3 = 10571,
  CODE_FOR_adddi3 = 10572,
   CODE_FOR_addti3 = CODE_FOR_nothing,
  CODE_FOR_addqi_ext_1 = 10573,
  CODE_FOR_addvqi4 = 10574,
  CODE_FOR_addvhi4 = 10575,
  CODE_FOR_addvsi4 = 10576,
  CODE_FOR_addvdi4 = 10577,
   CODE_FOR_addvti4 = CODE_FOR_nothing,
  CODE_FOR_uaddvqi4 = 10578,
  CODE_FOR_uaddvhi4 = 10579,
  CODE_FOR_uaddvsi4 = 10580,
  CODE_FOR_uaddvdi4 = 10581,
   CODE_FOR_uaddvti4 = CODE_FOR_nothing,
  CODE_FOR_subqi3 = 10582,
  CODE_FOR_subhi3 = 10583,
  CODE_FOR_subsi3 = 10584,
  CODE_FOR_subdi3 = 10585,
   CODE_FOR_subti3 = CODE_FOR_nothing,
  CODE_FOR_subvqi4 = 10586,
  CODE_FOR_subvhi4 = 10587,
  CODE_FOR_subvsi4 = 10588,
  CODE_FOR_subvdi4 = 10589,
   CODE_FOR_subvti4 = CODE_FOR_nothing,
  CODE_FOR_usubvqi4 = 10590,
  CODE_FOR_usubvhi4 = 10591,
  CODE_FOR_usubvsi4 = 10592,
   CODE_FOR_usubvdi4 = CODE_FOR_nothing,
  CODE_FOR_subqi_3 = 10593,
  CODE_FOR_subhi_3 = 10594,
  CODE_FOR_subsi_3 = 10595,
   CODE_FOR_subdi_3 = CODE_FOR_nothing,
  CODE_FOR_addcarrysi_0 = 10596,
  CODE_FOR_addcarrydi_0 = 10597,
  CODE_FOR_subborrowsi_0 = 10598,
  CODE_FOR_subborrowdi_0 = 10599,
  CODE_FOR_uaddcsi5 = 10600,
   CODE_FOR_uaddcdi5 = CODE_FOR_nothing,
  CODE_FOR_usubcsi5 = 10601,
   CODE_FOR_usubcdi5 = CODE_FOR_nothing,
  CODE_FOR_addqi3_cconly_overflow = 10602,
  CODE_FOR_usaddqi3 = 10603,
  CODE_FOR_usaddhi3 = 10604,
  CODE_FOR_usaddsi3 = 10605,
   CODE_FOR_usadddi3 = CODE_FOR_nothing,
  CODE_FOR_ussubqi3 = 10606,
  CODE_FOR_ussubhi3 = 10607,
  CODE_FOR_ussubsi3 = 10608,
   CODE_FOR_ussubdi3 = CODE_FOR_nothing,
   CODE_FOR_ustruncdiqi2 = CODE_FOR_nothing,
   CODE_FOR_ustruncdihi2 = CODE_FOR_nothing,
   CODE_FOR_ustruncdisi2 = CODE_FOR_nothing,
  CODE_FOR_ustruncsiqi2 = 10609,
  CODE_FOR_ustruncsihi2 = 10610,
  CODE_FOR_ustrunchiqi2 = 10611,
  CODE_FOR_addxf3 = 10612,
  CODE_FOR_subxf3 = 10613,
  CODE_FOR_addhf3 = 10614,
  CODE_FOR_subhf3 = 10615,
  CODE_FOR_addsf3 = 10616,
  CODE_FOR_subsf3 = 10617,
  CODE_FOR_adddf3 = 10618,
  CODE_FOR_subdf3 = 10619,
  CODE_FOR_mulhi3 = 10620,
  CODE_FOR_mulsi3 = 10621,
   CODE_FOR_muldi3 = CODE_FOR_nothing,
  CODE_FOR_mulqi3 = 10622,
  CODE_FOR_mulvhi4 = 10623,
  CODE_FOR_mulvsi4 = 10624,
   CODE_FOR_mulvdi4 = CODE_FOR_nothing,
  CODE_FOR_umulvhi4 = 10625,
  CODE_FOR_umulvsi4 = 10626,
   CODE_FOR_umulvdi4 = CODE_FOR_nothing,
  CODE_FOR_mulvqi4 = 10627,
  CODE_FOR_umulvqi4 = 10628,
  CODE_FOR_mulsidi3 = 10629,
  CODE_FOR_umulsidi3 = 10630,
   CODE_FOR_mulditi3 = CODE_FOR_nothing,
   CODE_FOR_umulditi3 = CODE_FOR_nothing,
  CODE_FOR_mulqihi3 = 10631,
  CODE_FOR_umulqihi3 = 10632,
  CODE_FOR_mulxf3 = 10633,
  CODE_FOR_mulhf3 = 10634,
  CODE_FOR_mulsf3 = 10635,
  CODE_FOR_muldf3 = 10636,
  CODE_FOR_divxf3 = 10637,
  CODE_FOR_divhf3 = 10638,
  CODE_FOR_divsf3 = 10639,
  CODE_FOR_divdf3 = 10640,
  CODE_FOR_divmodhi4 = 10641,
  CODE_FOR_udivmodhi4 = 10642,
  CODE_FOR_divmodsi4 = 10643,
  CODE_FOR_udivmodsi4 = 10644,
   CODE_FOR_divmoddi4 = CODE_FOR_nothing,
   CODE_FOR_udivmoddi4 = CODE_FOR_nothing,
  CODE_FOR_divmodqi4 = 10645,
  CODE_FOR_udivmodqi4 = 10646,
  CODE_FOR_testsi_ccno_1 = 10647,
   CODE_FOR_testdi_ccno_1 = CODE_FOR_nothing,
  CODE_FOR_testqi_ccz_1 = 10648,
  CODE_FOR_testqi_ext_1_ccno = 10649,
  CODE_FOR_andqi3 = 10650,
  CODE_FOR_andhi3 = 10651,
  CODE_FOR_andsi3 = 10652,
  CODE_FOR_anddi3 = 10653,
   CODE_FOR_andti3 = CODE_FOR_nothing,
  CODE_FOR_andqi_ext_1 = 10654,
  CODE_FOR_iorqi3 = 10655,
  CODE_FOR_xorqi3 = 10656,
  CODE_FOR_iorhi3 = 10657,
  CODE_FOR_xorhi3 = 10658,
  CODE_FOR_iorsi3 = 10659,
  CODE_FOR_xorsi3 = 10660,
  CODE_FOR_iordi3 = 10661,
  CODE_FOR_xordi3 = 10662,
   CODE_FOR_iorti3 = CODE_FOR_nothing,
   CODE_FOR_xorti3 = CODE_FOR_nothing,
  CODE_FOR_xorqi_ext_1_cc = 10663,
  CODE_FOR_negqi2 = 10664,
  CODE_FOR_neghi2 = 10665,
  CODE_FOR_negsi2 = 10666,
  CODE_FOR_negdi2 = 10667,
   CODE_FOR_negti2 = CODE_FOR_nothing,
  CODE_FOR_x86_negsi_ccc = 10668,
   CODE_FOR_x86_negdi_ccc = CODE_FOR_nothing,
  CODE_FOR_negvqi3 = 10669,
  CODE_FOR_negvhi3 = 10670,
  CODE_FOR_negvsi3 = 10671,
   CODE_FOR_negvdi3 = CODE_FOR_nothing,
  CODE_FOR_absqi2 = 10672,
  CODE_FOR_abshi2 = 10673,
  CODE_FOR_abssi2 = 10674,
  CODE_FOR_absdi2 = 10675,
   CODE_FOR_absti2 = CODE_FOR_nothing,
  CODE_FOR_abstf2 = 10676,
  CODE_FOR_negtf2 = 10677,
  CODE_FOR_abshf2 = 10678,
  CODE_FOR_neghf2 = 10679,
  CODE_FOR_abssf2 = 10680,
  CODE_FOR_negsf2 = 10681,
  CODE_FOR_absdf2 = 10682,
  CODE_FOR_negdf2 = 10683,
  CODE_FOR_absxf2 = 10684,
  CODE_FOR_negxf2 = 10685,
  CODE_FOR_copysignhf3 = 10686,
  CODE_FOR_copysignsf3 = 10687,
  CODE_FOR_copysigndf3 = 10688,
  CODE_FOR_copysigntf3 = 10689,
  CODE_FOR_xorsignhf3 = 10690,
  CODE_FOR_xorsignsf3 = 10691,
  CODE_FOR_xorsigndf3 = 10692,
  CODE_FOR_one_cmplqi2 = 10693,
  CODE_FOR_one_cmplhi2 = 10694,
  CODE_FOR_one_cmplsi2 = 10695,
  CODE_FOR_one_cmpldi2 = 10696,
   CODE_FOR_one_cmplti2 = CODE_FOR_nothing,
  CODE_FOR_ashlqi3 = 10697,
  CODE_FOR_ashlhi3 = 10698,
  CODE_FOR_ashlsi3 = 10699,
  CODE_FOR_ashldi3 = 10700,
   CODE_FOR_ashlti3 = CODE_FOR_nothing,
  CODE_FOR_x86_shiftsi_adj_1 = 10701,
   CODE_FOR_x86_shiftdi_adj_1 = CODE_FOR_nothing,
  CODE_FOR_x86_shiftsi_adj_2 = 10702,
   CODE_FOR_x86_shiftdi_adj_2 = CODE_FOR_nothing,
  CODE_FOR_lshrqi3 = 10703,
  CODE_FOR_ashrqi3 = 10704,
  CODE_FOR_lshrhi3 = 10705,
  CODE_FOR_ashrhi3 = 10706,
  CODE_FOR_lshrsi3 = 10707,
  CODE_FOR_ashrsi3 = 10708,
  CODE_FOR_lshrdi3 = 10709,
  CODE_FOR_ashrdi3 = 10710,
   CODE_FOR_lshrti3 = CODE_FOR_nothing,
   CODE_FOR_ashrti3 = CODE_FOR_nothing,
  CODE_FOR_x86_shiftsi_adj_3 = 10711,
   CODE_FOR_x86_shiftdi_adj_3 = CODE_FOR_nothing,
   CODE_FOR_rotlti3 = CODE_FOR_nothing,
   CODE_FOR_rotrti3 = CODE_FOR_nothing,
  CODE_FOR_rotldi3 = 10712,
  CODE_FOR_rotrdi3 = 10713,
  CODE_FOR_rotlqi3 = 10714,
  CODE_FOR_rotrqi3 = 10715,
  CODE_FOR_rotlhi3 = 10716,
  CODE_FOR_rotrhi3 = 10717,
  CODE_FOR_rotlsi3 = 10718,
  CODE_FOR_rotrsi3 = 10719,
  CODE_FOR_setcc_si_slp = 10720,
  CODE_FOR_indirect_jump = 10721,
  CODE_FOR_tablejump = 10722,
  CODE_FOR_call = 10723,
  CODE_FOR_sibcall = 10724,
  CODE_FOR_call_pop = 10725,
  CODE_FOR_call_value = 10726,
  CODE_FOR_sibcall_value = 10727,
  CODE_FOR_call_value_pop = 10728,
  CODE_FOR_untyped_call = 10729,
  CODE_FOR_memory_blockage = 10730,
  CODE_FOR_return = 10731,
  CODE_FOR_simple_return = 10732,
  CODE_FOR_simple_return_indirect_internal = 10733,
  CODE_FOR_prologue = 10734,
  CODE_FOR_set_got = 10735,
  CODE_FOR_set_got_labelled = 10736,
  CODE_FOR_epilogue = 10737,
  CODE_FOR_sibcall_epilogue = 10738,
  CODE_FOR_eh_return = 10739,
  CODE_FOR_leave_si = 10740,
  CODE_FOR_leave_di = 10741,
  CODE_FOR_split_stack_prologue = 10742,
  CODE_FOR_split_stack_space_check = 10743,
  CODE_FOR_ffssi2 = 10744,
   CODE_FOR_ffsdi2 = CODE_FOR_nothing,
  CODE_FOR_clzsi2 = 10745,
   CODE_FOR_clzdi2 = CODE_FOR_nothing,
  CODE_FOR_bmi2_bzhi_si3 = 10746,
   CODE_FOR_bmi2_bzhi_di3 = CODE_FOR_nothing,
   CODE_FOR_bswapdi2 = CODE_FOR_nothing,
  CODE_FOR_bswapsi2 = 10747,
  CODE_FOR_bswaphi2 = 10748,
  CODE_FOR_paritydi2 = 10749,
  CODE_FOR_paritysi2 = 10750,
  CODE_FOR_parityhi2 = 10751,
  CODE_FOR_parityqi2 = 10752,
  CODE_FOR_tls_global_dynamic_32 = 10753,
   CODE_FOR_tls_global_dynamic_64_si = CODE_FOR_nothing,
   CODE_FOR_tls_global_dynamic_64_di = CODE_FOR_nothing,
  CODE_FOR_tls_local_dynamic_base_32 = 10754,
   CODE_FOR_tls_local_dynamic_base_64_si = CODE_FOR_nothing,
   CODE_FOR_tls_local_dynamic_base_64_di = CODE_FOR_nothing,
  CODE_FOR_get_thread_pointersi = 10755,
  CODE_FOR_get_thread_pointerdi = 10756,
  CODE_FOR_tls_dynamic_gnu2_32 = 10757,
   CODE_FOR_tls_dynamic_gnu2_64_si = CODE_FOR_nothing,
   CODE_FOR_tls_dynamic_gnu2_64_di = CODE_FOR_nothing,
  CODE_FOR_rsqrtsf2 = 10758,
  CODE_FOR_sqrtsf2 = 10759,
  CODE_FOR_sqrtdf2 = 10760,
  CODE_FOR_hypotsf3 = 10761,
  CODE_FOR_hypotdf3 = 10762,
  CODE_FOR_fmodxf3 = 10763,
  CODE_FOR_fmodsf3 = 10764,
  CODE_FOR_fmoddf3 = 10765,
  CODE_FOR_remainderxf3 = 10766,
  CODE_FOR_remaindersf3 = 10767,
  CODE_FOR_remainderdf3 = 10768,
  CODE_FOR_sinsf2 = 10769,
  CODE_FOR_cossf2 = 10770,
  CODE_FOR_sindf2 = 10771,
  CODE_FOR_cosdf2 = 10772,
  CODE_FOR_sincossf3 = 10773,
  CODE_FOR_sincosdf3 = 10774,
  CODE_FOR_tanxf2 = 10775,
  CODE_FOR_tansf2 = 10776,
  CODE_FOR_tandf2 = 10777,
  CODE_FOR_atan2sf3 = 10778,
  CODE_FOR_atan2df3 = 10779,
  CODE_FOR_atanxf2 = 10780,
  CODE_FOR_atansf2 = 10781,
  CODE_FOR_atandf2 = 10782,
  CODE_FOR_asinxf2 = 10783,
  CODE_FOR_asinsf2 = 10784,
  CODE_FOR_asindf2 = 10785,
  CODE_FOR_acosxf2 = 10786,
  CODE_FOR_acossf2 = 10787,
  CODE_FOR_acosdf2 = 10788,
  CODE_FOR_sinhxf2 = 10789,
  CODE_FOR_sinhsf2 = 10790,
  CODE_FOR_sinhdf2 = 10791,
  CODE_FOR_coshxf2 = 10792,
  CODE_FOR_coshsf2 = 10793,
  CODE_FOR_coshdf2 = 10794,
  CODE_FOR_tanhxf2 = 10795,
  CODE_FOR_tanhsf2 = 10796,
  CODE_FOR_tanhdf2 = 10797,
  CODE_FOR_asinhxf2 = 10798,
  CODE_FOR_asinhsf2 = 10799,
  CODE_FOR_asinhdf2 = 10800,
  CODE_FOR_acoshxf2 = 10801,
  CODE_FOR_acoshsf2 = 10802,
  CODE_FOR_acoshdf2 = 10803,
  CODE_FOR_atanhxf2 = 10804,
  CODE_FOR_atanhsf2 = 10805,
  CODE_FOR_atanhdf2 = 10806,
  CODE_FOR_logxf2 = 10807,
  CODE_FOR_logsf2 = 10808,
  CODE_FOR_logdf2 = 10809,
  CODE_FOR_log10xf2 = 10810,
  CODE_FOR_log10sf2 = 10811,
  CODE_FOR_log10df2 = 10812,
  CODE_FOR_log2xf2 = 10813,
  CODE_FOR_log2sf2 = 10814,
  CODE_FOR_log2df2 = 10815,
  CODE_FOR_log1pxf2 = 10816,
  CODE_FOR_log1psf2 = 10817,
  CODE_FOR_log1pdf2 = 10818,
  CODE_FOR_logbxf2 = 10819,
  CODE_FOR_logbsf2 = 10820,
  CODE_FOR_logbdf2 = 10821,
  CODE_FOR_ilogbxf2 = 10822,
  CODE_FOR_ilogbsf2 = 10823,
  CODE_FOR_ilogbdf2 = 10824,
  CODE_FOR_expNcorexf3 = 10825,
  CODE_FOR_expxf2 = 10826,
  CODE_FOR_expsf2 = 10827,
  CODE_FOR_expdf2 = 10828,
  CODE_FOR_exp10xf2 = 10829,
  CODE_FOR_exp10sf2 = 10830,
  CODE_FOR_exp10df2 = 10831,
  CODE_FOR_exp2xf2 = 10832,
  CODE_FOR_exp2sf2 = 10833,
  CODE_FOR_exp2df2 = 10834,
  CODE_FOR_expm1xf2 = 10835,
  CODE_FOR_expm1sf2 = 10836,
  CODE_FOR_expm1df2 = 10837,
  CODE_FOR_ldexpxf3 = 10838,
  CODE_FOR_ldexpsf3 = 10839,
  CODE_FOR_ldexpdf3 = 10840,
  CODE_FOR_scalbxf3 = 10841,
  CODE_FOR_scalbsf3 = 10842,
  CODE_FOR_scalbdf3 = 10843,
  CODE_FOR_significandxf2 = 10844,
  CODE_FOR_significandsf2 = 10845,
  CODE_FOR_significanddf2 = 10846,
  CODE_FOR_rinthf2 = 10847,
  CODE_FOR_rintsf2 = 10848,
  CODE_FOR_rintdf2 = 10849,
  CODE_FOR_nearbyintxf2 = 10850,
  CODE_FOR_nearbyinthf2 = 10851,
  CODE_FOR_nearbyintsf2 = 10852,
  CODE_FOR_nearbyintdf2 = 10853,
  CODE_FOR_roundhf2 = 10854,
  CODE_FOR_roundsf2 = 10855,
  CODE_FOR_rounddf2 = 10856,
  CODE_FOR_roundxf2 = 10857,
  CODE_FOR_lroundhfhi2 = 10858,
  CODE_FOR_lroundhfsi2 = 10859,
   CODE_FOR_lroundhfdi2 = CODE_FOR_nothing,
  CODE_FOR_lrinthfsi2 = 10860,
   CODE_FOR_lrinthfdi2 = CODE_FOR_nothing,
  CODE_FOR_lrintsfsi2 = 10861,
   CODE_FOR_lrintsfdi2 = CODE_FOR_nothing,
  CODE_FOR_lrintdfsi2 = 10862,
   CODE_FOR_lrintdfdi2 = CODE_FOR_nothing,
  CODE_FOR_lroundsfhi2 = 10863,
  CODE_FOR_lrounddfhi2 = 10864,
  CODE_FOR_lroundxfhi2 = 10865,
  CODE_FOR_lroundsfsi2 = 10866,
  CODE_FOR_lrounddfsi2 = 10867,
  CODE_FOR_lroundxfsi2 = 10868,
  CODE_FOR_lroundsfdi2 = 10869,
  CODE_FOR_lrounddfdi2 = 10870,
  CODE_FOR_lroundxfdi2 = 10871,
  CODE_FOR_roundevenxf2 = 10872,
  CODE_FOR_floorxf2 = 10873,
  CODE_FOR_ceilxf2 = 10874,
  CODE_FOR_btruncxf2 = 10875,
  CODE_FOR_roundevenhf2 = 10876,
  CODE_FOR_floorhf2 = 10877,
  CODE_FOR_ceilhf2 = 10878,
  CODE_FOR_btrunchf2 = 10879,
  CODE_FOR_roundevensf2 = 10880,
  CODE_FOR_floorsf2 = 10881,
  CODE_FOR_ceilsf2 = 10882,
  CODE_FOR_btruncsf2 = 10883,
  CODE_FOR_roundevendf2 = 10884,
  CODE_FOR_floordf2 = 10885,
  CODE_FOR_ceildf2 = 10886,
  CODE_FOR_btruncdf2 = 10887,
  CODE_FOR_lfloorxfhi2 = 10888,
  CODE_FOR_lceilxfhi2 = 10889,
  CODE_FOR_lfloorxfsi2 = 10890,
  CODE_FOR_lceilxfsi2 = 10891,
  CODE_FOR_lfloorxfdi2 = 10892,
  CODE_FOR_lceilxfdi2 = 10893,
  CODE_FOR_lfloorhfsi2 = 10894,
  CODE_FOR_lceilhfsi2 = 10895,
   CODE_FOR_lfloorhfdi2 = CODE_FOR_nothing,
   CODE_FOR_lceilhfdi2 = CODE_FOR_nothing,
  CODE_FOR_lfloorsfsi2 = 10896,
  CODE_FOR_lceilsfsi2 = 10897,
   CODE_FOR_lfloorsfdi2 = CODE_FOR_nothing,
   CODE_FOR_lceilsfdi2 = CODE_FOR_nothing,
  CODE_FOR_lfloordfsi2 = 10898,
  CODE_FOR_lceildfsi2 = 10899,
   CODE_FOR_lfloordfdi2 = CODE_FOR_nothing,
   CODE_FOR_lceildfdi2 = CODE_FOR_nothing,
  CODE_FOR_signbittf2 = 10900,
  CODE_FOR_signbitxf2 = 10901,
  CODE_FOR_signbitdf2 = 10902,
  CODE_FOR_signbitsf2 = 10903,
  CODE_FOR_movmemsi = 10904,
   CODE_FOR_movmemdi = CODE_FOR_nothing,
  CODE_FOR_cpymemsi = 10905,
   CODE_FOR_cpymemdi = CODE_FOR_nothing,
  CODE_FOR_strmov = 10906,
  CODE_FOR_strmov_singleop = 10907,
  CODE_FOR_rep_mov = 10908,
  CODE_FOR_setmemsi = 10909,
   CODE_FOR_setmemdi = CODE_FOR_nothing,
  CODE_FOR_strset = 10910,
  CODE_FOR_strset_singleop = 10911,
  CODE_FOR_rep_stos = 10912,
  CODE_FOR_cmpmemsi = 10913,
  CODE_FOR_cmpstrnsi = 10914,
  CODE_FOR_cmpintqi = 10915,
  CODE_FOR_cmpstrnqi_nz_1 = 10916,
  CODE_FOR_cmpstrnqi_1 = 10917,
  CODE_FOR_strlensi = 10918,
  CODE_FOR_strlendi = 10919,
  CODE_FOR_strlenqi_1 = 10920,
  CODE_FOR_movqicc = 10921,
  CODE_FOR_movhicc = 10922,
  CODE_FOR_movsicc = 10923,
   CODE_FOR_movdicc = CODE_FOR_nothing,
  CODE_FOR_x86_movsicc_0_m1 = 10924,
   CODE_FOR_x86_movdicc_0_m1 = CODE_FOR_nothing,
  CODE_FOR_x86_movqicc_0_m1_neg = 10925,
  CODE_FOR_x86_movhicc_0_m1_neg = 10926,
  CODE_FOR_x86_movsicc_0_m1_neg = 10927,
   CODE_FOR_x86_movdicc_0_m1_neg = CODE_FOR_nothing,
  CODE_FOR_movhfcc = 10928,
  CODE_FOR_movsfcc = 10929,
  CODE_FOR_movdfcc = 10930,
  CODE_FOR_movxfcc = 10931,
  CODE_FOR_addqicc = 10932,
  CODE_FOR_addhicc = 10933,
  CODE_FOR_addsicc = 10934,
   CODE_FOR_adddicc = CODE_FOR_nothing,
  CODE_FOR_smaxqi3 = 10935,
  CODE_FOR_sminqi3 = 10936,
  CODE_FOR_umaxqi3 = 10937,
  CODE_FOR_uminqi3 = 10938,
  CODE_FOR_smaxhi3 = 10939,
  CODE_FOR_sminhi3 = 10940,
  CODE_FOR_umaxhi3 = 10941,
  CODE_FOR_uminhi3 = 10942,
  CODE_FOR_smaxsi3 = 10943,
  CODE_FOR_sminsi3 = 10944,
  CODE_FOR_umaxsi3 = 10945,
  CODE_FOR_uminsi3 = 10946,
  CODE_FOR_smaxdi3 = 10947,
  CODE_FOR_smindi3 = 10948,
  CODE_FOR_umaxdi3 = 10949,
  CODE_FOR_umindi3 = 10950,
   CODE_FOR_smaxti3 = CODE_FOR_nothing,
   CODE_FOR_sminti3 = CODE_FOR_nothing,
   CODE_FOR_umaxti3 = CODE_FOR_nothing,
   CODE_FOR_uminti3 = CODE_FOR_nothing,
  CODE_FOR_allocate_stack = 10951,
  CODE_FOR_probe_stack = 10952,
  CODE_FOR_builtin_setjmp_receiver = 10953,
  CODE_FOR_save_stack_nonlocal = 10954,
  CODE_FOR_restore_stack_nonlocal = 10955,
  CODE_FOR_stack_protect_set = 10956,
  CODE_FOR_stack_protect_test = 10957,
  CODE_FOR_prefetch = 10958,
  CODE_FOR_crc_revqisi4 = 10959,
  CODE_FOR_crc_revhisi4 = 10960,
  CODE_FOR_crc_revsisi4 = 10961,
  CODE_FOR_pause = 10962,
  CODE_FOR_xbegin = 10963,
  CODE_FOR_xtest = 10964,
  CODE_FOR_rdpkru = 10965,
  CODE_FOR_wrpkru = 10966,
  CODE_FOR_spaceshipsf4 = 10967,
  CODE_FOR_spaceshipdf4 = 10968,
  CODE_FOR_spaceshipxf4 = 10969,
  CODE_FOR_spaceshipqi4 = 10970,
  CODE_FOR_spaceshiphi4 = 10971,
  CODE_FOR_spaceshipsi4 = 10972,
   CODE_FOR_spaceshipdi4 = CODE_FOR_nothing,
  CODE_FOR_issignalingxf2 = 10973,
  CODE_FOR_movv8qi = 10974,
  CODE_FOR_movv4hi = 10975,
  CODE_FOR_movv2si = 10976,
  CODE_FOR_movv1di = 10977,
  CODE_FOR_movv2sf = 10978,
  CODE_FOR_movv4hf = 10979,
  CODE_FOR_movv4bf = 10980,
  CODE_FOR_movmisalignv8qi = 10981,
  CODE_FOR_movmisalignv4hi = 10982,
  CODE_FOR_movmisalignv2si = 10983,
  CODE_FOR_movmisalignv1di = 10984,
  CODE_FOR_movmisalignv2sf = 10985,
  CODE_FOR_movmisalignv4hf = 10986,
  CODE_FOR_movmisalignv4bf = 10987,
  CODE_FOR_movv4qi = 10988,
  CODE_FOR_movv2hi = 10989,
  CODE_FOR_movv1si = 10990,
  CODE_FOR_movv2hf = 10991,
  CODE_FOR_movv2bf = 10992,
  CODE_FOR_movmisalignv4qi = 10993,
  CODE_FOR_movmisalignv2hi = 10994,
  CODE_FOR_movmisalignv1si = 10995,
  CODE_FOR_movmisalignv2hf = 10996,
  CODE_FOR_movmisalignv2bf = 10997,
  CODE_FOR_movv2qi = 10998,
  CODE_FOR_movmisalignv2qi = 10999,
  CODE_FOR_movq_v2sf_to_sse = 11000,
  CODE_FOR_movq_v2si_to_sse = 11001,
  CODE_FOR_movq_v4hf_to_sse = 11002,
  CODE_FOR_movq_v4hi_to_sse = 11003,
   CODE_FOR_absv2sf2 = CODE_FOR_nothing,
   CODE_FOR_negv2sf2 = CODE_FOR_nothing,
   CODE_FOR_addv2sf3 = CODE_FOR_nothing,
   CODE_FOR_subv2sf3 = CODE_FOR_nothing,
   CODE_FOR_mulv2sf3 = CODE_FOR_nothing,
  CODE_FOR_mmx_addv2sf3 = 11004,
  CODE_FOR_mmx_subv2sf3 = 11005,
  CODE_FOR_mmx_subrv2sf3 = 11006,
  CODE_FOR_mmx_mulv2sf3 = 11007,
   CODE_FOR_divv2sf3 = CODE_FOR_nothing,
   CODE_FOR_smaxv2sf3 = CODE_FOR_nothing,
   CODE_FOR_sminv2sf3 = CODE_FOR_nothing,
  CODE_FOR_mmx_smaxv2sf3 = 11008,
  CODE_FOR_mmx_sminv2sf3 = 11009,
   CODE_FOR_sqrtv2sf2 = CODE_FOR_nothing,
  CODE_FOR_mmx_haddv2sf3 = 11010,
  CODE_FOR_mmx_haddsubv2sf3 = 11011,
   CODE_FOR_vec_addsubv2sf3 = CODE_FOR_nothing,
   CODE_FOR_vec_fmaddsubv2sf4 = CODE_FOR_nothing,
   CODE_FOR_vec_fmsubaddv2sf4 = CODE_FOR_nothing,
  CODE_FOR_mmx_eqv2sf3 = 11012,
   CODE_FOR_vec_cmpv2sfv2si = CODE_FOR_nothing,
   CODE_FOR_copysignv2sf3 = CODE_FOR_nothing,
   CODE_FOR_xorsignv2sf3 = CODE_FOR_nothing,
   CODE_FOR_signbitv2sf2 = CODE_FOR_nothing,
   CODE_FOR_fmav2sf4 = CODE_FOR_nothing,
   CODE_FOR_fmsv2sf4 = CODE_FOR_nothing,
   CODE_FOR_fnmav2sf4 = CODE_FOR_nothing,
   CODE_FOR_fnmsv2sf4 = CODE_FOR_nothing,
   CODE_FOR_fix_truncv2sfv2si2 = CODE_FOR_nothing,
   CODE_FOR_fixuns_truncv2sfv2si2 = CODE_FOR_nothing,
   CODE_FOR_floatv2siv2sf2 = CODE_FOR_nothing,
   CODE_FOR_floatunsv2siv2sf2 = CODE_FOR_nothing,
  CODE_FOR_vec_setv2sf = 11013,
  CODE_FOR_vec_extractv2sfsf = 11014,
  CODE_FOR_vec_initv2sfsf = 11015,
   CODE_FOR_nearbyintv2sf2 = CODE_FOR_nothing,
   CODE_FOR_rintv2sf2 = CODE_FOR_nothing,
   CODE_FOR_lrintv2sfv2si2 = CODE_FOR_nothing,
   CODE_FOR_ceilv2sf2 = CODE_FOR_nothing,
   CODE_FOR_lceilv2sfv2si2 = CODE_FOR_nothing,
   CODE_FOR_floorv2sf2 = CODE_FOR_nothing,
   CODE_FOR_lfloorv2sfv2si2 = CODE_FOR_nothing,
   CODE_FOR_btruncv2sf2 = CODE_FOR_nothing,
   CODE_FOR_roundv2sf2 = CODE_FOR_nothing,
   CODE_FOR_lroundv2sfv2si2 = CODE_FOR_nothing,
   CODE_FOR_divv4hf3 = CODE_FOR_nothing,
  CODE_FOR_movd_v2hf_to_sse = 11016,
  CODE_FOR_movd_v2bf_to_sse = 11017,
  CODE_FOR_movd_v2hi_to_sse = 11018,
  CODE_FOR_movd_v2hf_to_sse_reg = 11019,
  CODE_FOR_movd_v2bf_to_sse_reg = 11020,
  CODE_FOR_movd_v2hi_to_sse_reg = 11021,
  CODE_FOR_addv2hf3 = 11022,
  CODE_FOR_subv2hf3 = 11023,
  CODE_FOR_mulv2hf3 = 11024,
   CODE_FOR_addv4hf3 = CODE_FOR_nothing,
   CODE_FOR_subv4hf3 = CODE_FOR_nothing,
   CODE_FOR_mulv4hf3 = CODE_FOR_nothing,
  CODE_FOR_addv2bf3 = 11025,
  CODE_FOR_subv2bf3 = 11026,
  CODE_FOR_mulv2bf3 = 11027,
  CODE_FOR_divv2bf3 = 11028,
   CODE_FOR_addv4bf3 = CODE_FOR_nothing,
   CODE_FOR_subv4bf3 = CODE_FOR_nothing,
   CODE_FOR_mulv4bf3 = CODE_FOR_nothing,
   CODE_FOR_divv4bf3 = CODE_FOR_nothing,
  CODE_FOR_divv2hf3 = 11029,
  CODE_FOR_smaxv2hf3 = 11030,
  CODE_FOR_sminv2hf3 = 11031,
   CODE_FOR_smaxv4hf3 = CODE_FOR_nothing,
   CODE_FOR_sminv4hf3 = CODE_FOR_nothing,
  CODE_FOR_smaxv2bf3 = 11032,
  CODE_FOR_sminv2bf3 = 11033,
   CODE_FOR_smaxv4bf3 = CODE_FOR_nothing,
   CODE_FOR_sminv4bf3 = CODE_FOR_nothing,
  CODE_FOR_sqrtv2hf2 = 11034,
   CODE_FOR_sqrtv4hf2 = CODE_FOR_nothing,
  CODE_FOR_sqrtv2bf2 = 11035,
   CODE_FOR_sqrtv4bf2 = CODE_FOR_nothing,
  CODE_FOR_absv2bf2 = 11036,
  CODE_FOR_negv2bf2 = 11037,
   CODE_FOR_absv4bf2 = CODE_FOR_nothing,
   CODE_FOR_negv4bf2 = CODE_FOR_nothing,
  CODE_FOR_absv2hf2 = 11038,
  CODE_FOR_negv2hf2 = 11039,
   CODE_FOR_absv4hf2 = CODE_FOR_nothing,
   CODE_FOR_negv4hf2 = CODE_FOR_nothing,
   CODE_FOR_vec_cmpv4hfqi = CODE_FOR_nothing,
   CODE_FOR_vcond_mask_v4hfv4hi = CODE_FOR_nothing,
   CODE_FOR_vcond_mask_v4bfv4hi = CODE_FOR_nothing,
   CODE_FOR_vcond_mask_v4hfqi = CODE_FOR_nothing,
   CODE_FOR_vcond_mask_v4bfqi = CODE_FOR_nothing,
   CODE_FOR_vcond_mask_v4hiqi = CODE_FOR_nothing,
  CODE_FOR_vec_cmpv2hfqi = 11040,
  CODE_FOR_vcond_mask_v2hfv2hi = 11041,
  CODE_FOR_vcond_mask_v2bfv2hi = 11042,
  CODE_FOR_vcond_mask_v2hfqi = 11043,
  CODE_FOR_vcond_mask_v2bfqi = 11044,
  CODE_FOR_vcond_mask_v2hiqi = 11045,
  CODE_FOR_vec_cmpv2bfqi = 11046,
   CODE_FOR_vec_cmpv4bfqi = CODE_FOR_nothing,
  CODE_FOR_btruncv2hf2 = 11047,
   CODE_FOR_btruncv4hf2 = CODE_FOR_nothing,
  CODE_FOR_nearbyintv2hf2 = 11048,
   CODE_FOR_nearbyintv4hf2 = CODE_FOR_nothing,
  CODE_FOR_rintv2hf2 = 11049,
   CODE_FOR_rintv4hf2 = CODE_FOR_nothing,
  CODE_FOR_lrintv2hfv2hi2 = 11050,
   CODE_FOR_lrintv4hfv4hi2 = CODE_FOR_nothing,
  CODE_FOR_floorv2hf2 = 11051,
   CODE_FOR_floorv4hf2 = CODE_FOR_nothing,
  CODE_FOR_lfloorv2hfv2hi2 = 11052,
   CODE_FOR_lfloorv4hfv4hi2 = CODE_FOR_nothing,
  CODE_FOR_ceilv2hf2 = 11053,
   CODE_FOR_ceilv4hf2 = CODE_FOR_nothing,
  CODE_FOR_lceilv2hfv2hi2 = 11054,
   CODE_FOR_lceilv4hfv4hi2 = CODE_FOR_nothing,
  CODE_FOR_roundv2hf2 = 11055,
   CODE_FOR_roundv4hf2 = CODE_FOR_nothing,
  CODE_FOR_lroundv2hfv2hi2 = 11056,
   CODE_FOR_lroundv4hfv4hi2 = CODE_FOR_nothing,
  CODE_FOR_copysignv2bf3 = 11057,
   CODE_FOR_copysignv4bf3 = CODE_FOR_nothing,
  CODE_FOR_copysignv2hf3 = 11058,
   CODE_FOR_copysignv4hf3 = CODE_FOR_nothing,
  CODE_FOR_xorsignv2bf3 = 11059,
   CODE_FOR_xorsignv4bf3 = CODE_FOR_nothing,
  CODE_FOR_xorsignv2hf3 = 11060,
   CODE_FOR_xorsignv4hf3 = CODE_FOR_nothing,
  CODE_FOR_signbitv2bf2 = 11061,
   CODE_FOR_signbitv4bf2 = CODE_FOR_nothing,
  CODE_FOR_signbitv2hf2 = 11062,
   CODE_FOR_signbitv4hf2 = CODE_FOR_nothing,
  CODE_FOR_fmav2hf4 = 11063,
   CODE_FOR_fmav4hf4 = CODE_FOR_nothing,
  CODE_FOR_fmsv2hf4 = 11064,
   CODE_FOR_fmsv4hf4 = CODE_FOR_nothing,
  CODE_FOR_fnmav2hf4 = 11065,
   CODE_FOR_fnmav4hf4 = CODE_FOR_nothing,
  CODE_FOR_fnmsv2hf4 = 11066,
   CODE_FOR_fnmsv4hf4 = CODE_FOR_nothing,
   CODE_FOR_vec_fmaddsubv4hf4 = CODE_FOR_nothing,
   CODE_FOR_vec_fmsubaddv4hf4 = CODE_FOR_nothing,
  CODE_FOR_fmav2bf4 = 11067,
   CODE_FOR_fmav4bf4 = CODE_FOR_nothing,
  CODE_FOR_fmsv2bf4 = 11068,
   CODE_FOR_fmsv4bf4 = CODE_FOR_nothing,
  CODE_FOR_fnmav2bf4 = 11069,
   CODE_FOR_fnmav4bf4 = CODE_FOR_nothing,
  CODE_FOR_fnmsv2bf4 = 11070,
   CODE_FOR_fnmsv4bf4 = CODE_FOR_nothing,
   CODE_FOR_cmlav4hf4 = CODE_FOR_nothing,
   CODE_FOR_cmla_conjv4hf4 = CODE_FOR_nothing,
   CODE_FOR_cmulv4hf3 = CODE_FOR_nothing,
   CODE_FOR_cmul_conjv4hf3 = CODE_FOR_nothing,
  CODE_FOR_fix_truncv2hfv2hi2 = 11071,
  CODE_FOR_fixuns_truncv2hfv2hi2 = 11072,
   CODE_FOR_fix_truncv4hfv4hi2 = CODE_FOR_nothing,
   CODE_FOR_fixuns_truncv4hfv4hi2 = CODE_FOR_nothing,
   CODE_FOR_fix_truncv2hfv2si2 = CODE_FOR_nothing,
   CODE_FOR_fixuns_truncv2hfv2si2 = CODE_FOR_nothing,
  CODE_FOR_floatv2hiv2hf2 = 11073,
  CODE_FOR_floatunsv2hiv2hf2 = 11074,
   CODE_FOR_floatv4hiv4hf2 = CODE_FOR_nothing,
   CODE_FOR_floatunsv4hiv4hf2 = CODE_FOR_nothing,
   CODE_FOR_floatv2siv2hf2 = CODE_FOR_nothing,
   CODE_FOR_floatunsv2siv2hf2 = CODE_FOR_nothing,
   CODE_FOR_extendv2hfv2sf2 = CODE_FOR_nothing,
   CODE_FOR_truncv2sfv2hf2 = CODE_FOR_nothing,
   CODE_FOR_truncv2sfv2bf2 = CODE_FOR_nothing,
   CODE_FOR_extendv2bfv2sf2 = CODE_FOR_nothing,
   CODE_FOR_negv8qi2 = CODE_FOR_nothing,
   CODE_FOR_negv4hi2 = CODE_FOR_nothing,
   CODE_FOR_negv2si2 = CODE_FOR_nothing,
  CODE_FOR_negv4qi2 = 11075,
  CODE_FOR_negv2hi2 = 11076,
  CODE_FOR_mmx_addv8qi3 = 11077,
  CODE_FOR_mmx_subv8qi3 = 11078,
  CODE_FOR_mmx_addv4hi3 = 11079,
  CODE_FOR_mmx_subv4hi3 = 11080,
  CODE_FOR_mmx_addv2si3 = 11081,
  CODE_FOR_mmx_subv2si3 = 11082,
  CODE_FOR_mmx_addv1di3 = 11083,
  CODE_FOR_mmx_subv1di3 = 11084,
   CODE_FOR_addv8qi3 = CODE_FOR_nothing,
   CODE_FOR_subv8qi3 = CODE_FOR_nothing,
   CODE_FOR_addv4hi3 = CODE_FOR_nothing,
   CODE_FOR_subv4hi3 = CODE_FOR_nothing,
   CODE_FOR_addv2si3 = CODE_FOR_nothing,
   CODE_FOR_subv2si3 = CODE_FOR_nothing,
  CODE_FOR_mmx_ssaddv8qi3 = 11085,
  CODE_FOR_mmx_usaddv8qi3 = 11086,
  CODE_FOR_mmx_sssubv8qi3 = 11087,
  CODE_FOR_mmx_ussubv8qi3 = 11088,
  CODE_FOR_mmx_ssaddv4hi3 = 11089,
  CODE_FOR_mmx_usaddv4hi3 = 11090,
  CODE_FOR_mmx_sssubv4hi3 = 11091,
  CODE_FOR_mmx_ussubv4hi3 = 11092,
   CODE_FOR_ssaddv8qi3 = CODE_FOR_nothing,
   CODE_FOR_usaddv8qi3 = CODE_FOR_nothing,
   CODE_FOR_sssubv8qi3 = CODE_FOR_nothing,
   CODE_FOR_ussubv8qi3 = CODE_FOR_nothing,
   CODE_FOR_ssaddv4hi3 = CODE_FOR_nothing,
   CODE_FOR_usaddv4hi3 = CODE_FOR_nothing,
   CODE_FOR_sssubv4hi3 = CODE_FOR_nothing,
   CODE_FOR_ussubv4hi3 = CODE_FOR_nothing,
  CODE_FOR_mmx_mulv4hi3 = 11093,
   CODE_FOR_mulv4hi3 = CODE_FOR_nothing,
   CODE_FOR_mulv8qi3 = CODE_FOR_nothing,
  CODE_FOR_mulv4qi3 = 11094,
  CODE_FOR_mmx_smulv4hi3_highpart = 11095,
  CODE_FOR_mmx_umulv4hi3_highpart = 11096,
   CODE_FOR_smulv4hi3_highpart = CODE_FOR_nothing,
   CODE_FOR_umulv4hi3_highpart = CODE_FOR_nothing,
  CODE_FOR_mmx_pmaddwd = 11097,
  CODE_FOR_mmx_pmulhrwv4hi3 = 11098,
  CODE_FOR_sse2_umulv1siv1di3 = 11099,
  CODE_FOR_mmx_smaxv4hi3 = 11100,
  CODE_FOR_mmx_sminv4hi3 = 11101,
   CODE_FOR_smaxv4hi3 = CODE_FOR_nothing,
   CODE_FOR_sminv4hi3 = CODE_FOR_nothing,
  CODE_FOR_mmx_umaxv8qi3 = 11102,
  CODE_FOR_mmx_uminv8qi3 = 11103,
   CODE_FOR_umaxv8qi3 = CODE_FOR_nothing,
   CODE_FOR_uminv8qi3 = CODE_FOR_nothing,
   CODE_FOR_absv8qi2 = CODE_FOR_nothing,
   CODE_FOR_absv4hi2 = CODE_FOR_nothing,
   CODE_FOR_absv2si2 = CODE_FOR_nothing,
   CODE_FOR_ashrv4hi3 = CODE_FOR_nothing,
   CODE_FOR_ashrv2si3 = CODE_FOR_nothing,
   CODE_FOR_ashlv4hi3 = CODE_FOR_nothing,
   CODE_FOR_lshrv4hi3 = CODE_FOR_nothing,
   CODE_FOR_ashlv2si3 = CODE_FOR_nothing,
   CODE_FOR_lshrv2si3 = CODE_FOR_nothing,
   CODE_FOR_ashlv8qi3 = CODE_FOR_nothing,
   CODE_FOR_lshrv8qi3 = CODE_FOR_nothing,
   CODE_FOR_ashrv8qi3 = CODE_FOR_nothing,
  CODE_FOR_ashlv4qi3 = 11104,
  CODE_FOR_lshrv4qi3 = 11105,
  CODE_FOR_ashrv4qi3 = 11106,
   CODE_FOR_vashlv8qi3 = CODE_FOR_nothing,
   CODE_FOR_vlshrv8qi3 = CODE_FOR_nothing,
   CODE_FOR_vashrv8qi3 = CODE_FOR_nothing,
  CODE_FOR_vashlv4qi3 = 11107,
  CODE_FOR_vlshrv4qi3 = 11108,
  CODE_FOR_vashrv4qi3 = 11109,
   CODE_FOR_vec_shl_v2sf = CODE_FOR_nothing,
   CODE_FOR_vec_shl_v2si = CODE_FOR_nothing,
   CODE_FOR_vec_shl_v4hf = CODE_FOR_nothing,
   CODE_FOR_vec_shl_v4bf = CODE_FOR_nothing,
   CODE_FOR_vec_shl_v4hi = CODE_FOR_nothing,
   CODE_FOR_vec_shl_v8qi = CODE_FOR_nothing,
  CODE_FOR_vec_shl_v2hf = 11110,
  CODE_FOR_vec_shl_v2bf = 11111,
  CODE_FOR_vec_shl_v2hi = 11112,
  CODE_FOR_vec_shl_v4qi = 11113,
   CODE_FOR_vec_shr_v2sf = CODE_FOR_nothing,
   CODE_FOR_vec_shr_v2si = CODE_FOR_nothing,
   CODE_FOR_vec_shr_v4hf = CODE_FOR_nothing,
   CODE_FOR_vec_shr_v4bf = CODE_FOR_nothing,
   CODE_FOR_vec_shr_v4hi = CODE_FOR_nothing,
   CODE_FOR_vec_shr_v8qi = CODE_FOR_nothing,
  CODE_FOR_vec_shr_v2hf = 11114,
  CODE_FOR_vec_shr_v2bf = 11115,
  CODE_FOR_vec_shr_v2hi = 11116,
  CODE_FOR_vec_shr_v4qi = 11117,
  CODE_FOR_mmx_eqv8qi3 = 11118,
  CODE_FOR_mmx_eqv4hi3 = 11119,
  CODE_FOR_mmx_eqv2si3 = 11120,
   CODE_FOR_vec_cmpv8qiv8qi = CODE_FOR_nothing,
   CODE_FOR_vec_cmpv4hiv4hi = CODE_FOR_nothing,
   CODE_FOR_vec_cmpv2siv2si = CODE_FOR_nothing,
  CODE_FOR_vec_cmpv4qiv4qi = 11121,
  CODE_FOR_vec_cmpv2qiv2qi = 11122,
  CODE_FOR_vec_cmpv2hiv2hi = 11123,
   CODE_FOR_vec_cmpuv8qiv8qi = CODE_FOR_nothing,
   CODE_FOR_vec_cmpuv4hiv4hi = CODE_FOR_nothing,
   CODE_FOR_vec_cmpuv2siv2si = CODE_FOR_nothing,
  CODE_FOR_vec_cmpuv4qiv4qi = 11124,
  CODE_FOR_vec_cmpuv2qiv2qi = 11125,
  CODE_FOR_vec_cmpuv2hiv2hi = 11126,
   CODE_FOR_vcond_mask_v8qiv8qi = CODE_FOR_nothing,
   CODE_FOR_vcond_mask_v4hiv4hi = CODE_FOR_nothing,
   CODE_FOR_vcond_mask_v2siv2si = CODE_FOR_nothing,
   CODE_FOR_vcond_mask_v2sfv2si = CODE_FOR_nothing,
  CODE_FOR_vcond_mask_v4qiv4qi = 11127,
  CODE_FOR_vcond_mask_v2qiv2qi = 11128,
  CODE_FOR_vcond_mask_v2hiv2hi = 11129,
   CODE_FOR_one_cmplv8qi2 = CODE_FOR_nothing,
   CODE_FOR_one_cmplv4hi2 = CODE_FOR_nothing,
   CODE_FOR_one_cmplv2si2 = CODE_FOR_nothing,
   CODE_FOR_andnv8qi3 = CODE_FOR_nothing,
   CODE_FOR_andnv4hi3 = CODE_FOR_nothing,
   CODE_FOR_andnv2si3 = CODE_FOR_nothing,
  CODE_FOR_mmx_andv8qi3 = 11130,
  CODE_FOR_mmx_iorv8qi3 = 11131,
  CODE_FOR_mmx_xorv8qi3 = 11132,
  CODE_FOR_mmx_andv4hi3 = 11133,
  CODE_FOR_mmx_iorv4hi3 = 11134,
  CODE_FOR_mmx_xorv4hi3 = 11135,
  CODE_FOR_mmx_andv2si3 = 11136,
  CODE_FOR_mmx_iorv2si3 = 11137,
  CODE_FOR_mmx_xorv2si3 = 11138,
   CODE_FOR_andv8qi3 = CODE_FOR_nothing,
   CODE_FOR_iorv8qi3 = CODE_FOR_nothing,
   CODE_FOR_xorv8qi3 = CODE_FOR_nothing,
   CODE_FOR_andv4hi3 = CODE_FOR_nothing,
   CODE_FOR_iorv4hi3 = CODE_FOR_nothing,
   CODE_FOR_xorv4hi3 = CODE_FOR_nothing,
   CODE_FOR_andv2si3 = CODE_FOR_nothing,
   CODE_FOR_iorv2si3 = CODE_FOR_nothing,
   CODE_FOR_xorv2si3 = CODE_FOR_nothing,
  CODE_FOR_andv4qi3 = 11139,
  CODE_FOR_iorv4qi3 = 11140,
  CODE_FOR_xorv4qi3 = 11141,
  CODE_FOR_andv2qi3 = 11142,
  CODE_FOR_iorv2qi3 = 11143,
  CODE_FOR_xorv2qi3 = 11144,
  CODE_FOR_andv2hi3 = 11145,
  CODE_FOR_iorv2hi3 = 11146,
  CODE_FOR_xorv2hi3 = 11147,
   CODE_FOR_extendv4qiv4hi2 = CODE_FOR_nothing,
   CODE_FOR_zero_extendv4qiv4hi2 = CODE_FOR_nothing,
   CODE_FOR_extendv2hiv2si2 = CODE_FOR_nothing,
   CODE_FOR_zero_extendv2hiv2si2 = CODE_FOR_nothing,
   CODE_FOR_extendv2qiv2si2 = CODE_FOR_nothing,
   CODE_FOR_zero_extendv2qiv2si2 = CODE_FOR_nothing,
  CODE_FOR_extendv2qiv2hi2 = 11148,
  CODE_FOR_zero_extendv2qiv2hi2 = 11149,
   CODE_FOR_truncv4hiv4qi2 = CODE_FOR_nothing,
  CODE_FOR_truncv2hiv2qi2 = 11150,
   CODE_FOR_truncv2siv2qi2 = CODE_FOR_nothing,
   CODE_FOR_truncv2siv2hi2 = CODE_FOR_nothing,
   CODE_FOR_vec_pack_trunc_v4hi = CODE_FOR_nothing,
   CODE_FOR_vec_pack_trunc_v2si = CODE_FOR_nothing,
  CODE_FOR_vec_pack_trunc_v2hi = 11151,
   CODE_FOR_vec_unpacks_lo_v8qi = CODE_FOR_nothing,
   CODE_FOR_vec_unpacks_lo_v4hi = CODE_FOR_nothing,
   CODE_FOR_vec_unpacks_hi_v8qi = CODE_FOR_nothing,
   CODE_FOR_vec_unpacks_hi_v4hi = CODE_FOR_nothing,
   CODE_FOR_vec_unpacku_lo_v8qi = CODE_FOR_nothing,
   CODE_FOR_vec_unpacku_lo_v4hi = CODE_FOR_nothing,
   CODE_FOR_vec_unpacku_hi_v8qi = CODE_FOR_nothing,
   CODE_FOR_vec_unpacku_hi_v4hi = CODE_FOR_nothing,
  CODE_FOR_vec_unpacks_lo_v4qi = 11152,
  CODE_FOR_vec_unpacks_hi_v4qi = 11153,
  CODE_FOR_vec_unpacku_lo_v4qi = 11154,
  CODE_FOR_vec_unpacku_hi_v4qi = 11155,
  CODE_FOR_mmx_pshufw = 11156,
  CODE_FOR_vec_setv2si = 11157,
  CODE_FOR_vec_extractv2sisi = 11158,
  CODE_FOR_vec_initv2sisi = 11159,
  CODE_FOR_vec_setv4hf = 11160,
  CODE_FOR_vec_setv4bf = 11161,
  CODE_FOR_vec_setv4hi = 11162,
  CODE_FOR_vec_extractv4hfhf = 11163,
  CODE_FOR_vec_extractv4bfbf = 11164,
  CODE_FOR_vec_extractv4hihi = 11165,
  CODE_FOR_vec_initv4hihi = 11166,
   CODE_FOR_vec_initv4hfhf = CODE_FOR_nothing,
   CODE_FOR_vec_initv4bfbf = CODE_FOR_nothing,
   CODE_FOR_vec_setv8qi = CODE_FOR_nothing,
   CODE_FOR_vec_extractv8qiqi = CODE_FOR_nothing,
  CODE_FOR_vec_initv8qiqi = 11167,
  CODE_FOR_vec_setv2hf = 11168,
  CODE_FOR_vec_setv2bf = 11169,
  CODE_FOR_vec_setv2hi = 11170,
  CODE_FOR_vec_extractv2hfhf = 11171,
  CODE_FOR_vec_extractv2bfbf = 11172,
  CODE_FOR_vec_extractv2hihi = 11173,
  CODE_FOR_vec_setv4qi = 11174,
  CODE_FOR_vec_extractv4qiqi = 11175,
  CODE_FOR_vec_initv2hfhf = 11176,
  CODE_FOR_vec_initv2bfbf = 11177,
  CODE_FOR_vec_initv2hihi = 11178,
  CODE_FOR_vec_initv4qiqi = 11179,
  CODE_FOR_mmx_uavgv8qi3 = 11180,
  CODE_FOR_mmx_uavgv4hi3 = 11181,
   CODE_FOR_uavgv8qi3_ceil = CODE_FOR_nothing,
   CODE_FOR_uavgv4hi3_ceil = CODE_FOR_nothing,
  CODE_FOR_mmx_psadbw = 11182,
   CODE_FOR_reduc_and_scal_v8qi = CODE_FOR_nothing,
   CODE_FOR_reduc_ior_scal_v8qi = CODE_FOR_nothing,
   CODE_FOR_reduc_xor_scal_v8qi = CODE_FOR_nothing,
   CODE_FOR_reduc_and_scal_v4hi = CODE_FOR_nothing,
   CODE_FOR_reduc_ior_scal_v4hi = CODE_FOR_nothing,
   CODE_FOR_reduc_xor_scal_v4hi = CODE_FOR_nothing,
  CODE_FOR_reduc_and_scal_v4qi = 11183,
  CODE_FOR_reduc_ior_scal_v4qi = 11184,
  CODE_FOR_reduc_xor_scal_v4qi = 11185,
   CODE_FOR_reduc_plus_scal_v8qi = CODE_FOR_nothing,
   CODE_FOR_reduc_plus_scal_v4hi = CODE_FOR_nothing,
   CODE_FOR_reduc_smax_scal_v4hi = CODE_FOR_nothing,
   CODE_FOR_reduc_smin_scal_v4hi = CODE_FOR_nothing,
  CODE_FOR_reduc_smax_scal_v4qi = 11186,
  CODE_FOR_reduc_smin_scal_v4qi = 11187,
   CODE_FOR_reduc_umax_scal_v4hi = CODE_FOR_nothing,
   CODE_FOR_reduc_umin_scal_v4hi = CODE_FOR_nothing,
  CODE_FOR_reduc_umax_scal_v4qi = 11188,
  CODE_FOR_reduc_umin_scal_v4qi = 11189,
  CODE_FOR_reduc_plus_scal_v4qi = 11190,
   CODE_FOR_usadv8qi = CODE_FOR_nothing,
   CODE_FOR_usdot_prodv2siv8qi = CODE_FOR_nothing,
   CODE_FOR_sdot_prodv2siv8qi = CODE_FOR_nothing,
   CODE_FOR_udot_prodv2siv8qi = CODE_FOR_nothing,
   CODE_FOR_usdot_prodv2siv4hi = CODE_FOR_nothing,
   CODE_FOR_udot_prodv2siv4hi = CODE_FOR_nothing,
   CODE_FOR_sdot_prodv2siv4hi = CODE_FOR_nothing,
  CODE_FOR_mmx_maskmovq = 11191,
  CODE_FOR_mmx_emms = 11192,
  CODE_FOR_mmx_femms = 11193,
  CODE_FOR_movv64qi = 11194,
  CODE_FOR_movv32qi = 11195,
  CODE_FOR_movv16qi = 11196,
  CODE_FOR_movv32hi = 11197,
  CODE_FOR_movv16hi = 11198,
  CODE_FOR_movv8hi = 11199,
  CODE_FOR_movv16si = 11200,
  CODE_FOR_movv8si = 11201,
  CODE_FOR_movv4si = 11202,
  CODE_FOR_movv8di = 11203,
  CODE_FOR_movv4di = 11204,
  CODE_FOR_movv2di = 11205,
  CODE_FOR_movv4ti = 11206,
  CODE_FOR_movv2ti = 11207,
  CODE_FOR_movv1ti = 11208,
  CODE_FOR_movv32hf = 11209,
  CODE_FOR_movv16hf = 11210,
  CODE_FOR_movv8hf = 11211,
  CODE_FOR_movv32bf = 11212,
  CODE_FOR_movv16bf = 11213,
  CODE_FOR_movv8bf = 11214,
  CODE_FOR_movv16sf = 11215,
  CODE_FOR_movv8sf = 11216,
  CODE_FOR_movv4sf = 11217,
  CODE_FOR_movv8df = 11218,
  CODE_FOR_movv4df = 11219,
  CODE_FOR_movv2df = 11220,
  CODE_FOR_avx512f_loadv16si_mask = 11221,
  CODE_FOR_avx512vl_loadv8si_mask = 11222,
  CODE_FOR_avx512vl_loadv4si_mask = 11223,
  CODE_FOR_avx512f_loadv8di_mask = 11224,
  CODE_FOR_avx512vl_loadv4di_mask = 11225,
  CODE_FOR_avx512vl_loadv2di_mask = 11226,
  CODE_FOR_avx512f_loadv16sf_mask = 11227,
  CODE_FOR_avx512vl_loadv8sf_mask = 11228,
  CODE_FOR_avx512vl_loadv4sf_mask = 11229,
  CODE_FOR_avx512f_loadv8df_mask = 11230,
  CODE_FOR_avx512vl_loadv4df_mask = 11231,
  CODE_FOR_avx512vl_loadv2df_mask = 11232,
  CODE_FOR_avx512bw_loadv64qi_mask = 11233,
  CODE_FOR_avx512vl_loadv16qi_mask = 11234,
  CODE_FOR_avx512vl_loadv32qi_mask = 11235,
  CODE_FOR_avx512bw_loadv32hi_mask = 11236,
  CODE_FOR_avx512vl_loadv16hi_mask = 11237,
  CODE_FOR_avx512vl_loadv8hi_mask = 11238,
  CODE_FOR_avx512f_loadhf_mask = 11239,
  CODE_FOR_avx512f_loadsf_mask = 11240,
  CODE_FOR_avx512f_loaddf_mask = 11241,
  CODE_FOR_sse2_movq128 = 11242,
  CODE_FOR_movmisalignv64qi = 11243,
  CODE_FOR_movmisalignv32qi = 11244,
  CODE_FOR_movmisalignv16qi = 11245,
  CODE_FOR_movmisalignv32hi = 11246,
  CODE_FOR_movmisalignv16hi = 11247,
  CODE_FOR_movmisalignv8hi = 11248,
  CODE_FOR_movmisalignv16si = 11249,
  CODE_FOR_movmisalignv8si = 11250,
  CODE_FOR_movmisalignv4si = 11251,
  CODE_FOR_movmisalignv8di = 11252,
  CODE_FOR_movmisalignv4di = 11253,
  CODE_FOR_movmisalignv2di = 11254,
  CODE_FOR_movmisalignv4ti = 11255,
  CODE_FOR_movmisalignv2ti = 11256,
  CODE_FOR_movmisalignv1ti = 11257,
  CODE_FOR_movmisalignv32hf = 11258,
  CODE_FOR_movmisalignv16hf = 11259,
  CODE_FOR_movmisalignv8hf = 11260,
  CODE_FOR_movmisalignv32bf = 11261,
  CODE_FOR_movmisalignv16bf = 11262,
  CODE_FOR_movmisalignv8bf = 11263,
  CODE_FOR_movmisalignv16sf = 11264,
  CODE_FOR_movmisalignv8sf = 11265,
  CODE_FOR_movmisalignv4sf = 11266,
  CODE_FOR_movmisalignv8df = 11267,
  CODE_FOR_movmisalignv4df = 11268,
  CODE_FOR_movmisalignv2df = 11269,
   CODE_FOR_storentdi = CODE_FOR_nothing,
  CODE_FOR_storentsi = 11270,
  CODE_FOR_storentsf = 11271,
  CODE_FOR_storentdf = 11272,
  CODE_FOR_storentv8di = 11273,
  CODE_FOR_storentv4di = 11274,
  CODE_FOR_storentv2di = 11275,
  CODE_FOR_storentv16sf = 11276,
  CODE_FOR_storentv8sf = 11277,
  CODE_FOR_storentv4sf = 11278,
  CODE_FOR_storentv8df = 11279,
  CODE_FOR_storentv4df = 11280,
  CODE_FOR_storentv2df = 11281,
  CODE_FOR_kmovb = 11282,
  CODE_FOR_kmovw = 11283,
  CODE_FOR_kmovd = 11284,
  CODE_FOR_kmovq = 11285,
  CODE_FOR_kortestqi = 11286,
  CODE_FOR_kortesthi = 11287,
  CODE_FOR_kortestsi = 11288,
  CODE_FOR_kortestdi = 11289,
  CODE_FOR_absv32bf2 = 11290,
  CODE_FOR_negv32bf2 = 11291,
  CODE_FOR_absv16bf2 = 11292,
  CODE_FOR_negv16bf2 = 11293,
  CODE_FOR_absv8bf2 = 11294,
  CODE_FOR_negv8bf2 = 11295,
  CODE_FOR_absv32hf2 = 11296,
  CODE_FOR_negv32hf2 = 11297,
  CODE_FOR_absv16hf2 = 11298,
  CODE_FOR_negv16hf2 = 11299,
  CODE_FOR_absv8hf2 = 11300,
  CODE_FOR_negv8hf2 = 11301,
  CODE_FOR_absv16sf2 = 11302,
  CODE_FOR_negv16sf2 = 11303,
  CODE_FOR_absv8sf2 = 11304,
  CODE_FOR_negv8sf2 = 11305,
  CODE_FOR_absv4sf2 = 11306,
  CODE_FOR_negv4sf2 = 11307,
  CODE_FOR_absv8df2 = 11308,
  CODE_FOR_negv8df2 = 11309,
  CODE_FOR_absv4df2 = 11310,
  CODE_FOR_negv4df2 = 11311,
  CODE_FOR_absv2df2 = 11312,
  CODE_FOR_negv2df2 = 11313,
  CODE_FOR_cond_addv32hf = 11314,
  CODE_FOR_cond_subv32hf = 11315,
  CODE_FOR_cond_addv16hf = 11316,
  CODE_FOR_cond_subv16hf = 11317,
  CODE_FOR_cond_addv8hf = 11318,
  CODE_FOR_cond_subv8hf = 11319,
  CODE_FOR_cond_addv16sf = 11320,
  CODE_FOR_cond_subv16sf = 11321,
  CODE_FOR_cond_addv8sf = 11322,
  CODE_FOR_cond_subv8sf = 11323,
  CODE_FOR_cond_addv4sf = 11324,
  CODE_FOR_cond_subv4sf = 11325,
  CODE_FOR_cond_addv8df = 11326,
  CODE_FOR_cond_subv8df = 11327,
  CODE_FOR_cond_addv4df = 11328,
  CODE_FOR_cond_subv4df = 11329,
  CODE_FOR_cond_addv2df = 11330,
  CODE_FOR_cond_subv2df = 11331,
  CODE_FOR_addv32hf3 = 11332,
  CODE_FOR_addv32hf3_round = 11333,
  CODE_FOR_addv32hf3_mask = 11334,
  CODE_FOR_addv32hf3_mask_round = 11335,
  CODE_FOR_subv32hf3 = 11336,
  CODE_FOR_subv32hf3_round = 11337,
  CODE_FOR_subv32hf3_mask = 11338,
  CODE_FOR_subv32hf3_mask_round = 11339,
  CODE_FOR_addv16hf3 = 11340,
   CODE_FOR_addv16hf3_round = CODE_FOR_nothing,
  CODE_FOR_addv16hf3_mask = 11341,
   CODE_FOR_addv16hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_subv16hf3 = 11342,
   CODE_FOR_subv16hf3_round = CODE_FOR_nothing,
  CODE_FOR_subv16hf3_mask = 11343,
   CODE_FOR_subv16hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_addv8hf3 = 11344,
   CODE_FOR_addv8hf3_round = CODE_FOR_nothing,
  CODE_FOR_addv8hf3_mask = 11345,
   CODE_FOR_addv8hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_subv8hf3 = 11346,
   CODE_FOR_subv8hf3_round = CODE_FOR_nothing,
  CODE_FOR_subv8hf3_mask = 11347,
   CODE_FOR_subv8hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_addv16sf3 = 11348,
  CODE_FOR_addv16sf3_round = 11349,
  CODE_FOR_addv16sf3_mask = 11350,
  CODE_FOR_addv16sf3_mask_round = 11351,
  CODE_FOR_subv16sf3 = 11352,
  CODE_FOR_subv16sf3_round = 11353,
  CODE_FOR_subv16sf3_mask = 11354,
  CODE_FOR_subv16sf3_mask_round = 11355,
  CODE_FOR_addv8sf3 = 11356,
   CODE_FOR_addv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_addv8sf3_mask = 11357,
   CODE_FOR_addv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_subv8sf3 = 11358,
   CODE_FOR_subv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_subv8sf3_mask = 11359,
   CODE_FOR_subv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_addv4sf3 = 11360,
   CODE_FOR_addv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_addv4sf3_mask = 11361,
   CODE_FOR_addv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_subv4sf3 = 11362,
   CODE_FOR_subv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_subv4sf3_mask = 11363,
   CODE_FOR_subv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_addv8df3 = 11364,
  CODE_FOR_addv8df3_round = 11365,
  CODE_FOR_addv8df3_mask = 11366,
  CODE_FOR_addv8df3_mask_round = 11367,
  CODE_FOR_subv8df3 = 11368,
  CODE_FOR_subv8df3_round = 11369,
  CODE_FOR_subv8df3_mask = 11370,
  CODE_FOR_subv8df3_mask_round = 11371,
  CODE_FOR_addv4df3 = 11372,
   CODE_FOR_addv4df3_round = CODE_FOR_nothing,
  CODE_FOR_addv4df3_mask = 11373,
   CODE_FOR_addv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_subv4df3 = 11374,
   CODE_FOR_subv4df3_round = CODE_FOR_nothing,
  CODE_FOR_subv4df3_mask = 11375,
   CODE_FOR_subv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_addv2df3 = 11376,
   CODE_FOR_addv2df3_round = CODE_FOR_nothing,
  CODE_FOR_addv2df3_mask = 11377,
   CODE_FOR_addv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_subv2df3 = 11378,
   CODE_FOR_subv2df3_round = CODE_FOR_nothing,
  CODE_FOR_subv2df3_mask = 11379,
   CODE_FOR_subv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_addv32bf3 = 11380,
  CODE_FOR_addv32bf3_mask = 11381,
  CODE_FOR_subv32bf3 = 11382,
  CODE_FOR_subv32bf3_mask = 11383,
  CODE_FOR_addv16bf3 = 11384,
  CODE_FOR_addv16bf3_mask = 11385,
  CODE_FOR_subv16bf3 = 11386,
  CODE_FOR_subv16bf3_mask = 11387,
  CODE_FOR_addv8bf3 = 11388,
  CODE_FOR_addv8bf3_mask = 11389,
  CODE_FOR_subv8bf3 = 11390,
  CODE_FOR_subv8bf3_mask = 11391,
  CODE_FOR_cond_mulv32hf = 11392,
  CODE_FOR_cond_mulv16hf = 11393,
  CODE_FOR_cond_mulv8hf = 11394,
  CODE_FOR_cond_mulv16sf = 11395,
  CODE_FOR_cond_mulv8sf = 11396,
  CODE_FOR_cond_mulv4sf = 11397,
  CODE_FOR_cond_mulv8df = 11398,
  CODE_FOR_cond_mulv4df = 11399,
  CODE_FOR_cond_mulv2df = 11400,
  CODE_FOR_mulv32hf3 = 11401,
  CODE_FOR_mulv32hf3_round = 11402,
  CODE_FOR_mulv32hf3_mask = 11403,
  CODE_FOR_mulv32hf3_mask_round = 11404,
  CODE_FOR_mulv16hf3 = 11405,
   CODE_FOR_mulv16hf3_round = CODE_FOR_nothing,
  CODE_FOR_mulv16hf3_mask = 11406,
   CODE_FOR_mulv16hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_mulv8hf3 = 11407,
   CODE_FOR_mulv8hf3_round = CODE_FOR_nothing,
  CODE_FOR_mulv8hf3_mask = 11408,
   CODE_FOR_mulv8hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_mulv16sf3 = 11409,
  CODE_FOR_mulv16sf3_round = 11410,
  CODE_FOR_mulv16sf3_mask = 11411,
  CODE_FOR_mulv16sf3_mask_round = 11412,
  CODE_FOR_mulv8sf3 = 11413,
   CODE_FOR_mulv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_mulv8sf3_mask = 11414,
   CODE_FOR_mulv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_mulv4sf3 = 11415,
   CODE_FOR_mulv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_mulv4sf3_mask = 11416,
   CODE_FOR_mulv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_mulv8df3 = 11417,
  CODE_FOR_mulv8df3_round = 11418,
  CODE_FOR_mulv8df3_mask = 11419,
  CODE_FOR_mulv8df3_mask_round = 11420,
  CODE_FOR_mulv4df3 = 11421,
   CODE_FOR_mulv4df3_round = CODE_FOR_nothing,
  CODE_FOR_mulv4df3_mask = 11422,
   CODE_FOR_mulv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_mulv2df3 = 11423,
   CODE_FOR_mulv2df3_round = CODE_FOR_nothing,
  CODE_FOR_mulv2df3_mask = 11424,
   CODE_FOR_mulv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_mulv32bf3 = 11425,
  CODE_FOR_mulv32bf3_mask = 11426,
  CODE_FOR_mulv16bf3 = 11427,
  CODE_FOR_mulv16bf3_mask = 11428,
  CODE_FOR_mulv8bf3 = 11429,
  CODE_FOR_mulv8bf3_mask = 11430,
  CODE_FOR_divv8df3 = 11431,
  CODE_FOR_divv4df3 = 11432,
  CODE_FOR_divv2df3 = 11433,
  CODE_FOR_divv32hf3 = 11434,
  CODE_FOR_divv16hf3 = 11435,
  CODE_FOR_divv8hf3 = 11436,
  CODE_FOR_divv16sf3 = 11437,
  CODE_FOR_divv8sf3 = 11438,
  CODE_FOR_divv4sf3 = 11439,
  CODE_FOR_divv32bf3 = 11440,
  CODE_FOR_divv16bf3 = 11441,
  CODE_FOR_divv8bf3 = 11442,
  CODE_FOR_cond_divv32hf = 11443,
  CODE_FOR_cond_divv16hf = 11444,
  CODE_FOR_cond_divv8hf = 11445,
  CODE_FOR_cond_divv16sf = 11446,
  CODE_FOR_cond_divv8sf = 11447,
  CODE_FOR_cond_divv4sf = 11448,
  CODE_FOR_cond_divv8df = 11449,
  CODE_FOR_cond_divv4df = 11450,
  CODE_FOR_cond_divv2df = 11451,
  CODE_FOR_sqrtv32bf2 = 11452,
  CODE_FOR_sqrtv16bf2 = 11453,
  CODE_FOR_sqrtv8bf2 = 11454,
  CODE_FOR_sqrtv32hf2 = 11455,
  CODE_FOR_sqrtv16hf2 = 11456,
  CODE_FOR_sqrtv8hf2 = 11457,
  CODE_FOR_sqrtv8df2 = 11458,
  CODE_FOR_sqrtv4df2 = 11459,
  CODE_FOR_sqrtv2df2 = 11460,
  CODE_FOR_sqrtv16sf2 = 11461,
  CODE_FOR_sqrtv8sf2 = 11462,
  CODE_FOR_sqrtv4sf2 = 11463,
  CODE_FOR_rsqrtv8sf2 = 11464,
  CODE_FOR_rsqrtv4sf2 = 11465,
  CODE_FOR_rsqrtv32hf2 = 11466,
  CODE_FOR_rsqrtv16hf2 = 11467,
  CODE_FOR_rsqrtv8hf2 = 11468,
  CODE_FOR_cond_smaxv32hf = 11469,
  CODE_FOR_cond_sminv32hf = 11470,
  CODE_FOR_cond_smaxv16hf = 11471,
  CODE_FOR_cond_sminv16hf = 11472,
  CODE_FOR_cond_smaxv8hf = 11473,
  CODE_FOR_cond_sminv8hf = 11474,
  CODE_FOR_cond_smaxv16sf = 11475,
  CODE_FOR_cond_sminv16sf = 11476,
  CODE_FOR_cond_smaxv8sf = 11477,
  CODE_FOR_cond_sminv8sf = 11478,
  CODE_FOR_cond_smaxv4sf = 11479,
  CODE_FOR_cond_sminv4sf = 11480,
  CODE_FOR_cond_smaxv8df = 11481,
  CODE_FOR_cond_sminv8df = 11482,
  CODE_FOR_cond_smaxv4df = 11483,
  CODE_FOR_cond_sminv4df = 11484,
  CODE_FOR_cond_smaxv2df = 11485,
  CODE_FOR_cond_sminv2df = 11486,
  CODE_FOR_smaxv32hf3 = 11487,
  CODE_FOR_smaxv32hf3_round = 11488,
  CODE_FOR_smaxv32hf3_mask = 11489,
  CODE_FOR_smaxv32hf3_mask_round = 11490,
  CODE_FOR_sminv32hf3 = 11491,
  CODE_FOR_sminv32hf3_round = 11492,
  CODE_FOR_sminv32hf3_mask = 11493,
  CODE_FOR_sminv32hf3_mask_round = 11494,
  CODE_FOR_smaxv16hf3 = 11495,
   CODE_FOR_smaxv16hf3_round = CODE_FOR_nothing,
  CODE_FOR_smaxv16hf3_mask = 11496,
   CODE_FOR_smaxv16hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sminv16hf3 = 11497,
   CODE_FOR_sminv16hf3_round = CODE_FOR_nothing,
  CODE_FOR_sminv16hf3_mask = 11498,
   CODE_FOR_sminv16hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_smaxv8hf3 = 11499,
   CODE_FOR_smaxv8hf3_round = CODE_FOR_nothing,
  CODE_FOR_smaxv8hf3_mask = 11500,
   CODE_FOR_smaxv8hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sminv8hf3 = 11501,
   CODE_FOR_sminv8hf3_round = CODE_FOR_nothing,
  CODE_FOR_sminv8hf3_mask = 11502,
   CODE_FOR_sminv8hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_smaxv16sf3 = 11503,
  CODE_FOR_smaxv16sf3_round = 11504,
  CODE_FOR_smaxv16sf3_mask = 11505,
  CODE_FOR_smaxv16sf3_mask_round = 11506,
  CODE_FOR_sminv16sf3 = 11507,
  CODE_FOR_sminv16sf3_round = 11508,
  CODE_FOR_sminv16sf3_mask = 11509,
  CODE_FOR_sminv16sf3_mask_round = 11510,
  CODE_FOR_smaxv8sf3 = 11511,
   CODE_FOR_smaxv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_smaxv8sf3_mask = 11512,
   CODE_FOR_smaxv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sminv8sf3 = 11513,
   CODE_FOR_sminv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_sminv8sf3_mask = 11514,
   CODE_FOR_sminv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_smaxv4sf3 = 11515,
   CODE_FOR_smaxv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_smaxv4sf3_mask = 11516,
   CODE_FOR_smaxv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sminv4sf3 = 11517,
   CODE_FOR_sminv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_sminv4sf3_mask = 11518,
   CODE_FOR_sminv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_smaxv8df3 = 11519,
  CODE_FOR_smaxv8df3_round = 11520,
  CODE_FOR_smaxv8df3_mask = 11521,
  CODE_FOR_smaxv8df3_mask_round = 11522,
  CODE_FOR_sminv8df3 = 11523,
  CODE_FOR_sminv8df3_round = 11524,
  CODE_FOR_sminv8df3_mask = 11525,
  CODE_FOR_sminv8df3_mask_round = 11526,
  CODE_FOR_smaxv4df3 = 11527,
   CODE_FOR_smaxv4df3_round = CODE_FOR_nothing,
  CODE_FOR_smaxv4df3_mask = 11528,
   CODE_FOR_smaxv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sminv4df3 = 11529,
   CODE_FOR_sminv4df3_round = CODE_FOR_nothing,
  CODE_FOR_sminv4df3_mask = 11530,
   CODE_FOR_sminv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_smaxv2df3 = 11531,
   CODE_FOR_smaxv2df3_round = CODE_FOR_nothing,
  CODE_FOR_smaxv2df3_mask = 11532,
   CODE_FOR_smaxv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sminv2df3 = 11533,
   CODE_FOR_sminv2df3_round = CODE_FOR_nothing,
  CODE_FOR_sminv2df3_mask = 11534,
   CODE_FOR_sminv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_vmsmaxv8hf3 = 11535,
  CODE_FOR_avx512fp16_vmsmaxv8hf3_mask = 11536,
  CODE_FOR_avx512fp16_vmsmaxv8hf3_round = 11537,
  CODE_FOR_avx512fp16_vmsmaxv8hf3_mask_round = 11538,
  CODE_FOR_avx512fp16_vmsminv8hf3 = 11539,
  CODE_FOR_avx512fp16_vmsminv8hf3_mask = 11540,
  CODE_FOR_avx512fp16_vmsminv8hf3_round = 11541,
  CODE_FOR_avx512fp16_vmsminv8hf3_mask_round = 11542,
  CODE_FOR_sse_vmsmaxv4sf3 = 11543,
  CODE_FOR_sse_vmsmaxv4sf3_mask = 11544,
  CODE_FOR_sse_vmsmaxv4sf3_round = 11545,
  CODE_FOR_sse_vmsmaxv4sf3_mask_round = 11546,
  CODE_FOR_sse_vmsminv4sf3 = 11547,
  CODE_FOR_sse_vmsminv4sf3_mask = 11548,
  CODE_FOR_sse_vmsminv4sf3_round = 11549,
  CODE_FOR_sse_vmsminv4sf3_mask_round = 11550,
  CODE_FOR_sse2_vmsmaxv2df3 = 11551,
  CODE_FOR_sse2_vmsmaxv2df3_mask = 11552,
  CODE_FOR_sse2_vmsmaxv2df3_round = 11553,
  CODE_FOR_sse2_vmsmaxv2df3_mask_round = 11554,
  CODE_FOR_sse2_vmsminv2df3 = 11555,
  CODE_FOR_sse2_vmsminv2df3_mask = 11556,
  CODE_FOR_sse2_vmsminv2df3_round = 11557,
  CODE_FOR_sse2_vmsminv2df3_mask_round = 11558,
  CODE_FOR_sse3_haddv2df3 = 11559,
  CODE_FOR_reduc_plus_scal_v2df = 11560,
  CODE_FOR_reduc_plus_scal_v4sf = 11561,
  CODE_FOR_reduc_plus_scal_v8hf = 11562,
  CODE_FOR_reduc_plus_scal_v8hi = 11563,
  CODE_FOR_reduc_plus_scal_v4si = 11564,
  CODE_FOR_reduc_plus_scal_v2di = 11565,
  CODE_FOR_reduc_plus_scal_v16qi = 11566,
  CODE_FOR_reduc_plus_scal_v4df = 11567,
  CODE_FOR_reduc_plus_scal_v8sf = 11568,
  CODE_FOR_reduc_plus_scal_v16hf = 11569,
  CODE_FOR_reduc_plus_scal_v8df = 11570,
  CODE_FOR_reduc_plus_scal_v16sf = 11571,
  CODE_FOR_reduc_plus_scal_v32hf = 11572,
  CODE_FOR_reduc_plus_scal_v32qi = 11573,
  CODE_FOR_reduc_plus_scal_v16hi = 11574,
  CODE_FOR_reduc_plus_scal_v8si = 11575,
  CODE_FOR_reduc_plus_scal_v4di = 11576,
  CODE_FOR_reduc_plus_scal_v64qi = 11577,
  CODE_FOR_reduc_plus_scal_v32hi = 11578,
  CODE_FOR_reduc_plus_scal_v16si = 11579,
  CODE_FOR_reduc_plus_scal_v8di = 11580,
  CODE_FOR_reduc_smax_scal_v8hf = 11581,
  CODE_FOR_reduc_smin_scal_v8hf = 11582,
  CODE_FOR_reduc_smax_scal_v4sf = 11583,
  CODE_FOR_reduc_smin_scal_v4sf = 11584,
  CODE_FOR_reduc_smax_scal_v2df = 11585,
  CODE_FOR_reduc_smin_scal_v2df = 11586,
  CODE_FOR_reduc_smax_scal_v4si = 11587,
  CODE_FOR_reduc_smin_scal_v4si = 11588,
  CODE_FOR_reduc_smax_scal_v8hi = 11589,
  CODE_FOR_reduc_smin_scal_v8hi = 11590,
  CODE_FOR_reduc_smax_scal_v16qi = 11591,
  CODE_FOR_reduc_smin_scal_v16qi = 11592,
  CODE_FOR_reduc_smax_scal_v2di = 11593,
  CODE_FOR_reduc_smin_scal_v2di = 11594,
  CODE_FOR_reduc_smax_scal_v32qi = 11595,
  CODE_FOR_reduc_smin_scal_v32qi = 11596,
  CODE_FOR_reduc_smax_scal_v16hi = 11597,
  CODE_FOR_reduc_smin_scal_v16hi = 11598,
  CODE_FOR_reduc_smax_scal_v16hf = 11599,
  CODE_FOR_reduc_smin_scal_v16hf = 11600,
  CODE_FOR_reduc_smax_scal_v8si = 11601,
  CODE_FOR_reduc_smin_scal_v8si = 11602,
  CODE_FOR_reduc_smax_scal_v4di = 11603,
  CODE_FOR_reduc_smin_scal_v4di = 11604,
  CODE_FOR_reduc_smax_scal_v8sf = 11605,
  CODE_FOR_reduc_smin_scal_v8sf = 11606,
  CODE_FOR_reduc_smax_scal_v4df = 11607,
  CODE_FOR_reduc_smin_scal_v4df = 11608,
  CODE_FOR_reduc_smax_scal_v64qi = 11609,
  CODE_FOR_reduc_smin_scal_v64qi = 11610,
  CODE_FOR_reduc_smax_scal_v32hf = 11611,
  CODE_FOR_reduc_smin_scal_v32hf = 11612,
  CODE_FOR_reduc_smax_scal_v32hi = 11613,
  CODE_FOR_reduc_smin_scal_v32hi = 11614,
  CODE_FOR_reduc_smax_scal_v16si = 11615,
  CODE_FOR_reduc_smin_scal_v16si = 11616,
  CODE_FOR_reduc_smax_scal_v8di = 11617,
  CODE_FOR_reduc_smin_scal_v8di = 11618,
  CODE_FOR_reduc_smax_scal_v16sf = 11619,
  CODE_FOR_reduc_smin_scal_v16sf = 11620,
  CODE_FOR_reduc_smax_scal_v8df = 11621,
  CODE_FOR_reduc_smin_scal_v8df = 11622,
  CODE_FOR_reduc_umax_scal_v16si = 11623,
  CODE_FOR_reduc_umin_scal_v16si = 11624,
  CODE_FOR_reduc_umax_scal_v8di = 11625,
  CODE_FOR_reduc_umin_scal_v8di = 11626,
  CODE_FOR_reduc_umax_scal_v32hi = 11627,
  CODE_FOR_reduc_umin_scal_v32hi = 11628,
  CODE_FOR_reduc_umax_scal_v64qi = 11629,
  CODE_FOR_reduc_umin_scal_v64qi = 11630,
  CODE_FOR_reduc_umax_scal_v32qi = 11631,
  CODE_FOR_reduc_umin_scal_v32qi = 11632,
  CODE_FOR_reduc_umax_scal_v16hi = 11633,
  CODE_FOR_reduc_umin_scal_v16hi = 11634,
  CODE_FOR_reduc_umax_scal_v8si = 11635,
  CODE_FOR_reduc_umin_scal_v8si = 11636,
  CODE_FOR_reduc_umax_scal_v4di = 11637,
  CODE_FOR_reduc_umin_scal_v4di = 11638,
  CODE_FOR_reduc_umin_scal_v8hi = 11639,
  CODE_FOR_reduc_sbool_and_scal_qi = 11640,
  CODE_FOR_reduc_sbool_and_scal_hi = 11641,
  CODE_FOR_reduc_sbool_and_scal_si = 11642,
  CODE_FOR_reduc_sbool_and_scal_di = 11643,
  CODE_FOR_reduc_sbool_ior_scal_qi = 11644,
  CODE_FOR_reduc_sbool_ior_scal_hi = 11645,
  CODE_FOR_reduc_sbool_ior_scal_si = 11646,
  CODE_FOR_reduc_sbool_ior_scal_di = 11647,
  CODE_FOR_reduc_sbool_xor_scal_qi = 11648,
  CODE_FOR_reduc_sbool_xor_scal_hi = 11649,
  CODE_FOR_reduc_sbool_xor_scal_si = 11650,
   CODE_FOR_reduc_sbool_xor_scal_di = CODE_FOR_nothing,
  CODE_FOR_reduc_sbool_and_scal_v32qi = 11651,
  CODE_FOR_reduc_sbool_and_scal_v16qi = 11652,
  CODE_FOR_reduc_sbool_and_scal_v16hi = 11653,
  CODE_FOR_reduc_sbool_and_scal_v8hi = 11654,
  CODE_FOR_reduc_sbool_and_scal_v8si = 11655,
  CODE_FOR_reduc_sbool_and_scal_v4si = 11656,
  CODE_FOR_reduc_sbool_and_scal_v4di = 11657,
  CODE_FOR_reduc_sbool_and_scal_v2di = 11658,
  CODE_FOR_reduc_sbool_ior_scal_v32qi = 11659,
  CODE_FOR_reduc_sbool_ior_scal_v16qi = 11660,
  CODE_FOR_reduc_sbool_ior_scal_v16hi = 11661,
  CODE_FOR_reduc_sbool_ior_scal_v8hi = 11662,
  CODE_FOR_reduc_sbool_ior_scal_v8si = 11663,
  CODE_FOR_reduc_sbool_ior_scal_v4si = 11664,
  CODE_FOR_reduc_sbool_ior_scal_v4di = 11665,
  CODE_FOR_reduc_sbool_ior_scal_v2di = 11666,
  CODE_FOR_reduc_sbool_xor_scal_v32qi = 11667,
  CODE_FOR_reduc_sbool_xor_scal_v16qi = 11668,
  CODE_FOR_reduc_sbool_xor_scal_v4si = 11669,
  CODE_FOR_reduc_sbool_xor_scal_v2di = 11670,
  CODE_FOR_reduc_sbool_xor_scal_v8si = 11671,
  CODE_FOR_reduc_sbool_xor_scal_v4di = 11672,
  CODE_FOR_reduc_and_scal_v16qi = 11673,
  CODE_FOR_reduc_ior_scal_v16qi = 11674,
  CODE_FOR_reduc_xor_scal_v16qi = 11675,
  CODE_FOR_reduc_and_scal_v8hi = 11676,
  CODE_FOR_reduc_ior_scal_v8hi = 11677,
  CODE_FOR_reduc_xor_scal_v8hi = 11678,
  CODE_FOR_reduc_and_scal_v4si = 11679,
  CODE_FOR_reduc_ior_scal_v4si = 11680,
  CODE_FOR_reduc_xor_scal_v4si = 11681,
  CODE_FOR_reduc_and_scal_v2di = 11682,
  CODE_FOR_reduc_ior_scal_v2di = 11683,
  CODE_FOR_reduc_xor_scal_v2di = 11684,
  CODE_FOR_reduc_and_scal_v32qi = 11685,
  CODE_FOR_reduc_ior_scal_v32qi = 11686,
  CODE_FOR_reduc_xor_scal_v32qi = 11687,
  CODE_FOR_reduc_and_scal_v16hi = 11688,
  CODE_FOR_reduc_ior_scal_v16hi = 11689,
  CODE_FOR_reduc_xor_scal_v16hi = 11690,
  CODE_FOR_reduc_and_scal_v8si = 11691,
  CODE_FOR_reduc_ior_scal_v8si = 11692,
  CODE_FOR_reduc_xor_scal_v8si = 11693,
  CODE_FOR_reduc_and_scal_v4di = 11694,
  CODE_FOR_reduc_ior_scal_v4di = 11695,
  CODE_FOR_reduc_xor_scal_v4di = 11696,
  CODE_FOR_reduc_and_scal_v64qi = 11697,
  CODE_FOR_reduc_ior_scal_v64qi = 11698,
  CODE_FOR_reduc_xor_scal_v64qi = 11699,
  CODE_FOR_reduc_and_scal_v32hi = 11700,
  CODE_FOR_reduc_ior_scal_v32hi = 11701,
  CODE_FOR_reduc_xor_scal_v32hi = 11702,
  CODE_FOR_reduc_and_scal_v16si = 11703,
  CODE_FOR_reduc_ior_scal_v16si = 11704,
  CODE_FOR_reduc_xor_scal_v16si = 11705,
  CODE_FOR_reduc_and_scal_v8di = 11706,
  CODE_FOR_reduc_ior_scal_v8di = 11707,
  CODE_FOR_reduc_xor_scal_v8di = 11708,
  CODE_FOR_vec_cmpv16sihi = 11709,
  CODE_FOR_vec_cmpv8siqi = 11710,
  CODE_FOR_vec_cmpv4siqi = 11711,
  CODE_FOR_vec_cmpv8diqi = 11712,
  CODE_FOR_vec_cmpv4diqi = 11713,
  CODE_FOR_vec_cmpv2diqi = 11714,
  CODE_FOR_vec_cmpv32hfsi = 11715,
  CODE_FOR_vec_cmpv16hfhi = 11716,
  CODE_FOR_vec_cmpv8hfqi = 11717,
  CODE_FOR_vec_cmpv16sfhi = 11718,
  CODE_FOR_vec_cmpv8sfqi = 11719,
  CODE_FOR_vec_cmpv4sfqi = 11720,
  CODE_FOR_vec_cmpv8dfqi = 11721,
  CODE_FOR_vec_cmpv4dfqi = 11722,
  CODE_FOR_vec_cmpv2dfqi = 11723,
  CODE_FOR_vec_cmpv64qidi = 11724,
  CODE_FOR_vec_cmpv16qihi = 11725,
  CODE_FOR_vec_cmpv32qisi = 11726,
  CODE_FOR_vec_cmpv32hisi = 11727,
  CODE_FOR_vec_cmpv16hihi = 11728,
  CODE_FOR_vec_cmpv8hiqi = 11729,
  CODE_FOR_vec_cmpv32bfsi = 11730,
  CODE_FOR_vec_cmpv16bfhi = 11731,
  CODE_FOR_vec_cmpv8bfqi = 11732,
  CODE_FOR_vec_cmpv32qiv32qi = 11733,
  CODE_FOR_vec_cmpv16hiv16hi = 11734,
  CODE_FOR_vec_cmpv8siv8si = 11735,
  CODE_FOR_vec_cmpv4div4di = 11736,
  CODE_FOR_vec_cmpv16qiv16qi = 11737,
  CODE_FOR_vec_cmpv8hiv8hi = 11738,
  CODE_FOR_vec_cmpv4siv4si = 11739,
  CODE_FOR_vec_cmpv2div2di = 11740,
  CODE_FOR_vec_cmpv8sfv8si = 11741,
  CODE_FOR_vec_cmpv4dfv4di = 11742,
  CODE_FOR_vec_cmpv4sfv4si = 11743,
  CODE_FOR_vec_cmpv2dfv2di = 11744,
  CODE_FOR_vec_cmpuv16sihi = 11745,
  CODE_FOR_vec_cmpuv8siqi = 11746,
  CODE_FOR_vec_cmpuv4siqi = 11747,
  CODE_FOR_vec_cmpuv8diqi = 11748,
  CODE_FOR_vec_cmpuv4diqi = 11749,
  CODE_FOR_vec_cmpuv2diqi = 11750,
  CODE_FOR_vec_cmpuv64qidi = 11751,
  CODE_FOR_vec_cmpuv16qihi = 11752,
  CODE_FOR_vec_cmpuv32qisi = 11753,
  CODE_FOR_vec_cmpuv32hisi = 11754,
  CODE_FOR_vec_cmpuv16hihi = 11755,
  CODE_FOR_vec_cmpuv8hiqi = 11756,
  CODE_FOR_vec_cmpuv32qiv32qi = 11757,
  CODE_FOR_vec_cmpuv16hiv16hi = 11758,
  CODE_FOR_vec_cmpuv8siv8si = 11759,
  CODE_FOR_vec_cmpuv4div4di = 11760,
  CODE_FOR_vec_cmpuv16qiv16qi = 11761,
  CODE_FOR_vec_cmpuv8hiv8hi = 11762,
  CODE_FOR_vec_cmpuv4siv4si = 11763,
  CODE_FOR_vec_cmpuv2div2di = 11764,
  CODE_FOR_vec_cmpeqv2div2di = 11765,
  CODE_FOR_vec_cmpeqv1tiv1ti = 11766,
  CODE_FOR_vcond_mask_v16sihi = 11767,
  CODE_FOR_vcond_mask_v8siqi = 11768,
  CODE_FOR_vcond_mask_v4siqi = 11769,
  CODE_FOR_vcond_mask_v8diqi = 11770,
  CODE_FOR_vcond_mask_v4diqi = 11771,
  CODE_FOR_vcond_mask_v2diqi = 11772,
  CODE_FOR_vcond_mask_v16sfhi = 11773,
  CODE_FOR_vcond_mask_v8sfqi = 11774,
  CODE_FOR_vcond_mask_v4sfqi = 11775,
  CODE_FOR_vcond_mask_v8dfqi = 11776,
  CODE_FOR_vcond_mask_v4dfqi = 11777,
  CODE_FOR_vcond_mask_v2dfqi = 11778,
  CODE_FOR_vcond_mask_v64qidi = 11779,
  CODE_FOR_vcond_mask_v16qihi = 11780,
  CODE_FOR_vcond_mask_v32qisi = 11781,
  CODE_FOR_vcond_mask_v32hisi = 11782,
  CODE_FOR_vcond_mask_v16hihi = 11783,
  CODE_FOR_vcond_mask_v8hiqi = 11784,
  CODE_FOR_vcond_mask_v32hfsi = 11785,
  CODE_FOR_vcond_mask_v16hfhi = 11786,
  CODE_FOR_vcond_mask_v8hfqi = 11787,
  CODE_FOR_vcond_mask_v32bfsi = 11788,
  CODE_FOR_vcond_mask_v16bfhi = 11789,
  CODE_FOR_vcond_mask_v8bfqi = 11790,
  CODE_FOR_vcond_mask_v32qiv32qi = 11791,
  CODE_FOR_vcond_mask_v16hiv16hi = 11792,
  CODE_FOR_vcond_mask_v8siv8si = 11793,
  CODE_FOR_vcond_mask_v4div4di = 11794,
  CODE_FOR_vcond_mask_v16qiv16qi = 11795,
  CODE_FOR_vcond_mask_v8hiv8hi = 11796,
  CODE_FOR_vcond_mask_v4siv4si = 11797,
  CODE_FOR_vcond_mask_v2div2di = 11798,
  CODE_FOR_vcond_mask_v1tiv1ti = 11799,
  CODE_FOR_vcond_mask_v8sfv8si = 11800,
  CODE_FOR_vcond_mask_v4dfv4di = 11801,
  CODE_FOR_vcond_mask_v4sfv4si = 11802,
  CODE_FOR_vcond_mask_v2dfv2di = 11803,
  CODE_FOR_vcond_mask_qiqi = 11804,
  CODE_FOR_vcond_mask_hihi = 11805,
  CODE_FOR_vcond_mask_sisi = 11806,
  CODE_FOR_vcond_mask_didi = 11807,
  CODE_FOR_andv16bf3 = 11808,
   CODE_FOR_andv16bf3_mask = CODE_FOR_nothing,
  CODE_FOR_iorv16bf3 = 11809,
   CODE_FOR_iorv16bf3_mask = CODE_FOR_nothing,
  CODE_FOR_xorv16bf3 = 11810,
   CODE_FOR_xorv16bf3_mask = CODE_FOR_nothing,
  CODE_FOR_andv8bf3 = 11811,
   CODE_FOR_andv8bf3_mask = CODE_FOR_nothing,
  CODE_FOR_iorv8bf3 = 11812,
   CODE_FOR_iorv8bf3_mask = CODE_FOR_nothing,
  CODE_FOR_xorv8bf3 = 11813,
   CODE_FOR_xorv8bf3_mask = CODE_FOR_nothing,
  CODE_FOR_andv16hf3 = 11814,
   CODE_FOR_andv16hf3_mask = CODE_FOR_nothing,
  CODE_FOR_iorv16hf3 = 11815,
   CODE_FOR_iorv16hf3_mask = CODE_FOR_nothing,
  CODE_FOR_xorv16hf3 = 11816,
   CODE_FOR_xorv16hf3_mask = CODE_FOR_nothing,
  CODE_FOR_andv8hf3 = 11817,
   CODE_FOR_andv8hf3_mask = CODE_FOR_nothing,
  CODE_FOR_iorv8hf3 = 11818,
   CODE_FOR_iorv8hf3_mask = CODE_FOR_nothing,
  CODE_FOR_xorv8hf3 = 11819,
   CODE_FOR_xorv8hf3_mask = CODE_FOR_nothing,
  CODE_FOR_andv8sf3 = 11820,
  CODE_FOR_andv8sf3_mask = 11821,
  CODE_FOR_iorv8sf3 = 11822,
  CODE_FOR_iorv8sf3_mask = 11823,
  CODE_FOR_xorv8sf3 = 11824,
  CODE_FOR_xorv8sf3_mask = 11825,
  CODE_FOR_andv4sf3 = 11826,
  CODE_FOR_andv4sf3_mask = 11827,
  CODE_FOR_iorv4sf3 = 11828,
  CODE_FOR_iorv4sf3_mask = 11829,
  CODE_FOR_xorv4sf3 = 11830,
  CODE_FOR_xorv4sf3_mask = 11831,
  CODE_FOR_andv4df3 = 11832,
  CODE_FOR_andv4df3_mask = 11833,
  CODE_FOR_iorv4df3 = 11834,
  CODE_FOR_iorv4df3_mask = 11835,
  CODE_FOR_xorv4df3 = 11836,
  CODE_FOR_xorv4df3_mask = 11837,
  CODE_FOR_andv2df3 = 11838,
  CODE_FOR_andv2df3_mask = 11839,
  CODE_FOR_iorv2df3 = 11840,
  CODE_FOR_iorv2df3_mask = 11841,
  CODE_FOR_xorv2df3 = 11842,
  CODE_FOR_xorv2df3_mask = 11843,
  CODE_FOR_andv32bf3 = 11844,
   CODE_FOR_andv32bf3_mask = CODE_FOR_nothing,
  CODE_FOR_iorv32bf3 = 11845,
   CODE_FOR_iorv32bf3_mask = CODE_FOR_nothing,
  CODE_FOR_xorv32bf3 = 11846,
   CODE_FOR_xorv32bf3_mask = CODE_FOR_nothing,
  CODE_FOR_andv32hf3 = 11847,
   CODE_FOR_andv32hf3_mask = CODE_FOR_nothing,
  CODE_FOR_iorv32hf3 = 11848,
   CODE_FOR_iorv32hf3_mask = CODE_FOR_nothing,
  CODE_FOR_xorv32hf3 = 11849,
   CODE_FOR_xorv32hf3_mask = CODE_FOR_nothing,
  CODE_FOR_andv16sf3 = 11850,
  CODE_FOR_andv16sf3_mask = 11851,
  CODE_FOR_iorv16sf3 = 11852,
  CODE_FOR_iorv16sf3_mask = 11853,
  CODE_FOR_xorv16sf3 = 11854,
  CODE_FOR_xorv16sf3_mask = 11855,
  CODE_FOR_andv8df3 = 11856,
  CODE_FOR_andv8df3_mask = 11857,
  CODE_FOR_iorv8df3 = 11858,
  CODE_FOR_iorv8df3_mask = 11859,
  CODE_FOR_xorv8df3 = 11860,
  CODE_FOR_xorv8df3_mask = 11861,
  CODE_FOR_copysignv32bf3 = 11862,
  CODE_FOR_copysignv16bf3 = 11863,
  CODE_FOR_copysignv8bf3 = 11864,
  CODE_FOR_copysignv32hf3 = 11865,
  CODE_FOR_copysignv16hf3 = 11866,
  CODE_FOR_copysignv8hf3 = 11867,
  CODE_FOR_copysignv16sf3 = 11868,
  CODE_FOR_copysignv8sf3 = 11869,
  CODE_FOR_copysignv4sf3 = 11870,
  CODE_FOR_copysignv8df3 = 11871,
  CODE_FOR_copysignv4df3 = 11872,
  CODE_FOR_copysignv2df3 = 11873,
  CODE_FOR_xorsignv32bf3 = 11874,
  CODE_FOR_xorsignv16bf3 = 11875,
  CODE_FOR_xorsignv8bf3 = 11876,
  CODE_FOR_xorsignv32hf3 = 11877,
  CODE_FOR_xorsignv16hf3 = 11878,
  CODE_FOR_xorsignv8hf3 = 11879,
  CODE_FOR_xorsignv16sf3 = 11880,
  CODE_FOR_xorsignv8sf3 = 11881,
  CODE_FOR_xorsignv4sf3 = 11882,
  CODE_FOR_xorsignv8df3 = 11883,
  CODE_FOR_xorsignv4df3 = 11884,
  CODE_FOR_xorsignv2df3 = 11885,
  CODE_FOR_signbitv16sf2 = 11886,
  CODE_FOR_signbitv8sf2 = 11887,
  CODE_FOR_signbitv4sf2 = 11888,
  CODE_FOR_andtf3 = 11889,
  CODE_FOR_iortf3 = 11890,
  CODE_FOR_xortf3 = 11891,
  CODE_FOR_fmasf4 = 11892,
  CODE_FOR_fmadf4 = 11893,
  CODE_FOR_fmav4sf4 = 11894,
  CODE_FOR_fmav2df4 = 11895,
  CODE_FOR_fmav8sf4 = 11896,
  CODE_FOR_fmav4df4 = 11897,
  CODE_FOR_fmav16sf4 = 11898,
  CODE_FOR_fmav8df4 = 11899,
  CODE_FOR_fmahf4 = 11900,
  CODE_FOR_fmav8hf4 = 11901,
  CODE_FOR_fmav16hf4 = 11902,
  CODE_FOR_fmav32hf4 = 11903,
  CODE_FOR_fmav8bf4 = 11904,
  CODE_FOR_fmav16bf4 = 11905,
  CODE_FOR_fmav32bf4 = 11906,
  CODE_FOR_fmssf4 = 11907,
  CODE_FOR_fmsdf4 = 11908,
  CODE_FOR_fmsv4sf4 = 11909,
  CODE_FOR_fmsv2df4 = 11910,
  CODE_FOR_fmsv8sf4 = 11911,
  CODE_FOR_fmsv4df4 = 11912,
  CODE_FOR_fmsv16sf4 = 11913,
  CODE_FOR_fmsv8df4 = 11914,
  CODE_FOR_fmshf4 = 11915,
  CODE_FOR_fmsv8hf4 = 11916,
  CODE_FOR_fmsv16hf4 = 11917,
  CODE_FOR_fmsv32hf4 = 11918,
  CODE_FOR_fmsv8bf4 = 11919,
  CODE_FOR_fmsv16bf4 = 11920,
  CODE_FOR_fmsv32bf4 = 11921,
  CODE_FOR_fnmasf4 = 11922,
  CODE_FOR_fnmadf4 = 11923,
  CODE_FOR_fnmav4sf4 = 11924,
  CODE_FOR_fnmav2df4 = 11925,
  CODE_FOR_fnmav8sf4 = 11926,
  CODE_FOR_fnmav4df4 = 11927,
  CODE_FOR_fnmav16sf4 = 11928,
  CODE_FOR_fnmav8df4 = 11929,
  CODE_FOR_fnmahf4 = 11930,
  CODE_FOR_fnmav8hf4 = 11931,
  CODE_FOR_fnmav16hf4 = 11932,
  CODE_FOR_fnmav32hf4 = 11933,
  CODE_FOR_fnmav8bf4 = 11934,
  CODE_FOR_fnmav16bf4 = 11935,
  CODE_FOR_fnmav32bf4 = 11936,
  CODE_FOR_fnmssf4 = 11937,
  CODE_FOR_fnmsdf4 = 11938,
  CODE_FOR_fnmsv4sf4 = 11939,
  CODE_FOR_fnmsv2df4 = 11940,
  CODE_FOR_fnmsv8sf4 = 11941,
  CODE_FOR_fnmsv4df4 = 11942,
  CODE_FOR_fnmsv16sf4 = 11943,
  CODE_FOR_fnmsv8df4 = 11944,
  CODE_FOR_fnmshf4 = 11945,
  CODE_FOR_fnmsv8hf4 = 11946,
  CODE_FOR_fnmsv16hf4 = 11947,
  CODE_FOR_fnmsv32hf4 = 11948,
  CODE_FOR_fnmsv8bf4 = 11949,
  CODE_FOR_fnmsv16bf4 = 11950,
  CODE_FOR_fnmsv32bf4 = 11951,
  CODE_FOR_fma4i_fmadd_sf = 11952,
  CODE_FOR_fma4i_fmadd_df = 11953,
  CODE_FOR_fma4i_fmadd_v4sf = 11954,
  CODE_FOR_fma4i_fmadd_v2df = 11955,
  CODE_FOR_fma4i_fmadd_v8sf = 11956,
  CODE_FOR_fma4i_fmadd_v4df = 11957,
  CODE_FOR_fma4i_fmadd_v16sf = 11958,
  CODE_FOR_fma4i_fmadd_v8df = 11959,
  CODE_FOR_fma4i_fmsub_sf = 11960,
  CODE_FOR_fma4i_fmsub_df = 11961,
  CODE_FOR_fma4i_fmsub_v4sf = 11962,
  CODE_FOR_fma4i_fmsub_v2df = 11963,
  CODE_FOR_fma4i_fmsub_v8sf = 11964,
  CODE_FOR_fma4i_fmsub_v4df = 11965,
  CODE_FOR_fma4i_fmsub_v16sf = 11966,
  CODE_FOR_fma4i_fmsub_v8df = 11967,
  CODE_FOR_fma4i_fnmadd_sf = 11968,
  CODE_FOR_fma4i_fnmadd_df = 11969,
  CODE_FOR_fma4i_fnmadd_v4sf = 11970,
  CODE_FOR_fma4i_fnmadd_v2df = 11971,
  CODE_FOR_fma4i_fnmadd_v8sf = 11972,
  CODE_FOR_fma4i_fnmadd_v4df = 11973,
  CODE_FOR_fma4i_fnmadd_v16sf = 11974,
  CODE_FOR_fma4i_fnmadd_v8df = 11975,
  CODE_FOR_fma4i_fnmsub_sf = 11976,
  CODE_FOR_fma4i_fnmsub_df = 11977,
  CODE_FOR_fma4i_fnmsub_v4sf = 11978,
  CODE_FOR_fma4i_fnmsub_v2df = 11979,
  CODE_FOR_fma4i_fnmsub_v8sf = 11980,
  CODE_FOR_fma4i_fnmsub_v4df = 11981,
  CODE_FOR_fma4i_fnmsub_v16sf = 11982,
  CODE_FOR_fma4i_fnmsub_v8df = 11983,
  CODE_FOR_avx512bw_fmadd_v32hf_maskz = 11984,
  CODE_FOR_avx512bw_fmadd_v32hf_maskz_round = 11985,
  CODE_FOR_avx512vl_fmadd_v16hf_maskz = 11986,
  CODE_FOR_avx512vl_fmadd_v16hf_maskz_round = 11987,
  CODE_FOR_avx512fp16_fmadd_v8hf_maskz = 11988,
  CODE_FOR_avx512fp16_fmadd_v8hf_maskz_round = 11989,
  CODE_FOR_avx512f_fmadd_v16sf_maskz = 11990,
  CODE_FOR_avx512f_fmadd_v16sf_maskz_round = 11991,
  CODE_FOR_avx512vl_fmadd_v8sf_maskz = 11992,
  CODE_FOR_avx512vl_fmadd_v8sf_maskz_round = 11993,
  CODE_FOR_avx512vl_fmadd_v4sf_maskz = 11994,
  CODE_FOR_avx512vl_fmadd_v4sf_maskz_round = 11995,
  CODE_FOR_avx512f_fmadd_v8df_maskz = 11996,
  CODE_FOR_avx512f_fmadd_v8df_maskz_round = 11997,
  CODE_FOR_avx512vl_fmadd_v4df_maskz = 11998,
  CODE_FOR_avx512vl_fmadd_v4df_maskz_round = 11999,
  CODE_FOR_avx512vl_fmadd_v2df_maskz = 12000,
  CODE_FOR_avx512vl_fmadd_v2df_maskz_round = 12001,
  CODE_FOR_cond_fmav32hf = 12002,
  CODE_FOR_cond_fmav16hf = 12003,
  CODE_FOR_cond_fmav8hf = 12004,
  CODE_FOR_cond_fmav16sf = 12005,
  CODE_FOR_cond_fmav8sf = 12006,
  CODE_FOR_cond_fmav4sf = 12007,
  CODE_FOR_cond_fmav8df = 12008,
  CODE_FOR_cond_fmav4df = 12009,
  CODE_FOR_cond_fmav2df = 12010,
  CODE_FOR_avx512bw_fmsub_v32hf_maskz = 12011,
  CODE_FOR_avx512bw_fmsub_v32hf_maskz_round = 12012,
  CODE_FOR_avx512vl_fmsub_v16hf_maskz = 12013,
  CODE_FOR_avx512vl_fmsub_v16hf_maskz_round = 12014,
  CODE_FOR_avx512fp16_fmsub_v8hf_maskz = 12015,
  CODE_FOR_avx512fp16_fmsub_v8hf_maskz_round = 12016,
  CODE_FOR_avx512f_fmsub_v16sf_maskz = 12017,
  CODE_FOR_avx512f_fmsub_v16sf_maskz_round = 12018,
  CODE_FOR_avx512vl_fmsub_v8sf_maskz = 12019,
  CODE_FOR_avx512vl_fmsub_v8sf_maskz_round = 12020,
  CODE_FOR_avx512vl_fmsub_v4sf_maskz = 12021,
  CODE_FOR_avx512vl_fmsub_v4sf_maskz_round = 12022,
  CODE_FOR_avx512f_fmsub_v8df_maskz = 12023,
  CODE_FOR_avx512f_fmsub_v8df_maskz_round = 12024,
  CODE_FOR_avx512vl_fmsub_v4df_maskz = 12025,
  CODE_FOR_avx512vl_fmsub_v4df_maskz_round = 12026,
  CODE_FOR_avx512vl_fmsub_v2df_maskz = 12027,
  CODE_FOR_avx512vl_fmsub_v2df_maskz_round = 12028,
  CODE_FOR_cond_fmsv32hf = 12029,
  CODE_FOR_cond_fmsv16hf = 12030,
  CODE_FOR_cond_fmsv8hf = 12031,
  CODE_FOR_cond_fmsv16sf = 12032,
  CODE_FOR_cond_fmsv8sf = 12033,
  CODE_FOR_cond_fmsv4sf = 12034,
  CODE_FOR_cond_fmsv8df = 12035,
  CODE_FOR_cond_fmsv4df = 12036,
  CODE_FOR_cond_fmsv2df = 12037,
  CODE_FOR_avx512bw_fnmadd_v32hf_maskz = 12038,
  CODE_FOR_avx512bw_fnmadd_v32hf_maskz_round = 12039,
  CODE_FOR_avx512vl_fnmadd_v16hf_maskz = 12040,
  CODE_FOR_avx512vl_fnmadd_v16hf_maskz_round = 12041,
  CODE_FOR_avx512fp16_fnmadd_v8hf_maskz = 12042,
  CODE_FOR_avx512fp16_fnmadd_v8hf_maskz_round = 12043,
  CODE_FOR_avx512f_fnmadd_v16sf_maskz = 12044,
  CODE_FOR_avx512f_fnmadd_v16sf_maskz_round = 12045,
  CODE_FOR_avx512vl_fnmadd_v8sf_maskz = 12046,
  CODE_FOR_avx512vl_fnmadd_v8sf_maskz_round = 12047,
  CODE_FOR_avx512vl_fnmadd_v4sf_maskz = 12048,
  CODE_FOR_avx512vl_fnmadd_v4sf_maskz_round = 12049,
  CODE_FOR_avx512f_fnmadd_v8df_maskz = 12050,
  CODE_FOR_avx512f_fnmadd_v8df_maskz_round = 12051,
  CODE_FOR_avx512vl_fnmadd_v4df_maskz = 12052,
  CODE_FOR_avx512vl_fnmadd_v4df_maskz_round = 12053,
  CODE_FOR_avx512vl_fnmadd_v2df_maskz = 12054,
  CODE_FOR_avx512vl_fnmadd_v2df_maskz_round = 12055,
  CODE_FOR_cond_fnmav32hf = 12056,
  CODE_FOR_cond_fnmav16hf = 12057,
  CODE_FOR_cond_fnmav8hf = 12058,
  CODE_FOR_cond_fnmav16sf = 12059,
  CODE_FOR_cond_fnmav8sf = 12060,
  CODE_FOR_cond_fnmav4sf = 12061,
  CODE_FOR_cond_fnmav8df = 12062,
  CODE_FOR_cond_fnmav4df = 12063,
  CODE_FOR_cond_fnmav2df = 12064,
  CODE_FOR_avx512bw_fnmsub_v32hf_maskz = 12065,
  CODE_FOR_avx512bw_fnmsub_v32hf_maskz_round = 12066,
  CODE_FOR_avx512vl_fnmsub_v16hf_maskz = 12067,
  CODE_FOR_avx512vl_fnmsub_v16hf_maskz_round = 12068,
  CODE_FOR_avx512fp16_fnmsub_v8hf_maskz = 12069,
  CODE_FOR_avx512fp16_fnmsub_v8hf_maskz_round = 12070,
  CODE_FOR_avx512f_fnmsub_v16sf_maskz = 12071,
  CODE_FOR_avx512f_fnmsub_v16sf_maskz_round = 12072,
  CODE_FOR_avx512vl_fnmsub_v8sf_maskz = 12073,
  CODE_FOR_avx512vl_fnmsub_v8sf_maskz_round = 12074,
  CODE_FOR_avx512vl_fnmsub_v4sf_maskz = 12075,
  CODE_FOR_avx512vl_fnmsub_v4sf_maskz_round = 12076,
  CODE_FOR_avx512f_fnmsub_v8df_maskz = 12077,
  CODE_FOR_avx512f_fnmsub_v8df_maskz_round = 12078,
  CODE_FOR_avx512vl_fnmsub_v4df_maskz = 12079,
  CODE_FOR_avx512vl_fnmsub_v4df_maskz_round = 12080,
  CODE_FOR_avx512vl_fnmsub_v2df_maskz = 12081,
  CODE_FOR_avx512vl_fnmsub_v2df_maskz_round = 12082,
  CODE_FOR_cond_fnmsv32hf = 12083,
  CODE_FOR_cond_fnmsv16hf = 12084,
  CODE_FOR_cond_fnmsv8hf = 12085,
  CODE_FOR_cond_fnmsv16sf = 12086,
  CODE_FOR_cond_fnmsv8sf = 12087,
  CODE_FOR_cond_fnmsv4sf = 12088,
  CODE_FOR_cond_fnmsv8df = 12089,
  CODE_FOR_cond_fnmsv4df = 12090,
  CODE_FOR_cond_fnmsv2df = 12091,
  CODE_FOR_vec_fmaddsubv32hf4 = 12092,
  CODE_FOR_vec_fmaddsubv16hf4 = 12093,
  CODE_FOR_vec_fmaddsubv8hf4 = 12094,
  CODE_FOR_vec_fmaddsubv16sf4 = 12095,
  CODE_FOR_vec_fmaddsubv8sf4 = 12096,
  CODE_FOR_vec_fmaddsubv4sf4 = 12097,
  CODE_FOR_vec_fmaddsubv8df4 = 12098,
  CODE_FOR_vec_fmaddsubv4df4 = 12099,
  CODE_FOR_vec_fmaddsubv2df4 = 12100,
  CODE_FOR_vec_fmsubaddv32hf4 = 12101,
  CODE_FOR_vec_fmsubaddv16hf4 = 12102,
  CODE_FOR_vec_fmsubaddv8hf4 = 12103,
  CODE_FOR_vec_fmsubaddv16sf4 = 12104,
  CODE_FOR_vec_fmsubaddv8sf4 = 12105,
  CODE_FOR_vec_fmsubaddv4sf4 = 12106,
  CODE_FOR_vec_fmsubaddv8df4 = 12107,
  CODE_FOR_vec_fmsubaddv4df4 = 12108,
  CODE_FOR_vec_fmsubaddv2df4 = 12109,
  CODE_FOR_fmaddsub_v16sf = 12110,
  CODE_FOR_fmaddsub_v8sf = 12111,
  CODE_FOR_fmaddsub_v4sf = 12112,
  CODE_FOR_fmaddsub_v8df = 12113,
  CODE_FOR_fmaddsub_v4df = 12114,
  CODE_FOR_fmaddsub_v2df = 12115,
  CODE_FOR_avx512bw_fmaddsub_v32hf_maskz = 12116,
  CODE_FOR_avx512bw_fmaddsub_v32hf_maskz_round = 12117,
  CODE_FOR_avx512vl_fmaddsub_v16hf_maskz = 12118,
  CODE_FOR_avx512vl_fmaddsub_v16hf_maskz_round = 12119,
  CODE_FOR_avx512fp16_fmaddsub_v8hf_maskz = 12120,
  CODE_FOR_avx512fp16_fmaddsub_v8hf_maskz_round = 12121,
  CODE_FOR_avx512f_fmaddsub_v16sf_maskz = 12122,
  CODE_FOR_avx512f_fmaddsub_v16sf_maskz_round = 12123,
  CODE_FOR_avx512vl_fmaddsub_v8sf_maskz = 12124,
  CODE_FOR_avx512vl_fmaddsub_v8sf_maskz_round = 12125,
  CODE_FOR_avx512vl_fmaddsub_v4sf_maskz = 12126,
  CODE_FOR_avx512vl_fmaddsub_v4sf_maskz_round = 12127,
  CODE_FOR_avx512f_fmaddsub_v8df_maskz = 12128,
  CODE_FOR_avx512f_fmaddsub_v8df_maskz_round = 12129,
  CODE_FOR_avx512vl_fmaddsub_v4df_maskz = 12130,
  CODE_FOR_avx512vl_fmaddsub_v4df_maskz_round = 12131,
  CODE_FOR_avx512vl_fmaddsub_v2df_maskz = 12132,
  CODE_FOR_avx512vl_fmaddsub_v2df_maskz_round = 12133,
  CODE_FOR_avx512bw_fmsubadd_v32hf_maskz = 12134,
  CODE_FOR_avx512bw_fmsubadd_v32hf_maskz_round = 12135,
  CODE_FOR_avx512vl_fmsubadd_v16hf_maskz = 12136,
  CODE_FOR_avx512vl_fmsubadd_v16hf_maskz_round = 12137,
  CODE_FOR_avx512fp16_fmsubadd_v8hf_maskz = 12138,
  CODE_FOR_avx512fp16_fmsubadd_v8hf_maskz_round = 12139,
  CODE_FOR_avx512f_fmsubadd_v16sf_maskz = 12140,
  CODE_FOR_avx512f_fmsubadd_v16sf_maskz_round = 12141,
  CODE_FOR_avx512vl_fmsubadd_v8sf_maskz = 12142,
  CODE_FOR_avx512vl_fmsubadd_v8sf_maskz_round = 12143,
  CODE_FOR_avx512vl_fmsubadd_v4sf_maskz = 12144,
  CODE_FOR_avx512vl_fmsubadd_v4sf_maskz_round = 12145,
  CODE_FOR_avx512f_fmsubadd_v8df_maskz = 12146,
  CODE_FOR_avx512f_fmsubadd_v8df_maskz_round = 12147,
  CODE_FOR_avx512vl_fmsubadd_v4df_maskz = 12148,
  CODE_FOR_avx512vl_fmsubadd_v4df_maskz_round = 12149,
  CODE_FOR_avx512vl_fmsubadd_v2df_maskz = 12150,
  CODE_FOR_avx512vl_fmsubadd_v2df_maskz_round = 12151,
  CODE_FOR_fmai_vmfmadd_v8hf = 12152,
  CODE_FOR_fmai_vmfmadd_v8hf_round = 12153,
  CODE_FOR_fmai_vmfmadd_v4sf = 12154,
  CODE_FOR_fmai_vmfmadd_v4sf_round = 12155,
  CODE_FOR_fmai_vmfmadd_v2df = 12156,
  CODE_FOR_fmai_vmfmadd_v2df_round = 12157,
  CODE_FOR_fmai_vmfmsub_v8hf = 12158,
  CODE_FOR_fmai_vmfmsub_v8hf_round = 12159,
  CODE_FOR_fmai_vmfmsub_v4sf = 12160,
  CODE_FOR_fmai_vmfmsub_v4sf_round = 12161,
  CODE_FOR_fmai_vmfmsub_v2df = 12162,
  CODE_FOR_fmai_vmfmsub_v2df_round = 12163,
  CODE_FOR_fmai_vmfnmadd_v8hf = 12164,
  CODE_FOR_fmai_vmfnmadd_v8hf_round = 12165,
  CODE_FOR_fmai_vmfnmadd_v4sf = 12166,
  CODE_FOR_fmai_vmfnmadd_v4sf_round = 12167,
  CODE_FOR_fmai_vmfnmadd_v2df = 12168,
  CODE_FOR_fmai_vmfnmadd_v2df_round = 12169,
  CODE_FOR_fmai_vmfnmsub_v8hf = 12170,
  CODE_FOR_fmai_vmfnmsub_v8hf_round = 12171,
  CODE_FOR_fmai_vmfnmsub_v4sf = 12172,
  CODE_FOR_fmai_vmfnmsub_v4sf_round = 12173,
  CODE_FOR_fmai_vmfnmsub_v2df = 12174,
  CODE_FOR_fmai_vmfnmsub_v2df_round = 12175,
  CODE_FOR_avx512f_vmfmadd_v8hf_maskz = 12176,
  CODE_FOR_avx512f_vmfmadd_v8hf_maskz_round = 12177,
  CODE_FOR_avx512f_vmfmadd_v4sf_maskz = 12178,
  CODE_FOR_avx512f_vmfmadd_v4sf_maskz_round = 12179,
  CODE_FOR_avx512f_vmfmadd_v2df_maskz = 12180,
  CODE_FOR_avx512f_vmfmadd_v2df_maskz_round = 12181,
  CODE_FOR_avx512f_vmfnmadd_v8hf_maskz = 12182,
  CODE_FOR_avx512f_vmfnmadd_v8hf_maskz_round = 12183,
  CODE_FOR_avx512f_vmfnmadd_v4sf_maskz = 12184,
  CODE_FOR_avx512f_vmfnmadd_v4sf_maskz_round = 12185,
  CODE_FOR_avx512f_vmfnmadd_v2df_maskz = 12186,
  CODE_FOR_avx512f_vmfnmadd_v2df_maskz_round = 12187,
  CODE_FOR_fma4i_vmfmadd_v4sf = 12188,
  CODE_FOR_fma4i_vmfmadd_v2df = 12189,
  CODE_FOR_avx512bw_fmaddc_v32hf_mask1 = 12190,
  CODE_FOR_avx512bw_fmaddc_v32hf_mask1_round = 12191,
  CODE_FOR_avx512vl_fmaddc_v16hf_mask1 = 12192,
  CODE_FOR_avx512vl_fmaddc_v16hf_mask1_round = 12193,
  CODE_FOR_avx512fp16_fmaddc_v8hf_mask1 = 12194,
  CODE_FOR_avx512fp16_fmaddc_v8hf_mask1_round = 12195,
  CODE_FOR_avx512bw_fmaddc_v32hf_maskz = 12196,
  CODE_FOR_avx512bw_fmaddc_v32hf_maskz_round = 12197,
  CODE_FOR_avx512vl_fmaddc_v16hf_maskz = 12198,
  CODE_FOR_avx512vl_fmaddc_v16hf_maskz_round = 12199,
  CODE_FOR_avx512fp16_fmaddc_v8hf_maskz = 12200,
  CODE_FOR_avx512fp16_fmaddc_v8hf_maskz_round = 12201,
  CODE_FOR_avx512bw_fcmaddc_v32hf_mask1 = 12202,
  CODE_FOR_avx512bw_fcmaddc_v32hf_mask1_round = 12203,
  CODE_FOR_avx512vl_fcmaddc_v16hf_mask1 = 12204,
  CODE_FOR_avx512vl_fcmaddc_v16hf_mask1_round = 12205,
  CODE_FOR_avx512fp16_fcmaddc_v8hf_mask1 = 12206,
  CODE_FOR_avx512fp16_fcmaddc_v8hf_mask1_round = 12207,
  CODE_FOR_avx512bw_fcmaddc_v32hf_maskz = 12208,
  CODE_FOR_avx512bw_fcmaddc_v32hf_maskz_round = 12209,
  CODE_FOR_avx512vl_fcmaddc_v16hf_maskz = 12210,
  CODE_FOR_avx512vl_fcmaddc_v16hf_maskz_round = 12211,
  CODE_FOR_avx512fp16_fcmaddc_v8hf_maskz = 12212,
  CODE_FOR_avx512fp16_fcmaddc_v8hf_maskz_round = 12213,
  CODE_FOR_cmlav32hf4 = 12214,
  CODE_FOR_cmla_conjv32hf4 = 12215,
  CODE_FOR_cmlav16hf4 = 12216,
  CODE_FOR_cmla_conjv16hf4 = 12217,
  CODE_FOR_cmlav8hf4 = 12218,
  CODE_FOR_cmla_conjv8hf4 = 12219,
  CODE_FOR_cmulv32hf3 = 12220,
  CODE_FOR_cmul_conjv32hf3 = 12221,
  CODE_FOR_cmulv16hf3 = 12222,
  CODE_FOR_cmul_conjv16hf3 = 12223,
  CODE_FOR_cmulv8hf3 = 12224,
  CODE_FOR_cmul_conjv8hf3 = 12225,
  CODE_FOR_avx512fp16_fmaddcsh_v8hf_maskz = 12226,
  CODE_FOR_avx512fp16_fmaddcsh_v8hf_maskz_round = 12227,
  CODE_FOR_avx512fp16_fmaddcsh_v8hf_mask1 = 12228,
  CODE_FOR_avx512fp16_fmaddcsh_v8hf_mask1_round = 12229,
  CODE_FOR_avx512fp16_fcmaddcsh_v8hf_maskz = 12230,
  CODE_FOR_avx512fp16_fcmaddcsh_v8hf_maskz_round = 12231,
  CODE_FOR_avx512fp16_fcmaddcsh_v8hf_mask1 = 12232,
  CODE_FOR_avx512fp16_fcmaddcsh_v8hf_mask1_round = 12233,
  CODE_FOR_avx512fp16_fcmaddcsh_v8hf_mask3 = 12234,
  CODE_FOR_avx512fp16_fcmaddcsh_v8hf_mask3_round = 12235,
  CODE_FOR_avx512fp16_fmaddcsh_v8hf_mask3 = 12236,
  CODE_FOR_avx512fp16_fmaddcsh_v8hf_mask3_round = 12237,
  CODE_FOR_vec_unpacks_lo_v32hf = 12238,
  CODE_FOR_vec_unpacks_lo_v16hf = 12239,
  CODE_FOR_vec_unpacks_lo_v8hf = 12240,
  CODE_FOR_vec_unpacks_hi_v32hf = 12241,
  CODE_FOR_vec_unpacks_hi_v16hf = 12242,
  CODE_FOR_vec_unpacks_hi_v8hf = 12243,
  CODE_FOR_lrintv32hfv32hi2 = 12244,
  CODE_FOR_lrintv16hfv16hi2 = 12245,
  CODE_FOR_lrintv8hfv8hi2 = 12246,
  CODE_FOR_floatv8hiv8hf2 = 12247,
  CODE_FOR_floatunsv8hiv8hf2 = 12248,
  CODE_FOR_floatv16hiv16hf2 = 12249,
  CODE_FOR_floatunsv16hiv16hf2 = 12250,
  CODE_FOR_floatv32hiv32hf2 = 12251,
  CODE_FOR_floatunsv32hiv32hf2 = 12252,
  CODE_FOR_floatv8siv8hf2 = 12253,
  CODE_FOR_floatunsv8siv8hf2 = 12254,
  CODE_FOR_floatv16siv16hf2 = 12255,
  CODE_FOR_floatunsv16siv16hf2 = 12256,
  CODE_FOR_floatv8div8hf2 = 12257,
  CODE_FOR_floatunsv8div8hf2 = 12258,
  CODE_FOR_floatv4siv4hf2 = 12259,
  CODE_FOR_floatunsv4siv4hf2 = 12260,
  CODE_FOR_floatv4div4hf2 = 12261,
  CODE_FOR_floatunsv4div4hf2 = 12262,
  CODE_FOR_avx512fp16_floatv4siv4hf2 = 12263,
  CODE_FOR_avx512fp16_floatunsv4siv4hf2 = 12264,
  CODE_FOR_avx512fp16_floatv4div4hf2 = 12265,
  CODE_FOR_avx512fp16_floatunsv4div4hf2 = 12266,
  CODE_FOR_avx512fp16_vcvtdq2ph_v4si_mask = 12267,
  CODE_FOR_avx512fp16_vcvtudq2ph_v4si_mask = 12268,
  CODE_FOR_avx512fp16_vcvtqq2ph_v4di_mask = 12269,
  CODE_FOR_avx512fp16_vcvtuqq2ph_v4di_mask = 12270,
  CODE_FOR_floatv2div2hf2 = 12271,
  CODE_FOR_floatunsv2div2hf2 = 12272,
  CODE_FOR_avx512fp16_floatv2div2hf2 = 12273,
  CODE_FOR_avx512fp16_floatunsv2div2hf2 = 12274,
  CODE_FOR_avx512fp16_vcvtqq2ph_v2di_mask = 12275,
  CODE_FOR_avx512fp16_vcvtuqq2ph_v2di_mask = 12276,
  CODE_FOR_fix_truncv8hfv8hi2 = 12277,
  CODE_FOR_fixuns_truncv8hfv8hi2 = 12278,
  CODE_FOR_fix_truncv16hfv16hi2 = 12279,
  CODE_FOR_fixuns_truncv16hfv16hi2 = 12280,
  CODE_FOR_fix_truncv32hfv32hi2 = 12281,
  CODE_FOR_fixuns_truncv32hfv32hi2 = 12282,
  CODE_FOR_fix_truncv8hfv8si2 = 12283,
  CODE_FOR_fixuns_truncv8hfv8si2 = 12284,
  CODE_FOR_fix_truncv16hfv16si2 = 12285,
  CODE_FOR_fixuns_truncv16hfv16si2 = 12286,
  CODE_FOR_fix_truncv8hfv8di2 = 12287,
  CODE_FOR_fixuns_truncv8hfv8di2 = 12288,
  CODE_FOR_fix_truncv4hfv4si2 = 12289,
  CODE_FOR_fixuns_truncv4hfv4si2 = 12290,
  CODE_FOR_fix_truncv4hfv4di2 = 12291,
  CODE_FOR_fixuns_truncv4hfv4di2 = 12292,
  CODE_FOR_fix_truncv2hfv2di2 = 12293,
  CODE_FOR_fixuns_truncv2hfv2di2 = 12294,
  CODE_FOR_extendv8hfv8df2 = 12295,
  CODE_FOR_extendv16hfv16sf2 = 12296,
  CODE_FOR_extendv8hfv8sf2 = 12297,
  CODE_FOR_extendv4hfv4df2 = 12298,
  CODE_FOR_extendv4hfv4sf2 = 12299,
  CODE_FOR_extendv2hfv2df2 = 12300,
  CODE_FOR_truncv8dfv8hf2 = 12301,
  CODE_FOR_truncv16sfv16hf2 = 12302,
  CODE_FOR_truncv8sfv8hf2 = 12303,
  CODE_FOR_truncv4dfv4hf2 = 12304,
  CODE_FOR_truncv4sfv4hf2 = 12305,
  CODE_FOR_avx512fp16_truncv4dfv4hf2 = 12306,
  CODE_FOR_avx512fp16_truncv4sfv4hf2 = 12307,
  CODE_FOR_avx512fp16_vcvtpd2ph_v4df_mask = 12308,
  CODE_FOR_avx512fp16_vcvtps2ph_v4sf_mask = 12309,
  CODE_FOR_truncv2dfv2hf2 = 12310,
  CODE_FOR_avx512fp16_truncv2dfv2hf2 = 12311,
  CODE_FOR_avx512fp16_vcvtpd2ph_v2df_mask = 12312,
  CODE_FOR_floatunsv16siv16sf2 = 12313,
  CODE_FOR_floatunsv8siv8sf2 = 12314,
  CODE_FOR_floatunsv4siv4sf2 = 12315,
  CODE_FOR_fixuns_truncv8sfv8si2 = 12316,
  CODE_FOR_fixuns_truncv4sfv4si2 = 12317,
   CODE_FOR_floatv2siv2df2 = CODE_FOR_nothing,
   CODE_FOR_fix_truncv2dfv2si2 = CODE_FOR_nothing,
  CODE_FOR_avx512dq_floatv2div2sf2 = 12318,
  CODE_FOR_avx512dq_floatunsv2div2sf2 = 12319,
  CODE_FOR_floatv2div2sf2 = 12320,
  CODE_FOR_floatunsv2div2sf2 = 12321,
  CODE_FOR_vec_packs_float_v8di = 12322,
  CODE_FOR_vec_packu_float_v8di = 12323,
  CODE_FOR_vec_packs_float_v4di = 12324,
  CODE_FOR_vec_packu_float_v4di = 12325,
  CODE_FOR_vec_packs_float_v2di = 12326,
  CODE_FOR_vec_packu_float_v2di = 12327,
  CODE_FOR_vec_packs_float_v16si = 12328,
  CODE_FOR_vec_packu_float_v16si = 12329,
  CODE_FOR_vec_packs_float_v8si = 12330,
  CODE_FOR_vec_packu_float_v8si = 12331,
  CODE_FOR_vec_packs_float_v4si = 12332,
  CODE_FOR_vec_packu_float_v4si = 12333,
  CODE_FOR_floatv2div2sf2_mask = 12334,
  CODE_FOR_floatunsv2div2sf2_mask = 12335,
  CODE_FOR_avx_cvtpd2dq256_2 = 12336,
  CODE_FOR_fix_truncv2sfv2di2 = 12337,
  CODE_FOR_fixuns_truncv2sfv2di2 = 12338,
  CODE_FOR_vec_unpack_sfix_trunc_lo_v16sf = 12339,
  CODE_FOR_vec_unpack_ufix_trunc_lo_v16sf = 12340,
  CODE_FOR_vec_unpack_sfix_trunc_lo_v8sf = 12341,
  CODE_FOR_vec_unpack_ufix_trunc_lo_v8sf = 12342,
  CODE_FOR_vec_unpack_sfix_trunc_lo_v4sf = 12343,
  CODE_FOR_vec_unpack_ufix_trunc_lo_v4sf = 12344,
  CODE_FOR_vec_unpack_sfix_trunc_hi_v16sf = 12345,
  CODE_FOR_vec_unpack_ufix_trunc_hi_v16sf = 12346,
  CODE_FOR_vec_unpack_sfix_trunc_hi_v8sf = 12347,
  CODE_FOR_vec_unpack_ufix_trunc_hi_v8sf = 12348,
  CODE_FOR_vec_unpack_sfix_trunc_hi_v4sf = 12349,
  CODE_FOR_vec_unpack_ufix_trunc_hi_v4sf = 12350,
  CODE_FOR_vec_unpack_sfix_trunc_lo_v32hf = 12351,
  CODE_FOR_vec_unpack_ufix_trunc_lo_v32hf = 12352,
  CODE_FOR_vec_unpack_sfix_trunc_lo_v16hf = 12353,
  CODE_FOR_vec_unpack_ufix_trunc_lo_v16hf = 12354,
  CODE_FOR_vec_unpack_sfix_trunc_lo_v8hf = 12355,
  CODE_FOR_vec_unpack_ufix_trunc_lo_v8hf = 12356,
  CODE_FOR_vec_unpack_sfix_trunc_hi_v32hf = 12357,
  CODE_FOR_vec_unpack_ufix_trunc_hi_v32hf = 12358,
  CODE_FOR_vec_unpack_sfix_trunc_hi_v16hf = 12359,
  CODE_FOR_vec_unpack_ufix_trunc_hi_v16hf = 12360,
  CODE_FOR_vec_unpack_sfix_trunc_hi_v8hf = 12361,
  CODE_FOR_vec_unpack_ufix_trunc_hi_v8hf = 12362,
  CODE_FOR_avx_cvttpd2dq256_2 = 12363,
  CODE_FOR_sse2_cvtpd2ps = 12364,
  CODE_FOR_sse2_cvtpd2ps_mask = 12365,
  CODE_FOR_truncv8dfv8sf2 = 12366,
  CODE_FOR_truncv4dfv4sf2 = 12367,
  CODE_FOR_extendv8sfv8df2 = 12368,
  CODE_FOR_extendv4sfv4df2 = 12369,
  CODE_FOR_avx512bw_cvtmask2bv64qi = 12370,
  CODE_FOR_avx512vl_cvtmask2bv16qi = 12371,
  CODE_FOR_avx512vl_cvtmask2bv32qi = 12372,
  CODE_FOR_avx512bw_cvtmask2wv32hi = 12373,
  CODE_FOR_avx512vl_cvtmask2wv16hi = 12374,
  CODE_FOR_avx512vl_cvtmask2wv8hi = 12375,
  CODE_FOR_avx512f_cvtmask2dv16si = 12376,
  CODE_FOR_avx512vl_cvtmask2dv8si = 12377,
  CODE_FOR_avx512vl_cvtmask2dv4si = 12378,
  CODE_FOR_avx512f_cvtmask2qv8di = 12379,
  CODE_FOR_avx512vl_cvtmask2qv4di = 12380,
  CODE_FOR_avx512vl_cvtmask2qv2di = 12381,
   CODE_FOR_extendv2sfv2df2 = CODE_FOR_nothing,
  CODE_FOR_vec_unpacks_hi_v4sf = 12382,
  CODE_FOR_vec_unpacks_hi_v8sf = 12383,
  CODE_FOR_vec_unpacks_hi_v16sf = 12384,
  CODE_FOR_vec_unpacks_lo_v4sf = 12385,
  CODE_FOR_vec_unpacks_lo_v8sf = 12386,
  CODE_FOR_vec_unpacks_float_hi_v32hi = 12387,
  CODE_FOR_vec_unpacks_float_hi_v16hi = 12388,
  CODE_FOR_vec_unpacks_float_hi_v8hi = 12389,
  CODE_FOR_vec_unpacks_float_lo_v32hi = 12390,
  CODE_FOR_vec_unpacks_float_lo_v16hi = 12391,
  CODE_FOR_vec_unpacks_float_lo_v8hi = 12392,
  CODE_FOR_vec_unpacku_float_hi_v32hi = 12393,
  CODE_FOR_vec_unpacku_float_hi_v16hi = 12394,
  CODE_FOR_vec_unpacku_float_hi_v8hi = 12395,
  CODE_FOR_vec_unpacku_float_lo_v32hi = 12396,
  CODE_FOR_vec_unpacku_float_lo_v16hi = 12397,
  CODE_FOR_vec_unpacku_float_lo_v8hi = 12398,
  CODE_FOR_vec_unpacks_float_hi_v4si = 12399,
  CODE_FOR_vec_unpacks_float_lo_v4si = 12400,
  CODE_FOR_vec_unpacks_float_hi_v8si = 12401,
  CODE_FOR_vec_unpacks_float_lo_v8si = 12402,
  CODE_FOR_vec_unpacks_float_hi_v16si = 12403,
  CODE_FOR_vec_unpacks_float_lo_v16si = 12404,
  CODE_FOR_vec_unpacku_float_hi_v4si = 12405,
  CODE_FOR_vec_unpacku_float_lo_v4si = 12406,
  CODE_FOR_vec_unpacku_float_hi_v8si = 12407,
  CODE_FOR_vec_unpacku_float_hi_v16si = 12408,
  CODE_FOR_vec_unpacku_float_lo_v8si = 12409,
  CODE_FOR_vec_unpacku_float_lo_v16si = 12410,
  CODE_FOR_vec_pack_trunc_v8df = 12411,
  CODE_FOR_vec_pack_trunc_v4df = 12412,
  CODE_FOR_vec_pack_trunc_v16sf = 12413,
  CODE_FOR_vec_pack_trunc_v8sf = 12414,
  CODE_FOR_vec_pack_trunc_v4sf = 12415,
  CODE_FOR_vec_pack_trunc_v2df = 12416,
  CODE_FOR_vec_pack_sfix_trunc_v8df = 12417,
  CODE_FOR_vec_pack_sfix_trunc_v4df = 12418,
  CODE_FOR_vec_pack_sfix_trunc_v2df = 12419,
  CODE_FOR_vec_pack_ufix_trunc_v8df = 12420,
  CODE_FOR_vec_pack_ufix_trunc_v4df = 12421,
  CODE_FOR_vec_pack_ufix_trunc_v2df = 12422,
  CODE_FOR_avx512f_vec_pack_sfix_v8df = 12423,
  CODE_FOR_vec_pack_sfix_v4df = 12424,
  CODE_FOR_vec_pack_sfix_v2df = 12425,
  CODE_FOR_sse_movhlps_exp = 12426,
  CODE_FOR_sse_movlhps_exp = 12427,
  CODE_FOR_vec_interleave_highv8sf = 12428,
  CODE_FOR_vec_interleave_lowv8sf = 12429,
  CODE_FOR_avx_shufps256 = 12430,
  CODE_FOR_avx_shufps256_mask = 12431,
  CODE_FOR_sse_shufps = 12432,
  CODE_FOR_sse_shufps_mask = 12433,
  CODE_FOR_sse_loadhps_exp = 12434,
  CODE_FOR_sse_loadlps_exp = 12435,
  CODE_FOR_vec_setv16qi = 12436,
  CODE_FOR_vec_setv8hi = 12437,
  CODE_FOR_vec_setv8hf = 12438,
  CODE_FOR_vec_setv8bf = 12439,
  CODE_FOR_vec_setv4si = 12440,
  CODE_FOR_vec_setv2di = 12441,
  CODE_FOR_vec_setv4sf = 12442,
  CODE_FOR_vec_setv2df = 12443,
  CODE_FOR_vec_setv32qi = 12444,
  CODE_FOR_vec_setv16hi = 12445,
  CODE_FOR_vec_setv16hf = 12446,
  CODE_FOR_vec_setv16bf = 12447,
  CODE_FOR_vec_setv8si = 12448,
  CODE_FOR_vec_setv4di = 12449,
  CODE_FOR_vec_setv8sf = 12450,
  CODE_FOR_vec_setv4df = 12451,
  CODE_FOR_vec_setv64qi = 12452,
  CODE_FOR_vec_setv32hi = 12453,
  CODE_FOR_vec_setv32hf = 12454,
  CODE_FOR_vec_setv32bf = 12455,
  CODE_FOR_vec_setv16si = 12456,
  CODE_FOR_vec_setv8di = 12457,
  CODE_FOR_vec_setv16sf = 12458,
  CODE_FOR_vec_setv8df = 12459,
  CODE_FOR_avx512dq_vextractf64x2_mask = 12460,
  CODE_FOR_avx512dq_vextracti64x2_mask = 12461,
  CODE_FOR_avx512f_vextractf32x4_mask = 12462,
  CODE_FOR_avx512f_vextracti32x4_mask = 12463,
  CODE_FOR_avx512dq_vextractf32x8_mask = 12464,
  CODE_FOR_avx512dq_vextracti32x8_mask = 12465,
  CODE_FOR_avx512f_vextractf64x4_mask = 12466,
  CODE_FOR_avx512f_vextracti64x4_mask = 12467,
  CODE_FOR_avx512vl_vextractf128v8si = 12468,
  CODE_FOR_avx512vl_vextractf128v8sf = 12469,
  CODE_FOR_avx512vl_vextractf128v4di = 12470,
  CODE_FOR_avx512vl_vextractf128v4df = 12471,
  CODE_FOR_avx_vextractf128v32qi = 12472,
  CODE_FOR_avx_vextractf128v16hi = 12473,
  CODE_FOR_avx_vextractf128v8si = 12474,
  CODE_FOR_avx_vextractf128v4di = 12475,
  CODE_FOR_avx_vextractf128v8sf = 12476,
  CODE_FOR_avx_vextractf128v4df = 12477,
  CODE_FOR_avx_vextractf128v16hf = 12478,
  CODE_FOR_avx_vextractf128v16bf = 12479,
  CODE_FOR_vec_extractv64qiqi = 12480,
  CODE_FOR_vec_extractv32qiqi = 12481,
  CODE_FOR_vec_extractv16qiqi = 12482,
  CODE_FOR_vec_extractv32hihi = 12483,
  CODE_FOR_vec_extractv16hihi = 12484,
  CODE_FOR_vec_extractv8hihi = 12485,
  CODE_FOR_vec_extractv16sisi = 12486,
  CODE_FOR_vec_extractv8sisi = 12487,
  CODE_FOR_vec_extractv4sisi = 12488,
  CODE_FOR_vec_extractv8didi = 12489,
  CODE_FOR_vec_extractv4didi = 12490,
  CODE_FOR_vec_extractv2didi = 12491,
  CODE_FOR_vec_extractv32hfhf = 12492,
  CODE_FOR_vec_extractv16hfhf = 12493,
  CODE_FOR_vec_extractv8hfhf = 12494,
  CODE_FOR_vec_extractv32bfbf = 12495,
  CODE_FOR_vec_extractv16bfbf = 12496,
  CODE_FOR_vec_extractv8bfbf = 12497,
  CODE_FOR_vec_extractv16sfsf = 12498,
  CODE_FOR_vec_extractv8sfsf = 12499,
  CODE_FOR_vec_extractv4sfsf = 12500,
  CODE_FOR_vec_extractv8dfdf = 12501,
  CODE_FOR_vec_extractv4dfdf = 12502,
  CODE_FOR_vec_extractv2dfdf = 12503,
  CODE_FOR_vec_extractv4titi = 12504,
  CODE_FOR_vec_extractv2titi = 12505,
  CODE_FOR_vec_extractv32qiv16qi = 12506,
  CODE_FOR_vec_extractv16hiv8hi = 12507,
  CODE_FOR_vec_extractv16hfv8hf = 12508,
  CODE_FOR_vec_extractv16bfv8bf = 12509,
  CODE_FOR_vec_extractv8siv4si = 12510,
  CODE_FOR_vec_extractv4div2di = 12511,
  CODE_FOR_vec_extractv8sfv4sf = 12512,
  CODE_FOR_vec_extractv4dfv2df = 12513,
  CODE_FOR_vec_extractv64qiv32qi = 12514,
  CODE_FOR_vec_extractv32hiv16hi = 12515,
  CODE_FOR_vec_extractv32hfv16hf = 12516,
  CODE_FOR_vec_extractv32bfv16bf = 12517,
  CODE_FOR_vec_extractv16siv8si = 12518,
  CODE_FOR_vec_extractv8div4di = 12519,
  CODE_FOR_vec_extractv16sfv8sf = 12520,
  CODE_FOR_vec_extractv8dfv4df = 12521,
  CODE_FOR_vec_interleave_highv4df = 12522,
  CODE_FOR_vec_interleave_highv2df = 12523,
  CODE_FOR_vec_interleave_lowv4df = 12524,
  CODE_FOR_vec_interleave_lowv2df = 12525,
  CODE_FOR_avx512f_vternlogv16si_maskz = 12526,
  CODE_FOR_avx512vl_vternlogv8si_maskz = 12527,
  CODE_FOR_avx512vl_vternlogv4si_maskz = 12528,
  CODE_FOR_avx512f_vternlogv8di_maskz = 12529,
  CODE_FOR_avx512vl_vternlogv4di_maskz = 12530,
  CODE_FOR_avx512vl_vternlogv2di_maskz = 12531,
  CODE_FOR_avx512f_vternlogv16si_mask = 12532,
  CODE_FOR_avx512vl_vternlogv8si_mask = 12533,
  CODE_FOR_avx512vl_vternlogv4si_mask = 12534,
  CODE_FOR_avx512f_vternlogv8di_mask = 12535,
  CODE_FOR_avx512vl_vternlogv4di_mask = 12536,
  CODE_FOR_avx512vl_vternlogv2di_mask = 12537,
  CODE_FOR_avx512f_shufps512_mask = 12538,
  CODE_FOR_avx512f_fixupimmv16sf_maskz = 12539,
  CODE_FOR_avx512f_fixupimmv16sf_maskz_round = 12540,
  CODE_FOR_avx512vl_fixupimmv8sf_maskz = 12541,
  CODE_FOR_avx512vl_fixupimmv8sf_maskz_round = 12542,
  CODE_FOR_avx512vl_fixupimmv4sf_maskz = 12543,
  CODE_FOR_avx512vl_fixupimmv4sf_maskz_round = 12544,
  CODE_FOR_avx512f_fixupimmv8df_maskz = 12545,
  CODE_FOR_avx512f_fixupimmv8df_maskz_round = 12546,
  CODE_FOR_avx512vl_fixupimmv4df_maskz = 12547,
  CODE_FOR_avx512vl_fixupimmv4df_maskz_round = 12548,
  CODE_FOR_avx512vl_fixupimmv2df_maskz = 12549,
  CODE_FOR_avx512vl_fixupimmv2df_maskz_round = 12550,
  CODE_FOR_avx512f_sfixupimmv4sf_maskz = 12551,
  CODE_FOR_avx512f_sfixupimmv4sf_maskz_round = 12552,
  CODE_FOR_avx512f_sfixupimmv2df_maskz = 12553,
  CODE_FOR_avx512f_sfixupimmv2df_maskz_round = 12554,
  CODE_FOR_avx512f_shufpd512_mask = 12555,
  CODE_FOR_avx_shufpd256 = 12556,
  CODE_FOR_avx_shufpd256_mask = 12557,
  CODE_FOR_sse2_shufpd = 12558,
  CODE_FOR_sse2_shufpd_mask = 12559,
  CODE_FOR_sse2_loadhpd_exp = 12560,
  CODE_FOR_sse2_loadlpd_exp = 12561,
  CODE_FOR_truncv16siv16qi2 = 12562,
  CODE_FOR_truncv16siv16hi2 = 12563,
  CODE_FOR_truncv8div8si2 = 12564,
  CODE_FOR_truncv8div8hi2 = 12565,
  CODE_FOR_avx512f_ss_truncatev16siv16qi2_mask_store = 12566,
  CODE_FOR_avx512f_truncatev16siv16qi2_mask_store = 12567,
  CODE_FOR_avx512f_us_truncatev16siv16qi2_mask_store = 12568,
  CODE_FOR_avx512f_ss_truncatev16siv16hi2_mask_store = 12569,
  CODE_FOR_avx512f_truncatev16siv16hi2_mask_store = 12570,
  CODE_FOR_avx512f_us_truncatev16siv16hi2_mask_store = 12571,
  CODE_FOR_avx512f_ss_truncatev8div8si2_mask_store = 12572,
  CODE_FOR_avx512f_truncatev8div8si2_mask_store = 12573,
  CODE_FOR_avx512f_us_truncatev8div8si2_mask_store = 12574,
  CODE_FOR_avx512f_ss_truncatev8div8hi2_mask_store = 12575,
  CODE_FOR_avx512f_truncatev8div8hi2_mask_store = 12576,
  CODE_FOR_avx512f_us_truncatev8div8hi2_mask_store = 12577,
  CODE_FOR_truncv32hiv32qi2 = 12578,
  CODE_FOR_avx512bw_ss_truncatev32hiv32qi2_mask_store = 12579,
  CODE_FOR_avx512bw_truncatev32hiv32qi2_mask_store = 12580,
  CODE_FOR_avx512bw_us_truncatev32hiv32qi2_mask_store = 12581,
  CODE_FOR_truncv4div4si2 = 12582,
  CODE_FOR_truncv8siv8hi2 = 12583,
  CODE_FOR_truncv16hiv16qi2 = 12584,
  CODE_FOR_avx512vl_ss_truncatev4div4si2_mask_store = 12585,
  CODE_FOR_avx512vl_truncatev4div4si2_mask_store = 12586,
  CODE_FOR_avx512vl_us_truncatev4div4si2_mask_store = 12587,
  CODE_FOR_avx512vl_ss_truncatev8siv8hi2_mask_store = 12588,
  CODE_FOR_avx512vl_truncatev8siv8hi2_mask_store = 12589,
  CODE_FOR_avx512vl_us_truncatev8siv8hi2_mask_store = 12590,
  CODE_FOR_avx512vl_ss_truncatev16hiv16qi2_mask_store = 12591,
  CODE_FOR_avx512vl_truncatev16hiv16qi2_mask_store = 12592,
  CODE_FOR_avx512vl_us_truncatev16hiv16qi2_mask_store = 12593,
  CODE_FOR_truncv4div4qi2 = 12594,
  CODE_FOR_truncv2div2qi2 = 12595,
  CODE_FOR_truncv8siv8qi2 = 12596,
  CODE_FOR_truncv4siv4qi2 = 12597,
  CODE_FOR_truncv8hiv8qi2 = 12598,
  CODE_FOR_avx512vl_ss_truncatev2div2qi2_mask_store_2 = 12599,
  CODE_FOR_avx512vl_truncatev2div2qi2_mask_store_2 = 12600,
  CODE_FOR_avx512vl_us_truncatev2div2qi2_mask_store_2 = 12601,
  CODE_FOR_avx512vl_ss_truncatev4siv4qi2_mask_store_2 = 12602,
  CODE_FOR_avx512vl_truncatev4siv4qi2_mask_store_2 = 12603,
  CODE_FOR_avx512vl_us_truncatev4siv4qi2_mask_store_2 = 12604,
  CODE_FOR_avx512vl_ss_truncatev4div4qi2_mask_store_2 = 12605,
  CODE_FOR_avx512vl_truncatev4div4qi2_mask_store_2 = 12606,
  CODE_FOR_avx512vl_us_truncatev4div4qi2_mask_store_2 = 12607,
  CODE_FOR_avx512vl_ss_truncatev8hiv8qi2_mask_store_2 = 12608,
  CODE_FOR_avx512vl_truncatev8hiv8qi2_mask_store_2 = 12609,
  CODE_FOR_avx512vl_us_truncatev8hiv8qi2_mask_store_2 = 12610,
  CODE_FOR_avx512vl_ss_truncatev8siv8qi2_mask_store_2 = 12611,
  CODE_FOR_avx512vl_truncatev8siv8qi2_mask_store_2 = 12612,
  CODE_FOR_avx512vl_us_truncatev8siv8qi2_mask_store_2 = 12613,
  CODE_FOR_truncv4div4hi2 = 12614,
  CODE_FOR_truncv2div2hi2 = 12615,
  CODE_FOR_truncv4siv4hi2 = 12616,
  CODE_FOR_avx512vl_ss_truncatev4siv4hi2_mask_store_2 = 12617,
  CODE_FOR_avx512vl_truncatev4siv4hi2_mask_store_2 = 12618,
  CODE_FOR_avx512vl_us_truncatev4siv4hi2_mask_store_2 = 12619,
  CODE_FOR_avx512vl_ss_truncatev4div4hi2_mask_store_2 = 12620,
  CODE_FOR_avx512vl_truncatev4div4hi2_mask_store_2 = 12621,
  CODE_FOR_avx512vl_us_truncatev4div4hi2_mask_store_2 = 12622,
  CODE_FOR_avx512vl_ss_truncatev2div2hi2_mask_store_2 = 12623,
  CODE_FOR_avx512vl_truncatev2div2hi2_mask_store_2 = 12624,
  CODE_FOR_avx512vl_us_truncatev2div2hi2_mask_store_2 = 12625,
  CODE_FOR_truncv2div2si2 = 12626,
  CODE_FOR_avx512vl_ss_truncatev2div2si2_mask_store_2 = 12627,
  CODE_FOR_avx512vl_truncatev2div2si2_mask_store_2 = 12628,
  CODE_FOR_avx512vl_us_truncatev2div2si2_mask_store_2 = 12629,
  CODE_FOR_truncv8div8qi2 = 12630,
  CODE_FOR_avx512f_ss_truncatev8div16qi2_mask_store_2 = 12631,
  CODE_FOR_avx512f_truncatev8div16qi2_mask_store_2 = 12632,
  CODE_FOR_avx512f_us_truncatev8div16qi2_mask_store_2 = 12633,
  CODE_FOR_negv64qi2 = 12634,
  CODE_FOR_negv32qi2 = 12635,
  CODE_FOR_negv16qi2 = 12636,
  CODE_FOR_negv32hi2 = 12637,
  CODE_FOR_negv16hi2 = 12638,
  CODE_FOR_negv8hi2 = 12639,
  CODE_FOR_negv16si2 = 12640,
  CODE_FOR_negv8si2 = 12641,
  CODE_FOR_negv4si2 = 12642,
  CODE_FOR_negv8di2 = 12643,
  CODE_FOR_negv4di2 = 12644,
  CODE_FOR_negv2di2 = 12645,
  CODE_FOR_addv64qi3 = 12646,
  CODE_FOR_subv64qi3 = 12647,
  CODE_FOR_addv32qi3 = 12648,
  CODE_FOR_subv32qi3 = 12649,
  CODE_FOR_addv16qi3 = 12650,
  CODE_FOR_subv16qi3 = 12651,
  CODE_FOR_addv32hi3 = 12652,
  CODE_FOR_subv32hi3 = 12653,
  CODE_FOR_addv16hi3 = 12654,
  CODE_FOR_subv16hi3 = 12655,
  CODE_FOR_addv8hi3 = 12656,
  CODE_FOR_subv8hi3 = 12657,
  CODE_FOR_addv16si3 = 12658,
  CODE_FOR_subv16si3 = 12659,
  CODE_FOR_addv8si3 = 12660,
  CODE_FOR_subv8si3 = 12661,
  CODE_FOR_addv4si3 = 12662,
  CODE_FOR_subv4si3 = 12663,
  CODE_FOR_addv8di3 = 12664,
  CODE_FOR_subv8di3 = 12665,
  CODE_FOR_addv4di3 = 12666,
  CODE_FOR_subv4di3 = 12667,
  CODE_FOR_addv2di3 = 12668,
  CODE_FOR_subv2di3 = 12669,
  CODE_FOR_cond_addv64qi = 12670,
  CODE_FOR_cond_subv64qi = 12671,
  CODE_FOR_cond_addv32qi = 12672,
  CODE_FOR_cond_subv32qi = 12673,
  CODE_FOR_cond_addv16qi = 12674,
  CODE_FOR_cond_subv16qi = 12675,
  CODE_FOR_cond_addv32hi = 12676,
  CODE_FOR_cond_subv32hi = 12677,
  CODE_FOR_cond_addv16hi = 12678,
  CODE_FOR_cond_subv16hi = 12679,
  CODE_FOR_cond_addv8hi = 12680,
  CODE_FOR_cond_subv8hi = 12681,
  CODE_FOR_cond_addv16si = 12682,
  CODE_FOR_cond_subv16si = 12683,
  CODE_FOR_cond_addv8si = 12684,
  CODE_FOR_cond_subv8si = 12685,
  CODE_FOR_cond_addv4si = 12686,
  CODE_FOR_cond_subv4si = 12687,
  CODE_FOR_cond_addv8di = 12688,
  CODE_FOR_cond_subv8di = 12689,
  CODE_FOR_cond_addv4di = 12690,
  CODE_FOR_cond_subv4di = 12691,
  CODE_FOR_cond_addv2di = 12692,
  CODE_FOR_cond_subv2di = 12693,
  CODE_FOR_addv16si3_mask = 12694,
  CODE_FOR_subv16si3_mask = 12695,
  CODE_FOR_addv8si3_mask = 12696,
  CODE_FOR_subv8si3_mask = 12697,
  CODE_FOR_addv4si3_mask = 12698,
  CODE_FOR_subv4si3_mask = 12699,
  CODE_FOR_addv8di3_mask = 12700,
  CODE_FOR_subv8di3_mask = 12701,
  CODE_FOR_addv4di3_mask = 12702,
  CODE_FOR_subv4di3_mask = 12703,
  CODE_FOR_addv2di3_mask = 12704,
  CODE_FOR_subv2di3_mask = 12705,
  CODE_FOR_addv64qi3_mask = 12706,
  CODE_FOR_subv64qi3_mask = 12707,
  CODE_FOR_addv16qi3_mask = 12708,
  CODE_FOR_subv16qi3_mask = 12709,
  CODE_FOR_addv32qi3_mask = 12710,
  CODE_FOR_subv32qi3_mask = 12711,
  CODE_FOR_addv32hi3_mask = 12712,
  CODE_FOR_subv32hi3_mask = 12713,
  CODE_FOR_addv16hi3_mask = 12714,
  CODE_FOR_subv16hi3_mask = 12715,
  CODE_FOR_addv8hi3_mask = 12716,
  CODE_FOR_subv8hi3_mask = 12717,
  CODE_FOR_ssaddv64qi3 = 12718,
  CODE_FOR_ssaddv64qi3_mask = 12719,
  CODE_FOR_usaddv64qi3 = 12720,
  CODE_FOR_usaddv64qi3_mask = 12721,
  CODE_FOR_sssubv64qi3 = 12722,
  CODE_FOR_sssubv64qi3_mask = 12723,
  CODE_FOR_ussubv64qi3 = 12724,
  CODE_FOR_ussubv64qi3_mask = 12725,
  CODE_FOR_ssaddv32qi3 = 12726,
  CODE_FOR_ssaddv32qi3_mask = 12727,
  CODE_FOR_usaddv32qi3 = 12728,
  CODE_FOR_usaddv32qi3_mask = 12729,
  CODE_FOR_sssubv32qi3 = 12730,
  CODE_FOR_sssubv32qi3_mask = 12731,
  CODE_FOR_ussubv32qi3 = 12732,
  CODE_FOR_ussubv32qi3_mask = 12733,
  CODE_FOR_ssaddv16qi3 = 12734,
  CODE_FOR_ssaddv16qi3_mask = 12735,
  CODE_FOR_usaddv16qi3 = 12736,
  CODE_FOR_usaddv16qi3_mask = 12737,
  CODE_FOR_sssubv16qi3 = 12738,
  CODE_FOR_sssubv16qi3_mask = 12739,
  CODE_FOR_ussubv16qi3 = 12740,
  CODE_FOR_ussubv16qi3_mask = 12741,
  CODE_FOR_ssaddv32hi3 = 12742,
  CODE_FOR_ssaddv32hi3_mask = 12743,
  CODE_FOR_usaddv32hi3 = 12744,
  CODE_FOR_usaddv32hi3_mask = 12745,
  CODE_FOR_sssubv32hi3 = 12746,
  CODE_FOR_sssubv32hi3_mask = 12747,
  CODE_FOR_ussubv32hi3 = 12748,
  CODE_FOR_ussubv32hi3_mask = 12749,
  CODE_FOR_ssaddv16hi3 = 12750,
  CODE_FOR_ssaddv16hi3_mask = 12751,
  CODE_FOR_usaddv16hi3 = 12752,
  CODE_FOR_usaddv16hi3_mask = 12753,
  CODE_FOR_sssubv16hi3 = 12754,
  CODE_FOR_sssubv16hi3_mask = 12755,
  CODE_FOR_ussubv16hi3 = 12756,
  CODE_FOR_ussubv16hi3_mask = 12757,
  CODE_FOR_ssaddv8hi3 = 12758,
  CODE_FOR_ssaddv8hi3_mask = 12759,
  CODE_FOR_usaddv8hi3 = 12760,
  CODE_FOR_usaddv8hi3_mask = 12761,
  CODE_FOR_sssubv8hi3 = 12762,
  CODE_FOR_sssubv8hi3_mask = 12763,
  CODE_FOR_ussubv8hi3 = 12764,
  CODE_FOR_ussubv8hi3_mask = 12765,
  CODE_FOR_mulv64qi3 = 12766,
  CODE_FOR_mulv32qi3 = 12767,
  CODE_FOR_mulv16qi3 = 12768,
  CODE_FOR_cond_mulv8hi = 12769,
  CODE_FOR_cond_mulv16hi = 12770,
  CODE_FOR_cond_mulv32hi = 12771,
  CODE_FOR_mulv32hi3 = 12772,
  CODE_FOR_mulv32hi3_mask = 12773,
  CODE_FOR_mulv16hi3 = 12774,
  CODE_FOR_mulv16hi3_mask = 12775,
  CODE_FOR_mulv8hi3 = 12776,
  CODE_FOR_mulv8hi3_mask = 12777,
  CODE_FOR_smulv32hi3_highpart = 12778,
  CODE_FOR_smulv32hi3_highpart_mask = 12779,
  CODE_FOR_umulv32hi3_highpart = 12780,
  CODE_FOR_umulv32hi3_highpart_mask = 12781,
  CODE_FOR_smulv16hi3_highpart = 12782,
  CODE_FOR_smulv16hi3_highpart_mask = 12783,
  CODE_FOR_umulv16hi3_highpart = 12784,
  CODE_FOR_umulv16hi3_highpart_mask = 12785,
  CODE_FOR_smulv8hi3_highpart = 12786,
  CODE_FOR_smulv8hi3_highpart_mask = 12787,
  CODE_FOR_umulv8hi3_highpart = 12788,
  CODE_FOR_umulv8hi3_highpart_mask = 12789,
  CODE_FOR_vec_widen_umult_even_v16si = 12790,
  CODE_FOR_vec_widen_umult_even_v16si_mask = 12791,
  CODE_FOR_vec_widen_umult_even_v8si = 12792,
  CODE_FOR_vec_widen_umult_even_v8si_mask = 12793,
  CODE_FOR_vec_widen_umult_even_v4si = 12794,
  CODE_FOR_vec_widen_umult_even_v4si_mask = 12795,
  CODE_FOR_vec_widen_smult_even_v16si = 12796,
  CODE_FOR_vec_widen_smult_even_v16si_mask = 12797,
  CODE_FOR_vec_widen_smult_even_v8si = 12798,
  CODE_FOR_vec_widen_smult_even_v8si_mask = 12799,
  CODE_FOR_sse4_1_mulv2siv2di3 = 12800,
  CODE_FOR_sse4_1_mulv2siv2di3_mask = 12801,
  CODE_FOR_avx2_pmaddwd = 12802,
  CODE_FOR_sse2_pmaddwd = 12803,
  CODE_FOR_cond_mulv8di = 12804,
  CODE_FOR_cond_mulv4di = 12805,
  CODE_FOR_cond_mulv2di = 12806,
  CODE_FOR_avx512dq_mulv8di3 = 12807,
  CODE_FOR_avx512dq_mulv8di3_mask = 12808,
  CODE_FOR_avx512dq_mulv4di3 = 12809,
  CODE_FOR_avx512dq_mulv4di3_mask = 12810,
  CODE_FOR_avx512dq_mulv2di3 = 12811,
  CODE_FOR_avx512dq_mulv2di3_mask = 12812,
  CODE_FOR_cond_mulv16si = 12813,
  CODE_FOR_cond_mulv8si = 12814,
  CODE_FOR_cond_mulv4si = 12815,
  CODE_FOR_mulv16si3 = 12816,
  CODE_FOR_mulv16si3_mask = 12817,
  CODE_FOR_mulv8si3 = 12818,
  CODE_FOR_mulv8si3_mask = 12819,
  CODE_FOR_mulv4si3 = 12820,
  CODE_FOR_mulv4si3_mask = 12821,
  CODE_FOR_mulv8di3 = 12822,
  CODE_FOR_mulv4di3 = 12823,
  CODE_FOR_mulv2di3 = 12824,
  CODE_FOR_vec_widen_smult_hi_v32qi = 12825,
  CODE_FOR_vec_widen_umult_hi_v32qi = 12826,
  CODE_FOR_vec_widen_smult_hi_v16qi = 12827,
  CODE_FOR_vec_widen_umult_hi_v16qi = 12828,
  CODE_FOR_vec_widen_smult_hi_v16hi = 12829,
  CODE_FOR_vec_widen_umult_hi_v16hi = 12830,
  CODE_FOR_vec_widen_smult_hi_v8hi = 12831,
  CODE_FOR_vec_widen_umult_hi_v8hi = 12832,
  CODE_FOR_vec_widen_smult_hi_v8si = 12833,
  CODE_FOR_vec_widen_umult_hi_v8si = 12834,
  CODE_FOR_vec_widen_smult_hi_v4si = 12835,
  CODE_FOR_vec_widen_umult_hi_v4si = 12836,
  CODE_FOR_vec_widen_smult_lo_v32qi = 12837,
  CODE_FOR_vec_widen_umult_lo_v32qi = 12838,
  CODE_FOR_vec_widen_smult_lo_v16qi = 12839,
  CODE_FOR_vec_widen_umult_lo_v16qi = 12840,
  CODE_FOR_vec_widen_smult_lo_v16hi = 12841,
  CODE_FOR_vec_widen_umult_lo_v16hi = 12842,
  CODE_FOR_vec_widen_smult_lo_v8hi = 12843,
  CODE_FOR_vec_widen_umult_lo_v8hi = 12844,
  CODE_FOR_vec_widen_smult_lo_v8si = 12845,
  CODE_FOR_vec_widen_umult_lo_v8si = 12846,
  CODE_FOR_vec_widen_smult_lo_v4si = 12847,
  CODE_FOR_vec_widen_umult_lo_v4si = 12848,
  CODE_FOR_vec_widen_smult_even_v4si = 12849,
  CODE_FOR_vec_widen_smult_odd_v16si = 12850,
  CODE_FOR_vec_widen_umult_odd_v16si = 12851,
  CODE_FOR_vec_widen_smult_odd_v8si = 12852,
  CODE_FOR_vec_widen_umult_odd_v8si = 12853,
  CODE_FOR_vec_widen_smult_odd_v4si = 12854,
  CODE_FOR_vec_widen_umult_odd_v4si = 12855,
  CODE_FOR_sdot_prodv16siv32hi = 12856,
  CODE_FOR_sdot_prodv8siv16hi = 12857,
  CODE_FOR_sdot_prodv4siv8hi = 12858,
  CODE_FOR_sdot_prodv2div4si = 12859,
  CODE_FOR_uavgv64qi3_ceil = 12860,
  CODE_FOR_uavgv32qi3_ceil = 12861,
  CODE_FOR_uavgv16qi3_ceil = 12862,
  CODE_FOR_uavgv32hi3_ceil = 12863,
  CODE_FOR_uavgv16hi3_ceil = 12864,
  CODE_FOR_uavgv8hi3_ceil = 12865,
  CODE_FOR_usadv16qi = 12866,
  CODE_FOR_usadv32qi = 12867,
  CODE_FOR_usadv64qi = 12868,
  CODE_FOR_ashrv32hi3 = 12869,
  CODE_FOR_ashrv16si3 = 12870,
  CODE_FOR_ashrv8di3 = 12871,
  CODE_FOR_ashrv4di3 = 12872,
  CODE_FOR_vec_shl_v16qi = 12873,
  CODE_FOR_vec_shl_v8hi = 12874,
  CODE_FOR_vec_shl_v8hf = 12875,
  CODE_FOR_vec_shl_v8bf = 12876,
  CODE_FOR_vec_shl_v4si = 12877,
  CODE_FOR_vec_shl_v2di = 12878,
  CODE_FOR_vec_shl_v4sf = 12879,
  CODE_FOR_vec_shl_v2df = 12880,
  CODE_FOR_vec_shr_v16qi = 12881,
  CODE_FOR_vec_shr_v8hi = 12882,
  CODE_FOR_vec_shr_v8hf = 12883,
  CODE_FOR_vec_shr_v8bf = 12884,
  CODE_FOR_vec_shr_v4si = 12885,
  CODE_FOR_vec_shr_v2di = 12886,
  CODE_FOR_vec_shr_v4sf = 12887,
  CODE_FOR_vec_shr_v2df = 12888,
   CODE_FOR_ashlv1ti3 = CODE_FOR_nothing,
   CODE_FOR_lshrv1ti3 = CODE_FOR_nothing,
   CODE_FOR_ashrv1ti3 = CODE_FOR_nothing,
   CODE_FOR_rotlv1ti3 = CODE_FOR_nothing,
   CODE_FOR_rotrv1ti3 = CODE_FOR_nothing,
  CODE_FOR_smaxv32qi3 = 12889,
  CODE_FOR_sminv32qi3 = 12890,
  CODE_FOR_umaxv32qi3 = 12891,
  CODE_FOR_uminv32qi3 = 12892,
  CODE_FOR_smaxv16hi3 = 12893,
  CODE_FOR_sminv16hi3 = 12894,
  CODE_FOR_umaxv16hi3 = 12895,
  CODE_FOR_uminv16hi3 = 12896,
  CODE_FOR_smaxv8si3 = 12897,
  CODE_FOR_sminv8si3 = 12898,
  CODE_FOR_umaxv8si3 = 12899,
  CODE_FOR_uminv8si3 = 12900,
  CODE_FOR_smaxv64qi3 = 12901,
  CODE_FOR_sminv64qi3 = 12902,
  CODE_FOR_umaxv64qi3 = 12903,
  CODE_FOR_uminv64qi3 = 12904,
  CODE_FOR_smaxv32hi3 = 12905,
  CODE_FOR_sminv32hi3 = 12906,
  CODE_FOR_umaxv32hi3 = 12907,
  CODE_FOR_uminv32hi3 = 12908,
  CODE_FOR_smaxv16si3 = 12909,
  CODE_FOR_sminv16si3 = 12910,
  CODE_FOR_umaxv16si3 = 12911,
  CODE_FOR_uminv16si3 = 12912,
  CODE_FOR_cond_smaxv64qi = 12913,
  CODE_FOR_cond_sminv64qi = 12914,
  CODE_FOR_cond_umaxv64qi = 12915,
  CODE_FOR_cond_uminv64qi = 12916,
  CODE_FOR_cond_smaxv32qi = 12917,
  CODE_FOR_cond_sminv32qi = 12918,
  CODE_FOR_cond_umaxv32qi = 12919,
  CODE_FOR_cond_uminv32qi = 12920,
  CODE_FOR_cond_smaxv16qi = 12921,
  CODE_FOR_cond_sminv16qi = 12922,
  CODE_FOR_cond_umaxv16qi = 12923,
  CODE_FOR_cond_uminv16qi = 12924,
  CODE_FOR_cond_smaxv32hi = 12925,
  CODE_FOR_cond_sminv32hi = 12926,
  CODE_FOR_cond_umaxv32hi = 12927,
  CODE_FOR_cond_uminv32hi = 12928,
  CODE_FOR_cond_smaxv16hi = 12929,
  CODE_FOR_cond_sminv16hi = 12930,
  CODE_FOR_cond_umaxv16hi = 12931,
  CODE_FOR_cond_uminv16hi = 12932,
  CODE_FOR_cond_smaxv8hi = 12933,
  CODE_FOR_cond_sminv8hi = 12934,
  CODE_FOR_cond_umaxv8hi = 12935,
  CODE_FOR_cond_uminv8hi = 12936,
  CODE_FOR_cond_smaxv16si = 12937,
  CODE_FOR_cond_sminv16si = 12938,
  CODE_FOR_cond_umaxv16si = 12939,
  CODE_FOR_cond_uminv16si = 12940,
  CODE_FOR_cond_smaxv8si = 12941,
  CODE_FOR_cond_sminv8si = 12942,
  CODE_FOR_cond_umaxv8si = 12943,
  CODE_FOR_cond_uminv8si = 12944,
  CODE_FOR_cond_smaxv4si = 12945,
  CODE_FOR_cond_sminv4si = 12946,
  CODE_FOR_cond_umaxv4si = 12947,
  CODE_FOR_cond_uminv4si = 12948,
  CODE_FOR_cond_smaxv8di = 12949,
  CODE_FOR_cond_sminv8di = 12950,
  CODE_FOR_cond_umaxv8di = 12951,
  CODE_FOR_cond_uminv8di = 12952,
  CODE_FOR_cond_smaxv4di = 12953,
  CODE_FOR_cond_sminv4di = 12954,
  CODE_FOR_cond_umaxv4di = 12955,
  CODE_FOR_cond_uminv4di = 12956,
  CODE_FOR_cond_smaxv2di = 12957,
  CODE_FOR_cond_sminv2di = 12958,
  CODE_FOR_cond_umaxv2di = 12959,
  CODE_FOR_cond_uminv2di = 12960,
  CODE_FOR_smaxv64qi3_mask = 12961,
  CODE_FOR_sminv64qi3_mask = 12962,
  CODE_FOR_umaxv64qi3_mask = 12963,
  CODE_FOR_uminv64qi3_mask = 12964,
  CODE_FOR_smaxv32qi3_mask = 12965,
  CODE_FOR_sminv32qi3_mask = 12966,
  CODE_FOR_umaxv32qi3_mask = 12967,
  CODE_FOR_uminv32qi3_mask = 12968,
  CODE_FOR_smaxv16qi3_mask = 12969,
  CODE_FOR_sminv16qi3_mask = 12970,
  CODE_FOR_umaxv16qi3_mask = 12971,
  CODE_FOR_uminv16qi3_mask = 12972,
  CODE_FOR_smaxv32hi3_mask = 12973,
  CODE_FOR_sminv32hi3_mask = 12974,
  CODE_FOR_umaxv32hi3_mask = 12975,
  CODE_FOR_uminv32hi3_mask = 12976,
  CODE_FOR_smaxv16hi3_mask = 12977,
  CODE_FOR_sminv16hi3_mask = 12978,
  CODE_FOR_umaxv16hi3_mask = 12979,
  CODE_FOR_uminv16hi3_mask = 12980,
  CODE_FOR_smaxv8hi3_mask = 12981,
  CODE_FOR_sminv8hi3_mask = 12982,
  CODE_FOR_umaxv8hi3_mask = 12983,
  CODE_FOR_uminv8hi3_mask = 12984,
  CODE_FOR_smaxv16si3_mask = 12985,
  CODE_FOR_sminv16si3_mask = 12986,
  CODE_FOR_umaxv16si3_mask = 12987,
  CODE_FOR_uminv16si3_mask = 12988,
  CODE_FOR_smaxv8si3_mask = 12989,
  CODE_FOR_sminv8si3_mask = 12990,
  CODE_FOR_umaxv8si3_mask = 12991,
  CODE_FOR_uminv8si3_mask = 12992,
  CODE_FOR_smaxv4si3_mask = 12993,
  CODE_FOR_sminv4si3_mask = 12994,
  CODE_FOR_umaxv4si3_mask = 12995,
  CODE_FOR_uminv4si3_mask = 12996,
  CODE_FOR_smaxv8di3_mask = 12997,
  CODE_FOR_sminv8di3_mask = 12998,
  CODE_FOR_umaxv8di3_mask = 12999,
  CODE_FOR_uminv8di3_mask = 13000,
  CODE_FOR_smaxv4di3_mask = 13001,
  CODE_FOR_sminv4di3_mask = 13002,
  CODE_FOR_umaxv4di3_mask = 13003,
  CODE_FOR_uminv4di3_mask = 13004,
  CODE_FOR_smaxv2di3_mask = 13005,
  CODE_FOR_sminv2di3_mask = 13006,
  CODE_FOR_umaxv2di3_mask = 13007,
  CODE_FOR_uminv2di3_mask = 13008,
  CODE_FOR_smaxv8di3 = 13009,
  CODE_FOR_sminv8di3 = 13010,
  CODE_FOR_umaxv8di3 = 13011,
  CODE_FOR_uminv8di3 = 13012,
  CODE_FOR_smaxv4di3 = 13013,
  CODE_FOR_sminv4di3 = 13014,
  CODE_FOR_umaxv4di3 = 13015,
  CODE_FOR_uminv4di3 = 13016,
  CODE_FOR_smaxv2di3 = 13017,
  CODE_FOR_sminv2di3 = 13018,
  CODE_FOR_umaxv2di3 = 13019,
  CODE_FOR_uminv2di3 = 13020,
  CODE_FOR_smaxv16qi3 = 13021,
  CODE_FOR_sminv16qi3 = 13022,
  CODE_FOR_smaxv8hi3 = 13023,
  CODE_FOR_sminv8hi3 = 13024,
  CODE_FOR_smaxv4si3 = 13025,
  CODE_FOR_sminv4si3 = 13026,
  CODE_FOR_umaxv16qi3 = 13027,
  CODE_FOR_uminv16qi3 = 13028,
  CODE_FOR_umaxv8hi3 = 13029,
  CODE_FOR_uminv8hi3 = 13030,
  CODE_FOR_umaxv4si3 = 13031,
  CODE_FOR_uminv4si3 = 13032,
  CODE_FOR_avx512bw_eqv64qi3 = 13033,
  CODE_FOR_avx512bw_eqv64qi3_mask = 13034,
  CODE_FOR_avx512vl_eqv16qi3 = 13035,
  CODE_FOR_avx512vl_eqv16qi3_mask = 13036,
  CODE_FOR_avx512vl_eqv32qi3 = 13037,
  CODE_FOR_avx512vl_eqv32qi3_mask = 13038,
  CODE_FOR_avx512bw_eqv32hi3 = 13039,
  CODE_FOR_avx512bw_eqv32hi3_mask = 13040,
  CODE_FOR_avx512vl_eqv16hi3 = 13041,
  CODE_FOR_avx512vl_eqv16hi3_mask = 13042,
  CODE_FOR_avx512vl_eqv8hi3 = 13043,
  CODE_FOR_avx512vl_eqv8hi3_mask = 13044,
  CODE_FOR_avx512f_eqv16si3 = 13045,
  CODE_FOR_avx512f_eqv16si3_mask = 13046,
  CODE_FOR_avx512vl_eqv8si3 = 13047,
  CODE_FOR_avx512vl_eqv8si3_mask = 13048,
  CODE_FOR_avx512vl_eqv4si3 = 13049,
  CODE_FOR_avx512vl_eqv4si3_mask = 13050,
  CODE_FOR_avx512f_eqv8di3 = 13051,
  CODE_FOR_avx512f_eqv8di3_mask = 13052,
  CODE_FOR_avx512vl_eqv4di3 = 13053,
  CODE_FOR_avx512vl_eqv4di3_mask = 13054,
  CODE_FOR_avx512vl_eqv2di3 = 13055,
  CODE_FOR_avx512vl_eqv2di3_mask = 13056,
  CODE_FOR_avx512f_gtv16si3 = 13057,
  CODE_FOR_avx512f_gtv16si3_mask = 13058,
  CODE_FOR_avx512vl_gtv8si3 = 13059,
  CODE_FOR_avx512vl_gtv8si3_mask = 13060,
  CODE_FOR_avx512vl_gtv4si3 = 13061,
  CODE_FOR_avx512vl_gtv4si3_mask = 13062,
  CODE_FOR_avx512f_gtv8di3 = 13063,
  CODE_FOR_avx512f_gtv8di3_mask = 13064,
  CODE_FOR_avx512vl_gtv4di3 = 13065,
  CODE_FOR_avx512vl_gtv4di3_mask = 13066,
  CODE_FOR_avx512vl_gtv2di3 = 13067,
  CODE_FOR_avx512vl_gtv2di3_mask = 13068,
  CODE_FOR_avx512bw_gtv64qi3 = 13069,
  CODE_FOR_avx512bw_gtv64qi3_mask = 13070,
  CODE_FOR_avx512vl_gtv16qi3 = 13071,
  CODE_FOR_avx512vl_gtv16qi3_mask = 13072,
  CODE_FOR_avx512vl_gtv32qi3 = 13073,
  CODE_FOR_avx512vl_gtv32qi3_mask = 13074,
  CODE_FOR_avx512bw_gtv32hi3 = 13075,
  CODE_FOR_avx512bw_gtv32hi3_mask = 13076,
  CODE_FOR_avx512vl_gtv16hi3 = 13077,
  CODE_FOR_avx512vl_gtv16hi3_mask = 13078,
  CODE_FOR_avx512vl_gtv8hi3 = 13079,
  CODE_FOR_avx512vl_gtv8hi3_mask = 13080,
  CODE_FOR_vec_permv16qi = 13081,
  CODE_FOR_vec_permv8hi = 13082,
  CODE_FOR_vec_permv4si = 13083,
  CODE_FOR_vec_permv2di = 13084,
  CODE_FOR_vec_permv4sf = 13085,
  CODE_FOR_vec_permv2df = 13086,
  CODE_FOR_vec_permv8hf = 13087,
  CODE_FOR_vec_permv32qi = 13088,
  CODE_FOR_vec_permv16hi = 13089,
  CODE_FOR_vec_permv8si = 13090,
  CODE_FOR_vec_permv4di = 13091,
  CODE_FOR_vec_permv8sf = 13092,
  CODE_FOR_vec_permv4df = 13093,
  CODE_FOR_vec_permv16hf = 13094,
  CODE_FOR_vec_permv16sf = 13095,
  CODE_FOR_vec_permv8df = 13096,
  CODE_FOR_vec_permv16si = 13097,
  CODE_FOR_vec_permv8di = 13098,
  CODE_FOR_vec_permv32hi = 13099,
  CODE_FOR_vec_permv64qi = 13100,
  CODE_FOR_vec_permv32hf = 13101,
  CODE_FOR_one_cmplv16si2 = 13102,
  CODE_FOR_one_cmplv8di2 = 13103,
  CODE_FOR_one_cmplv64qi2 = 13104,
  CODE_FOR_one_cmplv32qi2 = 13105,
  CODE_FOR_one_cmplv16qi2 = 13106,
  CODE_FOR_one_cmplv32hi2 = 13107,
  CODE_FOR_one_cmplv16hi2 = 13108,
  CODE_FOR_one_cmplv8hi2 = 13109,
  CODE_FOR_one_cmplv8si2 = 13110,
  CODE_FOR_one_cmplv4si2 = 13111,
  CODE_FOR_one_cmplv4di2 = 13112,
  CODE_FOR_one_cmplv2di2 = 13113,
  CODE_FOR_avx512bw_andnotv64qi3 = 13114,
  CODE_FOR_avx2_andnotv32qi3 = 13115,
  CODE_FOR_sse2_andnotv16qi3 = 13116,
  CODE_FOR_avx512bw_andnotv32hi3 = 13117,
  CODE_FOR_avx2_andnotv16hi3 = 13118,
  CODE_FOR_sse2_andnotv8hi3 = 13119,
  CODE_FOR_avx512f_andnotv16si3 = 13120,
  CODE_FOR_avx2_andnotv8si3 = 13121,
  CODE_FOR_sse2_andnotv4si3 = 13122,
  CODE_FOR_avx512f_andnotv8di3 = 13123,
  CODE_FOR_avx2_andnotv4di3 = 13124,
  CODE_FOR_sse2_andnotv2di3 = 13125,
  CODE_FOR_andnv16si3 = 13126,
  CODE_FOR_andnv8di3 = 13127,
  CODE_FOR_andnv64qi3 = 13128,
  CODE_FOR_andnv32qi3 = 13129,
  CODE_FOR_andnv16qi3 = 13130,
  CODE_FOR_andnv32hi3 = 13131,
  CODE_FOR_andnv16hi3 = 13132,
  CODE_FOR_andnv8hi3 = 13133,
  CODE_FOR_andnv8si3 = 13134,
  CODE_FOR_andnv4si3 = 13135,
  CODE_FOR_andnv4di3 = 13136,
  CODE_FOR_andnv2di3 = 13137,
  CODE_FOR_avx512f_andnotv16si3_mask = 13138,
  CODE_FOR_avx2_andnotv8si3_mask = 13139,
  CODE_FOR_sse2_andnotv4si3_mask = 13140,
  CODE_FOR_avx512f_andnotv8di3_mask = 13141,
  CODE_FOR_avx2_andnotv4di3_mask = 13142,
  CODE_FOR_sse2_andnotv2di3_mask = 13143,
  CODE_FOR_andv16si3 = 13144,
  CODE_FOR_iorv16si3 = 13145,
  CODE_FOR_xorv16si3 = 13146,
  CODE_FOR_andv8di3 = 13147,
  CODE_FOR_iorv8di3 = 13148,
  CODE_FOR_xorv8di3 = 13149,
  CODE_FOR_andv64qi3 = 13150,
  CODE_FOR_iorv64qi3 = 13151,
  CODE_FOR_xorv64qi3 = 13152,
  CODE_FOR_andv32qi3 = 13153,
  CODE_FOR_iorv32qi3 = 13154,
  CODE_FOR_xorv32qi3 = 13155,
  CODE_FOR_andv16qi3 = 13156,
  CODE_FOR_iorv16qi3 = 13157,
  CODE_FOR_xorv16qi3 = 13158,
  CODE_FOR_andv32hi3 = 13159,
  CODE_FOR_iorv32hi3 = 13160,
  CODE_FOR_xorv32hi3 = 13161,
  CODE_FOR_andv16hi3 = 13162,
  CODE_FOR_iorv16hi3 = 13163,
  CODE_FOR_xorv16hi3 = 13164,
  CODE_FOR_andv8hi3 = 13165,
  CODE_FOR_iorv8hi3 = 13166,
  CODE_FOR_xorv8hi3 = 13167,
  CODE_FOR_andv8si3 = 13168,
  CODE_FOR_iorv8si3 = 13169,
  CODE_FOR_xorv8si3 = 13170,
  CODE_FOR_andv4si3 = 13171,
  CODE_FOR_iorv4si3 = 13172,
  CODE_FOR_xorv4si3 = 13173,
  CODE_FOR_andv4di3 = 13174,
  CODE_FOR_iorv4di3 = 13175,
  CODE_FOR_xorv4di3 = 13176,
  CODE_FOR_andv2di3 = 13177,
  CODE_FOR_iorv2di3 = 13178,
  CODE_FOR_xorv2di3 = 13179,
  CODE_FOR_cond_andv16si = 13180,
  CODE_FOR_cond_iorv16si = 13181,
  CODE_FOR_cond_xorv16si = 13182,
  CODE_FOR_cond_andv8si = 13183,
  CODE_FOR_cond_iorv8si = 13184,
  CODE_FOR_cond_xorv8si = 13185,
  CODE_FOR_cond_andv4si = 13186,
  CODE_FOR_cond_iorv4si = 13187,
  CODE_FOR_cond_xorv4si = 13188,
  CODE_FOR_cond_andv8di = 13189,
  CODE_FOR_cond_iorv8di = 13190,
  CODE_FOR_cond_xorv8di = 13191,
  CODE_FOR_cond_andv4di = 13192,
  CODE_FOR_cond_iorv4di = 13193,
  CODE_FOR_cond_xorv4di = 13194,
  CODE_FOR_cond_andv2di = 13195,
  CODE_FOR_cond_iorv2di = 13196,
  CODE_FOR_cond_xorv2di = 13197,
  CODE_FOR_andv16si3_mask = 13198,
  CODE_FOR_iorv16si3_mask = 13199,
  CODE_FOR_xorv16si3_mask = 13200,
  CODE_FOR_andv8si3_mask = 13201,
  CODE_FOR_iorv8si3_mask = 13202,
  CODE_FOR_xorv8si3_mask = 13203,
  CODE_FOR_andv4si3_mask = 13204,
  CODE_FOR_iorv4si3_mask = 13205,
  CODE_FOR_xorv4si3_mask = 13206,
  CODE_FOR_andv8di3_mask = 13207,
  CODE_FOR_iorv8di3_mask = 13208,
  CODE_FOR_xorv8di3_mask = 13209,
  CODE_FOR_andv4di3_mask = 13210,
  CODE_FOR_iorv4di3_mask = 13211,
  CODE_FOR_xorv4di3_mask = 13212,
  CODE_FOR_andv2di3_mask = 13213,
  CODE_FOR_iorv2di3_mask = 13214,
  CODE_FOR_xorv2di3_mask = 13215,
  CODE_FOR_one_cmplv1ti2 = 13216,
  CODE_FOR_vec_pack_trunc_v32hi = 13217,
  CODE_FOR_vec_pack_trunc_v16hi = 13218,
  CODE_FOR_vec_pack_trunc_v8hi = 13219,
  CODE_FOR_vec_pack_trunc_v16si = 13220,
  CODE_FOR_vec_pack_trunc_v8si = 13221,
  CODE_FOR_vec_pack_trunc_v4si = 13222,
  CODE_FOR_vec_pack_trunc_v8di = 13223,
  CODE_FOR_vec_pack_trunc_v4di = 13224,
  CODE_FOR_vec_pack_trunc_v2di = 13225,
  CODE_FOR_vec_pack_trunc_qi = 13226,
  CODE_FOR_vec_pack_trunc_hi = 13227,
  CODE_FOR_vec_pack_trunc_si = 13228,
  CODE_FOR_vec_pack_sbool_trunc_qi = 13229,
  CODE_FOR_vec_interleave_highv32qi = 13230,
  CODE_FOR_vec_interleave_highv16hi = 13231,
  CODE_FOR_vec_interleave_highv8si = 13232,
  CODE_FOR_vec_interleave_highv4di = 13233,
  CODE_FOR_vec_interleave_lowv32qi = 13234,
  CODE_FOR_vec_interleave_lowv16hi = 13235,
  CODE_FOR_vec_interleave_lowv8si = 13236,
  CODE_FOR_vec_interleave_lowv4di = 13237,
  CODE_FOR_avx512dq_vinsertf64x2_mask = 13238,
  CODE_FOR_avx512dq_vinserti64x2_mask = 13239,
  CODE_FOR_avx512f_vinsertf32x4_mask = 13240,
  CODE_FOR_avx512f_vinserti32x4_mask = 13241,
  CODE_FOR_avx512dq_vinsertf32x8_mask = 13242,
  CODE_FOR_avx512dq_vinserti32x8_mask = 13243,
  CODE_FOR_avx512f_vinsertf64x4_mask = 13244,
  CODE_FOR_avx512f_vinserti64x4_mask = 13245,
  CODE_FOR_avx512dq_shuf_i64x2_mask = 13246,
  CODE_FOR_avx512dq_shuf_f64x2_mask = 13247,
  CODE_FOR_avx512f_shuf_f64x2_mask = 13248,
  CODE_FOR_avx512f_shuf_i64x2_mask = 13249,
  CODE_FOR_avx512vl_shuf_i32x4_mask = 13250,
  CODE_FOR_avx512vl_shuf_f32x4_mask = 13251,
  CODE_FOR_avx512f_shuf_f32x4_mask = 13252,
  CODE_FOR_avx512f_shuf_i32x4_mask = 13253,
  CODE_FOR_avx512f_pshufdv3_mask = 13254,
  CODE_FOR_avx512vl_pshufdv3_mask = 13255,
  CODE_FOR_avx2_pshufdv3 = 13256,
  CODE_FOR_avx512vl_pshufd_mask = 13257,
  CODE_FOR_sse2_pshufd = 13258,
  CODE_FOR_avx512vl_pshuflwv3_mask = 13259,
  CODE_FOR_avx2_pshuflwv3 = 13260,
  CODE_FOR_avx512vl_pshuflw_mask = 13261,
  CODE_FOR_sse2_pshuflw = 13262,
  CODE_FOR_avx2_pshufhwv3 = 13263,
  CODE_FOR_avx512vl_pshufhwv3_mask = 13264,
  CODE_FOR_avx512vl_pshufhw_mask = 13265,
  CODE_FOR_sse2_pshufhw = 13266,
  CODE_FOR_sse2_loadd = 13267,
  CODE_FOR_vec_unpacks_lo_v64qi = 13268,
  CODE_FOR_vec_unpacks_lo_v32qi = 13269,
  CODE_FOR_vec_unpacks_lo_v16qi = 13270,
  CODE_FOR_vec_unpacks_lo_v32hi = 13271,
  CODE_FOR_vec_unpacks_lo_v16hi = 13272,
  CODE_FOR_vec_unpacks_lo_v8hi = 13273,
  CODE_FOR_vec_unpacks_lo_v16si = 13274,
  CODE_FOR_vec_unpacks_lo_v8si = 13275,
  CODE_FOR_vec_unpacks_lo_v4si = 13276,
  CODE_FOR_vec_unpacks_hi_v64qi = 13277,
  CODE_FOR_vec_unpacks_hi_v32qi = 13278,
  CODE_FOR_vec_unpacks_hi_v16qi = 13279,
  CODE_FOR_vec_unpacks_hi_v32hi = 13280,
  CODE_FOR_vec_unpacks_hi_v16hi = 13281,
  CODE_FOR_vec_unpacks_hi_v8hi = 13282,
  CODE_FOR_vec_unpacks_hi_v16si = 13283,
  CODE_FOR_vec_unpacks_hi_v8si = 13284,
  CODE_FOR_vec_unpacks_hi_v4si = 13285,
  CODE_FOR_vec_unpacku_lo_v64qi = 13286,
  CODE_FOR_vec_unpacku_lo_v32qi = 13287,
  CODE_FOR_vec_unpacku_lo_v16qi = 13288,
  CODE_FOR_vec_unpacku_lo_v32hi = 13289,
  CODE_FOR_vec_unpacku_lo_v16hi = 13290,
  CODE_FOR_vec_unpacku_lo_v8hi = 13291,
  CODE_FOR_vec_unpacku_lo_v16si = 13292,
  CODE_FOR_vec_unpacku_lo_v8si = 13293,
  CODE_FOR_vec_unpacku_lo_v4si = 13294,
  CODE_FOR_vec_unpacks_sbool_lo_qi = 13295,
  CODE_FOR_vec_unpacks_lo_hi = 13296,
  CODE_FOR_vec_unpacks_lo_si = 13297,
  CODE_FOR_vec_unpacks_lo_di = 13298,
  CODE_FOR_vec_unpacku_hi_v64qi = 13299,
  CODE_FOR_vec_unpacku_hi_v32qi = 13300,
  CODE_FOR_vec_unpacku_hi_v16qi = 13301,
  CODE_FOR_vec_unpacku_hi_v32hi = 13302,
  CODE_FOR_vec_unpacku_hi_v16hi = 13303,
  CODE_FOR_vec_unpacku_hi_v8hi = 13304,
  CODE_FOR_vec_unpacku_hi_v16si = 13305,
  CODE_FOR_vec_unpacku_hi_v8si = 13306,
  CODE_FOR_vec_unpacku_hi_v4si = 13307,
  CODE_FOR_vec_unpacks_sbool_hi_qi = 13308,
  CODE_FOR_vec_unpacks_hi_hi = 13309,
  CODE_FOR_vec_unpacks_hi_si = 13310,
  CODE_FOR_vec_unpacks_hi_di = 13311,
  CODE_FOR_avx512bw_uavgv64qi3 = 13312,
  CODE_FOR_avx512bw_uavgv64qi3_mask = 13313,
  CODE_FOR_avx2_uavgv32qi3 = 13314,
  CODE_FOR_avx2_uavgv32qi3_mask = 13315,
  CODE_FOR_sse2_uavgv16qi3 = 13316,
  CODE_FOR_sse2_uavgv16qi3_mask = 13317,
  CODE_FOR_avx512bw_uavgv32hi3 = 13318,
  CODE_FOR_avx512bw_uavgv32hi3_mask = 13319,
  CODE_FOR_avx2_uavgv16hi3 = 13320,
  CODE_FOR_avx2_uavgv16hi3_mask = 13321,
  CODE_FOR_sse2_uavgv8hi3 = 13322,
  CODE_FOR_sse2_uavgv8hi3_mask = 13323,
  CODE_FOR_avx512f_psadbw = 13324,
  CODE_FOR_avx2_psadbw = 13325,
  CODE_FOR_sse2_psadbw = 13326,
  CODE_FOR_sse2_maskmovdqu = 13327,
  CODE_FOR_ssse3_pmulhrswv8hi3_mask = 13328,
  CODE_FOR_avx2_pmulhrswv16hi3_mask = 13329,
  CODE_FOR_ssse3_pmulhrswv8hi3 = 13330,
  CODE_FOR_avx2_pmulhrswv16hi3 = 13331,
  CODE_FOR_smulhrsv32hi3 = 13332,
  CODE_FOR_smulhrsv16hi3 = 13333,
  CODE_FOR_smulhrsv8hi3 = 13334,
   CODE_FOR_smulhrsv4hi3 = CODE_FOR_nothing,
  CODE_FOR_ssse3_pmulhrswv4hi3 = 13335,
  CODE_FOR_smulhrsv2hi3 = 13336,
  CODE_FOR_ssse3_pshufbv8qi3 = 13337,
  CODE_FOR_absv64qi2 = 13338,
  CODE_FOR_absv32qi2 = 13339,
  CODE_FOR_absv16qi2 = 13340,
  CODE_FOR_absv32hi2 = 13341,
  CODE_FOR_absv16hi2 = 13342,
  CODE_FOR_absv8hi2 = 13343,
  CODE_FOR_absv16si2 = 13344,
  CODE_FOR_absv8si2 = 13345,
  CODE_FOR_absv4si2 = 13346,
  CODE_FOR_absv8di2 = 13347,
  CODE_FOR_absv4di2 = 13348,
  CODE_FOR_absv2di2 = 13349,
  CODE_FOR_avx2_pblendw = 13350,
  CODE_FOR_avx2_pblendph = 13351,
  CODE_FOR_avx2_pblendbf = 13352,
  CODE_FOR_avx2_pblendw_1 = 13353,
  CODE_FOR_avx2_pblendph_1 = 13354,
  CODE_FOR_avx2_pblendbf_1 = 13355,
  CODE_FOR_extendv16qiv16hi2 = 13356,
  CODE_FOR_zero_extendv16qiv16hi2 = 13357,
  CODE_FOR_extendv32qiv32hi2 = 13358,
  CODE_FOR_zero_extendv32qiv32hi2 = 13359,
  CODE_FOR_extendv8qiv8hi2 = 13360,
  CODE_FOR_zero_extendv8qiv8hi2 = 13361,
  CODE_FOR_extendv16qiv16si2 = 13362,
  CODE_FOR_zero_extendv16qiv16si2 = 13363,
  CODE_FOR_extendv8qiv8si2 = 13364,
  CODE_FOR_zero_extendv8qiv8si2 = 13365,
  CODE_FOR_extendv4qiv4si2 = 13366,
  CODE_FOR_zero_extendv4qiv4si2 = 13367,
  CODE_FOR_extendv16hiv16si2 = 13368,
  CODE_FOR_zero_extendv16hiv16si2 = 13369,
  CODE_FOR_extendv8hiv8si2 = 13370,
  CODE_FOR_zero_extendv8hiv8si2 = 13371,
  CODE_FOR_extendv4hiv4si2 = 13372,
  CODE_FOR_zero_extendv4hiv4si2 = 13373,
  CODE_FOR_extendv8qiv8di2 = 13374,
  CODE_FOR_zero_extendv8qiv8di2 = 13375,
  CODE_FOR_extendv4qiv4di2 = 13376,
  CODE_FOR_zero_extendv4qiv4di2 = 13377,
  CODE_FOR_extendv2qiv2di2 = 13378,
  CODE_FOR_zero_extendv2qiv2di2 = 13379,
  CODE_FOR_extendv8hiv8di2 = 13380,
  CODE_FOR_zero_extendv8hiv8di2 = 13381,
  CODE_FOR_extendv4hiv4di2 = 13382,
  CODE_FOR_zero_extendv4hiv4di2 = 13383,
  CODE_FOR_extendv2hiv2di2 = 13384,
  CODE_FOR_zero_extendv2hiv2di2 = 13385,
  CODE_FOR_extendv8siv8di2 = 13386,
  CODE_FOR_zero_extendv8siv8di2 = 13387,
  CODE_FOR_extendv4siv4di2 = 13388,
  CODE_FOR_zero_extendv4siv4di2 = 13389,
  CODE_FOR_extendv2siv2di2 = 13390,
  CODE_FOR_zero_extendv2siv2di2 = 13391,
  CODE_FOR_sse4_1_ptestzv16qi = 13392,
  CODE_FOR_sse4_1_ptestzv8hi = 13393,
  CODE_FOR_sse4_1_ptestzv4si = 13394,
  CODE_FOR_sse4_1_ptestzv2di = 13395,
  CODE_FOR_sse4_1_ptestzv1ti = 13396,
  CODE_FOR_sse4_1_ptestzv4sf = 13397,
  CODE_FOR_sse4_1_ptestzv2df = 13398,
  CODE_FOR_avx_ptestzv32qi = 13399,
  CODE_FOR_avx_ptestzv16hi = 13400,
  CODE_FOR_avx_ptestzv8si = 13401,
  CODE_FOR_avx_ptestzv4di = 13402,
  CODE_FOR_avx_ptestzv2ti = 13403,
  CODE_FOR_avx_ptestzv8sf = 13404,
  CODE_FOR_avx_ptestzv4df = 13405,
  CODE_FOR_sse4_1_ptestcv16qi = 13406,
  CODE_FOR_sse4_1_ptestcv8hi = 13407,
  CODE_FOR_sse4_1_ptestcv4si = 13408,
  CODE_FOR_sse4_1_ptestcv2di = 13409,
  CODE_FOR_sse4_1_ptestcv1ti = 13410,
  CODE_FOR_sse4_1_ptestcv4sf = 13411,
  CODE_FOR_sse4_1_ptestcv2df = 13412,
  CODE_FOR_avx_ptestcv32qi = 13413,
  CODE_FOR_avx_ptestcv16hi = 13414,
  CODE_FOR_avx_ptestcv8si = 13415,
  CODE_FOR_avx_ptestcv4di = 13416,
  CODE_FOR_avx_ptestcv2ti = 13417,
  CODE_FOR_avx_ptestcv8sf = 13418,
  CODE_FOR_avx_ptestcv4df = 13419,
  CODE_FOR_sse4_1_ptestv16qi = 13420,
  CODE_FOR_sse4_1_ptestv8hi = 13421,
  CODE_FOR_sse4_1_ptestv4si = 13422,
  CODE_FOR_sse4_1_ptestv2di = 13423,
  CODE_FOR_sse4_1_ptestv1ti = 13424,
  CODE_FOR_sse4_1_ptestv4sf = 13425,
  CODE_FOR_sse4_1_ptestv2df = 13426,
  CODE_FOR_avx_ptestv32qi = 13427,
  CODE_FOR_avx_ptestv16hi = 13428,
  CODE_FOR_avx_ptestv8si = 13429,
  CODE_FOR_avx_ptestv4di = 13430,
  CODE_FOR_avx_ptestv2ti = 13431,
  CODE_FOR_avx_ptestv8sf = 13432,
  CODE_FOR_avx_ptestv4df = 13433,
  CODE_FOR_nearbyintv32hf2 = 13434,
  CODE_FOR_nearbyintv16hf2 = 13435,
  CODE_FOR_nearbyintv8hf2 = 13436,
  CODE_FOR_nearbyintv16sf2 = 13437,
  CODE_FOR_nearbyintv8sf2 = 13438,
  CODE_FOR_nearbyintv4sf2 = 13439,
  CODE_FOR_nearbyintv8df2 = 13440,
  CODE_FOR_nearbyintv4df2 = 13441,
  CODE_FOR_nearbyintv2df2 = 13442,
  CODE_FOR_rintv32hf2 = 13443,
  CODE_FOR_rintv16hf2 = 13444,
  CODE_FOR_rintv8hf2 = 13445,
  CODE_FOR_rintv16sf2 = 13446,
  CODE_FOR_rintv8sf2 = 13447,
  CODE_FOR_rintv4sf2 = 13448,
  CODE_FOR_rintv8df2 = 13449,
  CODE_FOR_rintv4df2 = 13450,
  CODE_FOR_rintv2df2 = 13451,
  CODE_FOR_lrintv16sfv16si2 = 13452,
  CODE_FOR_lrintv8sfv8si2 = 13453,
  CODE_FOR_lrintv4sfv4si2 = 13454,
  CODE_FOR_lrintv8dfv8di2 = 13455,
  CODE_FOR_lrintv4dfv4di2 = 13456,
  CODE_FOR_lrintv2dfv2di2 = 13457,
  CODE_FOR_avx_roundps_sfix256 = 13458,
  CODE_FOR_sse4_1_roundps_sfix = 13459,
  CODE_FOR_avx512f_roundps512 = 13460,
  CODE_FOR_avx512f_roundpd512 = 13461,
  CODE_FOR_avx512f_roundps512_sfix = 13462,
  CODE_FOR_avx512f_roundpd_vec_pack_sfix512 = 13463,
  CODE_FOR_avx_roundpd_vec_pack_sfix256 = 13464,
  CODE_FOR_sse4_1_roundpd_vec_pack_sfix = 13465,
  CODE_FOR_floorv32hf2 = 13466,
  CODE_FOR_floorv16hf2 = 13467,
  CODE_FOR_floorv8hf2 = 13468,
  CODE_FOR_floorv16sf2 = 13469,
  CODE_FOR_floorv8sf2 = 13470,
  CODE_FOR_floorv4sf2 = 13471,
  CODE_FOR_floorv8df2 = 13472,
  CODE_FOR_floorv4df2 = 13473,
  CODE_FOR_floorv2df2 = 13474,
  CODE_FOR_lfloorv32hfv32hi2 = 13475,
  CODE_FOR_lfloorv16hfv16hi2 = 13476,
  CODE_FOR_lfloorv8hfv8hi2 = 13477,
  CODE_FOR_lfloorv16sfv16si2 = 13478,
  CODE_FOR_lfloorv8sfv8si2 = 13479,
  CODE_FOR_lfloorv4sfv4si2 = 13480,
  CODE_FOR_lfloorv8dfv8di2 = 13481,
  CODE_FOR_lfloorv4dfv4di2 = 13482,
  CODE_FOR_lfloorv2dfv2di2 = 13483,
  CODE_FOR_ceilv32hf2 = 13484,
  CODE_FOR_ceilv16hf2 = 13485,
  CODE_FOR_ceilv8hf2 = 13486,
  CODE_FOR_ceilv16sf2 = 13487,
  CODE_FOR_ceilv8sf2 = 13488,
  CODE_FOR_ceilv4sf2 = 13489,
  CODE_FOR_ceilv8df2 = 13490,
  CODE_FOR_ceilv4df2 = 13491,
  CODE_FOR_ceilv2df2 = 13492,
  CODE_FOR_lceilv32hfv32hi2 = 13493,
  CODE_FOR_lceilv16hfv16hi2 = 13494,
  CODE_FOR_lceilv8hfv8hi2 = 13495,
  CODE_FOR_lceilv16sfv16si2 = 13496,
  CODE_FOR_lceilv8sfv8si2 = 13497,
  CODE_FOR_lceilv4sfv4si2 = 13498,
  CODE_FOR_lceilv8dfv8di2 = 13499,
  CODE_FOR_lceilv4dfv4di2 = 13500,
  CODE_FOR_lceilv2dfv2di2 = 13501,
  CODE_FOR_btruncv32hf2 = 13502,
  CODE_FOR_btruncv16hf2 = 13503,
  CODE_FOR_btruncv8hf2 = 13504,
  CODE_FOR_btruncv16sf2 = 13505,
  CODE_FOR_btruncv8sf2 = 13506,
  CODE_FOR_btruncv4sf2 = 13507,
  CODE_FOR_btruncv8df2 = 13508,
  CODE_FOR_btruncv4df2 = 13509,
  CODE_FOR_btruncv2df2 = 13510,
  CODE_FOR_roundv32hf2 = 13511,
  CODE_FOR_roundv16hf2 = 13512,
  CODE_FOR_roundv8hf2 = 13513,
  CODE_FOR_roundv16sf2 = 13514,
  CODE_FOR_roundv8sf2 = 13515,
  CODE_FOR_roundv4sf2 = 13516,
  CODE_FOR_roundv8df2 = 13517,
  CODE_FOR_roundv4df2 = 13518,
  CODE_FOR_roundv2df2 = 13519,
  CODE_FOR_lroundv32hfv32hi2 = 13520,
  CODE_FOR_lroundv16hfv16hi2 = 13521,
  CODE_FOR_lroundv8hfv8hi2 = 13522,
  CODE_FOR_lroundv16sfv16si2 = 13523,
  CODE_FOR_lroundv8sfv8si2 = 13524,
  CODE_FOR_lroundv4sfv4si2 = 13525,
  CODE_FOR_lroundv8dfv8di2 = 13526,
  CODE_FOR_lroundv4dfv4di2 = 13527,
  CODE_FOR_lroundv2dfv2di2 = 13528,
  CODE_FOR_roundv16sf2_sfix = 13529,
  CODE_FOR_roundv8sf2_sfix = 13530,
  CODE_FOR_roundv4sf2_sfix = 13531,
  CODE_FOR_roundv8df2_vec_pack_sfix = 13532,
  CODE_FOR_roundv4df2_vec_pack_sfix = 13533,
  CODE_FOR_roundv2df2_vec_pack_sfix = 13534,
  CODE_FOR_rotlv8hi3 = 13535,
  CODE_FOR_rotlv4si3 = 13536,
  CODE_FOR_rotlv2di3 = 13537,
  CODE_FOR_rotrv8hi3 = 13538,
  CODE_FOR_rotrv4si3 = 13539,
  CODE_FOR_rotrv2di3 = 13540,
  CODE_FOR_vrotrv16qi3 = 13541,
  CODE_FOR_vrotrv8hi3 = 13542,
  CODE_FOR_vrotrv4si3 = 13543,
  CODE_FOR_vrotrv2di3 = 13544,
  CODE_FOR_vrotlv16qi3 = 13545,
  CODE_FOR_vrotlv8hi3 = 13546,
  CODE_FOR_vrotlv4si3 = 13547,
  CODE_FOR_vrotlv2di3 = 13548,
  CODE_FOR_vlshrv16qi3 = 13549,
  CODE_FOR_vlshrv8hi3 = 13550,
  CODE_FOR_vlshrv4si3 = 13551,
  CODE_FOR_vlshrv2di3 = 13552,
  CODE_FOR_vashlv64qi3 = 13553,
  CODE_FOR_vlshrv64qi3 = 13554,
  CODE_FOR_vashrv64qi3 = 13555,
  CODE_FOR_vashlv32qi3 = 13556,
  CODE_FOR_vlshrv32qi3 = 13557,
  CODE_FOR_vashrv32qi3 = 13558,
  CODE_FOR_vashlv32hi3 = 13559,
  CODE_FOR_vlshrv32hi3 = 13560,
  CODE_FOR_vashrv32hi3 = 13561,
  CODE_FOR_vashlv16hi3 = 13562,
  CODE_FOR_vlshrv16hi3 = 13563,
  CODE_FOR_vashrv16hi3 = 13564,
  CODE_FOR_vlshrv16si3 = 13565,
  CODE_FOR_vlshrv8di3 = 13566,
  CODE_FOR_vlshrv8si3 = 13567,
  CODE_FOR_vlshrv4di3 = 13568,
  CODE_FOR_vashrv8di3 = 13569,
  CODE_FOR_vashrv4di3 = 13570,
  CODE_FOR_vashrv16qi3 = 13571,
  CODE_FOR_vashrv8hi3 = 13572,
  CODE_FOR_vashrv2di3 = 13573,
  CODE_FOR_vashrv4si3 = 13574,
  CODE_FOR_vashrv16si3 = 13575,
  CODE_FOR_vashrv8si3 = 13576,
  CODE_FOR_vashlv16qi3 = 13577,
  CODE_FOR_vashlv8hi3 = 13578,
  CODE_FOR_vashlv4si3 = 13579,
  CODE_FOR_vashlv2di3 = 13580,
  CODE_FOR_vashlv16si3 = 13581,
  CODE_FOR_vashlv8di3 = 13582,
  CODE_FOR_vashlv8si3 = 13583,
  CODE_FOR_vashlv4di3 = 13584,
  CODE_FOR_ashlv64qi3 = 13585,
  CODE_FOR_lshrv64qi3 = 13586,
  CODE_FOR_ashrv64qi3 = 13587,
  CODE_FOR_ashlv32qi3 = 13588,
  CODE_FOR_lshrv32qi3 = 13589,
  CODE_FOR_ashrv32qi3 = 13590,
  CODE_FOR_ashlv16qi3 = 13591,
  CODE_FOR_lshrv16qi3 = 13592,
  CODE_FOR_ashrv16qi3 = 13593,
  CODE_FOR_rotlv64qi3 = 13594,
  CODE_FOR_rotrv64qi3 = 13595,
  CODE_FOR_rotlv32qi3 = 13596,
  CODE_FOR_rotrv32qi3 = 13597,
  CODE_FOR_rotlv16qi3 = 13598,
  CODE_FOR_rotrv16qi3 = 13599,
  CODE_FOR_ashrv2di3 = 13600,
  CODE_FOR_xop_vmfrczv4sf2 = 13601,
  CODE_FOR_xop_vmfrczv2df2 = 13602,
  CODE_FOR_avx_vzeroall = 13603,
  CODE_FOR_avx_vzeroupper = 13604,
  CODE_FOR_avx512f_vpermilv8df = 13605,
  CODE_FOR_avx512f_vpermilv8df_mask = 13606,
  CODE_FOR_avx_vpermilv4df = 13607,
  CODE_FOR_avx_vpermilv4df_mask = 13608,
  CODE_FOR_avx_vpermilv2df = 13609,
  CODE_FOR_avx_vpermilv2df_mask = 13610,
  CODE_FOR_avx512f_vpermilv16sf = 13611,
  CODE_FOR_avx512f_vpermilv16sf_mask = 13612,
  CODE_FOR_avx_vpermilv8sf = 13613,
  CODE_FOR_avx_vpermilv8sf_mask = 13614,
  CODE_FOR_avx_vpermilv4sf = 13615,
  CODE_FOR_avx_vpermilv4sf_mask = 13616,
  CODE_FOR_avx2_permv4di = 13617,
  CODE_FOR_avx2_permv4df = 13618,
  CODE_FOR_avx512vl_permv4di_mask = 13619,
  CODE_FOR_avx512vl_permv4df_mask = 13620,
  CODE_FOR_avx512f_permv8df = 13621,
  CODE_FOR_avx512f_permv8di = 13622,
  CODE_FOR_avx512f_permv8df_mask = 13623,
  CODE_FOR_avx512f_permv8di_mask = 13624,
  CODE_FOR_avx512f_vpermi2varv16si3_mask = 13625,
  CODE_FOR_avx512f_vpermi2varv16sf3_mask = 13626,
  CODE_FOR_avx512f_vpermi2varv8di3_mask = 13627,
  CODE_FOR_avx512f_vpermi2varv8df3_mask = 13628,
  CODE_FOR_avx512vl_vpermi2varv8si3_mask = 13629,
  CODE_FOR_avx512vl_vpermi2varv8sf3_mask = 13630,
  CODE_FOR_avx512vl_vpermi2varv4di3_mask = 13631,
  CODE_FOR_avx512vl_vpermi2varv4df3_mask = 13632,
  CODE_FOR_avx512vl_vpermi2varv4si3_mask = 13633,
  CODE_FOR_avx512vl_vpermi2varv4sf3_mask = 13634,
  CODE_FOR_avx512vl_vpermi2varv2di3_mask = 13635,
  CODE_FOR_avx512vl_vpermi2varv2df3_mask = 13636,
  CODE_FOR_avx512bw_vpermi2varv32hi3_mask = 13637,
  CODE_FOR_avx512vl_vpermi2varv16hi3_mask = 13638,
  CODE_FOR_avx512vl_vpermi2varv8hi3_mask = 13639,
  CODE_FOR_avx512bw_vpermi2varv64qi3_mask = 13640,
  CODE_FOR_avx512vl_vpermi2varv32qi3_mask = 13641,
  CODE_FOR_avx512vl_vpermi2varv16qi3_mask = 13642,
  CODE_FOR_avx512f_vpermt2varv16si3_maskz = 13643,
  CODE_FOR_avx512f_vpermt2varv16sf3_maskz = 13644,
  CODE_FOR_avx512f_vpermt2varv8di3_maskz = 13645,
  CODE_FOR_avx512f_vpermt2varv8df3_maskz = 13646,
  CODE_FOR_avx512vl_vpermt2varv8si3_maskz = 13647,
  CODE_FOR_avx512vl_vpermt2varv8sf3_maskz = 13648,
  CODE_FOR_avx512vl_vpermt2varv4di3_maskz = 13649,
  CODE_FOR_avx512vl_vpermt2varv4df3_maskz = 13650,
  CODE_FOR_avx512vl_vpermt2varv4si3_maskz = 13651,
  CODE_FOR_avx512vl_vpermt2varv4sf3_maskz = 13652,
  CODE_FOR_avx512vl_vpermt2varv2di3_maskz = 13653,
  CODE_FOR_avx512vl_vpermt2varv2df3_maskz = 13654,
  CODE_FOR_avx512bw_vpermt2varv32hi3_maskz = 13655,
  CODE_FOR_avx512vl_vpermt2varv16hi3_maskz = 13656,
  CODE_FOR_avx512vl_vpermt2varv8hi3_maskz = 13657,
  CODE_FOR_avx512bw_vpermt2varv64qi3_maskz = 13658,
  CODE_FOR_avx512vl_vpermt2varv32qi3_maskz = 13659,
  CODE_FOR_avx512vl_vpermt2varv16qi3_maskz = 13660,
  CODE_FOR_avx_vperm2f128v8si3 = 13661,
  CODE_FOR_avx_vperm2f128v8sf3 = 13662,
  CODE_FOR_avx_vperm2f128v4df3 = 13663,
  CODE_FOR_avx512vl_vinsertv8si = 13664,
  CODE_FOR_avx512vl_vinsertv8sf = 13665,
  CODE_FOR_avx512vl_vinsertv4di = 13666,
  CODE_FOR_avx512vl_vinsertv4df = 13667,
  CODE_FOR_avx_vinsertf128v32qi = 13668,
  CODE_FOR_avx_vinsertf128v16hi = 13669,
  CODE_FOR_avx_vinsertf128v8si = 13670,
  CODE_FOR_avx_vinsertf128v4di = 13671,
  CODE_FOR_avx_vinsertf128v8sf = 13672,
  CODE_FOR_avx_vinsertf128v4df = 13673,
  CODE_FOR_avx_vinsertf128v16hf = 13674,
  CODE_FOR_avx_vinsertf128v16bf = 13675,
  CODE_FOR_maskloadv4sfv4si_1 = 13676,
  CODE_FOR_maskloadv2dfv2di_1 = 13677,
  CODE_FOR_maskloadv4div4di_1 = 13678,
  CODE_FOR_maskloadv2div2di_1 = 13679,
  CODE_FOR_maskloadv8sfv8si_1 = 13680,
  CODE_FOR_maskloadv4dfv4di_1 = 13681,
  CODE_FOR_maskloadv8siv8si_1 = 13682,
  CODE_FOR_maskloadv4siv4si_1 = 13683,
  CODE_FOR_maskloadv4sfv4si = 13684,
  CODE_FOR_maskloadv2dfv2di = 13685,
  CODE_FOR_maskloadv4div4di = 13686,
  CODE_FOR_maskloadv2div2di = 13687,
  CODE_FOR_maskloadv8sfv8si = 13688,
  CODE_FOR_maskloadv4dfv4di = 13689,
  CODE_FOR_maskloadv8siv8si = 13690,
  CODE_FOR_maskloadv4siv4si = 13691,
  CODE_FOR_maskloadv16sihi = 13692,
  CODE_FOR_maskloadv8siqi = 13693,
  CODE_FOR_maskloadv4siqi = 13694,
  CODE_FOR_maskloadv8diqi = 13695,
  CODE_FOR_maskloadv4diqi = 13696,
  CODE_FOR_maskloadv2diqi = 13697,
  CODE_FOR_maskloadv16sfhi = 13698,
  CODE_FOR_maskloadv8sfqi = 13699,
  CODE_FOR_maskloadv4sfqi = 13700,
  CODE_FOR_maskloadv8dfqi = 13701,
  CODE_FOR_maskloadv4dfqi = 13702,
  CODE_FOR_maskloadv2dfqi = 13703,
  CODE_FOR_maskloadv64qidi = 13704,
  CODE_FOR_maskloadv16qihi = 13705,
  CODE_FOR_maskloadv32qisi = 13706,
  CODE_FOR_maskloadv32hisi = 13707,
  CODE_FOR_maskloadv16hihi = 13708,
  CODE_FOR_maskloadv8hiqi = 13709,
  CODE_FOR_maskloadv32hfsi = 13710,
  CODE_FOR_maskloadv16hfhi = 13711,
  CODE_FOR_maskloadv8hfqi = 13712,
  CODE_FOR_maskloadv32bfsi = 13713,
  CODE_FOR_maskloadv16bfhi = 13714,
  CODE_FOR_maskloadv8bfqi = 13715,
  CODE_FOR_maskstorev4sfv4si = 13716,
  CODE_FOR_maskstorev2dfv2di = 13717,
  CODE_FOR_maskstorev4div4di = 13718,
  CODE_FOR_maskstorev2div2di = 13719,
  CODE_FOR_maskstorev8sfv8si = 13720,
  CODE_FOR_maskstorev4dfv4di = 13721,
  CODE_FOR_maskstorev8siv8si = 13722,
  CODE_FOR_maskstorev4siv4si = 13723,
  CODE_FOR_maskstorev16sihi = 13724,
  CODE_FOR_maskstorev8siqi = 13725,
  CODE_FOR_maskstorev4siqi = 13726,
  CODE_FOR_maskstorev8diqi = 13727,
  CODE_FOR_maskstorev4diqi = 13728,
  CODE_FOR_maskstorev2diqi = 13729,
  CODE_FOR_maskstorev16sfhi = 13730,
  CODE_FOR_maskstorev8sfqi = 13731,
  CODE_FOR_maskstorev4sfqi = 13732,
  CODE_FOR_maskstorev8dfqi = 13733,
  CODE_FOR_maskstorev4dfqi = 13734,
  CODE_FOR_maskstorev2dfqi = 13735,
  CODE_FOR_maskstorev64qidi = 13736,
  CODE_FOR_maskstorev16qihi = 13737,
  CODE_FOR_maskstorev32qisi = 13738,
  CODE_FOR_maskstorev32hisi = 13739,
  CODE_FOR_maskstorev16hihi = 13740,
  CODE_FOR_maskstorev8hiqi = 13741,
  CODE_FOR_maskstorev32hfsi = 13742,
  CODE_FOR_maskstorev16hfhi = 13743,
  CODE_FOR_maskstorev8hfqi = 13744,
  CODE_FOR_maskstorev32bfsi = 13745,
  CODE_FOR_maskstorev16bfhi = 13746,
  CODE_FOR_maskstorev8bfqi = 13747,
  CODE_FOR_cbranchv64qi4 = 13748,
  CODE_FOR_cbranchv32qi4 = 13749,
  CODE_FOR_cbranchv16qi4 = 13750,
  CODE_FOR_cbranchv32hi4 = 13751,
  CODE_FOR_cbranchv16hi4 = 13752,
  CODE_FOR_cbranchv8hi4 = 13753,
  CODE_FOR_cbranchv16si4 = 13754,
  CODE_FOR_cbranchv8si4 = 13755,
  CODE_FOR_cbranchv4si4 = 13756,
  CODE_FOR_cbranchv8di4 = 13757,
  CODE_FOR_cbranchv4di4 = 13758,
  CODE_FOR_cbranchv2di4 = 13759,
  CODE_FOR_vec_initv64qiqi = 13760,
  CODE_FOR_vec_initv32qiqi = 13761,
  CODE_FOR_vec_initv16qiqi = 13762,
  CODE_FOR_vec_initv32hihi = 13763,
  CODE_FOR_vec_initv16hihi = 13764,
  CODE_FOR_vec_initv8hihi = 13765,
  CODE_FOR_vec_initv16sisi = 13766,
  CODE_FOR_vec_initv8sisi = 13767,
  CODE_FOR_vec_initv4sisi = 13768,
  CODE_FOR_vec_initv8didi = 13769,
  CODE_FOR_vec_initv4didi = 13770,
  CODE_FOR_vec_initv2didi = 13771,
  CODE_FOR_vec_initv32hfhf = 13772,
  CODE_FOR_vec_initv16hfhf = 13773,
  CODE_FOR_vec_initv8hfhf = 13774,
  CODE_FOR_vec_initv32bfbf = 13775,
  CODE_FOR_vec_initv16bfbf = 13776,
  CODE_FOR_vec_initv8bfbf = 13777,
  CODE_FOR_vec_initv16sfsf = 13778,
  CODE_FOR_vec_initv8sfsf = 13779,
  CODE_FOR_vec_initv4sfsf = 13780,
  CODE_FOR_vec_initv8dfdf = 13781,
  CODE_FOR_vec_initv4dfdf = 13782,
  CODE_FOR_vec_initv2dfdf = 13783,
  CODE_FOR_vec_initv4titi = 13784,
  CODE_FOR_vec_initv2titi = 13785,
  CODE_FOR_vec_initv64qiv32qi = 13786,
  CODE_FOR_vec_initv32qiv16qi = 13787,
  CODE_FOR_vec_initv16qiv8qi = 13788,
  CODE_FOR_vec_initv32hiv16hi = 13789,
  CODE_FOR_vec_initv16hiv8hi = 13790,
  CODE_FOR_vec_initv8hiv4hi = 13791,
  CODE_FOR_vec_initv16siv8si = 13792,
  CODE_FOR_vec_initv8siv4si = 13793,
  CODE_FOR_vec_initv4siv2si = 13794,
  CODE_FOR_vec_initv8div4di = 13795,
  CODE_FOR_vec_initv4div2di = 13796,
  CODE_FOR_vec_initv32hfv16hf = 13797,
  CODE_FOR_vec_initv16hfv8hf = 13798,
  CODE_FOR_vec_initv8hfv4hf = 13799,
  CODE_FOR_vec_initv32bfv16bf = 13800,
  CODE_FOR_vec_initv16bfv8bf = 13801,
  CODE_FOR_vec_initv8bfv4bf = 13802,
  CODE_FOR_vec_initv16sfv8sf = 13803,
  CODE_FOR_vec_initv8sfv4sf = 13804,
  CODE_FOR_vec_initv4sfv2sf = 13805,
  CODE_FOR_vec_initv8dfv4df = 13806,
  CODE_FOR_vec_initv4dfv2df = 13807,
  CODE_FOR_vec_initv4tiv2ti = 13808,
  CODE_FOR_cond_ashlv32hi = 13809,
  CODE_FOR_cond_lshrv32hi = 13810,
  CODE_FOR_cond_ashrv32hi = 13811,
  CODE_FOR_cond_ashlv16hi = 13812,
  CODE_FOR_cond_lshrv16hi = 13813,
  CODE_FOR_cond_ashrv16hi = 13814,
  CODE_FOR_cond_ashlv8hi = 13815,
  CODE_FOR_cond_lshrv8hi = 13816,
  CODE_FOR_cond_ashrv8hi = 13817,
  CODE_FOR_cond_ashlv16si = 13818,
  CODE_FOR_cond_lshrv16si = 13819,
  CODE_FOR_cond_ashrv16si = 13820,
  CODE_FOR_cond_ashlv8si = 13821,
  CODE_FOR_cond_lshrv8si = 13822,
  CODE_FOR_cond_ashrv8si = 13823,
  CODE_FOR_cond_ashlv4si = 13824,
  CODE_FOR_cond_lshrv4si = 13825,
  CODE_FOR_cond_ashrv4si = 13826,
  CODE_FOR_cond_ashlv8di = 13827,
  CODE_FOR_cond_lshrv8di = 13828,
  CODE_FOR_cond_ashrv8di = 13829,
  CODE_FOR_cond_ashlv4di = 13830,
  CODE_FOR_cond_lshrv4di = 13831,
  CODE_FOR_cond_ashrv4di = 13832,
  CODE_FOR_cond_ashlv2di = 13833,
  CODE_FOR_cond_lshrv2di = 13834,
  CODE_FOR_cond_ashrv2di = 13835,
  CODE_FOR_vcvtps2ph_mask = 13836,
  CODE_FOR_vcvtps2ph = 13837,
  CODE_FOR_avx512f_vcvtps2ph512_mask_sae = 13838,
  CODE_FOR_avx2_gathersiv2di = 13839,
  CODE_FOR_avx2_gathersiv2df = 13840,
  CODE_FOR_avx2_gathersiv4di = 13841,
  CODE_FOR_avx2_gathersiv4df = 13842,
  CODE_FOR_avx2_gathersiv4si = 13843,
  CODE_FOR_avx2_gathersiv4sf = 13844,
  CODE_FOR_avx2_gathersiv8si = 13845,
  CODE_FOR_avx2_gathersiv8sf = 13846,
  CODE_FOR_avx2_gatherdiv2di = 13847,
  CODE_FOR_avx2_gatherdiv2df = 13848,
  CODE_FOR_avx2_gatherdiv4di = 13849,
  CODE_FOR_avx2_gatherdiv4df = 13850,
  CODE_FOR_avx2_gatherdiv4si = 13851,
  CODE_FOR_avx2_gatherdiv4sf = 13852,
  CODE_FOR_avx2_gatherdiv8si = 13853,
  CODE_FOR_avx2_gatherdiv8sf = 13854,
  CODE_FOR_avx512f_gathersiv16si = 13855,
  CODE_FOR_avx512f_gathersiv16sf = 13856,
  CODE_FOR_avx512f_gathersiv8di = 13857,
  CODE_FOR_avx512f_gathersiv8df = 13858,
  CODE_FOR_avx512vl_gathersiv8si = 13859,
  CODE_FOR_avx512vl_gathersiv8sf = 13860,
  CODE_FOR_avx512vl_gathersiv4di = 13861,
  CODE_FOR_avx512vl_gathersiv4df = 13862,
  CODE_FOR_avx512vl_gathersiv4si = 13863,
  CODE_FOR_avx512vl_gathersiv4sf = 13864,
  CODE_FOR_avx512vl_gathersiv2di = 13865,
  CODE_FOR_avx512vl_gathersiv2df = 13866,
  CODE_FOR_avx512f_gatherdiv16si = 13867,
  CODE_FOR_avx512f_gatherdiv16sf = 13868,
  CODE_FOR_avx512f_gatherdiv8di = 13869,
  CODE_FOR_avx512f_gatherdiv8df = 13870,
  CODE_FOR_avx512vl_gatherdiv8si = 13871,
  CODE_FOR_avx512vl_gatherdiv8sf = 13872,
  CODE_FOR_avx512vl_gatherdiv4di = 13873,
  CODE_FOR_avx512vl_gatherdiv4df = 13874,
  CODE_FOR_avx512vl_gatherdiv4si = 13875,
  CODE_FOR_avx512vl_gatherdiv4sf = 13876,
  CODE_FOR_avx512vl_gatherdiv2di = 13877,
  CODE_FOR_avx512vl_gatherdiv2df = 13878,
  CODE_FOR_avx512f_scattersiv16si = 13879,
  CODE_FOR_avx512f_scattersiv16sf = 13880,
  CODE_FOR_avx512f_scattersiv8di = 13881,
  CODE_FOR_avx512f_scattersiv8df = 13882,
  CODE_FOR_avx512vl_scattersiv8si = 13883,
  CODE_FOR_avx512vl_scattersiv8sf = 13884,
  CODE_FOR_avx512vl_scattersiv4di = 13885,
  CODE_FOR_avx512vl_scattersiv4df = 13886,
  CODE_FOR_avx512vl_scattersiv4si = 13887,
  CODE_FOR_avx512vl_scattersiv4sf = 13888,
  CODE_FOR_avx512vl_scattersiv2di = 13889,
  CODE_FOR_avx512vl_scattersiv2df = 13890,
  CODE_FOR_avx512f_scatterdiv16si = 13891,
  CODE_FOR_avx512f_scatterdiv16sf = 13892,
  CODE_FOR_avx512f_scatterdiv8di = 13893,
  CODE_FOR_avx512f_scatterdiv8df = 13894,
  CODE_FOR_avx512vl_scatterdiv8si = 13895,
  CODE_FOR_avx512vl_scatterdiv8sf = 13896,
  CODE_FOR_avx512vl_scatterdiv4di = 13897,
  CODE_FOR_avx512vl_scatterdiv4df = 13898,
  CODE_FOR_avx512vl_scatterdiv4si = 13899,
  CODE_FOR_avx512vl_scatterdiv4sf = 13900,
  CODE_FOR_avx512vl_scatterdiv2di = 13901,
  CODE_FOR_avx512vl_scatterdiv2df = 13902,
  CODE_FOR_avx512f_expandv16si_maskz = 13903,
  CODE_FOR_avx512f_expandv16sf_maskz = 13904,
  CODE_FOR_avx512f_expandv8di_maskz = 13905,
  CODE_FOR_avx512f_expandv8df_maskz = 13906,
  CODE_FOR_avx512vl_expandv8si_maskz = 13907,
  CODE_FOR_avx512vl_expandv8sf_maskz = 13908,
  CODE_FOR_avx512vl_expandv4di_maskz = 13909,
  CODE_FOR_avx512vl_expandv4df_maskz = 13910,
  CODE_FOR_avx512vl_expandv4si_maskz = 13911,
  CODE_FOR_avx512vl_expandv4sf_maskz = 13912,
  CODE_FOR_avx512vl_expandv2di_maskz = 13913,
  CODE_FOR_avx512vl_expandv2df_maskz = 13914,
  CODE_FOR_expandv64qi_maskz = 13915,
  CODE_FOR_expandv16qi_maskz = 13916,
  CODE_FOR_expandv32qi_maskz = 13917,
  CODE_FOR_expandv32hi_maskz = 13918,
  CODE_FOR_expandv16hi_maskz = 13919,
  CODE_FOR_expandv8hi_maskz = 13920,
  CODE_FOR_vpmadd52huqv8di_maskz = 13921,
  CODE_FOR_vpmadd52huqv4di_maskz = 13922,
  CODE_FOR_vpmadd52huqv2di_maskz = 13923,
  CODE_FOR_vpmadd52luqv8di_maskz = 13924,
  CODE_FOR_vpmadd52luqv4di_maskz = 13925,
  CODE_FOR_vpmadd52luqv2di_maskz = 13926,
  CODE_FOR_popcountv16si2 = 13927,
  CODE_FOR_popcountv8si2 = 13928,
  CODE_FOR_popcountv4si2 = 13929,
  CODE_FOR_popcountv8di2 = 13930,
  CODE_FOR_popcountv4di2 = 13931,
  CODE_FOR_popcountv2di2 = 13932,
  CODE_FOR_popcountv64qi2 = 13933,
  CODE_FOR_popcountv16qi2 = 13934,
  CODE_FOR_popcountv32qi2 = 13935,
  CODE_FOR_popcountv32hi2 = 13936,
  CODE_FOR_popcountv16hi2 = 13937,
  CODE_FOR_popcountv8hi2 = 13938,
  CODE_FOR_vpshrdv_v32hi_maskz = 13939,
  CODE_FOR_vpshrdv_v16si_maskz = 13940,
  CODE_FOR_vpshrdv_v8di_maskz = 13941,
  CODE_FOR_vpshrdv_v16hi_maskz = 13942,
  CODE_FOR_vpshrdv_v8si_maskz = 13943,
  CODE_FOR_vpshrdv_v4di_maskz = 13944,
  CODE_FOR_vpshrdv_v8hi_maskz = 13945,
  CODE_FOR_vpshrdv_v4si_maskz = 13946,
  CODE_FOR_vpshrdv_v2di_maskz = 13947,
  CODE_FOR_vpshldv_v32hi_maskz = 13948,
  CODE_FOR_vpshldv_v16si_maskz = 13949,
  CODE_FOR_vpshldv_v8di_maskz = 13950,
  CODE_FOR_vpshldv_v16hi_maskz = 13951,
  CODE_FOR_vpshldv_v8si_maskz = 13952,
  CODE_FOR_vpshldv_v4di_maskz = 13953,
  CODE_FOR_vpshldv_v8hi_maskz = 13954,
  CODE_FOR_vpshldv_v4si_maskz = 13955,
  CODE_FOR_vpshldv_v2di_maskz = 13956,
  CODE_FOR_usdot_prodv16siv64qi = 13957,
  CODE_FOR_usdot_prodv8siv32qi = 13958,
  CODE_FOR_usdot_prodv4siv16qi = 13959,
  CODE_FOR_vpdpbusd_v16si_maskz = 13960,
  CODE_FOR_vpdpbusd_v8si_maskz = 13961,
  CODE_FOR_vpdpbusd_v4si_maskz = 13962,
  CODE_FOR_vpdpbusds_v16si_maskz = 13963,
  CODE_FOR_vpdpbusds_v8si_maskz = 13964,
  CODE_FOR_vpdpbusds_v4si_maskz = 13965,
  CODE_FOR_vpdpwssd_v16si_maskz = 13966,
  CODE_FOR_vpdpwssd_v8si_maskz = 13967,
  CODE_FOR_vpdpwssd_v4si_maskz = 13968,
  CODE_FOR_vpdpwssds_v16si_maskz = 13969,
  CODE_FOR_vpdpwssds_v8si_maskz = 13970,
  CODE_FOR_vpdpwssds_v4si_maskz = 13971,
  CODE_FOR_movp2qi = 13972,
  CODE_FOR_movp2hi = 13973,
  CODE_FOR_avx512f_cvtne2ps2bf16_v32bf_maskz = 13974,
  CODE_FOR_avx512f_cvtne2ps2bf16_v16bf_maskz = 13975,
  CODE_FOR_avx512f_cvtne2ps2bf16_v8bf_maskz = 13976,
  CODE_FOR_truncv4sfv4bf2 = 13977,
  CODE_FOR_vcvtneps2bf16_v4sf = 13978,
  CODE_FOR_avx512f_cvtneps2bf16_v4sf_maskz = 13979,
  CODE_FOR_avx512f_cvtneps2bf16_v4sf_mask = 13980,
  CODE_FOR_avx512f_cvtneps2bf16_v16sf_maskz = 13981,
  CODE_FOR_avx512f_cvtneps2bf16_v8sf_maskz = 13982,
  CODE_FOR_truncv8sfv8bf2 = 13983,
  CODE_FOR_truncv16sfv16bf2 = 13984,
  CODE_FOR_extendv16bfv16sf2 = 13985,
  CODE_FOR_extendv8bfv8sf2 = 13986,
  CODE_FOR_extendv4bfv4sf2 = 13987,
  CODE_FOR_avx512f_dpbf16ps_v16sf_maskz = 13988,
  CODE_FOR_avx512f_dpbf16ps_v8sf_maskz = 13989,
  CODE_FOR_avx512f_dpbf16ps_v4sf_maskz = 13990,
  CODE_FOR_encodekey128u32 = 13991,
  CODE_FOR_encodekey256u32 = 13992,
  CODE_FOR_aesdecwide128klu8 = 13993,
  CODE_FOR_aesdecwide256klu8 = 13994,
  CODE_FOR_aesencwide128klu8 = 13995,
  CODE_FOR_aesencwide256klu8 = 13996,
  CODE_FOR_vec_duplicatev64qi = 13997,
  CODE_FOR_vec_duplicatev32qi = 13998,
  CODE_FOR_vec_duplicatev16qi = 13999,
  CODE_FOR_vec_duplicatev32hi = 14000,
  CODE_FOR_vec_duplicatev16hi = 14001,
  CODE_FOR_vec_duplicatev8hi = 14002,
  CODE_FOR_vec_duplicatev16si = 14003,
  CODE_FOR_vec_duplicatev8si = 14004,
  CODE_FOR_vec_duplicatev4si = 14005,
   CODE_FOR_vec_duplicatev8di = CODE_FOR_nothing,
   CODE_FOR_vec_duplicatev4di = CODE_FOR_nothing,
   CODE_FOR_vec_duplicatev2di = CODE_FOR_nothing,
  CODE_FOR_sdot_prodv16siv64qi = 14006,
  CODE_FOR_sdot_prodv8siv32qi = 14007,
  CODE_FOR_sdot_prodv4siv16qi = 14008,
  CODE_FOR_udot_prodv16siv64qi = 14009,
  CODE_FOR_udot_prodv8siv32qi = 14010,
  CODE_FOR_udot_prodv4siv16qi = 14011,
  CODE_FOR_vpdpbssd_v16si_maskz = 14012,
  CODE_FOR_vpdpbssds_v16si_maskz = 14013,
  CODE_FOR_vpdpbsud_v16si_maskz = 14014,
  CODE_FOR_vpdpbsuds_v16si_maskz = 14015,
  CODE_FOR_vpdpbuud_v16si_maskz = 14016,
  CODE_FOR_vpdpbuuds_v16si_maskz = 14017,
  CODE_FOR_vpdpbssd_v8si_maskz = 14018,
  CODE_FOR_vpdpbssds_v8si_maskz = 14019,
  CODE_FOR_vpdpbsud_v8si_maskz = 14020,
  CODE_FOR_vpdpbsuds_v8si_maskz = 14021,
  CODE_FOR_vpdpbuud_v8si_maskz = 14022,
  CODE_FOR_vpdpbuuds_v8si_maskz = 14023,
  CODE_FOR_vpdpbssd_v4si_maskz = 14024,
  CODE_FOR_vpdpbssds_v4si_maskz = 14025,
  CODE_FOR_vpdpbsud_v4si_maskz = 14026,
  CODE_FOR_vpdpbsuds_v4si_maskz = 14027,
  CODE_FOR_vpdpbuud_v4si_maskz = 14028,
  CODE_FOR_vpdpbuuds_v4si_maskz = 14029,
  CODE_FOR_vcvtbiasph2bf8v8hf = 14030,
  CODE_FOR_vcvtbiasph2bf8sv8hf = 14031,
  CODE_FOR_vcvtbiasph2hf8v8hf = 14032,
  CODE_FOR_vcvtbiasph2hf8sv8hf = 14033,
  CODE_FOR_vcvtbiasph2bf8v8hf_mask = 14034,
  CODE_FOR_vcvtbiasph2bf8sv8hf_mask = 14035,
  CODE_FOR_vcvtbiasph2hf8v8hf_mask = 14036,
  CODE_FOR_vcvtbiasph2hf8sv8hf_mask = 14037,
  CODE_FOR_vcvtph2bf8v8hf = 14038,
  CODE_FOR_vcvtph2bf8sv8hf = 14039,
  CODE_FOR_vcvtph2hf8v8hf = 14040,
  CODE_FOR_vcvtph2hf8sv8hf = 14041,
  CODE_FOR_vcvtph2bf8v8hf_mask = 14042,
  CODE_FOR_vcvtph2bf8sv8hf_mask = 14043,
  CODE_FOR_vcvtph2hf8v8hf_mask = 14044,
  CODE_FOR_vcvtph2hf8sv8hf_mask = 14045,
  CODE_FOR_usdot_prodv16siv32hi = 14046,
  CODE_FOR_usdot_prodv8siv16hi = 14047,
  CODE_FOR_usdot_prodv4siv8hi = 14048,
  CODE_FOR_udot_prodv16siv32hi = 14049,
  CODE_FOR_udot_prodv8siv16hi = 14050,
  CODE_FOR_udot_prodv4siv8hi = 14051,
  CODE_FOR_vpdpwusd_v16si_maskz = 14052,
  CODE_FOR_vpdpwusds_v16si_maskz = 14053,
  CODE_FOR_vpdpwsud_v16si_maskz = 14054,
  CODE_FOR_vpdpwsuds_v16si_maskz = 14055,
  CODE_FOR_vpdpwuud_v16si_maskz = 14056,
  CODE_FOR_vpdpwuuds_v16si_maskz = 14057,
  CODE_FOR_vpdpwusd_v8si_maskz = 14058,
  CODE_FOR_vpdpwusds_v8si_maskz = 14059,
  CODE_FOR_vpdpwsud_v8si_maskz = 14060,
  CODE_FOR_vpdpwsuds_v8si_maskz = 14061,
  CODE_FOR_vpdpwuud_v8si_maskz = 14062,
  CODE_FOR_vpdpwuuds_v8si_maskz = 14063,
  CODE_FOR_vpdpwusd_v4si_maskz = 14064,
  CODE_FOR_vpdpwusds_v4si_maskz = 14065,
  CODE_FOR_vpdpwsud_v4si_maskz = 14066,
  CODE_FOR_vpdpwsuds_v4si_maskz = 14067,
  CODE_FOR_vpdpwuud_v4si_maskz = 14068,
  CODE_FOR_vpdpwuuds_v4si_maskz = 14069,
  CODE_FOR_vdpphps_v16sf_maskz = 14070,
  CODE_FOR_vdpphps_v8sf_maskz = 14071,
  CODE_FOR_vdpphps_v4sf_maskz = 14072,
  CODE_FOR_smaxv32bf3 = 14073,
  CODE_FOR_sminv32bf3 = 14074,
  CODE_FOR_smaxv16bf3 = 14075,
  CODE_FOR_sminv16bf3 = 14076,
  CODE_FOR_smaxv8bf3 = 14077,
  CODE_FOR_sminv8bf3 = 14078,
  CODE_FOR_avx10_2_fmaddbf16_v32bf_maskz = 14079,
  CODE_FOR_avx10_2_fmaddbf16_v16bf_maskz = 14080,
  CODE_FOR_avx10_2_fmaddbf16_v8bf_maskz = 14081,
  CODE_FOR_avx10_2_fnmaddbf16_v32bf_maskz = 14082,
  CODE_FOR_avx10_2_fnmaddbf16_v16bf_maskz = 14083,
  CODE_FOR_avx10_2_fnmaddbf16_v8bf_maskz = 14084,
  CODE_FOR_avx10_2_fmsubbf16_v32bf_maskz = 14085,
  CODE_FOR_avx10_2_fmsubbf16_v16bf_maskz = 14086,
  CODE_FOR_avx10_2_fmsubbf16_v8bf_maskz = 14087,
  CODE_FOR_avx10_2_fnmsubbf16_v32bf_maskz = 14088,
  CODE_FOR_avx10_2_fnmsubbf16_v16bf_maskz = 14089,
  CODE_FOR_avx10_2_fnmsubbf16_v8bf_maskz = 14090,
  CODE_FOR_sse2_lfence = 14091,
  CODE_FOR_sse_sfence = 14092,
  CODE_FOR_sse2_mfence = 14093,
  CODE_FOR_mem_thread_fence = 14094,
  CODE_FOR_atomic_loadqi = 14095,
  CODE_FOR_atomic_loadhi = 14096,
  CODE_FOR_atomic_loadsi = 14097,
  CODE_FOR_atomic_loaddi = 14098,
  CODE_FOR_atomic_storeqi = 14099,
  CODE_FOR_atomic_storehi = 14100,
  CODE_FOR_atomic_storesi = 14101,
  CODE_FOR_atomic_storedi = 14102,
  CODE_FOR_atomic_compare_and_swapqi = 14103,
  CODE_FOR_atomic_compare_and_swaphi = 14104,
  CODE_FOR_atomic_compare_and_swapsi = 14105,
  CODE_FOR_atomic_compare_and_swapdi = 14106,
   CODE_FOR_atomic_compare_and_swapti = CODE_FOR_nothing,
  CODE_FOR_atomic_fetch_andqi = 14107,
  CODE_FOR_atomic_fetch_orqi = 14108,
  CODE_FOR_atomic_fetch_xorqi = 14109,
  CODE_FOR_atomic_fetch_andhi = 14110,
  CODE_FOR_atomic_fetch_orhi = 14111,
  CODE_FOR_atomic_fetch_xorhi = 14112,
  CODE_FOR_atomic_fetch_andsi = 14113,
  CODE_FOR_atomic_fetch_orsi = 14114,
  CODE_FOR_atomic_fetch_xorsi = 14115,
  CODE_FOR_atomic_and_fetchqi = 14116,
  CODE_FOR_atomic_or_fetchqi = 14117,
  CODE_FOR_atomic_xor_fetchqi = 14118,
  CODE_FOR_atomic_and_fetchhi = 14119,
  CODE_FOR_atomic_or_fetchhi = 14120,
  CODE_FOR_atomic_xor_fetchhi = 14121,
  CODE_FOR_atomic_and_fetchsi = 14122,
  CODE_FOR_atomic_or_fetchsi = 14123,
  CODE_FOR_atomic_xor_fetchsi = 14124,
  CODE_FOR_atomic_fetch_nandqi = 14125,
  CODE_FOR_atomic_fetch_nandhi = 14126,
  CODE_FOR_atomic_fetch_nandsi = 14127,
  CODE_FOR_atomic_nand_fetchqi = 14128,
  CODE_FOR_atomic_nand_fetchhi = 14129,
  CODE_FOR_atomic_nand_fetchsi = 14130,
  CODE_FOR_atomic_fetch_anddi = 14131,
  CODE_FOR_atomic_fetch_ordi = 14132,
  CODE_FOR_atomic_fetch_xordi = 14133,
   CODE_FOR_atomic_fetch_andti = CODE_FOR_nothing,
   CODE_FOR_atomic_fetch_orti = CODE_FOR_nothing,
   CODE_FOR_atomic_fetch_xorti = CODE_FOR_nothing,
  CODE_FOR_atomic_and_fetchdi = 14134,
  CODE_FOR_atomic_or_fetchdi = 14135,
  CODE_FOR_atomic_xor_fetchdi = 14136,
   CODE_FOR_atomic_and_fetchti = CODE_FOR_nothing,
   CODE_FOR_atomic_or_fetchti = CODE_FOR_nothing,
   CODE_FOR_atomic_xor_fetchti = CODE_FOR_nothing,
  CODE_FOR_atomic_fetch_nanddi = 14137,
   CODE_FOR_atomic_fetch_nandti = CODE_FOR_nothing,
  CODE_FOR_atomic_nand_fetchdi = 14138,
   CODE_FOR_atomic_nand_fetchti = CODE_FOR_nothing,
  CODE_FOR_atomic_bit_test_and_sethi = 14139,
  CODE_FOR_atomic_bit_test_and_setsi = 14140,
   CODE_FOR_atomic_bit_test_and_setdi = CODE_FOR_nothing,
  CODE_FOR_atomic_bit_test_and_complementhi = 14141,
  CODE_FOR_atomic_bit_test_and_complementsi = 14142,
   CODE_FOR_atomic_bit_test_and_complementdi = CODE_FOR_nothing,
  CODE_FOR_atomic_bit_test_and_resethi = 14143,
  CODE_FOR_atomic_bit_test_and_resetsi = 14144,
   CODE_FOR_atomic_bit_test_and_resetdi = CODE_FOR_nothing,
  CODE_FOR_atomic_add_fetch_cmp_0qi = 14145,
  CODE_FOR_atomic_sub_fetch_cmp_0qi = 14146,
  CODE_FOR_atomic_add_fetch_cmp_0hi = 14147,
  CODE_FOR_atomic_sub_fetch_cmp_0hi = 14148,
  CODE_FOR_atomic_add_fetch_cmp_0si = 14149,
  CODE_FOR_atomic_sub_fetch_cmp_0si = 14150,
   CODE_FOR_atomic_add_fetch_cmp_0di = CODE_FOR_nothing,
   CODE_FOR_atomic_sub_fetch_cmp_0di = CODE_FOR_nothing,
  CODE_FOR_atomic_and_fetch_cmp_0qi = 14151,
  CODE_FOR_atomic_or_fetch_cmp_0qi = 14152,
  CODE_FOR_atomic_xor_fetch_cmp_0qi = 14153,
  CODE_FOR_atomic_and_fetch_cmp_0hi = 14154,
  CODE_FOR_atomic_or_fetch_cmp_0hi = 14155,
  CODE_FOR_atomic_xor_fetch_cmp_0hi = 14156,
  CODE_FOR_atomic_and_fetch_cmp_0si = 14157,
  CODE_FOR_atomic_or_fetch_cmp_0si = 14158,
  CODE_FOR_atomic_xor_fetch_cmp_0si = 14159,
   CODE_FOR_atomic_and_fetch_cmp_0di = CODE_FOR_nothing,
   CODE_FOR_atomic_or_fetch_cmp_0di = CODE_FOR_nothing,
   CODE_FOR_atomic_xor_fetch_cmp_0di = CODE_FOR_nothing
};

const unsigned int NUM_INSN_CODES = 14160;
#endif /* GCC_INSN_CODES_H */
